open AST
open BinNums
open Coloring
open Coqlib
open Datatypes
open Errors
open LTL
open Lattice
open List0
open Locations
open Maps
open Op
open RTL
open RTLtyping
open Registers

val reg_option_live : reg option -> Regset.t -> Regset.t

val reg_sum_live : (reg, ident) sum -> Regset.t -> Regset.t

val reg_list_live : reg list -> Regset.t -> Regset.t

val transfer : coq_function -> node -> Regset.t -> Regset.t

module DS : 
 sig 
  module L : 
  sig 
    type t = Regset.t
    val beq : t -> t -> bool
    val bot : t
    val lub : t -> t -> t
  end
  
  val fixpoint :
    positive list PTree.t -> (positive -> L.t -> L.t) -> (positive * L.t)
    list -> L.t PMap.t option
 end

val analyze : coq_function -> Regset.t PMap.t option

val is_redundant_move : operation -> reg list -> reg -> (reg -> loc) -> bool

val transf_instr :
  coq_function -> Regset.t PMap.t -> (reg -> loc) -> LTL.node -> instruction
  -> LTL.instruction

val transf_fun :
  coq_function -> Regset.t PMap.t -> (reg -> loc) -> LTL.coq_function

val live0 : coq_function -> Regset.t PMap.t -> Regset.t

val transf_function : coq_function -> LTL.coq_function res

val transf_fundef : fundef -> LTL.fundef res

val transf_program : program -> LTL.program res

