open AST
open BinNums
open Datatypes
open Floats
open Integers
open Machregs

type ireg =
| EAX
| EBX
| ECX
| EDX
| ESI
| EDI
| EBP
| ESP

type freg =
| XMM0
| XMM1
| XMM2
| XMM3
| XMM4
| XMM5
| XMM6
| XMM7

val ireg_eq : ireg -> ireg -> bool

type crbit =
| ZF
| CF
| PF
| SOF

type preg =
| PC
| IR of ireg
| FR of freg
| ST0
| CR of crbit
| RA

type label = positive

type addrmode =
| Addrmode of ireg option * (ireg * Int.int) option
   * (Int.int, ident * Int.int) sum

type testcond =
| Cond_e
| Cond_ne
| Cond_b
| Cond_be
| Cond_ae
| Cond_a
| Cond_l
| Cond_le
| Cond_ge
| Cond_g
| Cond_p
| Cond_np

type instruction =
| Pmov_rr of ireg * ireg
| Pmov_ri of ireg * Int.int
| Pmov_ra of ireg * ident
| Pmov_rm of ireg * addrmode
| Pmov_mr of addrmode * ireg
| Pmovd_fr of freg * ireg
| Pmovd_rf of ireg * freg
| Pmovsd_ff of freg * freg
| Pmovsd_fi of freg * float
| Pmovsd_fm of freg * addrmode
| Pmovsd_mf of addrmode * freg
| Pfld_f of freg
| Pfld_m of addrmode
| Pfstp_f of freg
| Pfstp_m of addrmode
| Pxchg_rr of ireg * ireg
| Pmovb_mr of addrmode * ireg
| Pmovw_mr of addrmode * ireg
| Pmovzb_rr of ireg * ireg
| Pmovzb_rm of ireg * addrmode
| Pmovsb_rr of ireg * ireg
| Pmovsb_rm of ireg * addrmode
| Pmovzw_rr of ireg * ireg
| Pmovzw_rm of ireg * addrmode
| Pmovsw_rr of ireg * ireg
| Pmovsw_rm of ireg * addrmode
| Pcvtss2sd_fm of freg * addrmode
| Pcvtsd2ss_ff of freg * freg
| Pcvtsd2ss_mf of addrmode * freg
| Pcvttsd2si_rf of ireg * freg
| Pcvtsi2sd_fr of freg * ireg
| Plea of ireg * addrmode
| Pneg of ireg
| Psub_rr of ireg * ireg
| Pimul_rr of ireg * ireg
| Pimul_ri of ireg * Int.int
| Pdiv of ireg
| Pidiv of ireg
| Pand_rr of ireg * ireg
| Pand_ri of ireg * Int.int
| Por_rr of ireg * ireg
| Por_ri of ireg * Int.int
| Pxor_r of ireg
| Pxor_rr of ireg * ireg
| Pxor_ri of ireg * Int.int
| Psal_rcl of ireg
| Psal_ri of ireg * Int.int
| Pshr_rcl of ireg
| Pshr_ri of ireg * Int.int
| Psar_rcl of ireg
| Psar_ri of ireg * Int.int
| Pror_ri of ireg * Int.int
| Pcmp_rr of ireg * ireg
| Pcmp_ri of ireg * Int.int
| Ptest_rr of ireg * ireg
| Ptest_ri of ireg * Int.int
| Pcmov of testcond * ireg * ireg
| Psetcc of testcond * ireg
| Paddd_ff of freg * freg
| Psubd_ff of freg * freg
| Pmuld_ff of freg * freg
| Pdivd_ff of freg * freg
| Pnegd of freg
| Pabsd of freg
| Pcomisd_ff of freg * freg
| Pxorpd_f of freg
| Pjmp_l of label
| Pjmp_s of ident
| Pjmp_r of ireg
| Pjcc of testcond * label
| Pjcc2 of testcond * testcond * label
| Pjmptbl of ireg * label list
| Pcall_s of ident
| Pcall_r of ireg
| Pret
| Plabel of label
| Pallocframe of coq_Z * Int.int * Int.int
| Pfreeframe of coq_Z * Int.int * Int.int
| Pbuiltin of external_function * preg list * preg
| Pannot of external_function * annot_param list
and annot_param =
| APreg of preg
| APstack of memory_chunk * coq_Z

type code = instruction list

type fundef = code AST.fundef

type program = (fundef, unit) AST.program

val preg_of : mreg -> preg

