open AST
open Asm
open Coqlib
open Datatypes
open Errors
open Floats
open Integers
open List0
open Mach
open Machregs
open Op

let ireg_of :  mreg -> ireg res =
  fun r  ->
  match preg_of r with
  | IR mr -> OK mr
  | _ ->
    Error
      (msg
        ('A'::('s'::('m'::('g'::('e'::('n'::('.'::('i'::('r'::('e'::('g'::('_'::('o'::('f'::[])))))))))))))))

let freg_of :  mreg -> freg res =
  fun r  ->
  match preg_of r with
  | FR mr -> OK mr
  | _ ->
    Error
      (msg
        ('A'::('s'::('m'::('g'::('e'::('n'::('.'::('f'::('r'::('e'::('g'::('_'::('o'::('f'::[])))))))))))))))

let mk_mov :  preg -> preg -> Asm.code -> Asm.code res =
  fun rd rs k  ->
  match rd with
  | IR rd0 ->
    (match rs with
     | IR rs0 -> OK ((Pmov_rr (rd0, rs0)) :: k)
     | _ ->
       Error
         (msg
           ('A'::('s'::('m'::('g'::('e'::('n'::('.'::('m'::('k'::('_'::('m'::('o'::('v'::[])))))))))))))))
  | FR rd0 ->
    (match rs with
     | FR rs0 -> OK ((Pmovsd_ff (rd0, rs0)) :: k)
     | ST0 -> OK ((Pfstp_f rd0) :: k)
     | _ ->
       Error
         (msg
           ('A'::('s'::('m'::('g'::('e'::('n'::('.'::('m'::('k'::('_'::('m'::('o'::('v'::[])))))))))))))))
  | ST0 ->
    (match rs with
     | FR rs0 -> OK ((Pfld_f rs0) :: k)
     | _ ->
       Error
         (msg
           ('A'::('s'::('m'::('g'::('e'::('n'::('.'::('m'::('k'::('_'::('m'::('o'::('v'::[])))))))))))))))
  | _ ->
    Error
      (msg
        ('A'::('s'::('m'::('g'::('e'::('n'::('.'::('m'::('k'::('_'::('m'::('o'::('v'::[]))))))))))))))

let mk_shift :     (ireg -> Asm.instruction) -> ireg -> ireg -> Asm.code -> Asm.code res =
  fun shift_instr r1 r2 k  ->
  if ireg_eq r2 ECX
  then OK ((shift_instr r1) :: k)
  else if ireg_eq r1 ECX
       then OK ((Pxchg_rr (r2, ECX)) :: ((shift_instr r2) :: ((Pxchg_rr (r2,
              ECX)) :: k)))
       else OK ((Pmov_rr (ECX, r2)) :: ((shift_instr r1) :: k))

let mk_mov2 :  ireg -> ireg -> ireg -> ireg -> Asm.code -> Asm.code =
  fun src1 dst1 src2 dst2 k  ->
  if ireg_eq src1 dst1
  then (Pmov_rr (dst2, src2)) :: k
  else if ireg_eq src2 dst2
       then (Pmov_rr (dst1, src1)) :: k
       else if ireg_eq src2 dst1
            then if ireg_eq src1 dst2
                 then (Pxchg_rr (src1, src2)) :: k
                 else (Pmov_rr (dst2, src2)) :: ((Pmov_rr (dst1, src1)) :: k)
            else (Pmov_rr (dst1, src1)) :: ((Pmov_rr (dst2, src2)) :: k)

let mk_div :     (ireg -> Asm.instruction) -> ireg -> ireg -> Asm.code -> Asm.code res =
  fun div_instr r1 r2 k  ->
  if ireg_eq r1 EAX
  then if ireg_eq r2 EDX
       then OK ((Pmov_rr (ECX, EDX)) :: ((div_instr ECX) :: k))
       else OK ((div_instr r2) :: k)
  else OK ((Pmovd_fr (XMM7,
         EAX)) :: (mk_mov2 r1 EAX r2 ECX ((div_instr ECX) :: ((Pmov_rr (r1,
                    EAX)) :: ((Pmovd_rf (EAX, XMM7)) :: k)))))

let mk_mod :     (ireg -> Asm.instruction) -> ireg -> ireg -> Asm.code -> Asm.code res =
  fun div_instr r1 r2 k  ->
  if ireg_eq r1 EAX
  then if ireg_eq r2 EDX
       then OK ((Pmov_rr (ECX, EDX)) :: ((div_instr ECX) :: ((Pmov_rr (EAX,
              EDX)) :: k)))
       else OK ((div_instr r2) :: ((Pmov_rr (EAX, EDX)) :: k))
  else OK ((Pmovd_fr (XMM7,
         EAX)) :: (mk_mov2 r1 EAX r2 ECX ((div_instr ECX) :: ((Pmov_rr (r1,
                    EDX)) :: ((Pmovd_rf (EAX, XMM7)) :: k)))))

let mk_shrximm :  ireg -> Int.int -> Asm.code -> Asm.code res =
  fun r n k  ->
  let tmp = if ireg_eq r ECX then EDX else ECX in
  let p = Int.sub (Int.shl Int.one n) Int.one in
  OK ((Ptest_rr (r, r)) :: ((Plea (tmp, (Addrmode ((Some r), None, (Coq_inl
  p))))) :: ((Pcmov (Cond_l, r, tmp)) :: ((Psar_ri (r, n)) :: k))))

let low_ireg :  ireg -> bool =
    function
| EAX -> true
| EBX -> true
| ECX -> true
| EDX -> true
| _ -> false

let mk_intconv :     (ireg -> ireg -> Asm.instruction) -> ireg -> ireg -> Asm.code ->
    Asm.instruction list res =
  fun mk rd rs k  ->
  if low_ireg rs
  then OK ((mk rd rs) :: k)
  else OK ((Pmov_rr (EDX, rs)) :: ((mk rd EDX) :: k))

let addressing_mentions :  addrmode -> ireg -> bool =
  fun addr r  ->
  let (Addrmode (coqzz_base, displ, coqzz_const)) = addr in
  (||)
    (match coqzz_base with
     | Some r' -> proj_sumbool (ireg_eq r r')
     | None -> false)
    (match displ with
     | Some p -> let (r', sc) = p in proj_sumbool (ireg_eq r r')
     | None -> false)

let mk_smallstore :     (addrmode -> ireg -> Asm.instruction) -> addrmode -> ireg -> Asm.code ->
    Asm.instruction list res =
  fun sto addr rs k  ->
  if low_ireg rs
  then OK ((sto addr rs) :: k)
  else if addressing_mentions addr ECX
       then OK ((Plea (ECX, addr)) :: ((Pmov_rr (EDX,
              rs)) :: ((sto (Addrmode ((Some ECX), None, (Coq_inl Int.zero)))
                         EDX) :: k)))
       else OK ((Pmov_rr (ECX, rs)) :: ((sto addr ECX) :: k))

let loadind :     ireg -> Int.int -> typ -> mreg -> Asm.code -> Asm.instruction list res =
  fun coqzz_base ofs ty dst k  ->
  match ty with
  | Tint ->
    (match ireg_of dst with
     | OK x ->
       OK ((Pmov_rm (x, (Addrmode ((Some coqzz_base), None, (Coq_inl ofs))))) :: k)
     | Error msg0 -> Error msg0)
  | Tfloat ->
    (match preg_of dst with
     | FR r ->
       OK ((Pmovsd_fm (r, (Addrmode ((Some coqzz_base), None, (Coq_inl
         ofs))))) :: k)
     | ST0 ->
       OK ((Pfld_m (Addrmode ((Some coqzz_base), None, (Coq_inl ofs)))) :: k)
     | _ ->
       Error
         (msg
           ('A'::('s'::('m'::('g'::('e'::('n'::('.'::('l'::('o'::('a'::('d'::('i'::('n'::('d'::[]))))))))))))))))

let storeind :     mreg -> ireg -> Int.int -> typ -> Asm.code -> Asm.instruction list res =
  fun src coqzz_base ofs ty k  ->
  match ty with
  | Tint ->
    (match ireg_of src with
     | OK x ->
       OK ((Pmov_mr ((Addrmode ((Some coqzz_base), None, (Coq_inl ofs))), x)) :: k)
     | Error msg0 -> Error msg0)
  | Tfloat ->
    (match preg_of src with
     | FR r ->
       OK ((Pmovsd_mf ((Addrmode ((Some coqzz_base), None, (Coq_inl ofs))),
         r)) :: k)
     | ST0 ->
       OK ((Pfstp_m (Addrmode ((Some coqzz_base), None, (Coq_inl ofs)))) :: k)
     | _ ->
       Error
         (msg
           ('A'::('s'::('m'::('g'::('e'::('n'::('.'::('l'::('o'::('a'::('d'::('i'::('n'::('d'::[]))))))))))))))))

let transl_addressing :  addressing -> mreg list -> addrmode res =
  fun a args  ->
  match a with
  | Aindexed n ->
    (match args with
     | [] ->
       Error
         (msg
           ('A'::('s'::('m'::('g'::('e'::('n'::('.'::('t'::('r'::('a'::('n'::('s'::('l'::('_'::('a'::('d'::('d'::('r'::('e'::('s'::('s'::('i'::('n'::('g'::[])))))))))))))))))))))))))
     | a1 :: l ->
       (match l with
        | [] ->
          (match ireg_of a1 with
           | OK x -> OK (Addrmode ((Some x), None, (Coq_inl n)))
           | Error msg0 -> Error msg0)
        | m :: l0 ->
          Error
            (msg
              ('A'::('s'::('m'::('g'::('e'::('n'::('.'::('t'::('r'::('a'::('n'::('s'::('l'::('_'::('a'::('d'::('d'::('r'::('e'::('s'::('s'::('i'::('n'::('g'::[])))))))))))))))))))))))))))
  | Aindexed2 n ->
    (match args with
     | [] ->
       Error
         (msg
           ('A'::('s'::('m'::('g'::('e'::('n'::('.'::('t'::('r'::('a'::('n'::('s'::('l'::('_'::('a'::('d'::('d'::('r'::('e'::('s'::('s'::('i'::('n'::('g'::[])))))))))))))))))))))))))
     | a1 :: l ->
       (match l with
        | [] ->
          Error
            (msg
              ('A'::('s'::('m'::('g'::('e'::('n'::('.'::('t'::('r'::('a'::('n'::('s'::('l'::('_'::('a'::('d'::('d'::('r'::('e'::('s'::('s'::('i'::('n'::('g'::[])))))))))))))))))))))))))
        | a2 :: l0 ->
          (match l0 with
           | [] ->
             (match ireg_of a1 with
              | OK x ->
                (match ireg_of a2 with
                 | OK x0 ->
                   OK (Addrmode ((Some x), (Some (x0, Int.one)), (Coq_inl
                     n)))
                 | Error msg0 -> Error msg0)
              | Error msg0 -> Error msg0)
           | m :: l1 ->
             Error
               (msg
                 ('A'::('s'::('m'::('g'::('e'::('n'::('.'::('t'::('r'::('a'::('n'::('s'::('l'::('_'::('a'::('d'::('d'::('r'::('e'::('s'::('s'::('i'::('n'::('g'::[]))))))))))))))))))))))))))))
  | Ascaled (sc, n) ->
    (match args with
     | [] ->
       Error
         (msg
           ('A'::('s'::('m'::('g'::('e'::('n'::('.'::('t'::('r'::('a'::('n'::('s'::('l'::('_'::('a'::('d'::('d'::('r'::('e'::('s'::('s'::('i'::('n'::('g'::[])))))))))))))))))))))))))
     | a1 :: l ->
       (match l with
        | [] ->
          (match ireg_of a1 with
           | OK x -> OK (Addrmode (None, (Some (x, sc)), (Coq_inl n)))
           | Error msg0 -> Error msg0)
        | m :: l0 ->
          Error
            (msg
              ('A'::('s'::('m'::('g'::('e'::('n'::('.'::('t'::('r'::('a'::('n'::('s'::('l'::('_'::('a'::('d'::('d'::('r'::('e'::('s'::('s'::('i'::('n'::('g'::[])))))))))))))))))))))))))))
  | Aindexed2scaled (sc, n) ->
    (match args with
     | [] ->
       Error
         (msg
           ('A'::('s'::('m'::('g'::('e'::('n'::('.'::('t'::('r'::('a'::('n'::('s'::('l'::('_'::('a'::('d'::('d'::('r'::('e'::('s'::('s'::('i'::('n'::('g'::[])))))))))))))))))))))))))
     | a1 :: l ->
       (match l with
        | [] ->
          Error
            (msg
              ('A'::('s'::('m'::('g'::('e'::('n'::('.'::('t'::('r'::('a'::('n'::('s'::('l'::('_'::('a'::('d'::('d'::('r'::('e'::('s'::('s'::('i'::('n'::('g'::[])))))))))))))))))))))))))
        | a2 :: l0 ->
          (match l0 with
           | [] ->
             (match ireg_of a1 with
              | OK x ->
                (match ireg_of a2 with
                 | OK x0 ->
                   OK (Addrmode ((Some x), (Some (x0, sc)), (Coq_inl n)))
                 | Error msg0 -> Error msg0)
              | Error msg0 -> Error msg0)
           | m :: l1 ->
             Error
               (msg
                 ('A'::('s'::('m'::('g'::('e'::('n'::('.'::('t'::('r'::('a'::('n'::('s'::('l'::('_'::('a'::('d'::('d'::('r'::('e'::('s'::('s'::('i'::('n'::('g'::[]))))))))))))))))))))))))))))
  | Aglobal (id, ofs) ->
    (match args with
     | [] -> OK (Addrmode (None, None, (Coq_inr (id, ofs))))
     | m :: l ->
       Error
         (msg
           ('A'::('s'::('m'::('g'::('e'::('n'::('.'::('t'::('r'::('a'::('n'::('s'::('l'::('_'::('a'::('d'::('d'::('r'::('e'::('s'::('s'::('i'::('n'::('g'::[]))))))))))))))))))))))))))
  | Abased (id, ofs) ->
    (match args with
     | [] ->
       Error
         (msg
           ('A'::('s'::('m'::('g'::('e'::('n'::('.'::('t'::('r'::('a'::('n'::('s'::('l'::('_'::('a'::('d'::('d'::('r'::('e'::('s'::('s'::('i'::('n'::('g'::[])))))))))))))))))))))))))
     | a1 :: l ->
       (match l with
        | [] ->
          (match ireg_of a1 with
           | OK x -> OK (Addrmode ((Some x), None, (Coq_inr (id, ofs))))
           | Error msg0 -> Error msg0)
        | m :: l0 ->
          Error
            (msg
              ('A'::('s'::('m'::('g'::('e'::('n'::('.'::('t'::('r'::('a'::('n'::('s'::('l'::('_'::('a'::('d'::('d'::('r'::('e'::('s'::('s'::('i'::('n'::('g'::[])))))))))))))))))))))))))))
  | Abasedscaled (sc, id, ofs) ->
    (match args with
     | [] ->
       Error
         (msg
           ('A'::('s'::('m'::('g'::('e'::('n'::('.'::('t'::('r'::('a'::('n'::('s'::('l'::('_'::('a'::('d'::('d'::('r'::('e'::('s'::('s'::('i'::('n'::('g'::[])))))))))))))))))))))))))
     | a1 :: l ->
       (match l with
        | [] ->
          (match ireg_of a1 with
           | OK x ->
             OK (Addrmode (None, (Some (x, sc)), (Coq_inr (id, ofs))))
           | Error msg0 -> Error msg0)
        | m :: l0 ->
          Error
            (msg
              ('A'::('s'::('m'::('g'::('e'::('n'::('.'::('t'::('r'::('a'::('n'::('s'::('l'::('_'::('a'::('d'::('d'::('r'::('e'::('s'::('s'::('i'::('n'::('g'::[])))))))))))))))))))))))))))
  | Ainstack n ->
    (match args with
     | [] -> OK (Addrmode ((Some ESP), None, (Coq_inl n)))
     | m :: l ->
       Error
         (msg
           ('A'::('s'::('m'::('g'::('e'::('n'::('.'::('t'::('r'::('a'::('n'::('s'::('l'::('_'::('a'::('d'::('d'::('r'::('e'::('s'::('s'::('i'::('n'::('g'::[]))))))))))))))))))))))))))

let floatcomp :  comparison -> freg -> freg -> Asm.instruction =
  fun cmp r1 r2  ->
  match cmp with
  | Clt -> Pcomisd_ff (r2, r1)
  | Cle -> Pcomisd_ff (r2, r1)
  | _ -> Pcomisd_ff (r1, r2)

let transl_cond :  condition -> mreg list -> Asm.code -> Asm.code res =
  fun cond args k  ->
  match cond with
  | Ccompimm (c, n) ->
    (match args with
     | [] ->
       Error
         (msg
           ('A'::('s'::('m'::('g'::('e'::('n'::('.'::('t'::('r'::('a'::('n'::('s'::('l'::('_'::('c'::('o'::('n'::('d'::[])))))))))))))))))))
     | a1 :: l ->
       (match l with
        | [] ->
          (match ireg_of a1 with
           | OK x ->
             OK
               (if Int.eq_dec n Int.zero
                then (Ptest_rr (x, x)) :: k
                else (Pcmp_ri (x, n)) :: k)
           | Error msg0 -> Error msg0)
        | m :: l0 ->
          Error
            (msg
              ('A'::('s'::('m'::('g'::('e'::('n'::('.'::('t'::('r'::('a'::('n'::('s'::('l'::('_'::('c'::('o'::('n'::('d'::[])))))))))))))))))))))
  | Ccompuimm (c, n) ->
    (match args with
     | [] ->
       Error
         (msg
           ('A'::('s'::('m'::('g'::('e'::('n'::('.'::('t'::('r'::('a'::('n'::('s'::('l'::('_'::('c'::('o'::('n'::('d'::[])))))))))))))))))))
     | a1 :: l ->
       (match l with
        | [] ->
          (match ireg_of a1 with
           | OK x -> OK ((Pcmp_ri (x, n)) :: k)
           | Error msg0 -> Error msg0)
        | m :: l0 ->
          Error
            (msg
              ('A'::('s'::('m'::('g'::('e'::('n'::('.'::('t'::('r'::('a'::('n'::('s'::('l'::('_'::('c'::('o'::('n'::('d'::[])))))))))))))))))))))
  | Ccompf cmp ->
    (match args with
     | [] ->
       Error
         (msg
           ('A'::('s'::('m'::('g'::('e'::('n'::('.'::('t'::('r'::('a'::('n'::('s'::('l'::('_'::('c'::('o'::('n'::('d'::[])))))))))))))))))))
     | a1 :: l ->
       (match l with
        | [] ->
          Error
            (msg
              ('A'::('s'::('m'::('g'::('e'::('n'::('.'::('t'::('r'::('a'::('n'::('s'::('l'::('_'::('c'::('o'::('n'::('d'::[])))))))))))))))))))
        | a2 :: l0 ->
          (match l0 with
           | [] ->
             (match freg_of a1 with
              | OK x ->
                (match freg_of a2 with
                 | OK x0 -> OK ((floatcomp cmp x x0) :: k)
                 | Error msg0 -> Error msg0)
              | Error msg0 -> Error msg0)
           | m :: l1 ->
             Error
               (msg
                 ('A'::('s'::('m'::('g'::('e'::('n'::('.'::('t'::('r'::('a'::('n'::('s'::('l'::('_'::('c'::('o'::('n'::('d'::[]))))))))))))))))))))))
  | Cnotcompf cmp ->
    (match args with
     | [] ->
       Error
         (msg
           ('A'::('s'::('m'::('g'::('e'::('n'::('.'::('t'::('r'::('a'::('n'::('s'::('l'::('_'::('c'::('o'::('n'::('d'::[])))))))))))))))))))
     | a1 :: l ->
       (match l with
        | [] ->
          Error
            (msg
              ('A'::('s'::('m'::('g'::('e'::('n'::('.'::('t'::('r'::('a'::('n'::('s'::('l'::('_'::('c'::('o'::('n'::('d'::[])))))))))))))))))))
        | a2 :: l0 ->
          (match l0 with
           | [] ->
             (match freg_of a1 with
              | OK x ->
                (match freg_of a2 with
                 | OK x0 -> OK ((floatcomp cmp x x0) :: k)
                 | Error msg0 -> Error msg0)
              | Error msg0 -> Error msg0)
           | m :: l1 ->
             Error
               (msg
                 ('A'::('s'::('m'::('g'::('e'::('n'::('.'::('t'::('r'::('a'::('n'::('s'::('l'::('_'::('c'::('o'::('n'::('d'::[]))))))))))))))))))))))
  | Cmaskzero n ->
    (match args with
     | [] ->
       Error
         (msg
           ('A'::('s'::('m'::('g'::('e'::('n'::('.'::('t'::('r'::('a'::('n'::('s'::('l'::('_'::('c'::('o'::('n'::('d'::[])))))))))))))))))))
     | a1 :: l ->
       (match l with
        | [] ->
          (match ireg_of a1 with
           | OK x -> OK ((Ptest_ri (x, n)) :: k)
           | Error msg0 -> Error msg0)
        | m :: l0 ->
          Error
            (msg
              ('A'::('s'::('m'::('g'::('e'::('n'::('.'::('t'::('r'::('a'::('n'::('s'::('l'::('_'::('c'::('o'::('n'::('d'::[])))))))))))))))))))))
  | Cmasknotzero n ->
    (match args with
     | [] ->
       Error
         (msg
           ('A'::('s'::('m'::('g'::('e'::('n'::('.'::('t'::('r'::('a'::('n'::('s'::('l'::('_'::('c'::('o'::('n'::('d'::[])))))))))))))))))))
     | a1 :: l ->
       (match l with
        | [] ->
          (match ireg_of a1 with
           | OK x -> OK ((Ptest_ri (x, n)) :: k)
           | Error msg0 -> Error msg0)
        | m :: l0 ->
          Error
            (msg
              ('A'::('s'::('m'::('g'::('e'::('n'::('.'::('t'::('r'::('a'::('n'::('s'::('l'::('_'::('c'::('o'::('n'::('d'::[])))))))))))))))))))))
  | _ ->
    (match args with
     | [] ->
       Error
         (msg
           ('A'::('s'::('m'::('g'::('e'::('n'::('.'::('t'::('r'::('a'::('n'::('s'::('l'::('_'::('c'::('o'::('n'::('d'::[])))))))))))))))))))
     | a1 :: l ->
       (match l with
        | [] ->
          Error
            (msg
              ('A'::('s'::('m'::('g'::('e'::('n'::('.'::('t'::('r'::('a'::('n'::('s'::('l'::('_'::('c'::('o'::('n'::('d'::[])))))))))))))))))))
        | a2 :: l0 ->
          (match l0 with
           | [] ->
             (match ireg_of a1 with
              | OK x ->
                (match ireg_of a2 with
                 | OK x0 -> OK ((Pcmp_rr (x, x0)) :: k)
                 | Error msg0 -> Error msg0)
              | Error msg0 -> Error msg0)
           | m :: l1 ->
             Error
               (msg
                 ('A'::('s'::('m'::('g'::('e'::('n'::('.'::('t'::('r'::('a'::('n'::('s'::('l'::('_'::('c'::('o'::('n'::('d'::[]))))))))))))))))))))))

let testcond_for_signed_comparison :  comparison -> testcond =
    function
| Ceq -> Cond_e
| Cne -> Cond_ne
| Clt -> Cond_l
| Cle -> Cond_le
| Cgt -> Cond_g
| Cge -> Cond_ge

let testcond_for_unsigned_comparison :  comparison -> testcond =
    function
| Ceq -> Cond_e
| Cne -> Cond_ne
| Clt -> Cond_b
| Cle -> Cond_be
| Cgt -> Cond_a
| Cge -> Cond_ae

type extcond =
| Cond_base of testcond
| Cond_or of testcond * testcond
| Cond_and of testcond * testcond

let testcond_for_condition :  condition -> extcond =
    function
| Ccomp c -> Cond_base (testcond_for_signed_comparison c)
| Ccompu c -> Cond_base (testcond_for_unsigned_comparison c)
| Ccompimm (c, n) -> Cond_base (testcond_for_signed_comparison c)
| Ccompuimm (c, n) -> Cond_base (testcond_for_unsigned_comparison c)
| Ccompf c ->
  (match c with
   | Ceq -> Cond_and (Cond_np, Cond_e)
   | Cne -> Cond_or (Cond_p, Cond_ne)
   | Cle -> Cond_base Cond_ae
   | Cge -> Cond_base Cond_ae
   | _ -> Cond_base Cond_a)
| Cnotcompf c ->
  (match c with
   | Ceq -> Cond_or (Cond_p, Cond_ne)
   | Cne -> Cond_and (Cond_np, Cond_e)
   | Cle -> Cond_base Cond_b
   | Cge -> Cond_base Cond_b
   | _ -> Cond_base Cond_be)
| Cmaskzero n -> Cond_base Cond_e
| Cmasknotzero n -> Cond_base Cond_ne

let mk_setcc :  extcond -> ireg -> Asm.code -> Asm.instruction list =
  fun cond rd k  ->
  match cond with
  | Cond_base c -> (Psetcc (c, rd)) :: k
  | Cond_or (c1, c2) ->
    if ireg_eq rd EDX
    then (Psetcc (c1, EDX)) :: ((Psetcc (c2, ECX)) :: ((Por_rr (EDX,
           ECX)) :: k))
    else (Psetcc (c1, EDX)) :: ((Psetcc (c2, rd)) :: ((Por_rr (rd,
           EDX)) :: k))
  | Cond_and (c1, c2) ->
    if ireg_eq rd EDX
    then (Psetcc (c1, EDX)) :: ((Psetcc (c2, ECX)) :: ((Pand_rr (EDX,
           ECX)) :: k))
    else (Psetcc (c1, EDX)) :: ((Psetcc (c2, rd)) :: ((Pand_rr (rd,
           EDX)) :: k))

let mk_jcc :  extcond -> Asm.label -> Asm.code -> Asm.instruction list =
  fun cond lbl k  ->
  match cond with
  | Cond_base c -> (Pjcc (c, lbl)) :: k
  | Cond_or (c1, c2) -> (Pjcc (c1, lbl)) :: ((Pjcc (c2, lbl)) :: k)
  | Cond_and (c1, c2) -> (Pjcc2 (c1, c2, lbl)) :: k

let transl_op :     operation -> mreg list -> mreg -> Asm.code -> Asm.code res =
  fun op args res0 k  ->
  match op with
  | Omove ->
    (match args with
     | [] ->
       Error
         (msg
           ('A'::('s'::('m'::('g'::('e'::('n'::('.'::('t'::('r'::('a'::('n'::('s'::('l'::('_'::('o'::('p'::[])))))))))))))))))
     | a1 :: l ->
       (match l with
        | [] -> mk_mov (preg_of res0) (preg_of a1) k
        | m :: l0 ->
          Error
            (msg
              ('A'::('s'::('m'::('g'::('e'::('n'::('.'::('t'::('r'::('a'::('n'::('s'::('l'::('_'::('o'::('p'::[])))))))))))))))))))
  | Ointconst n ->
    (match args with
     | [] ->
       (match ireg_of res0 with
        | OK x ->
          OK
            ((if Int.eq_dec n Int.zero then Pxor_r x else Pmov_ri (x, n)) :: k)
        | Error msg0 -> Error msg0)
     | m :: l ->
       Error
         (msg
           ('A'::('s'::('m'::('g'::('e'::('n'::('.'::('t'::('r'::('a'::('n'::('s'::('l'::('_'::('o'::('p'::[]))))))))))))))))))
  | Ofloatconst f ->
    (match args with
     | [] ->
       (match freg_of res0 with
        | OK x ->
          OK
            ((if Float.eq_dec f Float.zero
              then Pxorpd_f x
              else Pmovsd_fi (x, f)) :: k)
        | Error msg0 -> Error msg0)
     | m :: l ->
       Error
         (msg
           ('A'::('s'::('m'::('g'::('e'::('n'::('.'::('t'::('r'::('a'::('n'::('s'::('l'::('_'::('o'::('p'::[]))))))))))))))))))
  | Oindirectsymbol id ->
    (match args with
     | [] ->
       (match ireg_of res0 with
        | OK x -> OK ((Pmov_ra (x, id)) :: k)
        | Error msg0 -> Error msg0)
     | m :: l ->
       Error
         (msg
           ('A'::('s'::('m'::('g'::('e'::('n'::('.'::('t'::('r'::('a'::('n'::('s'::('l'::('_'::('o'::('p'::[]))))))))))))))))))
  | Ocast8signed ->
    (match args with
     | [] ->
       Error
         (msg
           ('A'::('s'::('m'::('g'::('e'::('n'::('.'::('t'::('r'::('a'::('n'::('s'::('l'::('_'::('o'::('p'::[])))))))))))))))))
     | a1 :: l ->
       (match l with
        | [] ->
          (match ireg_of a1 with
           | OK x ->
             (match ireg_of res0 with
              | OK x0 -> mk_intconv (fun x1 x2 -> Pmovsb_rr (x1, x2)) x0 x k
              | Error msg0 -> Error msg0)
           | Error msg0 -> Error msg0)
        | m :: l0 ->
          Error
            (msg
              ('A'::('s'::('m'::('g'::('e'::('n'::('.'::('t'::('r'::('a'::('n'::('s'::('l'::('_'::('o'::('p'::[])))))))))))))))))))
  | Ocast8unsigned ->
    (match args with
     | [] ->
       Error
         (msg
           ('A'::('s'::('m'::('g'::('e'::('n'::('.'::('t'::('r'::('a'::('n'::('s'::('l'::('_'::('o'::('p'::[])))))))))))))))))
     | a1 :: l ->
       (match l with
        | [] ->
          (match ireg_of a1 with
           | OK x ->
             (match ireg_of res0 with
              | OK x0 -> mk_intconv (fun x1 x2 -> Pmovzb_rr (x1, x2)) x0 x k
              | Error msg0 -> Error msg0)
           | Error msg0 -> Error msg0)
        | m :: l0 ->
          Error
            (msg
              ('A'::('s'::('m'::('g'::('e'::('n'::('.'::('t'::('r'::('a'::('n'::('s'::('l'::('_'::('o'::('p'::[])))))))))))))))))))
  | Ocast16signed ->
    (match args with
     | [] ->
       Error
         (msg
           ('A'::('s'::('m'::('g'::('e'::('n'::('.'::('t'::('r'::('a'::('n'::('s'::('l'::('_'::('o'::('p'::[])))))))))))))))))
     | a1 :: l ->
       (match l with
        | [] ->
          (match ireg_of a1 with
           | OK x ->
             (match ireg_of res0 with
              | OK x0 -> mk_intconv (fun x1 x2 -> Pmovsw_rr (x1, x2)) x0 x k
              | Error msg0 -> Error msg0)
           | Error msg0 -> Error msg0)
        | m :: l0 ->
          Error
            (msg
              ('A'::('s'::('m'::('g'::('e'::('n'::('.'::('t'::('r'::('a'::('n'::('s'::('l'::('_'::('o'::('p'::[])))))))))))))))))))
  | Ocast16unsigned ->
    (match args with
     | [] ->
       Error
         (msg
           ('A'::('s'::('m'::('g'::('e'::('n'::('.'::('t'::('r'::('a'::('n'::('s'::('l'::('_'::('o'::('p'::[])))))))))))))))))
     | a1 :: l ->
       (match l with
        | [] ->
          (match ireg_of a1 with
           | OK x ->
             (match ireg_of res0 with
              | OK x0 -> mk_intconv (fun x1 x2 -> Pmovzw_rr (x1, x2)) x0 x k
              | Error msg0 -> Error msg0)
           | Error msg0 -> Error msg0)
        | m :: l0 ->
          Error
            (msg
              ('A'::('s'::('m'::('g'::('e'::('n'::('.'::('t'::('r'::('a'::('n'::('s'::('l'::('_'::('o'::('p'::[])))))))))))))))))))
  | Oneg ->
    (match args with
     | [] ->
       Error
         (msg
           ('A'::('s'::('m'::('g'::('e'::('n'::('.'::('t'::('r'::('a'::('n'::('s'::('l'::('_'::('o'::('p'::[])))))))))))))))))
     | a1 :: l ->
       (match l with
        | [] ->
          (match assertion (proj_sumbool (mreg_eq a1 res0)) with
           | OK x ->
             (match ireg_of res0 with
              | OK x0 -> OK ((Pneg x0) :: k)
              | Error msg0 -> Error msg0)
           | Error msg0 -> Error msg0)
        | m :: l0 ->
          Error
            (msg
              ('A'::('s'::('m'::('g'::('e'::('n'::('.'::('t'::('r'::('a'::('n'::('s'::('l'::('_'::('o'::('p'::[])))))))))))))))))))
  | Osub ->
    (match args with
     | [] ->
       Error
         (msg
           ('A'::('s'::('m'::('g'::('e'::('n'::('.'::('t'::('r'::('a'::('n'::('s'::('l'::('_'::('o'::('p'::[])))))))))))))))))
     | a1 :: l ->
       (match l with
        | [] ->
          Error
            (msg
              ('A'::('s'::('m'::('g'::('e'::('n'::('.'::('t'::('r'::('a'::('n'::('s'::('l'::('_'::('o'::('p'::[])))))))))))))))))
        | a2 :: l0 ->
          (match l0 with
           | [] ->
             (match assertion (proj_sumbool (mreg_eq a1 res0)) with
              | OK x ->
                (match ireg_of res0 with
                 | OK x0 ->
                   (match ireg_of a2 with
                    | OK x1 -> OK ((Psub_rr (x0, x1)) :: k)
                    | Error msg0 -> Error msg0)
                 | Error msg0 -> Error msg0)
              | Error msg0 -> Error msg0)
           | m :: l1 ->
             Error
               (msg
                 ('A'::('s'::('m'::('g'::('e'::('n'::('.'::('t'::('r'::('a'::('n'::('s'::('l'::('_'::('o'::('p'::[]))))))))))))))))))))
  | Omul ->
    (match args with
     | [] ->
       Error
         (msg
           ('A'::('s'::('m'::('g'::('e'::('n'::('.'::('t'::('r'::('a'::('n'::('s'::('l'::('_'::('o'::('p'::[])))))))))))))))))
     | a1 :: l ->
       (match l with
        | [] ->
          Error
            (msg
              ('A'::('s'::('m'::('g'::('e'::('n'::('.'::('t'::('r'::('a'::('n'::('s'::('l'::('_'::('o'::('p'::[])))))))))))))))))
        | a2 :: l0 ->
          (match l0 with
           | [] ->
             (match assertion (proj_sumbool (mreg_eq a1 res0)) with
              | OK x ->
                (match ireg_of res0 with
                 | OK x0 ->
                   (match ireg_of a2 with
                    | OK x1 -> OK ((Pimul_rr (x0, x1)) :: k)
                    | Error msg0 -> Error msg0)
                 | Error msg0 -> Error msg0)
              | Error msg0 -> Error msg0)
           | m :: l1 ->
             Error
               (msg
                 ('A'::('s'::('m'::('g'::('e'::('n'::('.'::('t'::('r'::('a'::('n'::('s'::('l'::('_'::('o'::('p'::[]))))))))))))))))))))
  | Omulimm n ->
    (match args with
     | [] ->
       Error
         (msg
           ('A'::('s'::('m'::('g'::('e'::('n'::('.'::('t'::('r'::('a'::('n'::('s'::('l'::('_'::('o'::('p'::[])))))))))))))))))
     | a1 :: l ->
       (match l with
        | [] ->
          (match assertion (proj_sumbool (mreg_eq a1 res0)) with
           | OK x ->
             (match ireg_of res0 with
              | OK x0 -> OK ((Pimul_ri (x0, n)) :: k)
              | Error msg0 -> Error msg0)
           | Error msg0 -> Error msg0)
        | m :: l0 ->
          Error
            (msg
              ('A'::('s'::('m'::('g'::('e'::('n'::('.'::('t'::('r'::('a'::('n'::('s'::('l'::('_'::('o'::('p'::[])))))))))))))))))))
  | Odiv ->
    (match args with
     | [] ->
       Error
         (msg
           ('A'::('s'::('m'::('g'::('e'::('n'::('.'::('t'::('r'::('a'::('n'::('s'::('l'::('_'::('o'::('p'::[])))))))))))))))))
     | a1 :: l ->
       (match l with
        | [] ->
          Error
            (msg
              ('A'::('s'::('m'::('g'::('e'::('n'::('.'::('t'::('r'::('a'::('n'::('s'::('l'::('_'::('o'::('p'::[])))))))))))))))))
        | a2 :: l0 ->
          (match l0 with
           | [] ->
             (match assertion (proj_sumbool (mreg_eq a1 res0)) with
              | OK x ->
                (match ireg_of res0 with
                 | OK x0 ->
                   (match ireg_of a2 with
                    | OK x1 -> mk_div (fun x2 -> Pidiv x2) x0 x1 k
                    | Error msg0 -> Error msg0)
                 | Error msg0 -> Error msg0)
              | Error msg0 -> Error msg0)
           | m :: l1 ->
             Error
               (msg
                 ('A'::('s'::('m'::('g'::('e'::('n'::('.'::('t'::('r'::('a'::('n'::('s'::('l'::('_'::('o'::('p'::[]))))))))))))))))))))
  | Odivu ->
    (match args with
     | [] ->
       Error
         (msg
           ('A'::('s'::('m'::('g'::('e'::('n'::('.'::('t'::('r'::('a'::('n'::('s'::('l'::('_'::('o'::('p'::[])))))))))))))))))
     | a1 :: l ->
       (match l with
        | [] ->
          Error
            (msg
              ('A'::('s'::('m'::('g'::('e'::('n'::('.'::('t'::('r'::('a'::('n'::('s'::('l'::('_'::('o'::('p'::[])))))))))))))))))
        | a2 :: l0 ->
          (match l0 with
           | [] ->
             (match assertion (proj_sumbool (mreg_eq a1 res0)) with
              | OK x ->
                (match ireg_of res0 with
                 | OK x0 ->
                   (match ireg_of a2 with
                    | OK x1 -> mk_div (fun x2 -> Pdiv x2) x0 x1 k
                    | Error msg0 -> Error msg0)
                 | Error msg0 -> Error msg0)
              | Error msg0 -> Error msg0)
           | m :: l1 ->
             Error
               (msg
                 ('A'::('s'::('m'::('g'::('e'::('n'::('.'::('t'::('r'::('a'::('n'::('s'::('l'::('_'::('o'::('p'::[]))))))))))))))))))))
  | Omod ->
    (match args with
     | [] ->
       Error
         (msg
           ('A'::('s'::('m'::('g'::('e'::('n'::('.'::('t'::('r'::('a'::('n'::('s'::('l'::('_'::('o'::('p'::[])))))))))))))))))
     | a1 :: l ->
       (match l with
        | [] ->
          Error
            (msg
              ('A'::('s'::('m'::('g'::('e'::('n'::('.'::('t'::('r'::('a'::('n'::('s'::('l'::('_'::('o'::('p'::[])))))))))))))))))
        | a2 :: l0 ->
          (match l0 with
           | [] ->
             (match assertion (proj_sumbool (mreg_eq a1 res0)) with
              | OK x ->
                (match ireg_of res0 with
                 | OK x0 ->
                   (match ireg_of a2 with
                    | OK x1 -> mk_mod (fun x2 -> Pidiv x2) x0 x1 k
                    | Error msg0 -> Error msg0)
                 | Error msg0 -> Error msg0)
              | Error msg0 -> Error msg0)
           | m :: l1 ->
             Error
               (msg
                 ('A'::('s'::('m'::('g'::('e'::('n'::('.'::('t'::('r'::('a'::('n'::('s'::('l'::('_'::('o'::('p'::[]))))))))))))))))))))
  | Omodu ->
    (match args with
     | [] ->
       Error
         (msg
           ('A'::('s'::('m'::('g'::('e'::('n'::('.'::('t'::('r'::('a'::('n'::('s'::('l'::('_'::('o'::('p'::[])))))))))))))))))
     | a1 :: l ->
       (match l with
        | [] ->
          Error
            (msg
              ('A'::('s'::('m'::('g'::('e'::('n'::('.'::('t'::('r'::('a'::('n'::('s'::('l'::('_'::('o'::('p'::[])))))))))))))))))
        | a2 :: l0 ->
          (match l0 with
           | [] ->
             (match assertion (proj_sumbool (mreg_eq a1 res0)) with
              | OK x ->
                (match ireg_of res0 with
                 | OK x0 ->
                   (match ireg_of a2 with
                    | OK x1 -> mk_mod (fun x2 -> Pdiv x2) x0 x1 k
                    | Error msg0 -> Error msg0)
                 | Error msg0 -> Error msg0)
              | Error msg0 -> Error msg0)
           | m :: l1 ->
             Error
               (msg
                 ('A'::('s'::('m'::('g'::('e'::('n'::('.'::('t'::('r'::('a'::('n'::('s'::('l'::('_'::('o'::('p'::[]))))))))))))))))))))
  | Oand ->
    (match args with
     | [] ->
       Error
         (msg
           ('A'::('s'::('m'::('g'::('e'::('n'::('.'::('t'::('r'::('a'::('n'::('s'::('l'::('_'::('o'::('p'::[])))))))))))))))))
     | a1 :: l ->
       (match l with
        | [] ->
          Error
            (msg
              ('A'::('s'::('m'::('g'::('e'::('n'::('.'::('t'::('r'::('a'::('n'::('s'::('l'::('_'::('o'::('p'::[])))))))))))))))))
        | a2 :: l0 ->
          (match l0 with
           | [] ->
             (match assertion (proj_sumbool (mreg_eq a1 res0)) with
              | OK x ->
                (match ireg_of res0 with
                 | OK x0 ->
                   (match ireg_of a2 with
                    | OK x1 -> OK ((Pand_rr (x0, x1)) :: k)
                    | Error msg0 -> Error msg0)
                 | Error msg0 -> Error msg0)
              | Error msg0 -> Error msg0)
           | m :: l1 ->
             Error
               (msg
                 ('A'::('s'::('m'::('g'::('e'::('n'::('.'::('t'::('r'::('a'::('n'::('s'::('l'::('_'::('o'::('p'::[]))))))))))))))))))))
  | Oandimm n ->
    (match args with
     | [] ->
       Error
         (msg
           ('A'::('s'::('m'::('g'::('e'::('n'::('.'::('t'::('r'::('a'::('n'::('s'::('l'::('_'::('o'::('p'::[])))))))))))))))))
     | a1 :: l ->
       (match l with
        | [] ->
          (match assertion (proj_sumbool (mreg_eq a1 res0)) with
           | OK x ->
             (match ireg_of res0 with
              | OK x0 -> OK ((Pand_ri (x0, n)) :: k)
              | Error msg0 -> Error msg0)
           | Error msg0 -> Error msg0)
        | m :: l0 ->
          Error
            (msg
              ('A'::('s'::('m'::('g'::('e'::('n'::('.'::('t'::('r'::('a'::('n'::('s'::('l'::('_'::('o'::('p'::[])))))))))))))))))))
  | Oor ->
    (match args with
     | [] ->
       Error
         (msg
           ('A'::('s'::('m'::('g'::('e'::('n'::('.'::('t'::('r'::('a'::('n'::('s'::('l'::('_'::('o'::('p'::[])))))))))))))))))
     | a1 :: l ->
       (match l with
        | [] ->
          Error
            (msg
              ('A'::('s'::('m'::('g'::('e'::('n'::('.'::('t'::('r'::('a'::('n'::('s'::('l'::('_'::('o'::('p'::[])))))))))))))))))
        | a2 :: l0 ->
          (match l0 with
           | [] ->
             (match assertion (proj_sumbool (mreg_eq a1 res0)) with
              | OK x ->
                (match ireg_of res0 with
                 | OK x0 ->
                   (match ireg_of a2 with
                    | OK x1 -> OK ((Por_rr (x0, x1)) :: k)
                    | Error msg0 -> Error msg0)
                 | Error msg0 -> Error msg0)
              | Error msg0 -> Error msg0)
           | m :: l1 ->
             Error
               (msg
                 ('A'::('s'::('m'::('g'::('e'::('n'::('.'::('t'::('r'::('a'::('n'::('s'::('l'::('_'::('o'::('p'::[]))))))))))))))))))))
  | Oorimm n ->
    (match args with
     | [] ->
       Error
         (msg
           ('A'::('s'::('m'::('g'::('e'::('n'::('.'::('t'::('r'::('a'::('n'::('s'::('l'::('_'::('o'::('p'::[])))))))))))))))))
     | a1 :: l ->
       (match l with
        | [] ->
          (match assertion (proj_sumbool (mreg_eq a1 res0)) with
           | OK x ->
             (match ireg_of res0 with
              | OK x0 -> OK ((Por_ri (x0, n)) :: k)
              | Error msg0 -> Error msg0)
           | Error msg0 -> Error msg0)
        | m :: l0 ->
          Error
            (msg
              ('A'::('s'::('m'::('g'::('e'::('n'::('.'::('t'::('r'::('a'::('n'::('s'::('l'::('_'::('o'::('p'::[])))))))))))))))))))
  | Oxor ->
    (match args with
     | [] ->
       Error
         (msg
           ('A'::('s'::('m'::('g'::('e'::('n'::('.'::('t'::('r'::('a'::('n'::('s'::('l'::('_'::('o'::('p'::[])))))))))))))))))
     | a1 :: l ->
       (match l with
        | [] ->
          Error
            (msg
              ('A'::('s'::('m'::('g'::('e'::('n'::('.'::('t'::('r'::('a'::('n'::('s'::('l'::('_'::('o'::('p'::[])))))))))))))))))
        | a2 :: l0 ->
          (match l0 with
           | [] ->
             (match assertion (proj_sumbool (mreg_eq a1 res0)) with
              | OK x ->
                (match ireg_of res0 with
                 | OK x0 ->
                   (match ireg_of a2 with
                    | OK x1 -> OK ((Pxor_rr (x0, x1)) :: k)
                    | Error msg0 -> Error msg0)
                 | Error msg0 -> Error msg0)
              | Error msg0 -> Error msg0)
           | m :: l1 ->
             Error
               (msg
                 ('A'::('s'::('m'::('g'::('e'::('n'::('.'::('t'::('r'::('a'::('n'::('s'::('l'::('_'::('o'::('p'::[]))))))))))))))))))))
  | Oxorimm n ->
    (match args with
     | [] ->
       Error
         (msg
           ('A'::('s'::('m'::('g'::('e'::('n'::('.'::('t'::('r'::('a'::('n'::('s'::('l'::('_'::('o'::('p'::[])))))))))))))))))
     | a1 :: l ->
       (match l with
        | [] ->
          (match assertion (proj_sumbool (mreg_eq a1 res0)) with
           | OK x ->
             (match ireg_of res0 with
              | OK x0 -> OK ((Pxor_ri (x0, n)) :: k)
              | Error msg0 -> Error msg0)
           | Error msg0 -> Error msg0)
        | m :: l0 ->
          Error
            (msg
              ('A'::('s'::('m'::('g'::('e'::('n'::('.'::('t'::('r'::('a'::('n'::('s'::('l'::('_'::('o'::('p'::[])))))))))))))))))))
  | Oshl ->
    (match args with
     | [] ->
       Error
         (msg
           ('A'::('s'::('m'::('g'::('e'::('n'::('.'::('t'::('r'::('a'::('n'::('s'::('l'::('_'::('o'::('p'::[])))))))))))))))))
     | a1 :: l ->
       (match l with
        | [] ->
          Error
            (msg
              ('A'::('s'::('m'::('g'::('e'::('n'::('.'::('t'::('r'::('a'::('n'::('s'::('l'::('_'::('o'::('p'::[])))))))))))))))))
        | a2 :: l0 ->
          (match l0 with
           | [] ->
             (match assertion (proj_sumbool (mreg_eq a1 res0)) with
              | OK x ->
                (match ireg_of res0 with
                 | OK x0 ->
                   (match ireg_of a2 with
                    | OK x1 -> mk_shift (fun x2 -> Psal_rcl x2) x0 x1 k
                    | Error msg0 -> Error msg0)
                 | Error msg0 -> Error msg0)
              | Error msg0 -> Error msg0)
           | m :: l1 ->
             Error
               (msg
                 ('A'::('s'::('m'::('g'::('e'::('n'::('.'::('t'::('r'::('a'::('n'::('s'::('l'::('_'::('o'::('p'::[]))))))))))))))))))))
  | Oshlimm n ->
    (match args with
     | [] ->
       Error
         (msg
           ('A'::('s'::('m'::('g'::('e'::('n'::('.'::('t'::('r'::('a'::('n'::('s'::('l'::('_'::('o'::('p'::[])))))))))))))))))
     | a1 :: l ->
       (match l with
        | [] ->
          (match assertion (proj_sumbool (mreg_eq a1 res0)) with
           | OK x ->
             (match ireg_of res0 with
              | OK x0 -> OK ((Psal_ri (x0, n)) :: k)
              | Error msg0 -> Error msg0)
           | Error msg0 -> Error msg0)
        | m :: l0 ->
          Error
            (msg
              ('A'::('s'::('m'::('g'::('e'::('n'::('.'::('t'::('r'::('a'::('n'::('s'::('l'::('_'::('o'::('p'::[])))))))))))))))))))
  | Oshr ->
    (match args with
     | [] ->
       Error
         (msg
           ('A'::('s'::('m'::('g'::('e'::('n'::('.'::('t'::('r'::('a'::('n'::('s'::('l'::('_'::('o'::('p'::[])))))))))))))))))
     | a1 :: l ->
       (match l with
        | [] ->
          Error
            (msg
              ('A'::('s'::('m'::('g'::('e'::('n'::('.'::('t'::('r'::('a'::('n'::('s'::('l'::('_'::('o'::('p'::[])))))))))))))))))
        | a2 :: l0 ->
          (match l0 with
           | [] ->
             (match assertion (proj_sumbool (mreg_eq a1 res0)) with
              | OK x ->
                (match ireg_of res0 with
                 | OK x0 ->
                   (match ireg_of a2 with
                    | OK x1 -> mk_shift (fun x2 -> Psar_rcl x2) x0 x1 k
                    | Error msg0 -> Error msg0)
                 | Error msg0 -> Error msg0)
              | Error msg0 -> Error msg0)
           | m :: l1 ->
             Error
               (msg
                 ('A'::('s'::('m'::('g'::('e'::('n'::('.'::('t'::('r'::('a'::('n'::('s'::('l'::('_'::('o'::('p'::[]))))))))))))))))))))
  | Oshrimm n ->
    (match args with
     | [] ->
       Error
         (msg
           ('A'::('s'::('m'::('g'::('e'::('n'::('.'::('t'::('r'::('a'::('n'::('s'::('l'::('_'::('o'::('p'::[])))))))))))))))))
     | a1 :: l ->
       (match l with
        | [] ->
          (match assertion (proj_sumbool (mreg_eq a1 res0)) with
           | OK x ->
             (match ireg_of res0 with
              | OK x0 -> OK ((Psar_ri (x0, n)) :: k)
              | Error msg0 -> Error msg0)
           | Error msg0 -> Error msg0)
        | m :: l0 ->
          Error
            (msg
              ('A'::('s'::('m'::('g'::('e'::('n'::('.'::('t'::('r'::('a'::('n'::('s'::('l'::('_'::('o'::('p'::[])))))))))))))))))))
  | Oshrximm n ->
    (match args with
     | [] ->
       Error
         (msg
           ('A'::('s'::('m'::('g'::('e'::('n'::('.'::('t'::('r'::('a'::('n'::('s'::('l'::('_'::('o'::('p'::[])))))))))))))))))
     | a1 :: l ->
       (match l with
        | [] ->
          (match assertion (proj_sumbool (mreg_eq a1 res0)) with
           | OK x ->
             (match ireg_of res0 with
              | OK x0 -> mk_shrximm x0 n k
              | Error msg0 -> Error msg0)
           | Error msg0 -> Error msg0)
        | m :: l0 ->
          Error
            (msg
              ('A'::('s'::('m'::('g'::('e'::('n'::('.'::('t'::('r'::('a'::('n'::('s'::('l'::('_'::('o'::('p'::[])))))))))))))))))))
  | Oshru ->
    (match args with
     | [] ->
       Error
         (msg
           ('A'::('s'::('m'::('g'::('e'::('n'::('.'::('t'::('r'::('a'::('n'::('s'::('l'::('_'::('o'::('p'::[])))))))))))))))))
     | a1 :: l ->
       (match l with
        | [] ->
          Error
            (msg
              ('A'::('s'::('m'::('g'::('e'::('n'::('.'::('t'::('r'::('a'::('n'::('s'::('l'::('_'::('o'::('p'::[])))))))))))))))))
        | a2 :: l0 ->
          (match l0 with
           | [] ->
             (match assertion (proj_sumbool (mreg_eq a1 res0)) with
              | OK x ->
                (match ireg_of res0 with
                 | OK x0 ->
                   (match ireg_of a2 with
                    | OK x1 -> mk_shift (fun x2 -> Pshr_rcl x2) x0 x1 k
                    | Error msg0 -> Error msg0)
                 | Error msg0 -> Error msg0)
              | Error msg0 -> Error msg0)
           | m :: l1 ->
             Error
               (msg
                 ('A'::('s'::('m'::('g'::('e'::('n'::('.'::('t'::('r'::('a'::('n'::('s'::('l'::('_'::('o'::('p'::[]))))))))))))))))))))
  | Oshruimm n ->
    (match args with
     | [] ->
       Error
         (msg
           ('A'::('s'::('m'::('g'::('e'::('n'::('.'::('t'::('r'::('a'::('n'::('s'::('l'::('_'::('o'::('p'::[])))))))))))))))))
     | a1 :: l ->
       (match l with
        | [] ->
          (match assertion (proj_sumbool (mreg_eq a1 res0)) with
           | OK x ->
             (match ireg_of res0 with
              | OK x0 -> OK ((Pshr_ri (x0, n)) :: k)
              | Error msg0 -> Error msg0)
           | Error msg0 -> Error msg0)
        | m :: l0 ->
          Error
            (msg
              ('A'::('s'::('m'::('g'::('e'::('n'::('.'::('t'::('r'::('a'::('n'::('s'::('l'::('_'::('o'::('p'::[])))))))))))))))))))
  | Ororimm n ->
    (match args with
     | [] ->
       Error
         (msg
           ('A'::('s'::('m'::('g'::('e'::('n'::('.'::('t'::('r'::('a'::('n'::('s'::('l'::('_'::('o'::('p'::[])))))))))))))))))
     | a1 :: l ->
       (match l with
        | [] ->
          (match assertion (proj_sumbool (mreg_eq a1 res0)) with
           | OK x ->
             (match ireg_of res0 with
              | OK x0 -> OK ((Pror_ri (x0, n)) :: k)
              | Error msg0 -> Error msg0)
           | Error msg0 -> Error msg0)
        | m :: l0 ->
          Error
            (msg
              ('A'::('s'::('m'::('g'::('e'::('n'::('.'::('t'::('r'::('a'::('n'::('s'::('l'::('_'::('o'::('p'::[])))))))))))))))))))
  | Olea addr ->
    (match transl_addressing addr args with
     | OK x ->
       (match ireg_of res0 with
        | OK x0 -> OK ((Plea (x0, x)) :: k)
        | Error msg0 -> Error msg0)
     | Error msg0 -> Error msg0)
  | Onegf ->
    (match args with
     | [] ->
       Error
         (msg
           ('A'::('s'::('m'::('g'::('e'::('n'::('.'::('t'::('r'::('a'::('n'::('s'::('l'::('_'::('o'::('p'::[])))))))))))))))))
     | a1 :: l ->
       (match l with
        | [] ->
          (match assertion (proj_sumbool (mreg_eq a1 res0)) with
           | OK x ->
             (match freg_of res0 with
              | OK x0 -> OK ((Pnegd x0) :: k)
              | Error msg0 -> Error msg0)
           | Error msg0 -> Error msg0)
        | m :: l0 ->
          Error
            (msg
              ('A'::('s'::('m'::('g'::('e'::('n'::('.'::('t'::('r'::('a'::('n'::('s'::('l'::('_'::('o'::('p'::[])))))))))))))))))))
  | Oabsf ->
    (match args with
     | [] ->
       Error
         (msg
           ('A'::('s'::('m'::('g'::('e'::('n'::('.'::('t'::('r'::('a'::('n'::('s'::('l'::('_'::('o'::('p'::[])))))))))))))))))
     | a1 :: l ->
       (match l with
        | [] ->
          (match assertion (proj_sumbool (mreg_eq a1 res0)) with
           | OK x ->
             (match freg_of res0 with
              | OK x0 -> OK ((Pabsd x0) :: k)
              | Error msg0 -> Error msg0)
           | Error msg0 -> Error msg0)
        | m :: l0 ->
          Error
            (msg
              ('A'::('s'::('m'::('g'::('e'::('n'::('.'::('t'::('r'::('a'::('n'::('s'::('l'::('_'::('o'::('p'::[])))))))))))))))))))
  | Oaddf ->
    (match args with
     | [] ->
       Error
         (msg
           ('A'::('s'::('m'::('g'::('e'::('n'::('.'::('t'::('r'::('a'::('n'::('s'::('l'::('_'::('o'::('p'::[])))))))))))))))))
     | a1 :: l ->
       (match l with
        | [] ->
          Error
            (msg
              ('A'::('s'::('m'::('g'::('e'::('n'::('.'::('t'::('r'::('a'::('n'::('s'::('l'::('_'::('o'::('p'::[])))))))))))))))))
        | a2 :: l0 ->
          (match l0 with
           | [] ->
             (match assertion (proj_sumbool (mreg_eq a1 res0)) with
              | OK x ->
                (match freg_of res0 with
                 | OK x0 ->
                   (match freg_of a2 with
                    | OK x1 -> OK ((Paddd_ff (x0, x1)) :: k)
                    | Error msg0 -> Error msg0)
                 | Error msg0 -> Error msg0)
              | Error msg0 -> Error msg0)
           | m :: l1 ->
             Error
               (msg
                 ('A'::('s'::('m'::('g'::('e'::('n'::('.'::('t'::('r'::('a'::('n'::('s'::('l'::('_'::('o'::('p'::[]))))))))))))))))))))
  | Osubf ->
    (match args with
     | [] ->
       Error
         (msg
           ('A'::('s'::('m'::('g'::('e'::('n'::('.'::('t'::('r'::('a'::('n'::('s'::('l'::('_'::('o'::('p'::[])))))))))))))))))
     | a1 :: l ->
       (match l with
        | [] ->
          Error
            (msg
              ('A'::('s'::('m'::('g'::('e'::('n'::('.'::('t'::('r'::('a'::('n'::('s'::('l'::('_'::('o'::('p'::[])))))))))))))))))
        | a2 :: l0 ->
          (match l0 with
           | [] ->
             (match assertion (proj_sumbool (mreg_eq a1 res0)) with
              | OK x ->
                (match freg_of res0 with
                 | OK x0 ->
                   (match freg_of a2 with
                    | OK x1 -> OK ((Psubd_ff (x0, x1)) :: k)
                    | Error msg0 -> Error msg0)
                 | Error msg0 -> Error msg0)
              | Error msg0 -> Error msg0)
           | m :: l1 ->
             Error
               (msg
                 ('A'::('s'::('m'::('g'::('e'::('n'::('.'::('t'::('r'::('a'::('n'::('s'::('l'::('_'::('o'::('p'::[]))))))))))))))))))))
  | Omulf ->
    (match args with
     | [] ->
       Error
         (msg
           ('A'::('s'::('m'::('g'::('e'::('n'::('.'::('t'::('r'::('a'::('n'::('s'::('l'::('_'::('o'::('p'::[])))))))))))))))))
     | a1 :: l ->
       (match l with
        | [] ->
          Error
            (msg
              ('A'::('s'::('m'::('g'::('e'::('n'::('.'::('t'::('r'::('a'::('n'::('s'::('l'::('_'::('o'::('p'::[])))))))))))))))))
        | a2 :: l0 ->
          (match l0 with
           | [] ->
             (match assertion (proj_sumbool (mreg_eq a1 res0)) with
              | OK x ->
                (match freg_of res0 with
                 | OK x0 ->
                   (match freg_of a2 with
                    | OK x1 -> OK ((Pmuld_ff (x0, x1)) :: k)
                    | Error msg0 -> Error msg0)
                 | Error msg0 -> Error msg0)
              | Error msg0 -> Error msg0)
           | m :: l1 ->
             Error
               (msg
                 ('A'::('s'::('m'::('g'::('e'::('n'::('.'::('t'::('r'::('a'::('n'::('s'::('l'::('_'::('o'::('p'::[]))))))))))))))))))))
  | Odivf ->
    (match args with
     | [] ->
       Error
         (msg
           ('A'::('s'::('m'::('g'::('e'::('n'::('.'::('t'::('r'::('a'::('n'::('s'::('l'::('_'::('o'::('p'::[])))))))))))))))))
     | a1 :: l ->
       (match l with
        | [] ->
          Error
            (msg
              ('A'::('s'::('m'::('g'::('e'::('n'::('.'::('t'::('r'::('a'::('n'::('s'::('l'::('_'::('o'::('p'::[])))))))))))))))))
        | a2 :: l0 ->
          (match l0 with
           | [] ->
             (match assertion (proj_sumbool (mreg_eq a1 res0)) with
              | OK x ->
                (match freg_of res0 with
                 | OK x0 ->
                   (match freg_of a2 with
                    | OK x1 -> OK ((Pdivd_ff (x0, x1)) :: k)
                    | Error msg0 -> Error msg0)
                 | Error msg0 -> Error msg0)
              | Error msg0 -> Error msg0)
           | m :: l1 ->
             Error
               (msg
                 ('A'::('s'::('m'::('g'::('e'::('n'::('.'::('t'::('r'::('a'::('n'::('s'::('l'::('_'::('o'::('p'::[]))))))))))))))))))))
  | Osingleoffloat ->
    (match args with
     | [] ->
       Error
         (msg
           ('A'::('s'::('m'::('g'::('e'::('n'::('.'::('t'::('r'::('a'::('n'::('s'::('l'::('_'::('o'::('p'::[])))))))))))))))))
     | a1 :: l ->
       (match l with
        | [] ->
          (match freg_of res0 with
           | OK x ->
             (match freg_of a1 with
              | OK x0 -> OK ((Pcvtsd2ss_ff (x, x0)) :: k)
              | Error msg0 -> Error msg0)
           | Error msg0 -> Error msg0)
        | m :: l0 ->
          Error
            (msg
              ('A'::('s'::('m'::('g'::('e'::('n'::('.'::('t'::('r'::('a'::('n'::('s'::('l'::('_'::('o'::('p'::[])))))))))))))))))))
  | Ointoffloat ->
    (match args with
     | [] ->
       Error
         (msg
           ('A'::('s'::('m'::('g'::('e'::('n'::('.'::('t'::('r'::('a'::('n'::('s'::('l'::('_'::('o'::('p'::[])))))))))))))))))
     | a1 :: l ->
       (match l with
        | [] ->
          (match ireg_of res0 with
           | OK x ->
             (match freg_of a1 with
              | OK x0 -> OK ((Pcvttsd2si_rf (x, x0)) :: k)
              | Error msg0 -> Error msg0)
           | Error msg0 -> Error msg0)
        | m :: l0 ->
          Error
            (msg
              ('A'::('s'::('m'::('g'::('e'::('n'::('.'::('t'::('r'::('a'::('n'::('s'::('l'::('_'::('o'::('p'::[])))))))))))))))))))
  | Ofloatofint ->
    (match args with
     | [] ->
       Error
         (msg
           ('A'::('s'::('m'::('g'::('e'::('n'::('.'::('t'::('r'::('a'::('n'::('s'::('l'::('_'::('o'::('p'::[])))))))))))))))))
     | a1 :: l ->
       (match l with
        | [] ->
          (match freg_of res0 with
           | OK x ->
             (match ireg_of a1 with
              | OK x0 -> OK ((Pcvtsi2sd_fr (x, x0)) :: k)
              | Error msg0 -> Error msg0)
           | Error msg0 -> Error msg0)
        | m :: l0 ->
          Error
            (msg
              ('A'::('s'::('m'::('g'::('e'::('n'::('.'::('t'::('r'::('a'::('n'::('s'::('l'::('_'::('o'::('p'::[])))))))))))))))))))
  | Ocmp c ->
    (match ireg_of res0 with
     | OK x -> transl_cond c args (mk_setcc (testcond_for_condition c) x k)
     | Error msg0 -> Error msg0)

let transl_load :     memory_chunk -> addressing -> mreg list -> mreg -> Asm.code -> Asm.code
    res =
  fun chunk addr args dest k  ->
  match transl_addressing addr args with
  | OK x ->
    (match chunk with
     | Mint8signed ->
       (match ireg_of dest with
        | OK x0 -> OK ((Pmovsb_rm (x0, x)) :: k)
        | Error msg0 -> Error msg0)
     | Mint8unsigned ->
       (match ireg_of dest with
        | OK x0 -> OK ((Pmovzb_rm (x0, x)) :: k)
        | Error msg0 -> Error msg0)
     | Mint16signed ->
       (match ireg_of dest with
        | OK x0 -> OK ((Pmovsw_rm (x0, x)) :: k)
        | Error msg0 -> Error msg0)
     | Mint16unsigned ->
       (match ireg_of dest with
        | OK x0 -> OK ((Pmovzw_rm (x0, x)) :: k)
        | Error msg0 -> Error msg0)
     | Mint32 ->
       (match ireg_of dest with
        | OK x0 -> OK ((Pmov_rm (x0, x)) :: k)
        | Error msg0 -> Error msg0)
     | Mfloat32 ->
       (match freg_of dest with
        | OK x0 -> OK ((Pcvtss2sd_fm (x0, x)) :: k)
        | Error msg0 -> Error msg0)
     | _ ->
       (match freg_of dest with
        | OK x0 -> OK ((Pmovsd_fm (x0, x)) :: k)
        | Error msg0 -> Error msg0))
  | Error msg0 -> Error msg0

let transl_store :     memory_chunk -> addressing -> mreg list -> mreg -> Asm.code -> Asm.code
    res =
  fun chunk addr args src k  ->
  match transl_addressing addr args with
  | OK x ->
    (match chunk with
     | Mint8signed ->
       (match ireg_of src with
        | OK x0 -> mk_smallstore (fun x1 x2 -> Pmovb_mr (x1, x2)) x x0 k
        | Error msg0 -> Error msg0)
     | Mint8unsigned ->
       (match ireg_of src with
        | OK x0 -> mk_smallstore (fun x1 x2 -> Pmovb_mr (x1, x2)) x x0 k
        | Error msg0 -> Error msg0)
     | Mint16signed ->
       (match ireg_of src with
        | OK x0 -> OK ((Pmovw_mr (x, x0)) :: k)
        | Error msg0 -> Error msg0)
     | Mint16unsigned ->
       (match ireg_of src with
        | OK x0 -> OK ((Pmovw_mr (x, x0)) :: k)
        | Error msg0 -> Error msg0)
     | Mint32 ->
       (match ireg_of src with
        | OK x0 -> OK ((Pmov_mr (x, x0)) :: k)
        | Error msg0 -> Error msg0)
     | Mfloat32 ->
       (match freg_of src with
        | OK x0 -> OK ((Pcvtsd2ss_mf (x, x0)) :: k)
        | Error msg0 -> Error msg0)
     | _ ->
       (match freg_of src with
        | OK x0 -> OK ((Pmovsd_mf (x, x0)) :: k)
        | Error msg0 -> Error msg0))
  | Error msg0 -> Error msg0

let transl_annot_param :  annot_param -> Asm.annot_param =
    function
| APreg r -> Asm.APreg (preg_of r)
| APstack (chunk, ofs) -> Asm.APstack (chunk, ofs)

let transl_instr :     coq_function -> instruction -> bool -> Asm.code -> Asm.instruction list
    res =
  fun f i edx_is_parent k  ->
  match i with
  | Mgetstack (ofs, ty, dst) -> loadind ESP ofs ty dst k
  | Msetstack (src, ofs, ty) -> storeind src ESP ofs ty k
  | Mgetparam (ofs, ty, dst) ->
    if edx_is_parent
    then loadind EDX ofs ty dst k
    else (match loadind EDX ofs ty dst k with
          | OK x -> loadind ESP f.fn_link_ofs Tint IT1 x
          | Error msg0 -> Error msg0)
  | Mop (op, args, res0) -> transl_op op args res0 k
  | Mload (chunk, addr, args, dst) -> transl_load chunk addr args dst k
  | Mstore (chunk, addr, args, src) -> transl_store chunk addr args src k
  | Mcall (sig0, s) ->
    (match s with
     | Coq_inl reg ->
       (match ireg_of reg with
        | OK x -> OK ((Pcall_r x) :: k)
        | Error msg0 -> Error msg0)
     | Coq_inr symb -> OK ((Pcall_s symb) :: k))
  | Mtailcall (sig0, s) ->
    (match s with
     | Coq_inl reg ->
       (match ireg_of reg with
        | OK x ->
          OK ((Pfreeframe (f.fn_stacksize, f.fn_retaddr_ofs,
            f.fn_link_ofs)) :: ((Pjmp_r x) :: k))
        | Error msg0 -> Error msg0)
     | Coq_inr symb ->
       OK ((Pfreeframe (f.fn_stacksize, f.fn_retaddr_ofs,
         f.fn_link_ofs)) :: ((Pjmp_s symb) :: k)))
  | Mbuiltin (ef, args, res0) ->
    OK ((Pbuiltin (ef, (map preg_of args), (preg_of res0))) :: k)
  | Mannot (ef, args) ->
    OK ((Pannot (ef, (map transl_annot_param args))) :: k)
  | Mlabel lbl -> OK ((Plabel lbl) :: k)
  | Mgoto lbl -> OK ((Pjmp_l lbl) :: k)
  | Mcond (cond, args, lbl) ->
    transl_cond cond args (mk_jcc (testcond_for_condition cond) lbl k)
  | Mjumptable (arg, tbl) ->
    (match ireg_of arg with
     | OK x -> OK ((Pjmptbl (x, tbl)) :: k)
     | Error msg0 -> Error msg0)
  | Mreturn ->
    OK ((Pfreeframe (f.fn_stacksize, f.fn_retaddr_ofs,
      f.fn_link_ofs)) :: (Pret :: k))

let edx_preserved :  bool -> instruction -> bool =
  fun before  ->
    function
| Msetstack (src, ofs, ty) -> before
| Mgetparam (ofs, ty, dst) -> negb (proj_sumbool (mreg_eq dst IT1))
| _ -> false

let rec transl_code :     coq_function -> instruction list -> bool -> Asm.instruction list res =
  fun f il edx_is_parent  ->
  match il with
  | [] -> OK []
  | i1 :: il' ->
    (match transl_code f il' (edx_preserved edx_is_parent i1) with
     | OK x -> transl_instr f i1 edx_is_parent x
     | Error msg0 -> Error msg0)

let transf_function :  coq_function -> Asm.code res =
  fun f  ->
  match transl_code f f.fn_code true with
  | OK x ->
    if zlt (list_length_z x) Int.max_unsigned
    then OK ((Pallocframe (f.fn_stacksize, f.fn_retaddr_ofs,
           f.fn_link_ofs)) :: x)
    else Error
           (msg
             ('c'::('o'::('d'::('e'::(' '::('s'::('i'::('z'::('e'::(' '::('e'::('x'::('c'::('e'::('e'::('d'::('e'::('d'::[])))))))))))))))))))
  | Error msg0 -> Error msg0

let transf_fundef :  fundef -> Asm.fundef res =
  fun f  ->
  transf_partial_fundef transf_function f

let transf_program :  program -> Asm.program res =
  fun p  ->
  transform_partial_program transf_fundef p

