open AST
open BinNums
open BinPos
open CombineOp
open Coqlib
open Datatypes
open Errors
open Kildall
open List0
open Maps
open Op
open RTL
open RTLtyping
open Registers

val eq_valnum : valnum -> valnum -> bool

val eq_list_valnum : valnum list -> valnum list -> bool

val eq_rhs : rhs -> rhs -> bool

type numbering = { num_next : valnum; num_eqs : (valnum * rhs) list;
                   num_reg : valnum PTree.t; num_val : reg list PMap.t }

val num_next : numbering -> valnum

val num_eqs : numbering -> (valnum * rhs) list

val num_reg : numbering -> valnum PTree.t

val num_val : numbering -> reg list PMap.t

val empty_numbering : numbering

val valnum_reg : numbering -> reg -> numbering * valnum

val valnum_regs : numbering -> reg list -> numbering * valnum list

val find_valnum_rhs : rhs -> (valnum * rhs) list -> valnum option

val find_valnum_num : valnum -> (valnum * rhs) list -> rhs option

val forget_reg : numbering -> reg -> reg list PMap.t

val update_reg : numbering -> reg -> valnum -> reg list PMap.t

val add_rhs : numbering -> reg -> rhs -> numbering

val add_op : numbering -> reg -> operation -> reg list -> numbering

val add_load :
  numbering -> reg -> memory_chunk -> addressing -> reg list -> numbering

val add_unknown : numbering -> reg -> numbering

val kill_eqs : (rhs -> bool) -> (valnum * rhs) list -> (valnum * rhs) list

val kill_equations : (rhs -> bool) -> numbering -> numbering

val filter_loads : rhs -> bool

val kill_loads : numbering -> numbering

val filter_after_store :
  memory_chunk -> addressing -> valnum list -> rhs -> bool

val add_store :
  numbering -> memory_chunk -> addressing -> reg list -> reg -> numbering

val reg_valnum : numbering -> valnum -> reg option

val regs_valnums : numbering -> valnum list -> reg list option

val find_rhs : numbering -> rhs -> reg option

val reduce_rec :
  ((valnum -> rhs option) -> 'a1 -> valnum list -> ('a1 * valnum list)
  option) -> numbering -> nat -> 'a1 -> valnum list -> ('a1 * reg list)
  option

val reduce :
  ((valnum -> rhs option) -> 'a1 -> valnum list -> ('a1 * valnum list)
  option) -> numbering -> 'a1 -> reg list -> valnum list -> 'a1 * reg list

module Solver : 
 sig 
  module L : 
  sig 
    type t = numbering
    val top : t
  end
  
  val fixpoint :
    positive list PTree.t -> (positive -> L.t -> L.t) -> positive -> L.t
    PMap.t option
 end

val transfer : coq_function -> node -> numbering -> numbering

val analyze : coq_function -> numbering PMap.t option

val transf_instr : numbering -> instruction -> instruction

val transf_code : numbering PMap.t -> code -> code

val transf_function : coq_function -> coq_function res

val transf_fundef : fundef -> fundef res

val transf_program : program -> program res

