open AST
open BinInt
open BinNums
open Cminor
open Coqlib
open Csharpminor
open Datatypes
open Errors
open Floats
open Integers
open List0
open Maps

type compilenv = coq_Z PTree.t

let rec uncast_int :  Int.int -> Cminor.expr -> Cminor.expr =
  fun m e  ->
    match e with
| Cminor.Eunop (u, e1) ->
  (match u with
   | Ocast8unsigned ->
     if Int.eq
          (Int.coq_and
            (Int.repr (Zpos (Coq_xI (Coq_xI (Coq_xI (Coq_xI (Coq_xI (Coq_xI
              (Coq_xI Coq_xH))))))))) m) m
     then uncast_int m e1
     else e
   | Ocast8signed ->
     if Int.eq
          (Int.coq_and
            (Int.repr (Zpos (Coq_xI (Coq_xI (Coq_xI (Coq_xI (Coq_xI (Coq_xI
              (Coq_xI Coq_xH))))))))) m) m
     then uncast_int m e1
     else e
   | Ocast16unsigned ->
     if Int.eq
          (Int.coq_and
            (Int.repr (Zpos (Coq_xI (Coq_xI (Coq_xI (Coq_xI (Coq_xI (Coq_xI
              (Coq_xI (Coq_xI (Coq_xI (Coq_xI (Coq_xI (Coq_xI (Coq_xI (Coq_xI
              (Coq_xI Coq_xH))))))))))))))))) m) m
     then uncast_int m e1
     else e
   | Ocast16signed ->
     if Int.eq
          (Int.coq_and
            (Int.repr (Zpos (Coq_xI (Coq_xI (Coq_xI (Coq_xI (Coq_xI (Coq_xI
              (Coq_xI (Coq_xI (Coq_xI (Coq_xI (Coq_xI (Coq_xI (Coq_xI (Coq_xI
              (Coq_xI Coq_xH))))))))))))))))) m) m
     then uncast_int m e1
     else e
   | _ -> e)
| Cminor.Ebinop (b, e1, e0) ->
  (match b with
   | Oand ->
     (match e0 with
      | Cminor.Econst c ->
        (match c with
         | Cminor.Ointconst n ->
           if Int.eq (Int.coq_and n m) m then uncast_int m e1 else e
         | _ -> e)
      | _ -> e)
   | Oshr ->
     (match e0 with
      | Cminor.Econst c ->
        (match c with
         | Cminor.Ointconst n ->
           if Int.eq (Int.shru (Int.shl m n) n) m
           then Cminor.Ebinop (Oshr, (uncast_int (Int.shl m n) e1),
                  (Cminor.Econst (Cminor.Ointconst n)))
           else e
         | _ -> e)
      | _ -> e)
   | Oshru ->
     (match e0 with
      | Cminor.Econst c ->
        (match c with
         | Cminor.Ointconst n ->
           if Int.eq (Int.shru (Int.shl m n) n) m
           then Cminor.Ebinop (Oshru, (uncast_int (Int.shl m n) e1),
                  (Cminor.Econst (Cminor.Ointconst n)))
           else e
         | _ -> e)
      | _ -> e)
   | _ -> e)
| _ -> e

let uncast_int8 :  Cminor.expr -> Cminor.expr =
  fun e  ->
  uncast_int
    (Int.repr (Zpos (Coq_xI (Coq_xI (Coq_xI (Coq_xI (Coq_xI (Coq_xI (Coq_xI
      Coq_xH))))))))) e

let uncast_int16 :  Cminor.expr -> Cminor.expr =
  fun e  ->
  uncast_int
    (Int.repr (Zpos (Coq_xI (Coq_xI (Coq_xI (Coq_xI (Coq_xI (Coq_xI (Coq_xI
      (Coq_xI (Coq_xI (Coq_xI (Coq_xI (Coq_xI (Coq_xI (Coq_xI (Coq_xI
      Coq_xH))))))))))))))))) e

let rec uncast_float32 :  Cminor.expr -> Cminor.expr =
  fun e  ->
    match e with
| Cminor.Eunop (u, e1) ->
  (match u with
   | Osingleoffloat -> uncast_float32 e1
   | _ -> e)
| _ -> e

let store_arg :  memory_chunk -> Cminor.expr -> Cminor.expr =
  fun chunk e  ->
  match chunk with
  | Mint8signed -> uncast_int8 e
  | Mint8unsigned -> uncast_int8 e
  | Mint16signed -> uncast_int16 e
  | Mint16unsigned -> uncast_int16 e
  | Mfloat32 -> uncast_float32 e
  | _ -> e

let make_store :     memory_chunk -> Cminor.expr -> Cminor.expr -> Cminor.stmt =
  fun chunk e1 e2  ->
  Cminor.Sstore (chunk, e1, (store_arg chunk e2))

let make_stackaddr :  coq_Z -> Cminor.expr =
  fun ofs  ->
  Cminor.Econst (Oaddrstack (Int.repr ofs))

let make_globaladdr :  ident -> Cminor.expr =
  fun id  ->
  Cminor.Econst (Oaddrsymbol (id, Int.zero))

let make_unop :  Cminor.unary_operation -> Cminor.expr -> Cminor.expr =
  fun op e  ->
  match op with
  | Ocast8unsigned -> Cminor.Eunop (Ocast8unsigned, (uncast_int8 e))
  | Ocast8signed -> Cminor.Eunop (Ocast8signed, (uncast_int8 e))
  | Ocast16unsigned -> Cminor.Eunop (Ocast16unsigned, (uncast_int16 e))
  | Ocast16signed -> Cminor.Eunop (Ocast16signed, (uncast_int16 e))
  | Osingleoffloat -> Cminor.Eunop (Osingleoffloat, (uncast_float32 e))
  | _ -> Cminor.Eunop (op, e)

type approx =
| Any
| Int1
| Int7
| Int8s
| Int8u
| Int15
| Int16s
| Int16u
| Float32

module Approx = 
 struct 
  (** val bge : approx -> approx -> bool **)
  
  let bge x y =
    match x with
    | Any -> true
    | Int1 ->
      (match y with
       | Int1 -> true
       | _ -> false)
    | Int7 ->
      (match y with
       | Int1 -> true
       | Int7 -> true
       | _ -> false)
    | Int8s ->
      (match y with
       | Int1 -> true
       | Int7 -> true
       | Int8s -> true
       | _ -> false)
    | Int8u ->
      (match y with
       | Int1 -> true
       | Int7 -> true
       | Int8u -> true
       | _ -> false)
    | Int15 ->
      (match y with
       | Int1 -> true
       | Int7 -> true
       | Int8u -> true
       | Int15 -> true
       | _ -> false)
    | Int16s ->
      (match y with
       | Any -> false
       | Int16u -> false
       | Float32 -> false
       | _ -> true)
    | Int16u ->
      (match y with
       | Any -> false
       | Int8s -> false
       | Int16s -> false
       | Float32 -> false
       | _ -> true)
    | Float32 ->
      (match y with
       | Float32 -> true
       | _ -> false)
  
  (** val of_int : Int.int -> approx **)
  
  let of_int n =
    if (||) (proj_sumbool (Int.eq_dec n Int.zero))
         (proj_sumbool (Int.eq_dec n Int.one))
    then Int1
    else if Int.eq_dec n (Int.zero_ext (Zpos (Coq_xI (Coq_xI Coq_xH))) n)
         then Int7
         else if Int.eq_dec n
                   (Int.zero_ext (Zpos (Coq_xO (Coq_xO (Coq_xO Coq_xH)))) n)
              then Int8u
              else if Int.eq_dec n
                        (Int.sign_ext (Zpos (Coq_xO (Coq_xO (Coq_xO
                          Coq_xH)))) n)
                   then Int8s
                   else if Int.eq_dec n
                             (Int.zero_ext (Zpos (Coq_xI (Coq_xI (Coq_xI
                               Coq_xH)))) n)
                        then Int15
                        else if Int.eq_dec n
                                  (Int.zero_ext (Zpos (Coq_xO (Coq_xO (Coq_xO
                                    (Coq_xO Coq_xH))))) n)
                             then Int16u
                             else if Int.eq_dec n
                                       (Int.sign_ext (Zpos (Coq_xO (Coq_xO
                                         (Coq_xO (Coq_xO Coq_xH))))) n)
                                  then Int16s
                                  else Any
  
  (** val of_float : float -> approx **)
  
  let of_float n =
    if Float.eq_dec n (Float.singleoffloat n) then Float32 else Any
  
  (** val of_chunk : memory_chunk -> approx **)
  
  let of_chunk = function
  | Mint8signed -> Int8s
  | Mint8unsigned -> Int8u
  | Mint16signed -> Int16s
  | Mint16unsigned -> Int16u
  | Mfloat32 -> Float32
  | _ -> Any
  
  (** val unop : Cminor.unary_operation -> approx -> approx **)
  
  let unop : Cminor.unary_operation -> approx -> approx =
    fun op a ->
    match op with
    | Ocast8unsigned -> Int8u
    | Ocast8signed -> Int8s
    | Ocast16unsigned -> Int16u
    | Ocast16signed -> Int16s
    | Osingleoffloat -> Float32
    | _ -> Any
  
  (** val unop_is_redundant : Cminor.unary_operation -> approx -> bool **)
  
  let unop_is_redundant op a =
    match op with
    | Ocast8unsigned -> bge Int8u a
    | Ocast8signed -> bge Int8s a
    | Ocast16unsigned -> bge Int16u a
    | Ocast16signed -> bge Int16s a
    | Osingleoffloat -> bge Float32 a
    | _ -> false
  
  (** val bitwise_and : approx -> approx -> approx **)
  
  let bitwise_and a1 a2 =
    if (||) (bge Int1 a1) (bge Int1 a2)
    then Int1
    else if (||) (bge Int8u a1) (bge Int8u a2)
         then Int8u
         else if (||) (bge Int16u a1) (bge Int16u a2) then Int16u else Any
  
  (** val bitwise_or : approx -> approx -> approx **)
  
  let bitwise_or a1 a2 =
    if (&&) (bge Int1 a1) (bge Int1 a2)
    then Int1
    else if (&&) (bge Int8u a1) (bge Int8u a2)
         then Int8u
         else if (&&) (bge Int16u a1) (bge Int16u a2) then Int16u else Any
  
  (** val binop : Cminor.binary_operation -> approx -> approx -> approx **)
  
  let binop op a1 a2 =
    match op with
    | Oand -> bitwise_and a1 a2
    | Oor -> bitwise_or a1 a2
    | Oxor -> bitwise_or a1 a2
    | Ocmp c -> Int1
    | Ocmpu c -> Int1
    | Ocmpf c -> Int1
    | _ -> Any
 end

let var_addr :  compilenv -> ident -> Cminor.expr =
  fun cenv id  ->
  match PTree.get id cenv with
  | Some ofs -> make_stackaddr ofs
  | None -> make_globaladdr id

let transl_constant :  constant -> Cminor.constant * approx =
    function
| Ointconst n -> ((Cminor.Ointconst n), (Approx.of_int n))
| Ofloatconst n -> ((Cminor.Ofloatconst n), (Approx.of_float n))

let rec transl_expr :  compilenv -> expr -> (Cminor.expr * approx) res =
  fun cenv  ->
    function
| Evar id -> OK ((Cminor.Evar id), Any)
| Eaddrof id -> OK ((var_addr cenv id), Any)
| Econst cst ->
  let (tcst, a) = transl_constant cst in OK ((Cminor.Econst tcst), a)
| Eunop (op, e1) ->
  (match transl_expr cenv e1 with
   | OK p ->
     let (x, y) = p in
     if Approx.unop_is_redundant op y
     then OK (x, y)
     else OK ((make_unop op x), (Approx.unop op y))
   | Error msg0 -> Error msg0)
| Ebinop (op, e1, e2) ->
  (match transl_expr cenv e1 with
   | OK p ->
     let (x, y) = p in
     (match transl_expr cenv e2 with
      | OK p0 ->
        let (x0, y0) = p0 in
        OK ((Cminor.Ebinop (op, x, x0)), (Approx.binop op y y0))
      | Error msg0 -> Error msg0)
   | Error msg0 -> Error msg0)
| Eload (chunk, e0) ->
  (match transl_expr cenv e0 with
   | OK p ->
     let (x, y) = p in
     OK ((Cminor.Eload (chunk, x)), (Approx.of_chunk chunk))
   | Error msg0 -> Error msg0)

let rec transl_exprlist :  compilenv -> expr list -> Cminor.expr list res =
  fun cenv  ->
    function
| [] -> OK []
| e1 :: e2 ->
  (match transl_expr cenv e1 with
   | OK p ->
     let (x, y) = p in
     (match transl_exprlist cenv e2 with
      | OK x0 -> OK (x :: x0)
      | Error msg0 -> Error msg0)
   | Error msg0 -> Error msg0)

type exit_env = bool list

let rec shift_exit :  exit_env -> nat -> nat =
  fun e n  ->
  match e with
  | [] -> n
  | b :: e' ->
    if b
    then (match n with
          | O -> O
          | S m -> S (shift_exit e' m))
    else S (shift_exit e' n)

let rec switch_table :  lbl_stmt -> nat -> (Int.int * nat) list =
  fun ls k  ->
  match ls with
  | LSdefault s -> []
  | LScase (ni, s, rem) -> (ni, k) :: (switch_table rem (S k))

let rec switch_env :  lbl_stmt -> exit_env -> exit_env =
  fun ls e  ->
  match ls with
  | LSdefault s -> e
  | LScase (i, s, ls') -> false :: (switch_env ls' e)

let rec transl_stmt :  compilenv -> exit_env -> stmt -> Cminor.stmt res =
  fun cenv xenv  ->
    function
| Sskip -> OK Cminor.Sskip
| Sset (id, e) ->
  (match transl_expr cenv e with
   | OK p -> let (x, y) = p in OK (Sassign (id, x))
   | Error msg0 -> Error msg0)
| Sstore (chunk, e1, e2) ->
  (match transl_expr cenv e1 with
   | OK p ->
     let (x, y) = p in
     (match transl_expr cenv e2 with
      | OK p0 -> let (x0, y0) = p0 in OK (make_store chunk x x0)
      | Error msg0 -> Error msg0)
   | Error msg0 -> Error msg0)
| Scall (optid, sig0, e, el) ->
  (match transl_expr cenv e with
   | OK p ->
     let (x, y) = p in
     (match transl_exprlist cenv el with
      | OK x0 -> OK (Cminor.Scall (optid, sig0, x, x0))
      | Error msg0 -> Error msg0)
   | Error msg0 -> Error msg0)
| Sbuiltin (optid, ef, el) ->
  (match transl_exprlist cenv el with
   | OK x -> OK (Cminor.Sbuiltin (optid, ef, x))
   | Error msg0 -> Error msg0)
| Sseq (s1, s2) ->
  (match transl_stmt cenv xenv s1 with
   | OK x ->
     (match transl_stmt cenv xenv s2 with
      | OK x0 -> OK (Cminor.Sseq (x, x0))
      | Error msg0 -> Error msg0)
   | Error msg0 -> Error msg0)
| Sifthenelse (e, s1, s2) ->
  (match transl_expr cenv e with
   | OK p ->
     let (x, y) = p in
     (match transl_stmt cenv xenv s1 with
      | OK x0 ->
        (match transl_stmt cenv xenv s2 with
         | OK x1 -> OK (Cminor.Sifthenelse (x, x0, x1))
         | Error msg0 -> Error msg0)
      | Error msg0 -> Error msg0)
   | Error msg0 -> Error msg0)
| Sloop s0 ->
  (match transl_stmt cenv xenv s0 with
   | OK x -> OK (Cminor.Sloop x)
   | Error msg0 -> Error msg0)
| Sblock s0 ->
  (match transl_stmt cenv (true :: xenv) s0 with
   | OK x -> OK (Cminor.Sblock x)
   | Error msg0 -> Error msg0)
| Sexit n -> OK (Cminor.Sexit (shift_exit xenv n))
| Sswitch (e, ls) ->
  let cases = switch_table ls O in
  let coqzz_default = length cases in
  (match transl_expr cenv e with
   | OK p ->
     let (x, y) = p in
     transl_lblstmt cenv (switch_env ls xenv) ls (Cminor.Sswitch (x, cases,
       coqzz_default))
   | Error msg0 -> Error msg0)
| Sreturn o ->
  (match o with
   | Some e ->
     (match transl_expr cenv e with
      | OK p -> let (x, y) = p in OK (Cminor.Sreturn (Some x))
      | Error msg0 -> Error msg0)
   | None -> OK (Cminor.Sreturn None))
| Slabel (lbl, s0) ->
  (match transl_stmt cenv xenv s0 with
   | OK x -> OK (Cminor.Slabel (lbl, x))
   | Error msg0 -> Error msg0)
| Sgoto lbl -> OK (Cminor.Sgoto lbl)

and transl_lblstmt :     compilenv -> exit_env -> lbl_stmt -> Cminor.stmt -> Cminor.stmt res =
  fun cenv xenv ls body  ->
  match ls with
  | LSdefault s ->
    (match transl_stmt cenv xenv s with
     | OK x -> OK (Cminor.Sseq ((Cminor.Sblock body), x))
     | Error msg0 -> Error msg0)
  | LScase (i, s, ls') ->
    (match transl_stmt cenv xenv s with
     | OK x ->
       transl_lblstmt cenv (tl xenv) ls' (Cminor.Sseq ((Cminor.Sblock body),
         x))
     | Error msg0 -> Error msg0)

let block_alignment :  coq_Z -> coq_Z =
  fun sz  ->
  if zlt sz (Zpos (Coq_xO Coq_xH))
  then Zpos Coq_xH
  else if zlt sz (Zpos (Coq_xO (Coq_xO Coq_xH)))
       then Zpos (Coq_xO Coq_xH)
       else if zlt sz (Zpos (Coq_xO (Coq_xO (Coq_xO Coq_xH))))
            then Zpos (Coq_xO (Coq_xO Coq_xH))
            else Zpos (Coq_xO (Coq_xO (Coq_xO Coq_xH)))

let assign_variable :     (compilenv * coq_Z) -> (ident * coq_Z) -> compilenv * coq_Z =
  fun cenv_stacksize  ->
    function
| (id, sz) ->
  let (cenv, stacksize) = cenv_stacksize in
  let ofs = align stacksize (block_alignment sz) in
  ((PTree.set id ofs cenv), (Z.add ofs (Z.max Z0 sz)))

let assign_variables :     (compilenv * coq_Z) -> (ident * coq_Z) list -> compilenv * coq_Z =
  fun cenv_stacksize vars  ->
  fold_left assign_variable vars cenv_stacksize

module VarSort = 
 struct 
  (** val merge :
      (ident * coq_Z) list -> (ident * coq_Z) list -> (ident * coq_Z) list **)
  
  let rec merge :
      (ident * coq_Z) list -> (ident * coq_Z) list -> (ident * coq_Z) list =
    fun l1 l2 ->
    let rec merge_aux l3 =
      match l1 with
      | [] -> l3
      | a1 :: l1' ->
        (match l3 with
         | [] -> l1
         | a2 :: l2' ->
           if proj_sumbool (zle (snd a1) (snd a2))
           then a1 :: (merge l1' l3)
           else a2 :: (merge_aux l2'))
    in merge_aux l2
  
  (** val merge_list_to_stack :
      (ident * coq_Z) list option list -> (ident * coq_Z) list ->
      (ident * coq_Z) list option list **)
  
  let rec merge_list_to_stack stack l =
    match stack with
    | [] -> (Some l) :: []
    | y :: stack' ->
      (match y with
       | Some l' -> None :: (merge_list_to_stack stack' (merge l' l))
       | None -> (Some l) :: stack')
  
  (** val merge_stack :
      (ident * coq_Z) list option list -> (ident * coq_Z) list **)
  
  let rec merge_stack :
      (ident * coq_Z) list option list -> (ident * coq_Z) list = function
  | [] -> []
  | y :: stack' ->
    (match y with
     | Some l -> merge l (merge_stack stack')
     | None -> merge_stack stack')
  
  (** val iter_merge :
      (ident * coq_Z) list option list -> (ident * coq_Z) list ->
      (ident * coq_Z) list **)
  
  let rec iter_merge stack = function
  | [] -> merge_stack stack
  | a :: l' -> iter_merge (merge_list_to_stack stack (a :: [])) l'
  
  (** val sort : (ident * coq_Z) list -> (ident * coq_Z) list **)
  
  let sort x =
    iter_merge [] x
  
  (** val flatten_stack :
      (ident * coq_Z) list option list -> (ident * coq_Z) list **)
  
  let rec flatten_stack :
      (ident * coq_Z) list option list -> (ident * coq_Z) list = function
  | [] -> []
  | o :: stack' ->
    (match o with
     | Some l -> app l (flatten_stack stack')
     | None -> flatten_stack stack')
 end

let build_compilenv :  coq_function -> compilenv * coq_Z =
  fun f  ->
  assign_variables (PTree.empty, Z0) (VarSort.sort f.fn_vars)

let transl_funbody :     compilenv -> coq_Z -> coq_function -> Cminor.coq_function res =
  fun cenv stacksize f  ->
  match transl_stmt cenv [] f.fn_body with
  | OK x ->
    OK { Cminor.fn_sig = f.fn_sig; Cminor.fn_params = f.fn_params;
      Cminor.fn_vars = f.fn_temps; fn_stackspace = stacksize;
      Cminor.fn_body = x }
  | Error msg0 -> Error msg0

let transl_function :  coq_function -> Cminor.coq_function res =
  fun f  ->
  let (cenv, stacksize) = build_compilenv f in
  if zle stacksize Int.max_unsigned
  then transl_funbody cenv stacksize f
  else Error
         (msg
           ('C'::('m'::('i'::('n'::('o'::('r'::('g'::('e'::('n'::(':'::(' '::('t'::('o'::('o'::(' '::('m'::('a'::('n'::('y'::(' '::('l'::('o'::('c'::('a'::('l'::(' '::('v'::('a'::('r'::('i'::('a'::('b'::('l'::('e'::('s'::(','::(' '::('s'::('t'::('a'::('c'::('k'::(' '::('s'::('i'::('z'::('e'::(' '::('e'::('x'::('c'::('e'::('e'::('d'::('e'::('d'::[])))))))))))))))))))))))))))))))))))))))))))))))))))))))))

let transl_fundef :  fundef -> Cminor.fundef res =
  fun f  ->
  transf_partial_fundef transl_function f

let transl_program :  program -> Cminor.program res =
  fun p  ->
  transform_partial_program transl_fundef p

