open AST
open BinInt
open BinNums
open Cminor
open Coqlib
open Csharpminor
open Datatypes
open Errors
open Floats
open Integers
open List0
open Maps

type compilenv = coq_Z PTree.t

val uncast_int : Int.int -> Cminor.expr -> Cminor.expr

val uncast_int8 : Cminor.expr -> Cminor.expr

val uncast_int16 : Cminor.expr -> Cminor.expr

val uncast_float32 : Cminor.expr -> Cminor.expr

val store_arg : memory_chunk -> Cminor.expr -> Cminor.expr

val make_store : memory_chunk -> Cminor.expr -> Cminor.expr -> Cminor.stmt

val make_stackaddr : coq_Z -> Cminor.expr

val make_globaladdr : ident -> Cminor.expr

val make_unop : Cminor.unary_operation -> Cminor.expr -> Cminor.expr

type approx =
| Any
| Int1
| Int7
| Int8s
| Int8u
| Int15
| Int16s
| Int16u
| Float32

module Approx : 
 sig 
  val bge : approx -> approx -> bool
  
  val of_int : Int.int -> approx
  
  val of_float : float -> approx
  
  val of_chunk : memory_chunk -> approx
  
  val unop : Cminor.unary_operation -> approx -> approx
  
  val unop_is_redundant : Cminor.unary_operation -> approx -> bool
  
  val bitwise_and : approx -> approx -> approx
  
  val bitwise_or : approx -> approx -> approx
  
  val binop : Cminor.binary_operation -> approx -> approx -> approx
 end

val var_addr : compilenv -> ident -> Cminor.expr

val transl_constant : constant -> Cminor.constant * approx

val transl_expr : compilenv -> expr -> (Cminor.expr * approx) res

val transl_exprlist : compilenv -> expr list -> Cminor.expr list res

type exit_env = bool list

val shift_exit : exit_env -> nat -> nat

val switch_table : lbl_stmt -> nat -> (Int.int * nat) list

val switch_env : lbl_stmt -> exit_env -> exit_env

val transl_stmt : compilenv -> exit_env -> stmt -> Cminor.stmt res

val transl_lblstmt :
  compilenv -> exit_env -> lbl_stmt -> Cminor.stmt -> Cminor.stmt res

val block_alignment : coq_Z -> coq_Z

val assign_variable :
  (compilenv * coq_Z) -> (ident * coq_Z) -> compilenv * coq_Z

val assign_variables :
  (compilenv * coq_Z) -> (ident * coq_Z) list -> compilenv * coq_Z

module VarSort : 
 sig 
  val merge :
    (ident * coq_Z) list -> (ident * coq_Z) list -> (ident * coq_Z) list
  
  val merge_list_to_stack :
    (ident * coq_Z) list option list -> (ident * coq_Z) list ->
    (ident * coq_Z) list option list
  
  val merge_stack : (ident * coq_Z) list option list -> (ident * coq_Z) list
  
  val iter_merge :
    (ident * coq_Z) list option list -> (ident * coq_Z) list ->
    (ident * coq_Z) list
  
  val sort : (ident * coq_Z) list -> (ident * coq_Z) list
  
  val flatten_stack :
    (ident * coq_Z) list option list -> (ident * coq_Z) list
 end

val build_compilenv : coq_function -> compilenv * coq_Z

val transl_funbody :
  compilenv -> coq_Z -> coq_function -> Cminor.coq_function res

val transl_function : coq_function -> Cminor.coq_function res

val transl_fundef : fundef -> Cminor.fundef res

val transl_program : program -> Cminor.program res

