open AST
open BinNums
open Integers
open Op
open SelectOp

type valnum = positive

type rhs =
| Op of operation * valnum list
| Load of memory_chunk * addressing * valnum list

let combine_compimm_ne_0 :     (valnum -> rhs option) -> valnum -> (condition * valnum list) option =
  fun get x  ->
  match get x with
  | Some r ->
    (match r with
     | Op (o, ys) ->
       (match o with
        | Oandimm n -> Some ((Cmasknotzero n), ys)
        | Ocmp c -> Some (c, ys)
        | _ -> None)
     | Load (m, a, l) -> None)
  | None -> None

let combine_compimm_eq_0 :     (valnum -> rhs option) -> valnum -> (condition * valnum list) option =
  fun get x  ->
  match get x with
  | Some r ->
    (match r with
     | Op (o, ys) ->
       (match o with
        | Oandimm n -> Some ((Cmaskzero n), ys)
        | Ocmp c -> Some ((negate_condition c), ys)
        | _ -> None)
     | Load (m, a, l) -> None)
  | None -> None

let combine_compimm_eq_1 :     (valnum -> rhs option) -> valnum -> (condition * valnum list) option =
  fun get x  ->
  match get x with
  | Some r ->
    (match r with
     | Op (o, ys) ->
       (match o with
        | Ocmp c -> Some (c, ys)
        | _ -> None)
     | Load (m, a, l) -> None)
  | None -> None

let combine_compimm_ne_1 :     (valnum -> rhs option) -> valnum -> (condition * valnum list) option =
  fun get x  ->
  match get x with
  | Some r ->
    (match r with
     | Op (o, ys) ->
       (match o with
        | Ocmp c -> Some ((negate_condition c), ys)
        | _ -> None)
     | Load (m, a, l) -> None)
  | None -> None

let combine_cond :     (valnum -> rhs option) -> condition -> valnum list -> (condition * valnum
    list) option =
  fun get cond args  ->
  match cond with
  | Ccompimm (c, n) ->
    (match c with
     | Ceq ->
       (match args with
        | [] -> None
        | x :: l ->
          (match l with
           | [] ->
             if Int.eq_dec n Int.zero
             then combine_compimm_eq_0 get x
             else if Int.eq_dec n Int.one
                  then combine_compimm_eq_1 get x
                  else None
           | v :: l0 -> None))
     | Cne ->
       (match args with
        | [] -> None
        | x :: l ->
          (match l with
           | [] ->
             if Int.eq_dec n Int.zero
             then combine_compimm_ne_0 get x
             else if Int.eq_dec n Int.one
                  then combine_compimm_ne_1 get x
                  else None
           | v :: l0 -> None))
     | _ -> None)
  | Ccompuimm (c, n) ->
    (match c with
     | Ceq ->
       (match args with
        | [] -> None
        | x :: l ->
          (match l with
           | [] ->
             if Int.eq_dec n Int.zero
             then combine_compimm_eq_0 get x
             else if Int.eq_dec n Int.one
                  then combine_compimm_eq_1 get x
                  else None
           | v :: l0 -> None))
     | Cne ->
       (match args with
        | [] -> None
        | x :: l ->
          (match l with
           | [] ->
             if Int.eq_dec n Int.zero
             then combine_compimm_ne_0 get x
             else if Int.eq_dec n Int.one
                  then combine_compimm_ne_1 get x
                  else None
           | v :: l0 -> None))
     | _ -> None)
  | _ -> None

let combine_addr :     (valnum -> rhs option) -> addressing -> valnum list ->
    (addressing * valnum list) option =
  fun get addr args  ->
  match addr with
  | Aindexed n ->
    (match args with
     | [] -> None
     | x :: l ->
       (match l with
        | [] ->
          (match get x with
           | Some r ->
             (match r with
              | Op (o, ys) ->
                (match o with
                 | Olea a -> Some ((offset_addressing a n), ys)
                 | _ -> None)
              | Load (m, a, l0) -> None)
           | None -> None)
        | v :: l0 -> None))
  | _ -> None

let combine_op :     (valnum -> rhs option) -> operation -> valnum list -> (operation * valnum
    list) option =
  fun get op args  ->
  match op with
  | Olea addr ->
    (match combine_addr get addr args with
     | Some p -> let (addr', args') = p in Some ((Olea addr'), args')
     | None -> None)
  | Ocmp cond ->
    (match combine_cond get cond args with
     | Some p -> let (cond', args') = p in Some ((Ocmp cond'), args')
     | None -> None)
  | _ -> None

