open AST
open BinInt
open BinNums
open ConstpropOp
open Coqlib
open Datatypes
open Floats
open Integers
open Lattice
open List0
open Maps
open Op
open RTL
open Registers

module Approx = 
 struct 
  type t = approx
  
  (** val eq_dec : t -> t -> bool **)
  
  let eq_dec x y =
    match x with
    | Novalue ->
      (match y with
       | Novalue -> true
       | _ -> false)
    | Unknown ->
      (match y with
       | Unknown -> true
       | _ -> false)
    | I x0 ->
      (match y with
       | I i0 -> Int.eq_dec x0 i0
       | _ -> false)
    | F x0 ->
      (match y with
       | F f0 -> Float.eq_dec x0 f0
       | _ -> false)
    | G (x0, x1) ->
      (match y with
       | G (i1, i2) -> if ident_eq x0 i1 then Int.eq_dec x1 i2 else false
       | _ -> false)
    | ConstpropOp.S x0 ->
      (match y with
       | ConstpropOp.S i0 -> Int.eq_dec x0 i0
       | _ -> false)
  
  (** val beq : t -> t -> bool **)
  
  let beq x y =
    if eq_dec x y then true else false
  
  (** val bot : approx **)
  
  let bot =
    Novalue
  
  (** val top : approx **)
  
  let top =
    Unknown
  
  (** val lub : t -> t -> t **)
  
  let lub x y =
    if eq_dec x y
    then x
    else (match x with
          | Novalue -> y
          | _ ->
            (match y with
             | Novalue -> x
             | _ -> Unknown))
 end

type t' =
| Bot_except of Approx.t PTree.t
| Top_except of Approx.t PTree.t
      
module D = 
 struct 
  
  type t = t'
  
  (** val get : positive -> t -> Approx.t **)
  
  let get p = function
  | Bot_except m ->
    (match PTree.get p m with
     | Some x0 -> x0
     | None -> Approx.bot)
  | Top_except m ->
    (match PTree.get p m with
     | Some x0 -> x0
     | None -> Approx.top)
  
  (** val set : positive -> Approx.t -> t -> t **)
  
  let set p v = function
  | Bot_except m ->
    Bot_except
      (if Approx.beq v Approx.bot then PTree.remove p m else PTree.set p v m)
  | Top_except m ->
    Top_except
      (if Approx.beq v Approx.top then PTree.remove p m else PTree.set p v m)
  
  (** val beq : t -> t -> bool **)
  
  let beq x y =
    match x with
    | Bot_except m ->
      (match y with
       | Bot_except n -> PTree.beq Approx.beq m n
       | Top_except t0 -> false)
    | Top_except m ->
      (match y with
       | Bot_except t0 -> false
       | Top_except n -> PTree.beq Approx.beq m n)
  
  (** val bot : t' **)
  
  let bot =
    Bot_except PTree.empty
  
  (** val top : t' **)
  
  let top =
    Top_except PTree.empty
  
  (** val opt_beq : Approx.t option -> Approx.t option -> bool **)
  
  let opt_beq ox oy =
    match ox with
    | Some x ->
      (match oy with
       | Some y -> Approx.beq x y
       | None -> false)
    | None ->
      (match oy with
       | Some t0 -> false
       | None -> true)
  
  type changed =
  | Unchanged
  | Changed of Approx.t PTree.t
  
  (** val combine_l :
      (Approx.t option -> Approx.t option -> Approx.t option) -> Approx.t
      PTree.t -> changed **)
  
  let rec combine_l :
      (Approx.t option -> Approx.t option -> Approx.t option) -> Approx.t
      PTree.t -> changed =
    fun f -> function
  | PTree.Leaf -> Unchanged
  | PTree.Node (l, o, r) ->
    let o' = f o None in
    (match combine_l f l with
     | Unchanged ->
       (match combine_l f r with
        | Unchanged ->
          if opt_beq o' o
          then Unchanged
          else Changed (PTree.coq_Node' l o' r)
        | Changed r' -> Changed (PTree.coq_Node' l o' r'))
     | Changed l' ->
       (match combine_l f r with
        | Unchanged -> Changed (PTree.coq_Node' l' o' r)
        | Changed r' -> Changed (PTree.coq_Node' l' o' r')))
  
  (** val combine_r :
      (Approx.t option -> Approx.t option -> Approx.t option) -> Approx.t
      PTree.t -> changed **)
  
  let rec combine_r :
      (Approx.t option -> Approx.t option -> Approx.t option) -> Approx.t
      PTree.t -> changed =
    fun f -> function
  | PTree.Leaf -> Unchanged
  | PTree.Node (l, o, r) ->
    let o' = f None o in
    (match combine_r f l with
     | Unchanged ->
       (match combine_r f r with
        | Unchanged ->
          if opt_beq o' o
          then Unchanged
          else Changed (PTree.coq_Node' l o' r)
        | Changed r' -> Changed (PTree.coq_Node' l o' r'))
     | Changed l' ->
       (match combine_r f r with
        | Unchanged -> Changed (PTree.coq_Node' l' o' r)
        | Changed r' -> Changed (PTree.coq_Node' l' o' r')))
  
  type changed2 =
  | Same
  | Same1
  | Same2
  | CC of Approx.t PTree.t
  
  (** val xcombine :
      (Approx.t option -> Approx.t option -> Approx.t option) -> Approx.t
      PTree.t -> Approx.t PTree.t -> changed2 **)
  
  let rec xcombine f m1 m2 =
    match m1 with
    | PTree.Leaf ->
      (match m2 with
       | PTree.Leaf -> Same
       | PTree.Node (t0, o, t1) ->
         (match combine_r f m2 with
          | Unchanged -> Same2
          | Changed m -> CC m))
    | PTree.Node (l1, o1, r1) ->
      (match m2 with
       | PTree.Leaf ->
         (match combine_l f m1 with
          | Unchanged -> Same1
          | Changed m -> CC m)
       | PTree.Node (l2, o2, r2) ->
         let o = f o1 o2 in
         (match xcombine f l1 l2 with
          | Same ->
            (match xcombine f r1 r2 with
             | Same ->
               if opt_beq o o1
               then if opt_beq o o2 then Same else Same1
               else if opt_beq o o2
                    then Same2
                    else CC (PTree.coq_Node' l1 o r1)
             | Same1 ->
               if opt_beq o o1 then Same1 else CC (PTree.coq_Node' l1 o r1)
             | Same2 ->
               if opt_beq o o2 then Same2 else CC (PTree.coq_Node' l2 o r2)
             | CC r -> CC (PTree.coq_Node' l1 o r))
          | Same1 ->
            (match xcombine f r1 r2 with
             | Same2 -> CC (PTree.coq_Node' l1 o r2)
             | CC r -> CC (PTree.coq_Node' l1 o r)
             | _ ->
               if opt_beq o o1 then Same1 else CC (PTree.coq_Node' l1 o r1))
          | Same2 ->
            (match xcombine f r1 r2 with
             | Same1 -> CC (PTree.coq_Node' l2 o r1)
             | CC r -> CC (PTree.coq_Node' l2 o r)
             | _ ->
               if opt_beq o o2 then Same2 else CC (PTree.coq_Node' l2 o r2))
          | CC l ->
            (match xcombine f r1 r2 with
             | Same2 -> CC (PTree.coq_Node' l o r2)
             | CC r -> CC (PTree.coq_Node' l o r)
             | _ -> CC (PTree.coq_Node' l o r1))))
  
  (** val combine :
      (Approx.t option -> Approx.t option -> Approx.t option) -> Approx.t
      PTree.t -> Approx.t PTree.t -> Approx.t PTree.t **)
  
  let combine f m1 m2 =
    match xcombine f m1 m2 with
    | Same2 -> m2
    | CC m -> m
    | _ -> m1
  
  (** val opt_lub : Approx.t -> Approx.t -> Approx.t option **)
  
  let opt_lub x y =
    let z = Approx.lub x y in
    if Approx.beq z Approx.top then None else Some z
  
  (** val lub : t -> t -> t **)
  
  let lub x y =
    match x with
    | Bot_except m ->
      (match y with
       | Bot_except n ->
         Bot_except
           (combine (fun a b ->
             match a with
             | Some u ->
               (match b with
                | Some v -> Some (Approx.lub u v)
                | None -> a)
             | None -> b) m n)
       | Top_except n ->
         Top_except
           (combine (fun a b ->
             match a with
             | Some u ->
               (match b with
                | Some v -> opt_lub u v
                | None -> None)
             | None -> b) m n))
    | Top_except m ->
      (match y with
       | Bot_except n ->
         Top_except
           (combine (fun a b ->
             match a with
             | Some u ->
               (match b with
                | Some v -> opt_lub u v
                | None -> a)
             | None -> None) m n)
       | Top_except n ->
         Top_except
           (combine (fun a b ->
             match a with
             | Some u ->
               (match b with
                | Some v -> opt_lub u v
                | None -> None)
             | None -> None) m n))
 end

type global_approx = init_data list PTree.t

let rec eval_load_init :     memory_chunk -> coq_Z -> init_data list -> approx =
  fun chunk pos  ->
    function
| [] -> Unknown
| i :: il' ->
  (match i with
   | Init_int8 n ->
     if zeq pos Z0
     then (match chunk with
           | Mint8signed ->
             I (Int.sign_ext (Zpos (Coq_xO (Coq_xO (Coq_xO Coq_xH)))) n)
           | Mint8unsigned ->
             I (Int.zero_ext (Zpos (Coq_xO (Coq_xO (Coq_xO Coq_xH)))) n)
           | _ -> Unknown)
     else eval_load_init chunk (Z.sub pos (Zpos Coq_xH)) il'
   | Init_int16 n ->
     if zeq pos Z0
     then (match chunk with
           | Mint16signed ->
             I
               (Int.sign_ext (Zpos (Coq_xO (Coq_xO (Coq_xO (Coq_xO
                 Coq_xH))))) n)
           | Mint16unsigned ->
             I
               (Int.zero_ext (Zpos (Coq_xO (Coq_xO (Coq_xO (Coq_xO
                 Coq_xH))))) n)
           | _ -> Unknown)
     else eval_load_init chunk (Z.sub pos (Zpos (Coq_xO Coq_xH))) il'
   | Init_int32 n ->
     if zeq pos Z0
     then (match chunk with
           | Mint32 -> I n
           | _ -> Unknown)
     else eval_load_init chunk (Z.sub pos (Zpos (Coq_xO (Coq_xO Coq_xH))))
            il'
   | Init_float32 n ->
     if zeq pos Z0
     then (match chunk with
           | Mfloat32 ->
             if propagate_float_constants ()
             then F (Float.singleoffloat n)
             else Unknown
           | _ -> Unknown)
     else eval_load_init chunk (Z.sub pos (Zpos (Coq_xO (Coq_xO Coq_xH))))
            il'
   | Init_float64 n ->
     if zeq pos Z0
     then (match chunk with
           | Mfloat64 ->
             if propagate_float_constants () then F n else Unknown
           | _ -> Unknown)
     else eval_load_init chunk
            (Z.sub pos (Zpos (Coq_xO (Coq_xO (Coq_xO Coq_xH))))) il'
   | Init_space n -> eval_load_init chunk (Z.sub pos (Z.max n Z0)) il'
   | Init_addrof (symb, ofs) ->
     if zeq pos Z0
     then (match chunk with
           | Mint32 -> G (symb, ofs)
           | _ -> Unknown)
     else eval_load_init chunk (Z.sub pos (Zpos (Coq_xO (Coq_xO Coq_xH))))
            il')

let eval_static_load :     global_approx -> memory_chunk -> approx -> approx =
  fun gapp chunk  ->
    function
| G (symb, ofs) ->
  (match PTree.get symb gapp with
   | Some il -> eval_load_init chunk (Int.unsigned ofs) il
   | None -> Unknown)
| _ -> Unknown

let approx_reg :  D.t -> reg -> Approx.t =
  fun app r  ->
  D.get r app

let approx_regs :  D.t -> reg list -> Approx.t list =
  fun app rl  ->
  map (approx_reg app) rl

let transfer :  global_approx -> coq_function -> node -> D.t -> D.t =
  fun gapp f pc before  ->
  match PTree.get pc f.fn_code with
  | Some i ->
    (match i with
     | Iop (op, args, res, s) ->
       let a = eval_static_operation op (approx_regs before args) in
       D.set res a before
     | Iload (chunk, addr, args, dst, s) ->
       let a =
         eval_static_load gapp chunk
           (eval_static_addressing addr (approx_regs before args))
       in
       D.set dst a before
     | Icall (sig0, ros, args, res, s) -> D.set res Unknown before
     | Ibuiltin (ef, args, res, s) -> D.set res Unknown before
     | _ -> before)
  | None -> before

module DS = 
 struct 
  module L = 
   struct 
    type t = t'
    
    (** val get : positive -> t' -> Approx.t **)
    
    let get p = function
    | Bot_except m ->
      (match PTree.get p m with
       | Some x0 -> x0
       | None -> Approx.bot)
    | Top_except m ->
      (match PTree.get p m with
       | Some x0 -> x0
       | None -> Approx.top)
    
    (** val set : positive -> Approx.t -> t' -> t' **)
    
    let set p v = function
    | Bot_except m ->
      Bot_except
        (if Approx.beq v Approx.bot
         then PTree.remove p m
         else PTree.set p v m)
    | Top_except m ->
      Top_except
        (if Approx.beq v Approx.top
         then PTree.remove p m
         else PTree.set p v m)
    
    (** val beq : t' -> t' -> bool **)
    
    let beq x y =
      match x with
      | Bot_except m ->
        (match y with
         | Bot_except n -> PTree.beq Approx.beq m n
         | Top_except t0 -> false)
      | Top_except m ->
        (match y with
         | Bot_except t0 -> false
         | Top_except n -> PTree.beq Approx.beq m n)
    
    (** val bot : t' **)
    
    let bot =
      Bot_except PTree.empty
    
    (** val top : t' **)
    
    let top =
      Top_except PTree.empty
    
    (** val opt_beq : Approx.t option -> Approx.t option -> bool **)
    
    let opt_beq ox oy =
      match ox with
      | Some x ->
        (match oy with
         | Some y -> Approx.beq x y
         | None -> false)
      | None ->
        (match oy with
         | Some t0 -> false
         | None -> true)
    
    type changed =
    | Unchanged
    | Changed of Approx.t PTree.t
    
    (** val combine_l :
        (Approx.t option -> Approx.t option -> Approx.t option) -> Approx.t
        PTree.t -> changed **)
    
    let rec combine_l :
        (Approx.t option -> Approx.t option -> Approx.t option) -> Approx.t
        PTree.t -> changed =
      fun f -> function
    | PTree.Leaf -> Unchanged
    | PTree.Node (l, o, r) ->
      let o' = f o None in
      (match combine_l f l with
       | Unchanged ->
         (match combine_l f r with
          | Unchanged ->
            if opt_beq o' o
            then Unchanged
            else Changed (PTree.coq_Node' l o' r)
          | Changed r' -> Changed (PTree.coq_Node' l o' r'))
       | Changed l' ->
         (match combine_l f r with
          | Unchanged -> Changed (PTree.coq_Node' l' o' r)
          | Changed r' -> Changed (PTree.coq_Node' l' o' r')))
    
    (** val combine_r :
        (Approx.t option -> Approx.t option -> Approx.t option) -> Approx.t
        PTree.t -> changed **)
    
    let rec combine_r :
        (Approx.t option -> Approx.t option -> Approx.t option) -> Approx.t
        PTree.t -> changed =
      fun f -> function
    | PTree.Leaf -> Unchanged
    | PTree.Node (l, o, r) ->
      let o' = f None o in
      (match combine_r f l with
       | Unchanged ->
         (match combine_r f r with
          | Unchanged ->
            if opt_beq o' o
            then Unchanged
            else Changed (PTree.coq_Node' l o' r)
          | Changed r' -> Changed (PTree.coq_Node' l o' r'))
       | Changed l' ->
         (match combine_r f r with
          | Unchanged -> Changed (PTree.coq_Node' l' o' r)
          | Changed r' -> Changed (PTree.coq_Node' l' o' r')))
    
    type changed2 =
    | Same
    | Same1
    | Same2
    | CC of Approx.t PTree.t
    
    (** val xcombine :
        (Approx.t option -> Approx.t option -> Approx.t option) -> Approx.t
        PTree.t -> Approx.t PTree.t -> changed2 **)
    
    let rec xcombine f m1 m2 =
      match m1 with
      | PTree.Leaf ->
        (match m2 with
         | PTree.Leaf -> Same
         | PTree.Node (t0, o, t1) ->
           (match combine_r f m2 with
            | Unchanged -> Same2
            | Changed m -> CC m))
      | PTree.Node (l1, o1, r1) ->
        (match m2 with
         | PTree.Leaf ->
           (match combine_l f m1 with
            | Unchanged -> Same1
            | Changed m -> CC m)
         | PTree.Node (l2, o2, r2) ->
           let o = f o1 o2 in
           (match xcombine f l1 l2 with
            | Same ->
              (match xcombine f r1 r2 with
               | Same ->
                 if opt_beq o o1
                 then if opt_beq o o2 then Same else Same1
                 else if opt_beq o o2
                      then Same2
                      else CC (PTree.coq_Node' l1 o r1)
               | Same1 ->
                 if opt_beq o o1 then Same1 else CC (PTree.coq_Node' l1 o r1)
               | Same2 ->
                 if opt_beq o o2 then Same2 else CC (PTree.coq_Node' l2 o r2)
               | CC r -> CC (PTree.coq_Node' l1 o r))
            | Same1 ->
              (match xcombine f r1 r2 with
               | Same2 -> CC (PTree.coq_Node' l1 o r2)
               | CC r -> CC (PTree.coq_Node' l1 o r)
               | _ ->
                 if opt_beq o o1 then Same1 else CC (PTree.coq_Node' l1 o r1))
            | Same2 ->
              (match xcombine f r1 r2 with
               | Same1 -> CC (PTree.coq_Node' l2 o r1)
               | CC r -> CC (PTree.coq_Node' l2 o r)
               | _ ->
                 if opt_beq o o2 then Same2 else CC (PTree.coq_Node' l2 o r2))
            | CC l ->
              (match xcombine f r1 r2 with
               | Same2 -> CC (PTree.coq_Node' l o r2)
               | CC r -> CC (PTree.coq_Node' l o r)
               | _ -> CC (PTree.coq_Node' l o r1))))
    
    (** val combine :
        (Approx.t option -> Approx.t option -> Approx.t option) -> Approx.t
        PTree.t -> Approx.t PTree.t -> Approx.t PTree.t **)
    
    let combine f m1 m2 =
      match xcombine f m1 m2 with
      | Same2 -> m2
      | CC m -> m
      | _ -> m1
    
    (** val opt_lub : Approx.t -> Approx.t -> Approx.t option **)
    
    let opt_lub x y =
      let z = Approx.lub x y in
      if Approx.beq z Approx.top then None else Some z
    
    (** val lub : t' -> t' -> t' **)
    
    let lub x y =
      match x with
      | Bot_except m ->
        (match y with
         | Bot_except n ->
           Bot_except
             (combine (fun a b ->
               match a with
               | Some u ->
                 (match b with
                  | Some v -> Some (Approx.lub u v)
                  | None -> a)
               | None -> b) m n)
         | Top_except n ->
           Top_except
             (combine (fun a b ->
               match a with
               | Some u ->
                 (match b with
                  | Some v -> opt_lub u v
                  | None -> None)
               | None -> b) m n))
      | Top_except m ->
        (match y with
         | Bot_except n ->
           Top_except
             (combine (fun a b ->
               match a with
               | Some u ->
                 (match b with
                  | Some v -> opt_lub u v
                  | None -> a)
               | None -> None) m n)
         | Top_except n ->
           Top_except
             (combine (fun a b ->
               match a with
               | Some u ->
                 (match b with
                  | Some v -> opt_lub u v
                  | None -> None)
               | None -> None) m n))
   end
  
  (** val fixpoint :
      positive list PTree.t -> (positive -> L.t -> L.t) -> (positive * L.t)
      list -> L.t PMap.t option **)
  
  let fixpoint :
      positive list PTree.t -> (positive -> L.t -> L.t) -> (positive * L.t)
      list -> L.t PMap.t option =
    fun x y z ->
    failwith "AXIOM TO BE REALIZED"
 end

let analyze :  global_approx -> coq_function -> D.t PMap.t =
  fun gapp f  ->
  match DS.fixpoint (successors f) (transfer gapp f) ((f.fn_entrypoint,
          D.top) :: []) with
  | Some res -> res
  | None -> PMap.init D.top

let transf_ros :  D.t -> (reg, ident) sum -> (reg, ident) sum =
  fun app ros  ->
    match ros with
| Coq_inl r ->
  (match D.get r app with
   | G (symb, ofs) -> if Int.eq ofs Int.zero then Coq_inr symb else ros
   | _ -> ros)
| Coq_inr s -> ros

let generate_float_constants :  unit -> bool =
  fun _ -> !Clflags.option_ffloatconstprop > 2

let const_for_result :  approx -> operation option =
    function
| I n -> Some (Ointconst n)
| F n -> if generate_float_constants () then Some (Ofloatconst n) else None
| G (symb, ofs) -> Some (coq_Oaddrsymbol symb ofs)
| ConstpropOp.S ofs -> Some (coq_Oaddrstack ofs)
| _ -> None

let rec successor_rec :  nat -> coq_function -> D.t -> node -> node =
  fun n f app pc  ->
  match n with
  | O -> pc
  | S n' ->
    (match PTree.get pc f.fn_code with
     | Some i ->
       (match i with
        | Inop s -> successor_rec n' f app s
        | Icond (cond, args, s1, s2) ->
          (match eval_static_condition cond (approx_regs app args) with
           | Some b -> if b then s1 else s2
           | None -> pc)
        | _ -> pc)
     | None -> pc)

let num_iter :  nat =
  S (S (S (S (S (S (S (S (S (S O)))))))))

let successor :  coq_function -> D.t -> node -> node =
  fun f app pc  ->
  successor_rec num_iter f app pc

let rec annot_strength_reduction :     D.t -> annot_arg list -> reg list -> annot_arg list * reg list =
  fun app targs args  ->
  match targs with
  | [] -> (targs, args)
  | targ :: targs' ->
    (match targ with
     | AA_arg ty ->
       (match args with
        | [] ->
          let (targs'', args'') = annot_strength_reduction app targs' args in
          ((targ :: targs''), args'')
        | arg :: args' ->
          let (targs'', args'') = annot_strength_reduction app targs' args'
          in
          (match ty with
           | Tint ->
             (match approx_reg app arg with
              | I n -> (((AA_int n) :: targs''), args'')
              | _ -> (((AA_arg ty) :: targs''), (arg :: args'')))
           | Tfloat ->
             (match approx_reg app arg with
              | F n -> (((AA_float n) :: targs''), args'')
              | _ -> (((AA_arg ty) :: targs''), (arg :: args'')))))
     | _ ->
       let (targs'', args'') = annot_strength_reduction app targs' args in
       ((targ :: targs''), args''))

let builtin_strength_reduction :     D.t -> external_function -> reg list -> external_function * reg list =
  fun app ef args  ->
  match ef with
  | EF_vload chunk ->
    (match args with
     | [] -> (ef, args)
     | r1 :: l ->
       (match l with
        | [] ->
          (match approx_reg app r1 with
           | G (symb, n1) -> ((EF_vload_global (chunk, symb, n1)), [])
           | _ -> (ef, args))
        | r :: l0 -> (ef, args)))
  | EF_vstore chunk ->
    (match args with
     | [] -> (ef, args)
     | r1 :: l ->
       (match l with
        | [] -> (ef, args)
        | r2 :: l0 ->
          (match l0 with
           | [] ->
             (match approx_reg app r1 with
              | G (symb, n1) ->
                ((EF_vstore_global (chunk, symb, n1)), (r2 :: []))
              | _ -> (ef, args))
           | r :: l1 -> (ef, args))))
  | EF_annot (text, targs) ->
    let (targs', args') = annot_strength_reduction app targs args in
    ((EF_annot (text, targs')), args')
  | _ -> (ef, args)

let transf_instr :     global_approx -> coq_function -> D.t PMap.t -> node -> instruction ->
    instruction =
  fun gapp f apps pc instr  ->
  let app = PMap.get pc apps in
  (match instr with
   | Iop (op, args, res, s) ->
     let a = eval_static_operation op (approx_regs app args) in
     let s' = successor f (D.set res a app) s in
     (match const_for_result a with
      | Some cop -> Iop (cop, [], res, s')
      | None ->
        let (op', args') =
          op_strength_reduction op args (approx_regs app args)
        in
        Iop (op', args', res, s'))
   | Iload (chunk, addr, args, dst, s) ->
     let a =
       eval_static_load gapp chunk
         (eval_static_addressing addr (approx_regs app args))
     in
     (match const_for_result a with
      | Some cop -> Iop (cop, [], dst, s)
      | None ->
        let (addr', args') =
          addr_strength_reduction addr args (approx_regs app args)
        in
        Iload (chunk, addr', args', dst, s))
   | Istore (chunk, addr, args, src, s) ->
     let (addr', args') =
       addr_strength_reduction addr args (approx_regs app args)
     in
     Istore (chunk, addr', args', src, s)
   | Icall (sig0, ros, args, res, s) ->
     Icall (sig0, (transf_ros app ros), args, res, s)
   | Itailcall (sig0, ros, args) ->
     Itailcall (sig0, (transf_ros app ros), args)
   | Ibuiltin (ef, args, res, s) ->
     let (ef', args') = builtin_strength_reduction app ef args in
     Ibuiltin (ef', args', res, s)
   | Icond (cond, args, s1, s2) ->
     (match eval_static_condition cond (approx_regs app args) with
      | Some b -> if b then Inop s1 else Inop s2
      | None ->
        let (cond', args') =
          cond_strength_reduction cond args (approx_regs app args)
        in
        Icond (cond', args', s1, s2))
   | Ijumptable (arg, tbl) ->
     (match approx_reg app arg with
      | I n ->
        (match list_nth_z tbl (Int.unsigned n) with
         | Some s -> Inop s
         | None -> instr)
      | _ -> instr)
   | _ -> instr)

let transf_code :     global_approx -> coq_function -> D.t PMap.t -> code -> code =
  fun gapp f app instrs  ->
  PTree.map (transf_instr gapp f app) instrs

let transf_function :  global_approx -> coq_function -> coq_function =
  fun gapp f  ->
  let approxs = analyze gapp f in
  { fn_sig = f.fn_sig; fn_params = f.fn_params; fn_stacksize =
  f.fn_stacksize; fn_code = (transf_code gapp f approxs f.fn_code);
  fn_entrypoint = f.fn_entrypoint }

let transf_fundef :  global_approx -> fundef -> fundef =
  fun gapp fd  ->
  transf_fundef (transf_function gapp) fd

let rec make_global_approx :     global_approx -> (ident * (fundef, unit) globdef) list -> global_approx =
  fun gapp  ->
    function
| [] -> gapp
| p :: gdl' ->
  let (id, gl) = p in
  let gapp1 =
    match gl with
    | Gfun f -> PTree.remove id gapp
    | Gvar gv ->
      if (&&) gv.gvar_readonly (negb gv.gvar_volatile)
      then PTree.set id gv.gvar_init gapp
      else PTree.remove id gapp
  in
  make_global_approx gapp1 gdl'

let transf_program :  program -> program =
  fun p  ->
  let gapp = make_global_approx PTree.empty p.prog_defs in
  transform_program (transf_fundef gapp) p

