open AST
open BinInt
open BinNums
open ConstpropOp
open Coqlib
open Datatypes
open Floats
open Integers
open Lattice
open List0
open Maps
open Op
open RTL
open Registers

module Approx : 
 sig 
  type t = approx
  
  val eq_dec : t -> t -> bool
  
  val beq : t -> t -> bool
  
  val bot : approx
  
  val top : approx
  
  val lub : t -> t -> t
 end

type t' =
| Bot_except of Approx.t PTree.t
| Top_except of Approx.t PTree.t

module D : 
 sig 
  type t = t'
  
  val get : positive -> t -> Approx.t
  
  val set : positive -> Approx.t -> t -> t
  
  val beq : t -> t -> bool
  
  val bot : t'
  
  val top : t'
  
  val opt_beq : Approx.t option -> Approx.t option -> bool
  
  type changed =
  | Unchanged
  | Changed of Approx.t PTree.t
  
  val combine_l :
    (Approx.t option -> Approx.t option -> Approx.t option) -> Approx.t
    PTree.t -> changed
  
  val combine_r :
    (Approx.t option -> Approx.t option -> Approx.t option) -> Approx.t
    PTree.t -> changed
  
  type changed2 =
  | Same
  | Same1
  | Same2
  | CC of Approx.t PTree.t
  
  val xcombine :
    (Approx.t option -> Approx.t option -> Approx.t option) -> Approx.t
    PTree.t -> Approx.t PTree.t -> changed2
  
  val combine :
    (Approx.t option -> Approx.t option -> Approx.t option) -> Approx.t
    PTree.t -> Approx.t PTree.t -> Approx.t PTree.t
  
  val opt_lub : Approx.t -> Approx.t -> Approx.t option
  
  val lub : t -> t -> t
 end

type global_approx = init_data list PTree.t

val eval_load_init : memory_chunk -> coq_Z -> init_data list -> approx

val eval_static_load : global_approx -> memory_chunk -> approx -> approx

val approx_reg : D.t -> reg -> Approx.t

val approx_regs : D.t -> reg list -> Approx.t list

val transfer : global_approx -> coq_function -> node -> D.t -> D.t

module DS : 
 sig 
  module L : 
  sig 
    type t = t'
    val beq : t -> t -> bool
    val bot : t
    val lub : t -> t -> t
  end
  
  val fixpoint :
    positive list PTree.t -> (positive -> L.t -> L.t) -> (positive * L.t)
    list -> L.t PMap.t option
 end

val analyze : global_approx -> coq_function -> D.t PMap.t

val transf_ros : D.t -> (reg, ident) sum -> (reg, ident) sum

val generate_float_constants : unit -> bool

val const_for_result : approx -> operation option

val successor_rec : nat -> coq_function -> D.t -> node -> node

val num_iter : nat

val successor : coq_function -> D.t -> node -> node

val annot_strength_reduction :
  D.t -> annot_arg list -> reg list -> annot_arg list * reg list

val builtin_strength_reduction :
  D.t -> external_function -> reg list -> external_function * reg list

val transf_instr :
  global_approx -> coq_function -> D.t PMap.t -> node -> instruction ->
  instruction

val transf_code : global_approx -> coq_function -> D.t PMap.t -> code -> code

val transf_function : global_approx -> coq_function -> coq_function

val transf_fundef : global_approx -> fundef -> fundef

val make_global_approx :
  global_approx -> (ident * (fundef, unit) globdef) list -> global_approx

val transf_program : program -> program

