open AST
open BinNums
open Datatypes
open Floats
open Integers
open Op
open Registers

type approx =
| Novalue
| Unknown
| I of Int.int
| F of float
| G of ident * Int.int
| S of Int.int

type eval_static_condition_cases =
| Coq_eval_static_condition_case1 of comparison * Int.int * Int.int
| Coq_eval_static_condition_case2 of comparison * Int.int * Int.int
| Coq_eval_static_condition_case3 of comparison * Int.int * Int.int
| Coq_eval_static_condition_case4 of comparison * Int.int * Int.int
| Coq_eval_static_condition_case5 of comparison * float * float
| Coq_eval_static_condition_case6 of comparison * float * float
| Coq_eval_static_condition_case7 of Int.int * Int.int
| Coq_eval_static_condition_case8 of Int.int * Int.int
| Coq_eval_static_condition_default of condition * approx list

let eval_static_condition_match :     condition -> approx list -> eval_static_condition_cases =
  fun cond vl  ->
  match cond with
  | Ccomp c ->
    let cond0 = Ccomp c in
    (match vl with
     | [] -> Coq_eval_static_condition_default (cond0, [])
     | a :: l ->
       (match a with
        | I n1 ->
          (match l with
           | [] -> Coq_eval_static_condition_default (cond0, ((I n1) :: []))
           | a0 :: l0 ->
             (match a0 with
              | I n2 ->
                (match l0 with
                 | [] -> Coq_eval_static_condition_case1 (c, n1, n2)
                 | a1 :: l1 ->
                   Coq_eval_static_condition_default (cond0, ((I n1) :: ((I
                     n2) :: (a1 :: l1)))))
              | x ->
                Coq_eval_static_condition_default (cond0, ((I
                  n1) :: (x :: l0)))))
        | x -> Coq_eval_static_condition_default (cond0, (x :: l))))
  | Ccompu c ->
    let cond0 = Ccompu c in
    (match vl with
     | [] -> Coq_eval_static_condition_default (cond0, [])
     | a :: l ->
       (match a with
        | I n1 ->
          (match l with
           | [] -> Coq_eval_static_condition_default (cond0, ((I n1) :: []))
           | a0 :: l0 ->
             (match a0 with
              | I n2 ->
                (match l0 with
                 | [] -> Coq_eval_static_condition_case2 (c, n1, n2)
                 | a1 :: l1 ->
                   Coq_eval_static_condition_default (cond0, ((I n1) :: ((I
                     n2) :: (a1 :: l1)))))
              | x ->
                Coq_eval_static_condition_default (cond0, ((I
                  n1) :: (x :: l0)))))
        | x -> Coq_eval_static_condition_default (cond0, (x :: l))))
  | Ccompimm (c, n) ->
    let cond0 = Ccompimm (c, n) in
    (match vl with
     | [] -> Coq_eval_static_condition_default (cond0, [])
     | a :: l ->
       (match a with
        | I n1 ->
          (match l with
           | [] -> Coq_eval_static_condition_case3 (c, n, n1)
           | a0 :: l0 ->
             Coq_eval_static_condition_default (cond0, ((I
               n1) :: (a0 :: l0))))
        | x -> Coq_eval_static_condition_default (cond0, (x :: l))))
  | Ccompuimm (c, n) ->
    let cond0 = Ccompuimm (c, n) in
    (match vl with
     | [] -> Coq_eval_static_condition_default (cond0, [])
     | a :: l ->
       (match a with
        | I n1 ->
          (match l with
           | [] -> Coq_eval_static_condition_case4 (c, n, n1)
           | a0 :: l0 ->
             Coq_eval_static_condition_default (cond0, ((I
               n1) :: (a0 :: l0))))
        | x -> Coq_eval_static_condition_default (cond0, (x :: l))))
  | Ccompf c ->
    let cond0 = Ccompf c in
    (match vl with
     | [] -> Coq_eval_static_condition_default (cond0, [])
     | a :: l ->
       (match a with
        | F n1 ->
          (match l with
           | [] -> Coq_eval_static_condition_default (cond0, ((F n1) :: []))
           | a0 :: l0 ->
             (match a0 with
              | F n2 ->
                (match l0 with
                 | [] -> Coq_eval_static_condition_case5 (c, n1, n2)
                 | a1 :: l1 ->
                   Coq_eval_static_condition_default (cond0, ((F n1) :: ((F
                     n2) :: (a1 :: l1)))))
              | x ->
                Coq_eval_static_condition_default (cond0, ((F
                  n1) :: (x :: l0)))))
        | x -> Coq_eval_static_condition_default (cond0, (x :: l))))
  | Cnotcompf c ->
    let cond0 = Cnotcompf c in
    (match vl with
     | [] -> Coq_eval_static_condition_default (cond0, [])
     | a :: l ->
       (match a with
        | F n1 ->
          (match l with
           | [] -> Coq_eval_static_condition_default (cond0, ((F n1) :: []))
           | a0 :: l0 ->
             (match a0 with
              | F n2 ->
                (match l0 with
                 | [] -> Coq_eval_static_condition_case6 (c, n1, n2)
                 | a1 :: l1 ->
                   Coq_eval_static_condition_default (cond0, ((F n1) :: ((F
                     n2) :: (a1 :: l1)))))
              | x ->
                Coq_eval_static_condition_default (cond0, ((F
                  n1) :: (x :: l0)))))
        | x -> Coq_eval_static_condition_default (cond0, (x :: l))))
  | Cmaskzero n ->
    let cond0 = Cmaskzero n in
    (match vl with
     | [] -> Coq_eval_static_condition_default (cond0, [])
     | a :: l ->
       (match a with
        | I n1 ->
          (match l with
           | [] -> Coq_eval_static_condition_case7 (n, n1)
           | a0 :: l0 ->
             Coq_eval_static_condition_default (cond0, ((I
               n1) :: (a0 :: l0))))
        | x -> Coq_eval_static_condition_default (cond0, (x :: l))))
  | Cmasknotzero n ->
    let cond0 = Cmasknotzero n in
    (match vl with
     | [] -> Coq_eval_static_condition_default (cond0, [])
     | a :: l ->
       (match a with
        | I n1 ->
          (match l with
           | [] -> Coq_eval_static_condition_case8 (n, n1)
           | a0 :: l0 ->
             Coq_eval_static_condition_default (cond0, ((I
               n1) :: (a0 :: l0))))
        | x -> Coq_eval_static_condition_default (cond0, (x :: l))))

let eval_static_condition :  condition -> approx list -> bool option =
  fun cond vl  ->
  match eval_static_condition_match cond vl with
  | Coq_eval_static_condition_case1 (c, n1, n2) -> Some (Int.cmp c n1 n2)
  | Coq_eval_static_condition_case2 (c, n1, n2) -> Some (Int.cmpu c n1 n2)
  | Coq_eval_static_condition_case3 (c, n, n1) -> Some (Int.cmp c n1 n)
  | Coq_eval_static_condition_case4 (c, n, n1) -> Some (Int.cmpu c n1 n)
  | Coq_eval_static_condition_case5 (c, n1, n2) -> Some (Float.cmp c n1 n2)
  | Coq_eval_static_condition_case6 (c, n1, n2) ->
    Some (negb (Float.cmp c n1 n2))
  | Coq_eval_static_condition_case7 (n, n1) ->
    Some (Int.eq (Int.coq_and n1 n) Int.zero)
  | Coq_eval_static_condition_case8 (n, n1) ->
    Some (negb (Int.eq (Int.coq_and n1 n) Int.zero))
  | Coq_eval_static_condition_default (cond0, vl0) -> None

let eval_static_condition_val :  condition -> approx list -> approx =
  fun cond vl  ->
  match eval_static_condition cond vl with
  | Some b -> I (if b then Int.one else Int.zero)
  | None -> Unknown

let eval_static_intoffloat :  float -> approx =
  fun f  ->
  match Float.intoffloat f with
  | Some x -> I x
  | None -> Unknown

type eval_static_addressing_cases =
| Coq_eval_static_addressing_case1 of Int.int * Int.int
| Coq_eval_static_addressing_case2 of Int.int * ident * Int.int
| Coq_eval_static_addressing_case3 of Int.int * Int.int
| Coq_eval_static_addressing_case4 of Int.int * Int.int * Int.int
| Coq_eval_static_addressing_case5 of Int.int * ident * Int.int * Int.int
| Coq_eval_static_addressing_case6 of Int.int * Int.int * ident * Int.int
| Coq_eval_static_addressing_case7 of Int.int * Int.int * Int.int
| Coq_eval_static_addressing_case8 of Int.int * Int.int * Int.int
| Coq_eval_static_addressing_case9 of Int.int * Int.int * Int.int
| Coq_eval_static_addressing_case10 of Int.int * Int.int * Int.int * Int.int
| Coq_eval_static_addressing_case11 of Int.int * Int.int * ident * Int.int
   * Int.int
| Coq_eval_static_addressing_case12 of Int.int * Int.int * Int.int * Int.int
| Coq_eval_static_addressing_case13 of ident * Int.int
| Coq_eval_static_addressing_case14 of ident * Int.int * Int.int
| Coq_eval_static_addressing_case15 of Int.int * ident * Int.int * Int.int
| Coq_eval_static_addressing_case16 of Int.int
| Coq_eval_static_addressing_default of addressing * approx list

let eval_static_addressing_match :     addressing -> approx list -> eval_static_addressing_cases =
  fun addr vl  ->
  match addr with
  | Aindexed n ->
    let addr0 = Aindexed n in
    (match vl with
     | [] -> Coq_eval_static_addressing_default (addr0, [])
     | a :: l ->
       (match a with
        | I n1 ->
          (match l with
           | [] -> Coq_eval_static_addressing_case1 (n, n1)
           | a0 :: l0 ->
             Coq_eval_static_addressing_default (addr0, ((I
               n1) :: (a0 :: l0))))
        | G (id, ofs) ->
          (match l with
           | [] -> Coq_eval_static_addressing_case2 (n, id, ofs)
           | a0 :: l0 ->
             Coq_eval_static_addressing_default (addr0, ((G (id,
               ofs)) :: (a0 :: l0))))
        | S ofs ->
          (match l with
           | [] -> Coq_eval_static_addressing_case3 (n, ofs)
           | a0 :: l0 ->
             Coq_eval_static_addressing_default (addr0, ((S
               ofs) :: (a0 :: l0))))
        | x -> Coq_eval_static_addressing_default (addr0, (x :: l))))
  | Aindexed2 n ->
    let addr0 = Aindexed2 n in
    (match vl with
     | [] -> Coq_eval_static_addressing_default (addr0, [])
     | a :: l ->
       (match a with
        | I n1 ->
          (match l with
           | [] -> Coq_eval_static_addressing_default (addr0, ((I n1) :: []))
           | a0 :: l0 ->
             (match a0 with
              | I n2 ->
                (match l0 with
                 | [] -> Coq_eval_static_addressing_case4 (n, n1, n2)
                 | a1 :: l1 ->
                   Coq_eval_static_addressing_default (addr0, ((I n1) :: ((I
                     n2) :: (a1 :: l1)))))
              | G (id, ofs) ->
                (match l0 with
                 | [] -> Coq_eval_static_addressing_case6 (n, n1, id, ofs)
                 | a1 :: l1 ->
                   Coq_eval_static_addressing_default (addr0, ((I n1) :: ((G
                     (id, ofs)) :: (a1 :: l1)))))
              | S ofs ->
                (match l0 with
                 | [] -> Coq_eval_static_addressing_case8 (n, n1, ofs)
                 | a1 :: l1 ->
                   Coq_eval_static_addressing_default (addr0, ((I n1) :: ((S
                     ofs) :: (a1 :: l1)))))
              | x ->
                Coq_eval_static_addressing_default (addr0, ((I
                  n1) :: (x :: l0)))))
        | G (id, ofs) ->
          (match l with
           | [] ->
             Coq_eval_static_addressing_default (addr0, ((G (id,
               ofs)) :: []))
           | a0 :: l0 ->
             (match a0 with
              | I n2 ->
                (match l0 with
                 | [] -> Coq_eval_static_addressing_case5 (n, id, ofs, n2)
                 | a1 :: l1 ->
                   Coq_eval_static_addressing_default (addr0, ((G (id,
                     ofs)) :: ((I n2) :: (a1 :: l1)))))
              | x ->
                Coq_eval_static_addressing_default (addr0, ((G (id,
                  ofs)) :: (x :: l0)))))
        | S ofs ->
          (match l with
           | [] ->
             Coq_eval_static_addressing_default (addr0, ((S ofs) :: []))
           | a0 :: l0 ->
             (match a0 with
              | I n2 ->
                (match l0 with
                 | [] -> Coq_eval_static_addressing_case7 (n, ofs, n2)
                 | a1 :: l1 ->
                   Coq_eval_static_addressing_default (addr0, ((S ofs) :: ((I
                     n2) :: (a1 :: l1)))))
              | x ->
                Coq_eval_static_addressing_default (addr0, ((S
                  ofs) :: (x :: l0)))))
        | x -> Coq_eval_static_addressing_default (addr0, (x :: l))))
  | Ascaled (sc, n) ->
    let addr0 = Ascaled (sc, n) in
    (match vl with
     | [] -> Coq_eval_static_addressing_default (addr0, [])
     | a :: l ->
       (match a with
        | I n1 ->
          (match l with
           | [] -> Coq_eval_static_addressing_case9 (sc, n, n1)
           | a0 :: l0 ->
             Coq_eval_static_addressing_default (addr0, ((I
               n1) :: (a0 :: l0))))
        | x -> Coq_eval_static_addressing_default (addr0, (x :: l))))
  | Aindexed2scaled (sc, n) ->
    let addr0 = Aindexed2scaled (sc, n) in
    (match vl with
     | [] -> Coq_eval_static_addressing_default (addr0, [])
     | a :: l ->
       (match a with
        | I n1 ->
          (match l with
           | [] -> Coq_eval_static_addressing_default (addr0, ((I n1) :: []))
           | a0 :: l0 ->
             (match a0 with
              | I n2 ->
                (match l0 with
                 | [] -> Coq_eval_static_addressing_case10 (sc, n, n1, n2)
                 | a1 :: l1 ->
                   Coq_eval_static_addressing_default (addr0, ((I n1) :: ((I
                     n2) :: (a1 :: l1)))))
              | x ->
                Coq_eval_static_addressing_default (addr0, ((I
                  n1) :: (x :: l0)))))
        | G (id, ofs) ->
          (match l with
           | [] ->
             Coq_eval_static_addressing_default (addr0, ((G (id,
               ofs)) :: []))
           | a0 :: l0 ->
             (match a0 with
              | I n2 ->
                (match l0 with
                 | [] ->
                   Coq_eval_static_addressing_case11 (sc, n, id, ofs, n2)
                 | a1 :: l1 ->
                   Coq_eval_static_addressing_default (addr0, ((G (id,
                     ofs)) :: ((I n2) :: (a1 :: l1)))))
              | x ->
                Coq_eval_static_addressing_default (addr0, ((G (id,
                  ofs)) :: (x :: l0)))))
        | S ofs ->
          (match l with
           | [] ->
             Coq_eval_static_addressing_default (addr0, ((S ofs) :: []))
           | a0 :: l0 ->
             (match a0 with
              | I n2 ->
                (match l0 with
                 | [] -> Coq_eval_static_addressing_case12 (sc, n, ofs, n2)
                 | a1 :: l1 ->
                   Coq_eval_static_addressing_default (addr0, ((S ofs) :: ((I
                     n2) :: (a1 :: l1)))))
              | x ->
                Coq_eval_static_addressing_default (addr0, ((S
                  ofs) :: (x :: l0)))))
        | x -> Coq_eval_static_addressing_default (addr0, (x :: l))))
  | Aglobal (id, ofs) ->
    (match vl with
     | [] -> Coq_eval_static_addressing_case13 (id, ofs)
     | a :: l ->
       Coq_eval_static_addressing_default ((Aglobal (id, ofs)), (a :: l)))
  | Abased (id, ofs) ->
    let addr0 = Abased (id, ofs) in
    (match vl with
     | [] -> Coq_eval_static_addressing_default (addr0, [])
     | a :: l ->
       (match a with
        | I n1 ->
          (match l with
           | [] -> Coq_eval_static_addressing_case14 (id, ofs, n1)
           | a0 :: l0 ->
             Coq_eval_static_addressing_default (addr0, ((I
               n1) :: (a0 :: l0))))
        | x -> Coq_eval_static_addressing_default (addr0, (x :: l))))
  | Abasedscaled (sc, id, ofs) ->
    let addr0 = Abasedscaled (sc, id, ofs) in
    (match vl with
     | [] -> Coq_eval_static_addressing_default (addr0, [])
     | a :: l ->
       (match a with
        | I n1 ->
          (match l with
           | [] -> Coq_eval_static_addressing_case15 (sc, id, ofs, n1)
           | a0 :: l0 ->
             Coq_eval_static_addressing_default (addr0, ((I
               n1) :: (a0 :: l0))))
        | x -> Coq_eval_static_addressing_default (addr0, (x :: l))))
  | Ainstack ofs ->
    (match vl with
     | [] -> Coq_eval_static_addressing_case16 ofs
     | a :: l ->
       Coq_eval_static_addressing_default ((Ainstack ofs), (a :: l)))

let eval_static_addressing :  addressing -> approx list -> approx =
  fun addr vl  ->
  match eval_static_addressing_match addr vl with
  | Coq_eval_static_addressing_case1 (n, n1) -> I (Int.add n1 n)
  | Coq_eval_static_addressing_case2 (n, id, ofs) -> G (id, (Int.add ofs n))
  | Coq_eval_static_addressing_case3 (n, ofs) -> S (Int.add ofs n)
  | Coq_eval_static_addressing_case4 (n, n1, n2) ->
    I (Int.add (Int.add n1 n2) n)
  | Coq_eval_static_addressing_case5 (n, id, ofs, n2) ->
    G (id, (Int.add (Int.add ofs n2) n))
  | Coq_eval_static_addressing_case6 (n, n1, id, ofs) ->
    G (id, (Int.add (Int.add ofs n1) n))
  | Coq_eval_static_addressing_case7 (n, ofs, n2) ->
    S (Int.add (Int.add ofs n2) n)
  | Coq_eval_static_addressing_case8 (n, n1, ofs) ->
    S (Int.add (Int.add ofs n1) n)
  | Coq_eval_static_addressing_case9 (sc, n, n1) ->
    I (Int.add (Int.mul n1 sc) n)
  | Coq_eval_static_addressing_case10 (sc, n, n1, n2) ->
    I (Int.add n1 (Int.add (Int.mul n2 sc) n))
  | Coq_eval_static_addressing_case11 (sc, n, id, ofs, n2) ->
    G (id, (Int.add ofs (Int.add (Int.mul n2 sc) n)))
  | Coq_eval_static_addressing_case12 (sc, n, ofs, n2) ->
    S (Int.add ofs (Int.add (Int.mul n2 sc) n))
  | Coq_eval_static_addressing_case13 (id, ofs) -> G (id, ofs)
  | Coq_eval_static_addressing_case14 (id, ofs, n1) ->
    G (id, (Int.add ofs n1))
  | Coq_eval_static_addressing_case15 (sc, id, ofs, n1) ->
    G (id, (Int.add ofs (Int.mul sc n1)))
  | Coq_eval_static_addressing_case16 ofs -> S ofs
  | Coq_eval_static_addressing_default (addr0, vl0) -> Unknown

let propagate_float_constants :  unit -> bool =
  fun _ -> !Clflags.option_ffloatconstprop > 1

type eval_static_operation_cases =
| Coq_eval_static_operation_case1 of approx
| Coq_eval_static_operation_case2 of Int.int
| Coq_eval_static_operation_case3 of float
| Coq_eval_static_operation_case4 of Int.int
| Coq_eval_static_operation_case5 of Int.int
| Coq_eval_static_operation_case6 of Int.int
| Coq_eval_static_operation_case7 of Int.int
| Coq_eval_static_operation_case8 of Int.int
| Coq_eval_static_operation_case9 of Int.int * Int.int
| Coq_eval_static_operation_case10 of ident * Int.int * Int.int
| Coq_eval_static_operation_case11 of Int.int * Int.int
| Coq_eval_static_operation_case12 of Int.int * Int.int
| Coq_eval_static_operation_case13 of Int.int * Int.int
| Coq_eval_static_operation_case14 of Int.int * Int.int
| Coq_eval_static_operation_case15 of Int.int * Int.int
| Coq_eval_static_operation_case16 of Int.int * Int.int
| Coq_eval_static_operation_case17 of Int.int * Int.int
| Coq_eval_static_operation_case18 of Int.int * Int.int
| Coq_eval_static_operation_case19 of Int.int * Int.int
| Coq_eval_static_operation_case20 of Int.int * Int.int
| Coq_eval_static_operation_case21 of Int.int * Int.int
| Coq_eval_static_operation_case22 of Int.int * Int.int
| Coq_eval_static_operation_case23 of Int.int * Int.int
| Coq_eval_static_operation_case24 of Int.int * Int.int
| Coq_eval_static_operation_case25 of Int.int * Int.int
| Coq_eval_static_operation_case26 of Int.int * Int.int
| Coq_eval_static_operation_case27 of Int.int * Int.int
| Coq_eval_static_operation_case28 of Int.int * Int.int
| Coq_eval_static_operation_case29 of Int.int * Int.int
| Coq_eval_static_operation_case30 of Int.int * Int.int
| Coq_eval_static_operation_case31 of addressing * approx list
| Coq_eval_static_operation_case32 of float
| Coq_eval_static_operation_case33 of float
| Coq_eval_static_operation_case34 of float * float
| Coq_eval_static_operation_case35 of float * float
| Coq_eval_static_operation_case36 of float * float
| Coq_eval_static_operation_case37 of float * float
| Coq_eval_static_operation_case38 of float
| Coq_eval_static_operation_case39 of float
| Coq_eval_static_operation_case40 of Int.int
| Coq_eval_static_operation_case41 of condition * approx list
| Coq_eval_static_operation_default of operation * approx list

let eval_static_operation_match :     operation -> approx list -> eval_static_operation_cases =
  fun op vl  ->
  match op with
  | Omove ->
    let op0 = Omove in
    (match vl with
     | [] -> Coq_eval_static_operation_default (op0, [])
     | v1 :: l ->
       (match l with
        | [] -> Coq_eval_static_operation_case1 v1
        | a :: l0 ->
          Coq_eval_static_operation_default (op0, (v1 :: (a :: l0)))))
  | Ointconst n ->
    (match vl with
     | [] -> Coq_eval_static_operation_case2 n
     | a :: l -> Coq_eval_static_operation_default ((Ointconst n), (a :: l)))
  | Ofloatconst n ->
    (match vl with
     | [] -> Coq_eval_static_operation_case3 n
     | a :: l ->
       Coq_eval_static_operation_default ((Ofloatconst n), (a :: l)))
  | Oindirectsymbol i ->
    Coq_eval_static_operation_default ((Oindirectsymbol i), vl)
  | Ocast8signed ->
    let op0 = Ocast8signed in
    (match vl with
     | [] -> Coq_eval_static_operation_default (op0, [])
     | a :: l ->
       (match a with
        | I n1 ->
          (match l with
           | [] -> Coq_eval_static_operation_case4 n1
           | a0 :: l0 ->
             Coq_eval_static_operation_default (op0, ((I n1) :: (a0 :: l0))))
        | x -> Coq_eval_static_operation_default (op0, (x :: l))))
  | Ocast8unsigned ->
    let op0 = Ocast8unsigned in
    (match vl with
     | [] -> Coq_eval_static_operation_default (op0, [])
     | a :: l ->
       (match a with
        | I n1 ->
          (match l with
           | [] -> Coq_eval_static_operation_case5 n1
           | a0 :: l0 ->
             Coq_eval_static_operation_default (op0, ((I n1) :: (a0 :: l0))))
        | x -> Coq_eval_static_operation_default (op0, (x :: l))))
  | Ocast16signed ->
    let op0 = Ocast16signed in
    (match vl with
     | [] -> Coq_eval_static_operation_default (op0, [])
     | a :: l ->
       (match a with
        | I n1 ->
          (match l with
           | [] -> Coq_eval_static_operation_case6 n1
           | a0 :: l0 ->
             Coq_eval_static_operation_default (op0, ((I n1) :: (a0 :: l0))))
        | x -> Coq_eval_static_operation_default (op0, (x :: l))))
  | Ocast16unsigned ->
    let op0 = Ocast16unsigned in
    (match vl with
     | [] -> Coq_eval_static_operation_default (op0, [])
     | a :: l ->
       (match a with
        | I n1 ->
          (match l with
           | [] -> Coq_eval_static_operation_case7 n1
           | a0 :: l0 ->
             Coq_eval_static_operation_default (op0, ((I n1) :: (a0 :: l0))))
        | x -> Coq_eval_static_operation_default (op0, (x :: l))))
  | Oneg ->
    let op0 = Oneg in
    (match vl with
     | [] -> Coq_eval_static_operation_default (op0, [])
     | a :: l ->
       (match a with
        | I n1 ->
          (match l with
           | [] -> Coq_eval_static_operation_case8 n1
           | a0 :: l0 ->
             Coq_eval_static_operation_default (op0, ((I n1) :: (a0 :: l0))))
        | x -> Coq_eval_static_operation_default (op0, (x :: l))))
  | Osub ->
    let op0 = Osub in
    (match vl with
     | [] -> Coq_eval_static_operation_default (op0, [])
     | a :: l ->
       (match a with
        | I n1 ->
          (match l with
           | [] -> Coq_eval_static_operation_default (op0, ((I n1) :: []))
           | a0 :: l0 ->
             (match a0 with
              | I n2 ->
                (match l0 with
                 | [] -> Coq_eval_static_operation_case9 (n1, n2)
                 | a1 :: l1 ->
                   Coq_eval_static_operation_default (op0, ((I n1) :: ((I
                     n2) :: (a1 :: l1)))))
              | x ->
                Coq_eval_static_operation_default (op0, ((I
                  n1) :: (x :: l0)))))
        | G (s1, n1) ->
          (match l with
           | [] ->
             Coq_eval_static_operation_default (op0, ((G (s1, n1)) :: []))
           | a0 :: l0 ->
             (match a0 with
              | I n2 ->
                (match l0 with
                 | [] -> Coq_eval_static_operation_case10 (s1, n1, n2)
                 | a1 :: l1 ->
                   Coq_eval_static_operation_default (op0, ((G (s1,
                     n1)) :: ((I n2) :: (a1 :: l1)))))
              | x ->
                Coq_eval_static_operation_default (op0, ((G (s1,
                  n1)) :: (x :: l0)))))
        | x -> Coq_eval_static_operation_default (op0, (x :: l))))
  | Omul ->
    let op0 = Omul in
    (match vl with
     | [] -> Coq_eval_static_operation_default (op0, [])
     | a :: l ->
       (match a with
        | I n1 ->
          (match l with
           | [] -> Coq_eval_static_operation_default (op0, ((I n1) :: []))
           | a0 :: l0 ->
             (match a0 with
              | I n2 ->
                (match l0 with
                 | [] -> Coq_eval_static_operation_case11 (n1, n2)
                 | a1 :: l1 ->
                   Coq_eval_static_operation_default (op0, ((I n1) :: ((I
                     n2) :: (a1 :: l1)))))
              | x ->
                Coq_eval_static_operation_default (op0, ((I
                  n1) :: (x :: l0)))))
        | x -> Coq_eval_static_operation_default (op0, (x :: l))))
  | Omulimm n ->
    let op0 = Omulimm n in
    (match vl with
     | [] -> Coq_eval_static_operation_default (op0, [])
     | a :: l ->
       (match a with
        | I n1 ->
          (match l with
           | [] -> Coq_eval_static_operation_case12 (n, n1)
           | a0 :: l0 ->
             Coq_eval_static_operation_default (op0, ((I n1) :: (a0 :: l0))))
        | x -> Coq_eval_static_operation_default (op0, (x :: l))))
  | Odiv ->
    let op0 = Odiv in
    (match vl with
     | [] -> Coq_eval_static_operation_default (op0, [])
     | a :: l ->
       (match a with
        | I n1 ->
          (match l with
           | [] -> Coq_eval_static_operation_default (op0, ((I n1) :: []))
           | a0 :: l0 ->
             (match a0 with
              | I n2 ->
                (match l0 with
                 | [] -> Coq_eval_static_operation_case13 (n1, n2)
                 | a1 :: l1 ->
                   Coq_eval_static_operation_default (op0, ((I n1) :: ((I
                     n2) :: (a1 :: l1)))))
              | x ->
                Coq_eval_static_operation_default (op0, ((I
                  n1) :: (x :: l0)))))
        | x -> Coq_eval_static_operation_default (op0, (x :: l))))
  | Odivu ->
    let op0 = Odivu in
    (match vl with
     | [] -> Coq_eval_static_operation_default (op0, [])
     | a :: l ->
       (match a with
        | I n1 ->
          (match l with
           | [] -> Coq_eval_static_operation_default (op0, ((I n1) :: []))
           | a0 :: l0 ->
             (match a0 with
              | I n2 ->
                (match l0 with
                 | [] -> Coq_eval_static_operation_case14 (n1, n2)
                 | a1 :: l1 ->
                   Coq_eval_static_operation_default (op0, ((I n1) :: ((I
                     n2) :: (a1 :: l1)))))
              | x ->
                Coq_eval_static_operation_default (op0, ((I
                  n1) :: (x :: l0)))))
        | x -> Coq_eval_static_operation_default (op0, (x :: l))))
  | Omod ->
    let op0 = Omod in
    (match vl with
     | [] -> Coq_eval_static_operation_default (op0, [])
     | a :: l ->
       (match a with
        | I n1 ->
          (match l with
           | [] -> Coq_eval_static_operation_default (op0, ((I n1) :: []))
           | a0 :: l0 ->
             (match a0 with
              | I n2 ->
                (match l0 with
                 | [] -> Coq_eval_static_operation_case15 (n1, n2)
                 | a1 :: l1 ->
                   Coq_eval_static_operation_default (op0, ((I n1) :: ((I
                     n2) :: (a1 :: l1)))))
              | x ->
                Coq_eval_static_operation_default (op0, ((I
                  n1) :: (x :: l0)))))
        | x -> Coq_eval_static_operation_default (op0, (x :: l))))
  | Omodu ->
    let op0 = Omodu in
    (match vl with
     | [] -> Coq_eval_static_operation_default (op0, [])
     | a :: l ->
       (match a with
        | I n1 ->
          (match l with
           | [] -> Coq_eval_static_operation_default (op0, ((I n1) :: []))
           | a0 :: l0 ->
             (match a0 with
              | I n2 ->
                (match l0 with
                 | [] -> Coq_eval_static_operation_case16 (n1, n2)
                 | a1 :: l1 ->
                   Coq_eval_static_operation_default (op0, ((I n1) :: ((I
                     n2) :: (a1 :: l1)))))
              | x ->
                Coq_eval_static_operation_default (op0, ((I
                  n1) :: (x :: l0)))))
        | x -> Coq_eval_static_operation_default (op0, (x :: l))))
  | Oand ->
    let op0 = Oand in
    (match vl with
     | [] -> Coq_eval_static_operation_default (op0, [])
     | a :: l ->
       (match a with
        | I n1 ->
          (match l with
           | [] -> Coq_eval_static_operation_default (op0, ((I n1) :: []))
           | a0 :: l0 ->
             (match a0 with
              | I n2 ->
                (match l0 with
                 | [] -> Coq_eval_static_operation_case17 (n1, n2)
                 | a1 :: l1 ->
                   Coq_eval_static_operation_default (op0, ((I n1) :: ((I
                     n2) :: (a1 :: l1)))))
              | x ->
                Coq_eval_static_operation_default (op0, ((I
                  n1) :: (x :: l0)))))
        | x -> Coq_eval_static_operation_default (op0, (x :: l))))
  | Oandimm n ->
    let op0 = Oandimm n in
    (match vl with
     | [] -> Coq_eval_static_operation_default (op0, [])
     | a :: l ->
       (match a with
        | I n1 ->
          (match l with
           | [] -> Coq_eval_static_operation_case18 (n, n1)
           | a0 :: l0 ->
             Coq_eval_static_operation_default (op0, ((I n1) :: (a0 :: l0))))
        | x -> Coq_eval_static_operation_default (op0, (x :: l))))
  | Oor ->
    let op0 = Oor in
    (match vl with
     | [] -> Coq_eval_static_operation_default (op0, [])
     | a :: l ->
       (match a with
        | I n1 ->
          (match l with
           | [] -> Coq_eval_static_operation_default (op0, ((I n1) :: []))
           | a0 :: l0 ->
             (match a0 with
              | I n2 ->
                (match l0 with
                 | [] -> Coq_eval_static_operation_case19 (n1, n2)
                 | a1 :: l1 ->
                   Coq_eval_static_operation_default (op0, ((I n1) :: ((I
                     n2) :: (a1 :: l1)))))
              | x ->
                Coq_eval_static_operation_default (op0, ((I
                  n1) :: (x :: l0)))))
        | x -> Coq_eval_static_operation_default (op0, (x :: l))))
  | Oorimm n ->
    let op0 = Oorimm n in
    (match vl with
     | [] -> Coq_eval_static_operation_default (op0, [])
     | a :: l ->
       (match a with
        | I n1 ->
          (match l with
           | [] -> Coq_eval_static_operation_case20 (n, n1)
           | a0 :: l0 ->
             Coq_eval_static_operation_default (op0, ((I n1) :: (a0 :: l0))))
        | x -> Coq_eval_static_operation_default (op0, (x :: l))))
  | Oxor ->
    let op0 = Oxor in
    (match vl with
     | [] -> Coq_eval_static_operation_default (op0, [])
     | a :: l ->
       (match a with
        | I n1 ->
          (match l with
           | [] -> Coq_eval_static_operation_default (op0, ((I n1) :: []))
           | a0 :: l0 ->
             (match a0 with
              | I n2 ->
                (match l0 with
                 | [] -> Coq_eval_static_operation_case21 (n1, n2)
                 | a1 :: l1 ->
                   Coq_eval_static_operation_default (op0, ((I n1) :: ((I
                     n2) :: (a1 :: l1)))))
              | x ->
                Coq_eval_static_operation_default (op0, ((I
                  n1) :: (x :: l0)))))
        | x -> Coq_eval_static_operation_default (op0, (x :: l))))
  | Oxorimm n ->
    let op0 = Oxorimm n in
    (match vl with
     | [] -> Coq_eval_static_operation_default (op0, [])
     | a :: l ->
       (match a with
        | I n1 ->
          (match l with
           | [] -> Coq_eval_static_operation_case22 (n, n1)
           | a0 :: l0 ->
             Coq_eval_static_operation_default (op0, ((I n1) :: (a0 :: l0))))
        | x -> Coq_eval_static_operation_default (op0, (x :: l))))
  | Oshl ->
    let op0 = Oshl in
    (match vl with
     | [] -> Coq_eval_static_operation_default (op0, [])
     | a :: l ->
       (match a with
        | I n1 ->
          (match l with
           | [] -> Coq_eval_static_operation_default (op0, ((I n1) :: []))
           | a0 :: l0 ->
             (match a0 with
              | I n2 ->
                (match l0 with
                 | [] -> Coq_eval_static_operation_case23 (n1, n2)
                 | a1 :: l1 ->
                   Coq_eval_static_operation_default (op0, ((I n1) :: ((I
                     n2) :: (a1 :: l1)))))
              | x ->
                Coq_eval_static_operation_default (op0, ((I
                  n1) :: (x :: l0)))))
        | x -> Coq_eval_static_operation_default (op0, (x :: l))))
  | Oshlimm n ->
    let op0 = Oshlimm n in
    (match vl with
     | [] -> Coq_eval_static_operation_default (op0, [])
     | a :: l ->
       (match a with
        | I n1 ->
          (match l with
           | [] -> Coq_eval_static_operation_case24 (n, n1)
           | a0 :: l0 ->
             Coq_eval_static_operation_default (op0, ((I n1) :: (a0 :: l0))))
        | x -> Coq_eval_static_operation_default (op0, (x :: l))))
  | Oshr ->
    let op0 = Oshr in
    (match vl with
     | [] -> Coq_eval_static_operation_default (op0, [])
     | a :: l ->
       (match a with
        | I n1 ->
          (match l with
           | [] -> Coq_eval_static_operation_default (op0, ((I n1) :: []))
           | a0 :: l0 ->
             (match a0 with
              | I n2 ->
                (match l0 with
                 | [] -> Coq_eval_static_operation_case25 (n1, n2)
                 | a1 :: l1 ->
                   Coq_eval_static_operation_default (op0, ((I n1) :: ((I
                     n2) :: (a1 :: l1)))))
              | x ->
                Coq_eval_static_operation_default (op0, ((I
                  n1) :: (x :: l0)))))
        | x -> Coq_eval_static_operation_default (op0, (x :: l))))
  | Oshrimm n ->
    let op0 = Oshrimm n in
    (match vl with
     | [] -> Coq_eval_static_operation_default (op0, [])
     | a :: l ->
       (match a with
        | I n1 ->
          (match l with
           | [] -> Coq_eval_static_operation_case26 (n, n1)
           | a0 :: l0 ->
             Coq_eval_static_operation_default (op0, ((I n1) :: (a0 :: l0))))
        | x -> Coq_eval_static_operation_default (op0, (x :: l))))
  | Oshrximm n ->
    let op0 = Oshrximm n in
    (match vl with
     | [] -> Coq_eval_static_operation_default (op0, [])
     | a :: l ->
       (match a with
        | I n1 ->
          (match l with
           | [] -> Coq_eval_static_operation_case27 (n, n1)
           | a0 :: l0 ->
             Coq_eval_static_operation_default (op0, ((I n1) :: (a0 :: l0))))
        | x -> Coq_eval_static_operation_default (op0, (x :: l))))
  | Oshru ->
    let op0 = Oshru in
    (match vl with
     | [] -> Coq_eval_static_operation_default (op0, [])
     | a :: l ->
       (match a with
        | I n1 ->
          (match l with
           | [] -> Coq_eval_static_operation_default (op0, ((I n1) :: []))
           | a0 :: l0 ->
             (match a0 with
              | I n2 ->
                (match l0 with
                 | [] -> Coq_eval_static_operation_case28 (n1, n2)
                 | a1 :: l1 ->
                   Coq_eval_static_operation_default (op0, ((I n1) :: ((I
                     n2) :: (a1 :: l1)))))
              | x ->
                Coq_eval_static_operation_default (op0, ((I
                  n1) :: (x :: l0)))))
        | x -> Coq_eval_static_operation_default (op0, (x :: l))))
  | Oshruimm n ->
    let op0 = Oshruimm n in
    (match vl with
     | [] -> Coq_eval_static_operation_default (op0, [])
     | a :: l ->
       (match a with
        | I n1 ->
          (match l with
           | [] -> Coq_eval_static_operation_case29 (n, n1)
           | a0 :: l0 ->
             Coq_eval_static_operation_default (op0, ((I n1) :: (a0 :: l0))))
        | x -> Coq_eval_static_operation_default (op0, (x :: l))))
  | Ororimm n ->
    let op0 = Ororimm n in
    (match vl with
     | [] -> Coq_eval_static_operation_default (op0, [])
     | a :: l ->
       (match a with
        | I n1 ->
          (match l with
           | [] -> Coq_eval_static_operation_case30 (n, n1)
           | a0 :: l0 ->
             Coq_eval_static_operation_default (op0, ((I n1) :: (a0 :: l0))))
        | x -> Coq_eval_static_operation_default (op0, (x :: l))))
  | Olea mode -> Coq_eval_static_operation_case31 (mode, vl)
  | Onegf ->
    let op0 = Onegf in
    (match vl with
     | [] -> Coq_eval_static_operation_default (op0, [])
     | a :: l ->
       (match a with
        | F n1 ->
          (match l with
           | [] -> Coq_eval_static_operation_case32 n1
           | a0 :: l0 ->
             Coq_eval_static_operation_default (op0, ((F n1) :: (a0 :: l0))))
        | x -> Coq_eval_static_operation_default (op0, (x :: l))))
  | Oabsf ->
    let op0 = Oabsf in
    (match vl with
     | [] -> Coq_eval_static_operation_default (op0, [])
     | a :: l ->
       (match a with
        | F n1 ->
          (match l with
           | [] -> Coq_eval_static_operation_case33 n1
           | a0 :: l0 ->
             Coq_eval_static_operation_default (op0, ((F n1) :: (a0 :: l0))))
        | x -> Coq_eval_static_operation_default (op0, (x :: l))))
  | Oaddf ->
    let op0 = Oaddf in
    (match vl with
     | [] -> Coq_eval_static_operation_default (op0, [])
     | a :: l ->
       (match a with
        | F n1 ->
          (match l with
           | [] -> Coq_eval_static_operation_default (op0, ((F n1) :: []))
           | a0 :: l0 ->
             (match a0 with
              | F n2 ->
                (match l0 with
                 | [] -> Coq_eval_static_operation_case34 (n1, n2)
                 | a1 :: l1 ->
                   Coq_eval_static_operation_default (op0, ((F n1) :: ((F
                     n2) :: (a1 :: l1)))))
              | x ->
                Coq_eval_static_operation_default (op0, ((F
                  n1) :: (x :: l0)))))
        | x -> Coq_eval_static_operation_default (op0, (x :: l))))
  | Osubf ->
    let op0 = Osubf in
    (match vl with
     | [] -> Coq_eval_static_operation_default (op0, [])
     | a :: l ->
       (match a with
        | F n1 ->
          (match l with
           | [] -> Coq_eval_static_operation_default (op0, ((F n1) :: []))
           | a0 :: l0 ->
             (match a0 with
              | F n2 ->
                (match l0 with
                 | [] -> Coq_eval_static_operation_case35 (n1, n2)
                 | a1 :: l1 ->
                   Coq_eval_static_operation_default (op0, ((F n1) :: ((F
                     n2) :: (a1 :: l1)))))
              | x ->
                Coq_eval_static_operation_default (op0, ((F
                  n1) :: (x :: l0)))))
        | x -> Coq_eval_static_operation_default (op0, (x :: l))))
  | Omulf ->
    let op0 = Omulf in
    (match vl with
     | [] -> Coq_eval_static_operation_default (op0, [])
     | a :: l ->
       (match a with
        | F n1 ->
          (match l with
           | [] -> Coq_eval_static_operation_default (op0, ((F n1) :: []))
           | a0 :: l0 ->
             (match a0 with
              | F n2 ->
                (match l0 with
                 | [] -> Coq_eval_static_operation_case36 (n1, n2)
                 | a1 :: l1 ->
                   Coq_eval_static_operation_default (op0, ((F n1) :: ((F
                     n2) :: (a1 :: l1)))))
              | x ->
                Coq_eval_static_operation_default (op0, ((F
                  n1) :: (x :: l0)))))
        | x -> Coq_eval_static_operation_default (op0, (x :: l))))
  | Odivf ->
    let op0 = Odivf in
    (match vl with
     | [] -> Coq_eval_static_operation_default (op0, [])
     | a :: l ->
       (match a with
        | F n1 ->
          (match l with
           | [] -> Coq_eval_static_operation_default (op0, ((F n1) :: []))
           | a0 :: l0 ->
             (match a0 with
              | F n2 ->
                (match l0 with
                 | [] -> Coq_eval_static_operation_case37 (n1, n2)
                 | a1 :: l1 ->
                   Coq_eval_static_operation_default (op0, ((F n1) :: ((F
                     n2) :: (a1 :: l1)))))
              | x ->
                Coq_eval_static_operation_default (op0, ((F
                  n1) :: (x :: l0)))))
        | x -> Coq_eval_static_operation_default (op0, (x :: l))))
  | Osingleoffloat ->
    let op0 = Osingleoffloat in
    (match vl with
     | [] -> Coq_eval_static_operation_default (op0, [])
     | a :: l ->
       (match a with
        | F n1 ->
          (match l with
           | [] -> Coq_eval_static_operation_case38 n1
           | a0 :: l0 ->
             Coq_eval_static_operation_default (op0, ((F n1) :: (a0 :: l0))))
        | x -> Coq_eval_static_operation_default (op0, (x :: l))))
  | Ointoffloat ->
    let op0 = Ointoffloat in
    (match vl with
     | [] -> Coq_eval_static_operation_default (op0, [])
     | a :: l ->
       (match a with
        | F n1 ->
          (match l with
           | [] -> Coq_eval_static_operation_case39 n1
           | a0 :: l0 ->
             Coq_eval_static_operation_default (op0, ((F n1) :: (a0 :: l0))))
        | x -> Coq_eval_static_operation_default (op0, (x :: l))))
  | Ofloatofint ->
    let op0 = Ofloatofint in
    (match vl with
     | [] -> Coq_eval_static_operation_default (op0, [])
     | a :: l ->
       (match a with
        | I n1 ->
          (match l with
           | [] -> Coq_eval_static_operation_case40 n1
           | a0 :: l0 ->
             Coq_eval_static_operation_default (op0, ((I n1) :: (a0 :: l0))))
        | x -> Coq_eval_static_operation_default (op0, (x :: l))))
  | Ocmp c -> Coq_eval_static_operation_case41 (c, vl)

let eval_static_operation :  operation -> approx list -> approx =
  fun op vl  ->
  match eval_static_operation_match op vl with
  | Coq_eval_static_operation_case1 v1 -> v1
  | Coq_eval_static_operation_case2 n -> I n
  | Coq_eval_static_operation_case3 n ->
    if propagate_float_constants () then F n else Unknown
  | Coq_eval_static_operation_case4 n1 ->
    I (Int.sign_ext (Zpos (Coq_xO (Coq_xO (Coq_xO Coq_xH)))) n1)
  | Coq_eval_static_operation_case5 n1 ->
    I (Int.zero_ext (Zpos (Coq_xO (Coq_xO (Coq_xO Coq_xH)))) n1)
  | Coq_eval_static_operation_case6 n1 ->
    I (Int.sign_ext (Zpos (Coq_xO (Coq_xO (Coq_xO (Coq_xO Coq_xH))))) n1)
  | Coq_eval_static_operation_case7 n1 ->
    I (Int.zero_ext (Zpos (Coq_xO (Coq_xO (Coq_xO (Coq_xO Coq_xH))))) n1)
  | Coq_eval_static_operation_case8 n1 -> I (Int.neg n1)
  | Coq_eval_static_operation_case9 (n1, n2) -> I (Int.sub n1 n2)
  | Coq_eval_static_operation_case10 (s1, n1, n2) -> G (s1, (Int.sub n1 n2))
  | Coq_eval_static_operation_case11 (n1, n2) -> I (Int.mul n1 n2)
  | Coq_eval_static_operation_case12 (n, n1) -> I (Int.mul n1 n)
  | Coq_eval_static_operation_case13 (n1, n2) ->
    if Int.eq n2 Int.zero
    then Unknown
    else if (&&) (Int.eq n1 (Int.repr Int.min_signed)) (Int.eq n2 Int.mone)
         then Unknown
         else I (Int.divs n1 n2)
  | Coq_eval_static_operation_case14 (n1, n2) ->
    if Int.eq n2 Int.zero then Unknown else I (Int.divu n1 n2)
  | Coq_eval_static_operation_case15 (n1, n2) ->
    if Int.eq n2 Int.zero
    then Unknown
    else if (&&) (Int.eq n1 (Int.repr Int.min_signed)) (Int.eq n2 Int.mone)
         then Unknown
         else I (Int.mods n1 n2)
  | Coq_eval_static_operation_case16 (n1, n2) ->
    if Int.eq n2 Int.zero then Unknown else I (Int.modu n1 n2)
  | Coq_eval_static_operation_case17 (n1, n2) -> I (Int.coq_and n1 n2)
  | Coq_eval_static_operation_case18 (n, n1) -> I (Int.coq_and n1 n)
  | Coq_eval_static_operation_case19 (n1, n2) -> I (Int.coq_or n1 n2)
  | Coq_eval_static_operation_case20 (n, n1) -> I (Int.coq_or n1 n)
  | Coq_eval_static_operation_case21 (n1, n2) -> I (Int.xor n1 n2)
  | Coq_eval_static_operation_case22 (n, n1) -> I (Int.xor n1 n)
  | Coq_eval_static_operation_case23 (n1, n2) ->
    if Int.ltu n2 Int.iwordsize then I (Int.shl n1 n2) else Unknown
  | Coq_eval_static_operation_case24 (n, n1) ->
    if Int.ltu n Int.iwordsize then I (Int.shl n1 n) else Unknown
  | Coq_eval_static_operation_case25 (n1, n2) ->
    if Int.ltu n2 Int.iwordsize then I (Int.shr n1 n2) else Unknown
  | Coq_eval_static_operation_case26 (n, n1) ->
    if Int.ltu n Int.iwordsize then I (Int.shr n1 n) else Unknown
  | Coq_eval_static_operation_case27 (n, n1) ->
    if Int.ltu n (Int.repr (Zpos (Coq_xI (Coq_xI (Coq_xI (Coq_xI Coq_xH))))))
    then I (Int.shrx n1 n)
    else Unknown
  | Coq_eval_static_operation_case28 (n1, n2) ->
    if Int.ltu n2 Int.iwordsize then I (Int.shru n1 n2) else Unknown
  | Coq_eval_static_operation_case29 (n, n1) ->
    if Int.ltu n Int.iwordsize then I (Int.shru n1 n) else Unknown
  | Coq_eval_static_operation_case30 (n, n1) ->
    if Int.ltu n Int.iwordsize then I (Int.ror n1 n) else Unknown
  | Coq_eval_static_operation_case31 (mode, vl0) ->
    eval_static_addressing mode vl0
  | Coq_eval_static_operation_case32 n1 -> F (Float.neg n1)
  | Coq_eval_static_operation_case33 n1 -> F (Float.abs n1)
  | Coq_eval_static_operation_case34 (n1, n2) -> F (Float.add n1 n2)
  | Coq_eval_static_operation_case35 (n1, n2) -> F (Float.sub n1 n2)
  | Coq_eval_static_operation_case36 (n1, n2) -> F (Float.mul n1 n2)
  | Coq_eval_static_operation_case37 (n1, n2) -> F (Float.div n1 n2)
  | Coq_eval_static_operation_case38 n1 -> F (Float.singleoffloat n1)
  | Coq_eval_static_operation_case39 n1 -> eval_static_intoffloat n1
  | Coq_eval_static_operation_case40 n1 ->
    if propagate_float_constants () then F (Float.floatofint n1) else Unknown
  | Coq_eval_static_operation_case41 (c, vl0) ->
    eval_static_condition_val c vl0
  | Coq_eval_static_operation_default (op0, vl0) -> Unknown

type cond_strength_reduction_cases =
| Coq_cond_strength_reduction_case1 of comparison * reg * reg * Int.int
   * approx
| Coq_cond_strength_reduction_case2 of comparison * reg * reg * approx
   * Int.int
| Coq_cond_strength_reduction_case3 of comparison * reg * reg * Int.int
   * approx
| Coq_cond_strength_reduction_case4 of comparison * reg * reg * approx
   * Int.int
| Coq_cond_strength_reduction_default of condition * reg list * approx list

let cond_strength_reduction_match :     condition -> reg list -> approx list -> cond_strength_reduction_cases =
  fun cond args vl  ->
  match cond with
  | Ccomp c ->
    let cond0 = Ccomp c in
    (match args with
     | [] -> Coq_cond_strength_reduction_default (cond0, [], vl)
     | r1 :: l ->
       (match l with
        | [] -> Coq_cond_strength_reduction_default (cond0, (r1 :: []), vl)
        | r2 :: l0 ->
          (match l0 with
           | [] ->
             let args0 = r1 :: (r2 :: []) in
             (match vl with
              | [] -> Coq_cond_strength_reduction_default (cond0, args0, [])
              | v1 :: l1 ->
                (match v1 with
                 | Novalue ->
                   (match l1 with
                    | [] ->
                      Coq_cond_strength_reduction_default (cond0, args0,
                        (Novalue :: []))
                    | a :: l2 ->
                      (match a with
                       | Novalue ->
                         Coq_cond_strength_reduction_default (cond0, args0,
                           (Novalue :: (Novalue :: l2)))
                       | I n2 ->
                         (match l2 with
                          | [] ->
                            Coq_cond_strength_reduction_case2 (c, r1, r2,
                              Novalue, n2)
                          | a0 :: l3 ->
                            Coq_cond_strength_reduction_default (cond0,
                              args0, (Novalue :: ((I n2) :: (a0 :: l3)))))
                       | x ->
                         Coq_cond_strength_reduction_default (cond0, args0,
                           (Novalue :: (x :: l2)))))
                 | Unknown ->
                   (match l1 with
                    | [] ->
                      Coq_cond_strength_reduction_default (cond0, args0,
                        (Unknown :: []))
                    | a :: l2 ->
                      (match a with
                       | Unknown ->
                         Coq_cond_strength_reduction_default (cond0, args0,
                           (Unknown :: (Unknown :: l2)))
                       | I n2 ->
                         (match l2 with
                          | [] ->
                            Coq_cond_strength_reduction_case2 (c, r1, r2,
                              Unknown, n2)
                          | a0 :: l3 ->
                            Coq_cond_strength_reduction_default (cond0,
                              args0, (Unknown :: ((I n2) :: (a0 :: l3)))))
                       | x ->
                         Coq_cond_strength_reduction_default (cond0, args0,
                           (Unknown :: (x :: l2)))))
                 | I n1 ->
                   (match l1 with
                    | [] ->
                      Coq_cond_strength_reduction_default (cond0, args0, ((I
                        n1) :: []))
                    | v2 :: l2 ->
                      (match l2 with
                       | [] ->
                         Coq_cond_strength_reduction_case1 (c, r1, r2, n1,
                           v2)
                       | a :: l3 ->
                         Coq_cond_strength_reduction_default (cond0, args0,
                           ((I n1) :: (v2 :: (a :: l3))))))
                 | x ->
                   (match l1 with
                    | [] ->
                      Coq_cond_strength_reduction_default (cond0, args0,
                        (x :: []))
                    | a :: l2 ->
                      (match a with
                       | I n2 ->
                         (match l2 with
                          | [] ->
                            Coq_cond_strength_reduction_case2 (c, r1, r2, x,
                              n2)
                          | a0 :: l3 ->
                            Coq_cond_strength_reduction_default (cond0,
                              args0, (x :: ((I n2) :: (a0 :: l3)))))
                       | x0 ->
                         Coq_cond_strength_reduction_default (cond0, args0,
                           (x :: (x0 :: l2)))))))
           | r :: l1 ->
             Coq_cond_strength_reduction_default (cond0,
               (r1 :: (r2 :: (r :: l1))), vl))))
  | Ccompu c ->
    let cond0 = Ccompu c in
    (match args with
     | [] -> Coq_cond_strength_reduction_default (cond0, [], vl)
     | r1 :: l ->
       (match l with
        | [] -> Coq_cond_strength_reduction_default (cond0, (r1 :: []), vl)
        | r2 :: l0 ->
          (match l0 with
           | [] ->
             let args0 = r1 :: (r2 :: []) in
             (match vl with
              | [] -> Coq_cond_strength_reduction_default (cond0, args0, [])
              | v1 :: l1 ->
                (match v1 with
                 | Novalue ->
                   (match l1 with
                    | [] ->
                      Coq_cond_strength_reduction_default (cond0, args0,
                        (Novalue :: []))
                    | a :: l2 ->
                      (match a with
                       | Novalue ->
                         Coq_cond_strength_reduction_default (cond0, args0,
                           (Novalue :: (Novalue :: l2)))
                       | I n2 ->
                         (match l2 with
                          | [] ->
                            Coq_cond_strength_reduction_case4 (c, r1, r2,
                              Novalue, n2)
                          | a0 :: l3 ->
                            Coq_cond_strength_reduction_default (cond0,
                              args0, (Novalue :: ((I n2) :: (a0 :: l3)))))
                       | x ->
                         Coq_cond_strength_reduction_default (cond0, args0,
                           (Novalue :: (x :: l2)))))
                 | Unknown ->
                   (match l1 with
                    | [] ->
                      Coq_cond_strength_reduction_default (cond0, args0,
                        (Unknown :: []))
                    | a :: l2 ->
                      (match a with
                       | Unknown ->
                         Coq_cond_strength_reduction_default (cond0, args0,
                           (Unknown :: (Unknown :: l2)))
                       | I n2 ->
                         (match l2 with
                          | [] ->
                            Coq_cond_strength_reduction_case4 (c, r1, r2,
                              Unknown, n2)
                          | a0 :: l3 ->
                            Coq_cond_strength_reduction_default (cond0,
                              args0, (Unknown :: ((I n2) :: (a0 :: l3)))))
                       | x ->
                         Coq_cond_strength_reduction_default (cond0, args0,
                           (Unknown :: (x :: l2)))))
                 | I n1 ->
                   (match l1 with
                    | [] ->
                      Coq_cond_strength_reduction_default (cond0, args0, ((I
                        n1) :: []))
                    | v2 :: l2 ->
                      (match l2 with
                       | [] ->
                         Coq_cond_strength_reduction_case3 (c, r1, r2, n1,
                           v2)
                       | a :: l3 ->
                         Coq_cond_strength_reduction_default (cond0, args0,
                           ((I n1) :: (v2 :: (a :: l3))))))
                 | x ->
                   (match l1 with
                    | [] ->
                      Coq_cond_strength_reduction_default (cond0, args0,
                        (x :: []))
                    | a :: l2 ->
                      (match a with
                       | I n2 ->
                         (match l2 with
                          | [] ->
                            Coq_cond_strength_reduction_case4 (c, r1, r2, x,
                              n2)
                          | a0 :: l3 ->
                            Coq_cond_strength_reduction_default (cond0,
                              args0, (x :: ((I n2) :: (a0 :: l3)))))
                       | x0 ->
                         Coq_cond_strength_reduction_default (cond0, args0,
                           (x :: (x0 :: l2)))))))
           | r :: l1 ->
             Coq_cond_strength_reduction_default (cond0,
               (r1 :: (r2 :: (r :: l1))), vl))))
  | x -> Coq_cond_strength_reduction_default (x, args, vl)

let cond_strength_reduction :     condition -> reg list -> approx list -> condition * reg list =
  fun cond args vl  ->
  match cond_strength_reduction_match cond args vl with
  | Coq_cond_strength_reduction_case1 (c, r1, r2, n1, v2) ->
    ((Ccompimm ((swap_comparison c), n1)), (r2 :: []))
  | Coq_cond_strength_reduction_case2 (c, r1, r2, v1, n2) ->
    ((Ccompimm (c, n2)), (r1 :: []))
  | Coq_cond_strength_reduction_case3 (c, r1, r2, n1, v2) ->
    ((Ccompuimm ((swap_comparison c), n1)), (r2 :: []))
  | Coq_cond_strength_reduction_case4 (c, r1, r2, v1, n2) ->
    ((Ccompuimm (c, n2)), (r1 :: []))
  | Coq_cond_strength_reduction_default (cond0, args0, vl0) -> (cond0, args0)

type addr_strength_reduction_cases =
| Coq_addr_strength_reduction_case1 of Int.int * reg * ident * Int.int
| Coq_addr_strength_reduction_case2 of Int.int * reg * Int.int
| Coq_addr_strength_reduction_case3 of Int.int * reg * reg * ident * 
   Int.int * Int.int
| Coq_addr_strength_reduction_case4 of Int.int * reg * reg * Int.int * 
   ident * Int.int
| Coq_addr_strength_reduction_case5 of Int.int * reg * reg * Int.int
   * Int.int
| Coq_addr_strength_reduction_case6 of Int.int * reg * reg * Int.int
   * Int.int
| Coq_addr_strength_reduction_case7 of Int.int * reg * reg * ident * 
   Int.int * approx
| Coq_addr_strength_reduction_case8 of Int.int * reg * reg * approx * 
   ident * Int.int
| Coq_addr_strength_reduction_case9 of Int.int * reg * reg * Int.int * approx
| Coq_addr_strength_reduction_case10 of Int.int * reg * reg * approx
   * Int.int
| Coq_addr_strength_reduction_case11 of Int.int * Int.int * reg * reg * 
   ident * Int.int * Int.int
| Coq_addr_strength_reduction_case12 of Int.int * Int.int * reg * reg * 
   ident * Int.int * approx
| Coq_addr_strength_reduction_case13 of Int.int * Int.int * reg * reg
   * approx * Int.int
| Coq_addr_strength_reduction_case14 of ident * Int.int * reg * Int.int
| Coq_addr_strength_reduction_case15 of Int.int * ident * Int.int * reg
   * Int.int
| Coq_addr_strength_reduction_default of addressing * reg list * approx list

let addr_strength_reduction_match :     addressing -> reg list -> approx list -> addr_strength_reduction_cases =
  fun addr args vl  ->
  match addr with
  | Aindexed ofs ->
    let addr0 = Aindexed ofs in
    (match args with
     | [] -> Coq_addr_strength_reduction_default (addr0, [], vl)
     | r1 :: l ->
       (match l with
        | [] ->
          let args0 = r1 :: [] in
          (match vl with
           | [] -> Coq_addr_strength_reduction_default (addr0, args0, [])
           | a :: l0 ->
             (match a with
              | G (symb, n) ->
                (match l0 with
                 | [] -> Coq_addr_strength_reduction_case1 (ofs, r1, symb, n)
                 | a0 :: l1 ->
                   Coq_addr_strength_reduction_default (addr0, args0, ((G
                     (symb, n)) :: (a0 :: l1))))
              | S n ->
                (match l0 with
                 | [] -> Coq_addr_strength_reduction_case2 (ofs, r1, n)
                 | a0 :: l1 ->
                   Coq_addr_strength_reduction_default (addr0, args0, ((S
                     n) :: (a0 :: l1))))
              | x ->
                Coq_addr_strength_reduction_default (addr0, args0, (x :: l0))))
        | r :: l0 ->
          Coq_addr_strength_reduction_default (addr0, (r1 :: (r :: l0)), vl)))
  | Aindexed2 ofs ->
    let addr0 = Aindexed2 ofs in
    (match args with
     | [] -> Coq_addr_strength_reduction_default (addr0, [], vl)
     | r1 :: l ->
       (match l with
        | [] -> Coq_addr_strength_reduction_default (addr0, (r1 :: []), vl)
        | r2 :: l0 ->
          (match l0 with
           | [] ->
             let args0 = r1 :: (r2 :: []) in
             (match vl with
              | [] -> Coq_addr_strength_reduction_default (addr0, args0, [])
              | v1 :: l1 ->
                (match v1 with
                 | Novalue ->
                   let v2 = Novalue in
                   (match l1 with
                    | [] ->
                      Coq_addr_strength_reduction_default (addr0, args0,
                        (Novalue :: []))
                    | a :: l2 ->
                      (match a with
                       | Novalue ->
                         Coq_addr_strength_reduction_default (addr0, args0,
                           (Novalue :: (Novalue :: l2)))
                       | I n2 ->
                         (match l2 with
                          | [] ->
                            Coq_addr_strength_reduction_case10 (ofs, r1, r2,
                              v2, n2)
                          | a0 :: l3 ->
                            Coq_addr_strength_reduction_default (addr0,
                              args0, (Novalue :: ((I n2) :: (a0 :: l3)))))
                       | G (symb, n2) ->
                         (match l2 with
                          | [] ->
                            Coq_addr_strength_reduction_case8 (ofs, r1, r2,
                              v2, symb, n2)
                          | a0 :: l3 ->
                            Coq_addr_strength_reduction_default (addr0,
                              args0, (Novalue :: ((G (symb,
                              n2)) :: (a0 :: l3)))))
                       | x ->
                         Coq_addr_strength_reduction_default (addr0, args0,
                           (Novalue :: (x :: l2)))))
                 | Unknown ->
                   let v2 = Unknown in
                   (match l1 with
                    | [] ->
                      Coq_addr_strength_reduction_default (addr0, args0,
                        (Unknown :: []))
                    | a :: l2 ->
                      (match a with
                       | Unknown ->
                         Coq_addr_strength_reduction_default (addr0, args0,
                           (Unknown :: (Unknown :: l2)))
                       | I n2 ->
                         (match l2 with
                          | [] ->
                            Coq_addr_strength_reduction_case10 (ofs, r1, r2,
                              v2, n2)
                          | a0 :: l3 ->
                            Coq_addr_strength_reduction_default (addr0,
                              args0, (Unknown :: ((I n2) :: (a0 :: l3)))))
                       | G (symb, n2) ->
                         (match l2 with
                          | [] ->
                            Coq_addr_strength_reduction_case8 (ofs, r1, r2,
                              v2, symb, n2)
                          | a0 :: l3 ->
                            Coq_addr_strength_reduction_default (addr0,
                              args0, (Unknown :: ((G (symb,
                              n2)) :: (a0 :: l3)))))
                       | x ->
                         Coq_addr_strength_reduction_default (addr0, args0,
                           (Unknown :: (x :: l2)))))
                 | I n1 ->
                   (match l1 with
                    | [] ->
                      Coq_addr_strength_reduction_default (addr0, args0, ((I
                        n1) :: []))
                    | v2 :: l2 ->
                      (match v2 with
                       | G (symb, n2) ->
                         (match l2 with
                          | [] ->
                            Coq_addr_strength_reduction_case4 (ofs, r1, r2,
                              n1, symb, n2)
                          | a :: l3 ->
                            Coq_addr_strength_reduction_default (addr0,
                              args0, ((I n1) :: ((G (symb,
                              n2)) :: (a :: l3)))))
                       | S n2 ->
                         (match l2 with
                          | [] ->
                            Coq_addr_strength_reduction_case6 (ofs, r1, r2,
                              n1, n2)
                          | a :: l3 ->
                            Coq_addr_strength_reduction_default (addr0,
                              args0, ((I n1) :: ((S n2) :: (a :: l3)))))
                       | x ->
                         (match l2 with
                          | [] ->
                            Coq_addr_strength_reduction_case9 (ofs, r1, r2,
                              n1, x)
                          | a :: l3 ->
                            Coq_addr_strength_reduction_default (addr0,
                              args0, ((I n1) :: (x :: (a :: l3)))))))
                 | F f ->
                   let v2 = F f in
                   (match l1 with
                    | [] ->
                      Coq_addr_strength_reduction_default (addr0, args0, ((F
                        f) :: []))
                    | a :: l2 ->
                      (match a with
                       | I n2 ->
                         (match l2 with
                          | [] ->
                            Coq_addr_strength_reduction_case10 (ofs, r1, r2,
                              v2, n2)
                          | a0 :: l3 ->
                            Coq_addr_strength_reduction_default (addr0,
                              args0, ((F f) :: ((I n2) :: (a0 :: l3)))))
                       | G (symb, n2) ->
                         (match l2 with
                          | [] ->
                            Coq_addr_strength_reduction_case8 (ofs, r1, r2,
                              v2, symb, n2)
                          | a0 :: l3 ->
                            Coq_addr_strength_reduction_default (addr0,
                              args0, ((F f) :: ((G (symb,
                              n2)) :: (a0 :: l3)))))
                       | x ->
                         Coq_addr_strength_reduction_default (addr0, args0,
                           ((F f) :: (x :: l2)))))
                 | G (symb, n1) ->
                   (match l1 with
                    | [] ->
                      Coq_addr_strength_reduction_default (addr0, args0, ((G
                        (symb, n1)) :: []))
                    | v2 :: l2 ->
                      (match v2 with
                       | I n2 ->
                         (match l2 with
                          | [] ->
                            Coq_addr_strength_reduction_case3 (ofs, r1, r2,
                              symb, n1, n2)
                          | a :: l3 ->
                            Coq_addr_strength_reduction_default (addr0,
                              args0, ((G (symb, n1)) :: ((I
                              n2) :: (a :: l3)))))
                       | x ->
                         (match l2 with
                          | [] ->
                            Coq_addr_strength_reduction_case7 (ofs, r1, r2,
                              symb, n1, x)
                          | a :: l3 ->
                            Coq_addr_strength_reduction_default (addr0,
                              args0, ((G (symb, n1)) :: (x :: (a :: l3)))))))
                 | S n1 ->
                   (match l1 with
                    | [] ->
                      Coq_addr_strength_reduction_default (addr0, args0, ((S
                        n1) :: []))
                    | a :: l2 ->
                      (match a with
                       | I n2 ->
                         (match l2 with
                          | [] ->
                            Coq_addr_strength_reduction_case5 (ofs, r1, r2,
                              n1, n2)
                          | a0 :: l3 ->
                            Coq_addr_strength_reduction_default (addr0,
                              args0, ((S n1) :: ((I n2) :: (a0 :: l3)))))
                       | G (symb, n2) ->
                         (match l2 with
                          | [] ->
                            Coq_addr_strength_reduction_case8 (ofs, r1, r2,
                              (S n1), symb, n2)
                          | a0 :: l3 ->
                            Coq_addr_strength_reduction_default (addr0,
                              args0, ((S n1) :: ((G (symb,
                              n2)) :: (a0 :: l3)))))
                       | x ->
                         Coq_addr_strength_reduction_default (addr0, args0,
                           ((S n1) :: (x :: l2)))))))
           | r :: l1 ->
             Coq_addr_strength_reduction_default (addr0,
               (r1 :: (r2 :: (r :: l1))), vl))))
  | Aindexed2scaled (sc, ofs) ->
    let addr0 = Aindexed2scaled (sc, ofs) in
    (match args with
     | [] -> Coq_addr_strength_reduction_default (addr0, [], vl)
     | r1 :: l ->
       (match l with
        | [] -> Coq_addr_strength_reduction_default (addr0, (r1 :: []), vl)
        | r2 :: l0 ->
          (match l0 with
           | [] ->
             let args0 = r1 :: (r2 :: []) in
             (match vl with
              | [] -> Coq_addr_strength_reduction_default (addr0, args0, [])
              | v1 :: l1 ->
                (match v1 with
                 | Novalue ->
                   (match l1 with
                    | [] ->
                      Coq_addr_strength_reduction_default (addr0, args0,
                        (Novalue :: []))
                    | a :: l2 ->
                      (match a with
                       | Novalue ->
                         Coq_addr_strength_reduction_default (addr0, args0,
                           (Novalue :: (Novalue :: l2)))
                       | I n2 ->
                         (match l2 with
                          | [] ->
                            Coq_addr_strength_reduction_case13 (sc, ofs, r1,
                              r2, Novalue, n2)
                          | a0 :: l3 ->
                            Coq_addr_strength_reduction_default (addr0,
                              args0, (Novalue :: ((I n2) :: (a0 :: l3)))))
                       | x ->
                         Coq_addr_strength_reduction_default (addr0, args0,
                           (Novalue :: (x :: l2)))))
                 | Unknown ->
                   (match l1 with
                    | [] ->
                      Coq_addr_strength_reduction_default (addr0, args0,
                        (Unknown :: []))
                    | a :: l2 ->
                      (match a with
                       | Unknown ->
                         Coq_addr_strength_reduction_default (addr0, args0,
                           (Unknown :: (Unknown :: l2)))
                       | I n2 ->
                         (match l2 with
                          | [] ->
                            Coq_addr_strength_reduction_case13 (sc, ofs, r1,
                              r2, Unknown, n2)
                          | a0 :: l3 ->
                            Coq_addr_strength_reduction_default (addr0,
                              args0, (Unknown :: ((I n2) :: (a0 :: l3)))))
                       | x ->
                         Coq_addr_strength_reduction_default (addr0, args0,
                           (Unknown :: (x :: l2)))))
                 | G (symb, n1) ->
                   (match l1 with
                    | [] ->
                      Coq_addr_strength_reduction_default (addr0, args0, ((G
                        (symb, n1)) :: []))
                    | v2 :: l2 ->
                      (match v2 with
                       | I n2 ->
                         (match l2 with
                          | [] ->
                            Coq_addr_strength_reduction_case11 (sc, ofs, r1,
                              r2, symb, n1, n2)
                          | a :: l3 ->
                            Coq_addr_strength_reduction_default (addr0,
                              args0, ((G (symb, n1)) :: ((I
                              n2) :: (a :: l3)))))
                       | x ->
                         (match l2 with
                          | [] ->
                            Coq_addr_strength_reduction_case12 (sc, ofs, r1,
                              r2, symb, n1, x)
                          | a :: l3 ->
                            Coq_addr_strength_reduction_default (addr0,
                              args0, ((G (symb, n1)) :: (x :: (a :: l3)))))))
                 | x ->
                   (match l1 with
                    | [] ->
                      Coq_addr_strength_reduction_default (addr0, args0,
                        (x :: []))
                    | a :: l2 ->
                      (match a with
                       | I n2 ->
                         (match l2 with
                          | [] ->
                            Coq_addr_strength_reduction_case13 (sc, ofs, r1,
                              r2, x, n2)
                          | a0 :: l3 ->
                            Coq_addr_strength_reduction_default (addr0,
                              args0, (x :: ((I n2) :: (a0 :: l3)))))
                       | x0 ->
                         Coq_addr_strength_reduction_default (addr0, args0,
                           (x :: (x0 :: l2)))))))
           | r :: l1 ->
             Coq_addr_strength_reduction_default (addr0,
               (r1 :: (r2 :: (r :: l1))), vl))))
  | Abased (id, ofs) ->
    let addr0 = Abased (id, ofs) in
    (match args with
     | [] -> Coq_addr_strength_reduction_default (addr0, [], vl)
     | r1 :: l ->
       (match l with
        | [] ->
          let args0 = r1 :: [] in
          (match vl with
           | [] -> Coq_addr_strength_reduction_default (addr0, args0, [])
           | a :: l0 ->
             (match a with
              | I n1 ->
                (match l0 with
                 | [] -> Coq_addr_strength_reduction_case14 (id, ofs, r1, n1)
                 | a0 :: l1 ->
                   Coq_addr_strength_reduction_default (addr0, args0, ((I
                     n1) :: (a0 :: l1))))
              | x ->
                Coq_addr_strength_reduction_default (addr0, args0, (x :: l0))))
        | r :: l0 ->
          Coq_addr_strength_reduction_default (addr0, (r1 :: (r :: l0)), vl)))
  | Abasedscaled (sc, id, ofs) ->
    let addr0 = Abasedscaled (sc, id, ofs) in
    (match args with
     | [] -> Coq_addr_strength_reduction_default (addr0, [], vl)
     | r1 :: l ->
       (match l with
        | [] ->
          let args0 = r1 :: [] in
          (match vl with
           | [] -> Coq_addr_strength_reduction_default (addr0, args0, [])
           | a :: l0 ->
             (match a with
              | I n1 ->
                (match l0 with
                 | [] ->
                   Coq_addr_strength_reduction_case15 (sc, id, ofs, r1, n1)
                 | a0 :: l1 ->
                   Coq_addr_strength_reduction_default (addr0, args0, ((I
                     n1) :: (a0 :: l1))))
              | x ->
                Coq_addr_strength_reduction_default (addr0, args0, (x :: l0))))
        | r :: l0 ->
          Coq_addr_strength_reduction_default (addr0, (r1 :: (r :: l0)), vl)))
  | x -> Coq_addr_strength_reduction_default (x, args, vl)

let addr_strength_reduction :     addressing -> reg list -> approx list -> addressing * reg list =
  fun addr args vl  ->
  match addr_strength_reduction_match addr args vl with
  | Coq_addr_strength_reduction_case1 (ofs, r1, symb, n) ->
    ((Aglobal (symb, (Int.add n ofs))), [])
  | Coq_addr_strength_reduction_case2 (ofs, r1, n) ->
    ((Ainstack (Int.add n ofs)), [])
  | Coq_addr_strength_reduction_case3 (ofs, r1, r2, symb, n1, n2) ->
    ((Aglobal (symb, (Int.add (Int.add n1 n2) ofs))), [])
  | Coq_addr_strength_reduction_case4 (ofs, r1, r2, n1, symb, n2) ->
    ((Aglobal (symb, (Int.add (Int.add n1 n2) ofs))), [])
  | Coq_addr_strength_reduction_case5 (ofs, r1, r2, n1, n2) ->
    ((Ainstack (Int.add (Int.add n1 n2) ofs)), [])
  | Coq_addr_strength_reduction_case6 (ofs, r1, r2, n1, n2) ->
    ((Ainstack (Int.add (Int.add n1 n2) ofs)), [])
  | Coq_addr_strength_reduction_case7 (ofs, r1, r2, symb, n1, v2) ->
    ((Abased (symb, (Int.add n1 ofs))), (r2 :: []))
  | Coq_addr_strength_reduction_case8 (ofs, r1, r2, v1, symb, n2) ->
    ((Abased (symb, (Int.add n2 ofs))), (r1 :: []))
  | Coq_addr_strength_reduction_case9 (ofs, r1, r2, n1, v2) ->
    ((Aindexed (Int.add n1 ofs)), (r2 :: []))
  | Coq_addr_strength_reduction_case10 (ofs, r1, r2, v1, n2) ->
    ((Aindexed (Int.add n2 ofs)), (r1 :: []))
  | Coq_addr_strength_reduction_case11 (sc, ofs, r1, r2, symb, n1, n2) ->
    ((Aglobal (symb, (Int.add (Int.add n1 (Int.mul n2 sc)) ofs))), [])
  | Coq_addr_strength_reduction_case12 (sc, ofs, r1, r2, symb, n1, v2) ->
    ((Abasedscaled (sc, symb, (Int.add n1 ofs))), (r2 :: []))
  | Coq_addr_strength_reduction_case13 (sc, ofs, r1, r2, v1, n2) ->
    ((Aindexed (Int.add (Int.mul n2 sc) ofs)), (r1 :: []))
  | Coq_addr_strength_reduction_case14 (id, ofs, r1, n1) ->
    ((Aglobal (id, (Int.add ofs n1))), [])
  | Coq_addr_strength_reduction_case15 (sc, id, ofs, r1, n1) ->
    ((Aglobal (id, (Int.add ofs (Int.mul sc n1)))), [])
  | Coq_addr_strength_reduction_default (addr0, args0, vl0) -> (addr0, args0)

let make_addimm :  Int.int -> reg -> operation * reg list =
  fun n r  ->
  if Int.eq n Int.zero
  then (Omove, (r :: []))
  else ((Olea (Aindexed n)), (r :: []))

let make_shlimm :  Int.int -> reg -> operation * reg list =
  fun n r  ->
  if Int.eq n Int.zero then (Omove, (r :: [])) else ((Oshlimm n), (r :: []))

let make_shrimm :  Int.int -> reg -> operation * reg list =
  fun n r  ->
  if Int.eq n Int.zero then (Omove, (r :: [])) else ((Oshrimm n), (r :: []))

let make_shruimm :  Int.int -> reg -> operation * reg list =
  fun n r  ->
  if Int.eq n Int.zero then (Omove, (r :: [])) else ((Oshruimm n), (r :: []))

let make_mulimm :  Int.int -> reg -> operation * reg list =
  fun n r  ->
  if Int.eq n Int.zero
  then ((Ointconst Int.zero), [])
  else if Int.eq n Int.one
       then (Omove, (r :: []))
       else (match Int.is_power2 n with
             | Some l -> make_shlimm l r
             | None -> ((Omulimm n), (r :: [])))

let make_andimm :  Int.int -> reg -> operation * reg list =
  fun n r  ->
  if Int.eq n Int.zero
  then ((Ointconst Int.zero), [])
  else if Int.eq n Int.mone
       then (Omove, (r :: []))
       else ((Oandimm n), (r :: []))

let make_orimm :  Int.int -> reg -> operation * reg list =
  fun n r  ->
  if Int.eq n Int.zero
  then (Omove, (r :: []))
  else if Int.eq n Int.mone
       then ((Ointconst Int.mone), [])
       else ((Oorimm n), (r :: []))

let make_xorimm :  Int.int -> reg -> operation * reg list =
  fun n r  ->
  if Int.eq n Int.zero then (Omove, (r :: [])) else ((Oxorimm n), (r :: []))

let make_divimm :  Int.int -> reg -> reg -> operation * reg list =
  fun n r1 r2  ->
  match Int.is_power2 n with
  | Some l ->
    if Int.ltu l (Int.repr (Zpos (Coq_xI (Coq_xI (Coq_xI (Coq_xI Coq_xH))))))
    then ((Oshrximm l), (r1 :: []))
    else (Odiv, (r1 :: (r2 :: [])))
  | None -> (Odiv, (r1 :: (r2 :: [])))

let make_divuimm :  Int.int -> reg -> reg -> operation * reg list =
  fun n r1 r2  ->
  match Int.is_power2 n with
  | Some l -> make_shruimm l r1
  | None -> (Odivu, (r1 :: (r2 :: [])))

let make_moduimm :  Int.int -> reg -> reg -> operation * reg list =
  fun n r1 r2  ->
  match Int.is_power2 n with
  | Some l -> ((Oandimm (Int.sub n Int.one)), (r1 :: []))
  | None -> (Omodu, (r1 :: (r2 :: [])))

type op_strength_reduction_cases =
| Coq_op_strength_reduction_case1 of reg * reg * approx * Int.int
| Coq_op_strength_reduction_case2 of reg * reg * approx * Int.int
| Coq_op_strength_reduction_case3 of reg * reg * approx * Int.int
| Coq_op_strength_reduction_case4 of reg * reg * approx * Int.int
| Coq_op_strength_reduction_case5 of reg * reg * approx * Int.int
| Coq_op_strength_reduction_case6 of reg * reg * approx * Int.int
| Coq_op_strength_reduction_case7 of reg * reg * approx * Int.int
| Coq_op_strength_reduction_case8 of reg * reg * approx * Int.int
| Coq_op_strength_reduction_case9 of reg * reg * approx * Int.int
| Coq_op_strength_reduction_case10 of reg * reg * approx * Int.int
| Coq_op_strength_reduction_case11 of reg * reg * approx * Int.int
| Coq_op_strength_reduction_case12 of addressing * reg list * approx list
| Coq_op_strength_reduction_case13 of condition * reg list * approx list
| Coq_op_strength_reduction_default of operation * reg list * approx list

let op_strength_reduction_match :     operation -> reg list -> approx list -> op_strength_reduction_cases =
  fun op args vl  ->
  match op with
  | Osub ->
    let op0 = Osub in
    (match args with
     | [] -> Coq_op_strength_reduction_default (op0, [], vl)
     | r1 :: l ->
       (match l with
        | [] -> Coq_op_strength_reduction_default (op0, (r1 :: []), vl)
        | r2 :: l0 ->
          (match l0 with
           | [] ->
             let args0 = r1 :: (r2 :: []) in
             (match vl with
              | [] -> Coq_op_strength_reduction_default (op0, args0, [])
              | v1 :: l1 ->
                (match l1 with
                 | [] ->
                   Coq_op_strength_reduction_default (op0, args0, (v1 :: []))
                 | a :: l2 ->
                   (match a with
                    | I n2 ->
                      (match l2 with
                       | [] ->
                         Coq_op_strength_reduction_case1 (r1, r2, v1, n2)
                       | a0 :: l3 ->
                         Coq_op_strength_reduction_default (op0, args0,
                           (v1 :: ((I n2) :: (a0 :: l3)))))
                    | x ->
                      Coq_op_strength_reduction_default (op0, args0,
                        (v1 :: (x :: l2))))))
           | r :: l1 ->
             Coq_op_strength_reduction_default (op0,
               (r1 :: (r2 :: (r :: l1))), vl))))
  | Omul ->
    let op0 = Omul in
    (match args with
     | [] -> Coq_op_strength_reduction_default (op0, [], vl)
     | r1 :: l ->
       (match l with
        | [] -> Coq_op_strength_reduction_default (op0, (r1 :: []), vl)
        | r2 :: l0 ->
          (match l0 with
           | [] ->
             let args0 = r1 :: (r2 :: []) in
             (match vl with
              | [] -> Coq_op_strength_reduction_default (op0, args0, [])
              | v1 :: l1 ->
                (match l1 with
                 | [] ->
                   Coq_op_strength_reduction_default (op0, args0, (v1 :: []))
                 | a :: l2 ->
                   (match a with
                    | I n2 ->
                      (match l2 with
                       | [] ->
                         Coq_op_strength_reduction_case2 (r1, r2, v1, n2)
                       | a0 :: l3 ->
                         Coq_op_strength_reduction_default (op0, args0,
                           (v1 :: ((I n2) :: (a0 :: l3)))))
                    | x ->
                      Coq_op_strength_reduction_default (op0, args0,
                        (v1 :: (x :: l2))))))
           | r :: l1 ->
             Coq_op_strength_reduction_default (op0,
               (r1 :: (r2 :: (r :: l1))), vl))))
  | Odiv ->
    let op0 = Odiv in
    (match args with
     | [] -> Coq_op_strength_reduction_default (op0, [], vl)
     | r1 :: l ->
       (match l with
        | [] -> Coq_op_strength_reduction_default (op0, (r1 :: []), vl)
        | r2 :: l0 ->
          (match l0 with
           | [] ->
             let args0 = r1 :: (r2 :: []) in
             (match vl with
              | [] -> Coq_op_strength_reduction_default (op0, args0, [])
              | v1 :: l1 ->
                (match l1 with
                 | [] ->
                   Coq_op_strength_reduction_default (op0, args0, (v1 :: []))
                 | a :: l2 ->
                   (match a with
                    | I n2 ->
                      (match l2 with
                       | [] ->
                         Coq_op_strength_reduction_case3 (r1, r2, v1, n2)
                       | a0 :: l3 ->
                         Coq_op_strength_reduction_default (op0, args0,
                           (v1 :: ((I n2) :: (a0 :: l3)))))
                    | x ->
                      Coq_op_strength_reduction_default (op0, args0,
                        (v1 :: (x :: l2))))))
           | r :: l1 ->
             Coq_op_strength_reduction_default (op0,
               (r1 :: (r2 :: (r :: l1))), vl))))
  | Odivu ->
    let op0 = Odivu in
    (match args with
     | [] -> Coq_op_strength_reduction_default (op0, [], vl)
     | r1 :: l ->
       (match l with
        | [] -> Coq_op_strength_reduction_default (op0, (r1 :: []), vl)
        | r2 :: l0 ->
          (match l0 with
           | [] ->
             let args0 = r1 :: (r2 :: []) in
             (match vl with
              | [] -> Coq_op_strength_reduction_default (op0, args0, [])
              | v1 :: l1 ->
                (match l1 with
                 | [] ->
                   Coq_op_strength_reduction_default (op0, args0, (v1 :: []))
                 | a :: l2 ->
                   (match a with
                    | I n2 ->
                      (match l2 with
                       | [] ->
                         Coq_op_strength_reduction_case4 (r1, r2, v1, n2)
                       | a0 :: l3 ->
                         Coq_op_strength_reduction_default (op0, args0,
                           (v1 :: ((I n2) :: (a0 :: l3)))))
                    | x ->
                      Coq_op_strength_reduction_default (op0, args0,
                        (v1 :: (x :: l2))))))
           | r :: l1 ->
             Coq_op_strength_reduction_default (op0,
               (r1 :: (r2 :: (r :: l1))), vl))))
  | Omodu ->
    let op0 = Omodu in
    (match args with
     | [] -> Coq_op_strength_reduction_default (op0, [], vl)
     | r1 :: l ->
       (match l with
        | [] -> Coq_op_strength_reduction_default (op0, (r1 :: []), vl)
        | r2 :: l0 ->
          (match l0 with
           | [] ->
             let args0 = r1 :: (r2 :: []) in
             (match vl with
              | [] -> Coq_op_strength_reduction_default (op0, args0, [])
              | v1 :: l1 ->
                (match l1 with
                 | [] ->
                   Coq_op_strength_reduction_default (op0, args0, (v1 :: []))
                 | a :: l2 ->
                   (match a with
                    | I n2 ->
                      (match l2 with
                       | [] ->
                         Coq_op_strength_reduction_case5 (r1, r2, v1, n2)
                       | a0 :: l3 ->
                         Coq_op_strength_reduction_default (op0, args0,
                           (v1 :: ((I n2) :: (a0 :: l3)))))
                    | x ->
                      Coq_op_strength_reduction_default (op0, args0,
                        (v1 :: (x :: l2))))))
           | r :: l1 ->
             Coq_op_strength_reduction_default (op0,
               (r1 :: (r2 :: (r :: l1))), vl))))
  | Oand ->
    let op0 = Oand in
    (match args with
     | [] -> Coq_op_strength_reduction_default (op0, [], vl)
     | r1 :: l ->
       (match l with
        | [] -> Coq_op_strength_reduction_default (op0, (r1 :: []), vl)
        | r2 :: l0 ->
          (match l0 with
           | [] ->
             let args0 = r1 :: (r2 :: []) in
             (match vl with
              | [] -> Coq_op_strength_reduction_default (op0, args0, [])
              | v1 :: l1 ->
                (match l1 with
                 | [] ->
                   Coq_op_strength_reduction_default (op0, args0, (v1 :: []))
                 | a :: l2 ->
                   (match a with
                    | I n2 ->
                      (match l2 with
                       | [] ->
                         Coq_op_strength_reduction_case6 (r1, r2, v1, n2)
                       | a0 :: l3 ->
                         Coq_op_strength_reduction_default (op0, args0,
                           (v1 :: ((I n2) :: (a0 :: l3)))))
                    | x ->
                      Coq_op_strength_reduction_default (op0, args0,
                        (v1 :: (x :: l2))))))
           | r :: l1 ->
             Coq_op_strength_reduction_default (op0,
               (r1 :: (r2 :: (r :: l1))), vl))))
  | Oor ->
    let op0 = Oor in
    (match args with
     | [] -> Coq_op_strength_reduction_default (op0, [], vl)
     | r1 :: l ->
       (match l with
        | [] -> Coq_op_strength_reduction_default (op0, (r1 :: []), vl)
        | r2 :: l0 ->
          (match l0 with
           | [] ->
             let args0 = r1 :: (r2 :: []) in
             (match vl with
              | [] -> Coq_op_strength_reduction_default (op0, args0, [])
              | v1 :: l1 ->
                (match l1 with
                 | [] ->
                   Coq_op_strength_reduction_default (op0, args0, (v1 :: []))
                 | a :: l2 ->
                   (match a with
                    | I n2 ->
                      (match l2 with
                       | [] ->
                         Coq_op_strength_reduction_case7 (r1, r2, v1, n2)
                       | a0 :: l3 ->
                         Coq_op_strength_reduction_default (op0, args0,
                           (v1 :: ((I n2) :: (a0 :: l3)))))
                    | x ->
                      Coq_op_strength_reduction_default (op0, args0,
                        (v1 :: (x :: l2))))))
           | r :: l1 ->
             Coq_op_strength_reduction_default (op0,
               (r1 :: (r2 :: (r :: l1))), vl))))
  | Oxor ->
    let op0 = Oxor in
    (match args with
     | [] -> Coq_op_strength_reduction_default (op0, [], vl)
     | r1 :: l ->
       (match l with
        | [] -> Coq_op_strength_reduction_default (op0, (r1 :: []), vl)
        | r2 :: l0 ->
          (match l0 with
           | [] ->
             let args0 = r1 :: (r2 :: []) in
             (match vl with
              | [] -> Coq_op_strength_reduction_default (op0, args0, [])
              | v1 :: l1 ->
                (match l1 with
                 | [] ->
                   Coq_op_strength_reduction_default (op0, args0, (v1 :: []))
                 | a :: l2 ->
                   (match a with
                    | I n2 ->
                      (match l2 with
                       | [] ->
                         Coq_op_strength_reduction_case8 (r1, r2, v1, n2)
                       | a0 :: l3 ->
                         Coq_op_strength_reduction_default (op0, args0,
                           (v1 :: ((I n2) :: (a0 :: l3)))))
                    | x ->
                      Coq_op_strength_reduction_default (op0, args0,
                        (v1 :: (x :: l2))))))
           | r :: l1 ->
             Coq_op_strength_reduction_default (op0,
               (r1 :: (r2 :: (r :: l1))), vl))))
  | Oshl ->
    let op0 = Oshl in
    (match args with
     | [] -> Coq_op_strength_reduction_default (op0, [], vl)
     | r1 :: l ->
       (match l with
        | [] -> Coq_op_strength_reduction_default (op0, (r1 :: []), vl)
        | r2 :: l0 ->
          (match l0 with
           | [] ->
             let args0 = r1 :: (r2 :: []) in
             (match vl with
              | [] -> Coq_op_strength_reduction_default (op0, args0, [])
              | v1 :: l1 ->
                (match l1 with
                 | [] ->
                   Coq_op_strength_reduction_default (op0, args0, (v1 :: []))
                 | a :: l2 ->
                   (match a with
                    | I n2 ->
                      (match l2 with
                       | [] ->
                         Coq_op_strength_reduction_case9 (r1, r2, v1, n2)
                       | a0 :: l3 ->
                         Coq_op_strength_reduction_default (op0, args0,
                           (v1 :: ((I n2) :: (a0 :: l3)))))
                    | x ->
                      Coq_op_strength_reduction_default (op0, args0,
                        (v1 :: (x :: l2))))))
           | r :: l1 ->
             Coq_op_strength_reduction_default (op0,
               (r1 :: (r2 :: (r :: l1))), vl))))
  | Oshr ->
    let op0 = Oshr in
    (match args with
     | [] -> Coq_op_strength_reduction_default (op0, [], vl)
     | r1 :: l ->
       (match l with
        | [] -> Coq_op_strength_reduction_default (op0, (r1 :: []), vl)
        | r2 :: l0 ->
          (match l0 with
           | [] ->
             let args0 = r1 :: (r2 :: []) in
             (match vl with
              | [] -> Coq_op_strength_reduction_default (op0, args0, [])
              | v1 :: l1 ->
                (match l1 with
                 | [] ->
                   Coq_op_strength_reduction_default (op0, args0, (v1 :: []))
                 | a :: l2 ->
                   (match a with
                    | I n2 ->
                      (match l2 with
                       | [] ->
                         Coq_op_strength_reduction_case10 (r1, r2, v1, n2)
                       | a0 :: l3 ->
                         Coq_op_strength_reduction_default (op0, args0,
                           (v1 :: ((I n2) :: (a0 :: l3)))))
                    | x ->
                      Coq_op_strength_reduction_default (op0, args0,
                        (v1 :: (x :: l2))))))
           | r :: l1 ->
             Coq_op_strength_reduction_default (op0,
               (r1 :: (r2 :: (r :: l1))), vl))))
  | Oshru ->
    let op0 = Oshru in
    (match args with
     | [] -> Coq_op_strength_reduction_default (op0, [], vl)
     | r1 :: l ->
       (match l with
        | [] -> Coq_op_strength_reduction_default (op0, (r1 :: []), vl)
        | r2 :: l0 ->
          (match l0 with
           | [] ->
             let args0 = r1 :: (r2 :: []) in
             (match vl with
              | [] -> Coq_op_strength_reduction_default (op0, args0, [])
              | v1 :: l1 ->
                (match l1 with
                 | [] ->
                   Coq_op_strength_reduction_default (op0, args0, (v1 :: []))
                 | a :: l2 ->
                   (match a with
                    | I n2 ->
                      (match l2 with
                       | [] ->
                         Coq_op_strength_reduction_case11 (r1, r2, v1, n2)
                       | a0 :: l3 ->
                         Coq_op_strength_reduction_default (op0, args0,
                           (v1 :: ((I n2) :: (a0 :: l3)))))
                    | x ->
                      Coq_op_strength_reduction_default (op0, args0,
                        (v1 :: (x :: l2))))))
           | r :: l1 ->
             Coq_op_strength_reduction_default (op0,
               (r1 :: (r2 :: (r :: l1))), vl))))
  | Olea addr -> Coq_op_strength_reduction_case12 (addr, args, vl)
  | Ocmp c -> Coq_op_strength_reduction_case13 (c, args, vl)
  | x -> Coq_op_strength_reduction_default (x, args, vl)

let op_strength_reduction :     operation -> reg list -> approx list -> operation * reg list =
  fun op args vl  ->
  match op_strength_reduction_match op args vl with
  | Coq_op_strength_reduction_case1 (r1, r2, v1, n2) ->
    make_addimm (Int.neg n2) r1
  | Coq_op_strength_reduction_case2 (r1, r2, v1, n2) -> make_mulimm n2 r1
  | Coq_op_strength_reduction_case3 (r1, r2, v1, n2) -> make_divimm n2 r1 r2
  | Coq_op_strength_reduction_case4 (r1, r2, v1, n2) -> make_divuimm n2 r1 r2
  | Coq_op_strength_reduction_case5 (r1, r2, v1, n2) -> make_moduimm n2 r1 r2
  | Coq_op_strength_reduction_case6 (r1, r2, v1, n2) -> make_andimm n2 r1
  | Coq_op_strength_reduction_case7 (r1, r2, v1, n2) -> make_orimm n2 r1
  | Coq_op_strength_reduction_case8 (r1, r2, v1, n2) -> make_xorimm n2 r1
  | Coq_op_strength_reduction_case9 (r1, r2, v1, n2) -> make_shlimm n2 r1
  | Coq_op_strength_reduction_case10 (r1, r2, v1, n2) -> make_shrimm n2 r1
  | Coq_op_strength_reduction_case11 (r1, r2, v1, n2) -> make_shruimm n2 r1
  | Coq_op_strength_reduction_case12 (addr, args0, vl0) ->
    let (addr', args') = addr_strength_reduction addr args0 vl0 in
    ((Olea addr'), args')
  | Coq_op_strength_reduction_case13 (c, args0, vl0) ->
    let (c', args') = cond_strength_reduction c args0 vl0 in
    ((Ocmp c'), args')
  | Coq_op_strength_reduction_default (op0, args0, vl0) -> (op0, args0)

