open AST
open BinNums
open Datatypes
open Floats
open Integers
open Op
open Registers

type approx =
| Novalue
| Unknown
| I of Int.int
| F of float
| G of ident * Int.int
| S of Int.int

type eval_static_condition_cases =
| Coq_eval_static_condition_case1 of comparison * Int.int * Int.int
| Coq_eval_static_condition_case2 of comparison * Int.int * Int.int
| Coq_eval_static_condition_case3 of comparison * Int.int * Int.int
| Coq_eval_static_condition_case4 of comparison * Int.int * Int.int
| Coq_eval_static_condition_case5 of comparison * float * float
| Coq_eval_static_condition_case6 of comparison * float * float
| Coq_eval_static_condition_case7 of Int.int * Int.int
| Coq_eval_static_condition_case8 of Int.int * Int.int
| Coq_eval_static_condition_default of condition * approx list

val eval_static_condition_match :
  condition -> approx list -> eval_static_condition_cases

val eval_static_condition : condition -> approx list -> bool option

val eval_static_condition_val : condition -> approx list -> approx

val eval_static_intoffloat : float -> approx

type eval_static_addressing_cases =
| Coq_eval_static_addressing_case1 of Int.int * Int.int
| Coq_eval_static_addressing_case2 of Int.int * ident * Int.int
| Coq_eval_static_addressing_case3 of Int.int * Int.int
| Coq_eval_static_addressing_case4 of Int.int * Int.int * Int.int
| Coq_eval_static_addressing_case5 of Int.int * ident * Int.int * Int.int
| Coq_eval_static_addressing_case6 of Int.int * Int.int * ident * Int.int
| Coq_eval_static_addressing_case7 of Int.int * Int.int * Int.int
| Coq_eval_static_addressing_case8 of Int.int * Int.int * Int.int
| Coq_eval_static_addressing_case9 of Int.int * Int.int * Int.int
| Coq_eval_static_addressing_case10 of Int.int * Int.int * Int.int * Int.int
| Coq_eval_static_addressing_case11 of Int.int * Int.int * ident * Int.int
   * Int.int
| Coq_eval_static_addressing_case12 of Int.int * Int.int * Int.int * Int.int
| Coq_eval_static_addressing_case13 of ident * Int.int
| Coq_eval_static_addressing_case14 of ident * Int.int * Int.int
| Coq_eval_static_addressing_case15 of Int.int * ident * Int.int * Int.int
| Coq_eval_static_addressing_case16 of Int.int
| Coq_eval_static_addressing_default of addressing * approx list

val eval_static_addressing_match :
  addressing -> approx list -> eval_static_addressing_cases

val eval_static_addressing : addressing -> approx list -> approx

val propagate_float_constants : unit -> bool

type eval_static_operation_cases =
| Coq_eval_static_operation_case1 of approx
| Coq_eval_static_operation_case2 of Int.int
| Coq_eval_static_operation_case3 of float
| Coq_eval_static_operation_case4 of Int.int
| Coq_eval_static_operation_case5 of Int.int
| Coq_eval_static_operation_case6 of Int.int
| Coq_eval_static_operation_case7 of Int.int
| Coq_eval_static_operation_case8 of Int.int
| Coq_eval_static_operation_case9 of Int.int * Int.int
| Coq_eval_static_operation_case10 of ident * Int.int * Int.int
| Coq_eval_static_operation_case11 of Int.int * Int.int
| Coq_eval_static_operation_case12 of Int.int * Int.int
| Coq_eval_static_operation_case13 of Int.int * Int.int
| Coq_eval_static_operation_case14 of Int.int * Int.int
| Coq_eval_static_operation_case15 of Int.int * Int.int
| Coq_eval_static_operation_case16 of Int.int * Int.int
| Coq_eval_static_operation_case17 of Int.int * Int.int
| Coq_eval_static_operation_case18 of Int.int * Int.int
| Coq_eval_static_operation_case19 of Int.int * Int.int
| Coq_eval_static_operation_case20 of Int.int * Int.int
| Coq_eval_static_operation_case21 of Int.int * Int.int
| Coq_eval_static_operation_case22 of Int.int * Int.int
| Coq_eval_static_operation_case23 of Int.int * Int.int
| Coq_eval_static_operation_case24 of Int.int * Int.int
| Coq_eval_static_operation_case25 of Int.int * Int.int
| Coq_eval_static_operation_case26 of Int.int * Int.int
| Coq_eval_static_operation_case27 of Int.int * Int.int
| Coq_eval_static_operation_case28 of Int.int * Int.int
| Coq_eval_static_operation_case29 of Int.int * Int.int
| Coq_eval_static_operation_case30 of Int.int * Int.int
| Coq_eval_static_operation_case31 of addressing * approx list
| Coq_eval_static_operation_case32 of float
| Coq_eval_static_operation_case33 of float
| Coq_eval_static_operation_case34 of float * float
| Coq_eval_static_operation_case35 of float * float
| Coq_eval_static_operation_case36 of float * float
| Coq_eval_static_operation_case37 of float * float
| Coq_eval_static_operation_case38 of float
| Coq_eval_static_operation_case39 of float
| Coq_eval_static_operation_case40 of Int.int
| Coq_eval_static_operation_case41 of condition * approx list
| Coq_eval_static_operation_default of operation * approx list

val eval_static_operation_match :
  operation -> approx list -> eval_static_operation_cases

val eval_static_operation : operation -> approx list -> approx

type cond_strength_reduction_cases =
| Coq_cond_strength_reduction_case1 of comparison * reg * reg * Int.int
   * approx
| Coq_cond_strength_reduction_case2 of comparison * reg * reg * approx
   * Int.int
| Coq_cond_strength_reduction_case3 of comparison * reg * reg * Int.int
   * approx
| Coq_cond_strength_reduction_case4 of comparison * reg * reg * approx
   * Int.int
| Coq_cond_strength_reduction_default of condition * reg list * approx list

val cond_strength_reduction_match :
  condition -> reg list -> approx list -> cond_strength_reduction_cases

val cond_strength_reduction :
  condition -> reg list -> approx list -> condition * reg list

type addr_strength_reduction_cases =
| Coq_addr_strength_reduction_case1 of Int.int * reg * ident * Int.int
| Coq_addr_strength_reduction_case2 of Int.int * reg * Int.int
| Coq_addr_strength_reduction_case3 of Int.int * reg * reg * ident * 
   Int.int * Int.int
| Coq_addr_strength_reduction_case4 of Int.int * reg * reg * Int.int * 
   ident * Int.int
| Coq_addr_strength_reduction_case5 of Int.int * reg * reg * Int.int
   * Int.int
| Coq_addr_strength_reduction_case6 of Int.int * reg * reg * Int.int
   * Int.int
| Coq_addr_strength_reduction_case7 of Int.int * reg * reg * ident * 
   Int.int * approx
| Coq_addr_strength_reduction_case8 of Int.int * reg * reg * approx * 
   ident * Int.int
| Coq_addr_strength_reduction_case9 of Int.int * reg * reg * Int.int * approx
| Coq_addr_strength_reduction_case10 of Int.int * reg * reg * approx
   * Int.int
| Coq_addr_strength_reduction_case11 of Int.int * Int.int * reg * reg * 
   ident * Int.int * Int.int
| Coq_addr_strength_reduction_case12 of Int.int * Int.int * reg * reg * 
   ident * Int.int * approx
| Coq_addr_strength_reduction_case13 of Int.int * Int.int * reg * reg
   * approx * Int.int
| Coq_addr_strength_reduction_case14 of ident * Int.int * reg * Int.int
| Coq_addr_strength_reduction_case15 of Int.int * ident * Int.int * reg
   * Int.int
| Coq_addr_strength_reduction_default of addressing * reg list * approx list

val addr_strength_reduction_match :
  addressing -> reg list -> approx list -> addr_strength_reduction_cases

val addr_strength_reduction :
  addressing -> reg list -> approx list -> addressing * reg list

val make_addimm : Int.int -> reg -> operation * reg list

val make_shlimm : Int.int -> reg -> operation * reg list

val make_shrimm : Int.int -> reg -> operation * reg list

val make_shruimm : Int.int -> reg -> operation * reg list

val make_mulimm : Int.int -> reg -> operation * reg list

val make_andimm : Int.int -> reg -> operation * reg list

val make_orimm : Int.int -> reg -> operation * reg list

val make_xorimm : Int.int -> reg -> operation * reg list

val make_divimm : Int.int -> reg -> reg -> operation * reg list

val make_divuimm : Int.int -> reg -> reg -> operation * reg list

val make_moduimm : Int.int -> reg -> reg -> operation * reg list

type op_strength_reduction_cases =
| Coq_op_strength_reduction_case1 of reg * reg * approx * Int.int
| Coq_op_strength_reduction_case2 of reg * reg * approx * Int.int
| Coq_op_strength_reduction_case3 of reg * reg * approx * Int.int
| Coq_op_strength_reduction_case4 of reg * reg * approx * Int.int
| Coq_op_strength_reduction_case5 of reg * reg * approx * Int.int
| Coq_op_strength_reduction_case6 of reg * reg * approx * Int.int
| Coq_op_strength_reduction_case7 of reg * reg * approx * Int.int
| Coq_op_strength_reduction_case8 of reg * reg * approx * Int.int
| Coq_op_strength_reduction_case9 of reg * reg * approx * Int.int
| Coq_op_strength_reduction_case10 of reg * reg * approx * Int.int
| Coq_op_strength_reduction_case11 of reg * reg * approx * Int.int
| Coq_op_strength_reduction_case12 of addressing * reg list * approx list
| Coq_op_strength_reduction_case13 of condition * reg list * approx list
| Coq_op_strength_reduction_default of operation * reg list * approx list

val op_strength_reduction_match :
  operation -> reg list -> approx list -> op_strength_reduction_cases

val op_strength_reduction :
  operation -> reg list -> approx list -> operation * reg list

