open AST
open BinInt
open BinNums
open Datatypes
open List0
open Locations
open Machregs

let int_caller_save_regs :  mreg list =
  AX :: []

let float_caller_save_regs :  mreg list =
  X0 :: (X1 :: (X2 :: (X3 :: (X4 :: (X5 :: [])))))

let int_callee_save_regs :  mreg list =
  BX :: (SI :: (DI :: (BP :: [])))

let float_callee_save_regs :  mreg list =
  []

let destroyed_at_call_regs :  mreg list =
  app int_caller_save_regs float_caller_save_regs

let int_temporaries :  mreg list =
  IT1 :: (IT2 :: [])

let float_temporaries :  mreg list =
  FT1 :: (FT2 :: [])

let temporary_regs :  mreg list =
  IT1 :: (IT2 :: (FT1 :: (FT2 :: (FP0 :: []))))

let temporaries :  loc list =
  map (fun x -> R x) temporary_regs

let destroyed_at_move_regs :  mreg list =
  FP0 :: []

let destroyed_at_move :  loc list =
  map (fun x -> R x) destroyed_at_move_regs

let dummy_int_reg :  mreg =
  AX

let dummy_float_reg :  mreg =
  X0

let index_int_callee_save :  mreg -> coq_Z =
    function
| BX -> Z0
| SI -> Zpos Coq_xH
| DI -> Zpos (Coq_xO Coq_xH)
| BP -> Zpos (Coq_xI Coq_xH)
| _ -> Zneg Coq_xH

let index_float_callee_save :  mreg -> coq_Z =
  fun r  ->
  Zneg Coq_xH

let loc_result :  signature -> mreg =
  fun s  ->
  match s.sig_res with
  | Some t ->
    (match t with
     | Tint -> AX
     | Tfloat -> FP0)
  | None -> AX

let rec loc_arguments_rec :  typ list -> coq_Z -> loc list =
  fun tyl ofs  ->
  match tyl with
  | [] -> []
  | t :: tys ->
    (match t with
     | Tint ->
       (S (Outgoing (ofs,
         Tint))) :: (loc_arguments_rec tys (Z.add ofs (Zpos Coq_xH)))
     | Tfloat ->
       (S (Outgoing (ofs,
         Tfloat))) :: (loc_arguments_rec tys
                        (Z.add ofs (Zpos (Coq_xO Coq_xH)))))

let loc_arguments :  signature -> loc list =
  fun s  ->
  loc_arguments_rec s.sig_args Z0

let rec size_arguments_rec :  typ list -> coq_Z -> coq_Z =
  fun tyl ofs  ->
  match tyl with
  | [] -> ofs
  | t :: tys ->
    (match t with
     | Tint -> size_arguments_rec tys (Z.add ofs (Zpos Coq_xH))
     | Tfloat -> size_arguments_rec tys (Z.add ofs (Zpos (Coq_xO Coq_xH))))

let size_arguments :  signature -> coq_Z =
  fun s  ->
  size_arguments_rec s.sig_args Z0

