open AST
open BinNums
open Coqlib
open Ctypes
open Datatypes
open Floats
open Integers
open Memory
open Values

type unary_operation =
| Onotbool
| Onotint
| Oneg

type binary_operation =
| Oadd
| Osub
| Omul
| Odiv
| Omod
| Oand
| Oor
| Oxor
| Oshl
| Oshr
| Oeq
| One
| Olt
| Ogt
| Ole
| Oge

type incr_or_decr =
| Incr
| Decr

type classify_cast_cases =
| Coq_cast_case_neutral
| Coq_cast_case_i2i of intsize * signedness
| Coq_cast_case_f2f of floatsize
| Coq_cast_case_i2f of signedness * floatsize
| Coq_cast_case_f2i of intsize * signedness
| Coq_cast_case_f2bool
| Coq_cast_case_p2bool
| Coq_cast_case_struct of ident * fieldlist * ident * fieldlist
| Coq_cast_case_union of ident * fieldlist * ident * fieldlist
| Coq_cast_case_void
| Coq_cast_case_default

val classify_cast : coq_type -> coq_type -> classify_cast_cases

val cast_int_int : intsize -> signedness -> Int.int -> Int.int

val cast_int_float : signedness -> Int.int -> float

val cast_float_int : signedness -> float -> Int.int option

val cast_float_float : floatsize -> float -> float

val sem_cast : coq_val -> coq_type -> coq_type -> coq_val option

type classify_bool_cases =
| Coq_bool_case_i
| Coq_bool_case_f
| Coq_bool_case_p
| Coq_bool_default

val classify_bool : coq_type -> classify_bool_cases

val bool_val : coq_val -> coq_type -> bool option

val sem_notbool : coq_val -> coq_type -> coq_val option

type classify_neg_cases =
| Coq_neg_case_i of signedness
| Coq_neg_case_f
| Coq_neg_default

val classify_neg : coq_type -> classify_neg_cases

val sem_neg : coq_val -> coq_type -> coq_val option

type classify_notint_cases =
| Coq_notint_case_i of signedness
| Coq_notint_default

val classify_notint : coq_type -> classify_notint_cases

val sem_notint : coq_val -> coq_type -> coq_val option

type classify_add_cases =
| Coq_add_case_ii of signedness
| Coq_add_case_ff
| Coq_add_case_if of signedness
| Coq_add_case_fi of signedness
| Coq_add_case_pi of coq_type * attr
| Coq_add_case_ip of coq_type * attr
| Coq_add_default

val classify_add : coq_type -> coq_type -> classify_add_cases

val sem_add : coq_val -> coq_type -> coq_val -> coq_type -> coq_val option

type classify_sub_cases =
| Coq_sub_case_ii of signedness
| Coq_sub_case_ff
| Coq_sub_case_if of signedness
| Coq_sub_case_fi of signedness
| Coq_sub_case_pi of coq_type
| Coq_sub_case_pp of coq_type
| Coq_sub_default

val classify_sub : coq_type -> coq_type -> classify_sub_cases

val sem_sub : coq_val -> coq_type -> coq_val -> coq_type -> coq_val option

type classify_mul_cases =
| Coq_mul_case_ii of signedness
| Coq_mul_case_ff
| Coq_mul_case_if of signedness
| Coq_mul_case_fi of signedness
| Coq_mul_default

val classify_mul : coq_type -> coq_type -> classify_mul_cases

val sem_mul : coq_val -> coq_type -> coq_val -> coq_type -> coq_val option

type classify_div_cases =
| Coq_div_case_ii of signedness
| Coq_div_case_ff
| Coq_div_case_if of signedness
| Coq_div_case_fi of signedness
| Coq_div_default

val classify_div : coq_type -> coq_type -> classify_div_cases

val sem_div : coq_val -> coq_type -> coq_val -> coq_type -> coq_val option

type classify_binint_cases =
| Coq_binint_case_ii of signedness
| Coq_binint_default

val classify_binint : coq_type -> coq_type -> classify_binint_cases

val sem_mod : coq_val -> coq_type -> coq_val -> coq_type -> coq_val option

val sem_and : coq_val -> coq_type -> coq_val -> coq_type -> coq_val option

val sem_or : coq_val -> coq_type -> coq_val -> coq_type -> coq_val option

val sem_xor : coq_val -> coq_type -> coq_val -> coq_type -> coq_val option

type classify_shift_cases =
| Coq_shift_case_ii of signedness
| Coq_shift_default

val classify_shift : coq_type -> coq_type -> classify_shift_cases

val sem_shl : coq_val -> coq_type -> coq_val -> coq_type -> coq_val option

val sem_shr : coq_val -> coq_type -> coq_val -> coq_type -> coq_val option

type classify_cmp_cases =
| Coq_cmp_case_ii of signedness
| Coq_cmp_case_pp
| Coq_cmp_case_ff
| Coq_cmp_case_if of signedness
| Coq_cmp_case_fi of signedness
| Coq_cmp_default

val classify_cmp : coq_type -> coq_type -> classify_cmp_cases

val sem_cmp :
  comparison -> coq_val -> coq_type -> coq_val -> coq_type -> Mem.mem ->
  coq_val option

type classify_fun_cases =
| Coq_fun_case_f of typelist * coq_type
| Coq_fun_default

val classify_fun : coq_type -> classify_fun_cases

val sem_unary_operation :
  unary_operation -> coq_val -> coq_type -> coq_val option

val sem_binary_operation :
  binary_operation -> coq_val -> coq_type -> coq_val -> coq_type -> Mem.mem
  -> coq_val option

