open BinInt
open BinNums
open BinPos
open Datatypes
open List0
open ZArith_dec

let peq :  positive -> positive -> bool =
  fun x y -> Pos.eq_dec x y

let plt :  positive -> positive -> bool =
  fun x y  ->
  let c = Pos.compare x y in
  (match c with
   | Lt -> true
   | _ -> false)

let zeq :  coq_Z -> coq_Z -> bool =
  fun x y -> Z.eq_dec x y

let zlt :  coq_Z -> coq_Z -> bool =
fun x0 x1  -> coq_Z_lt_dec x0 x1 

let zle :  coq_Z -> coq_Z -> bool =
fun x0 x1  -> coq_Z_le_gt_dec x0 x1 

let coq_Zdivide_dec :  coq_Z -> coq_Z -> bool =
  fun p q  ->
  zeq (Z.modulo q p) Z0

let nat_of_Z :  coq_Z -> nat =
  fun x -> Z.to_nat x

let align :  coq_Z -> coq_Z -> coq_Z =
  fun n amount  ->
  Z.mul (Z.div (Z.sub (Z.add n amount) (Zpos Coq_xH)) amount) amount

let option_map :  ('a1 -> 'a2) -> 'a1 option -> 'a2 option =
  fun f  ->
    function
| Some y -> Some (f y)
| None -> None

let sum_left_map :  ('a1 -> 'a2) -> ('a1, 'a3) sum -> ('a2, 'a3) sum =
  fun f  ->
    function
| Coq_inl y -> Coq_inl (f y)
| Coq_inr z -> Coq_inr z

let rec list_length_z_aux :  'a1 list -> coq_Z -> coq_Z =
  fun l acc  ->
  match l with
  | [] -> acc
  | hd :: tl -> list_length_z_aux tl (Z.succ acc)

let list_length_z :  'a1 list -> coq_Z =
  fun l  ->
  list_length_z_aux l Z0

let rec list_nth_z :  'a1 list -> coq_Z -> 'a1 option =
  fun l n  ->
  match l with
  | [] -> None
  | hd :: tl -> if zeq n Z0 then Some hd else list_nth_z tl (Z.pred n)

let rec list_disjoint_dec :     ('a1 -> 'a1 -> bool) -> 'a1 list -> 'a1 list -> bool =
  fun eqA_dec l1 l2  ->
  match l1 with
  | [] -> true
  | y :: l ->
    if in_dec eqA_dec y l2 then false else list_disjoint_dec eqA_dec l l2

let rec list_norepet_dec :  ('a1 -> 'a1 -> bool) -> 'a1 list -> bool =
  fun eqA_dec  ->
    function
| [] -> true
| y :: l0 ->
  if list_norepet_dec eqA_dec l0
  then if in_dec eqA_dec y l0 then false else true
  else false

let rec list_repeat :  nat -> 'a1 -> 'a1 list =
  fun n x  ->
  match n with
  | O -> []
  | S m -> x :: (list_repeat m x)

let proj_sumbool :  bool -> bool =
    function
| true -> true
| false -> false

