open AST
open Cop
open Ctypes
open Datatypes
open Integers
open List0
open Values

type expr =
| Eval of coq_val * coq_type
| Evar of ident * coq_type
| Efield of expr * ident * coq_type
| Evalof of expr * coq_type
| Ederef of expr * coq_type
| Eaddrof of expr * coq_type
| Eunop of unary_operation * expr * coq_type
| Ebinop of binary_operation * expr * expr * coq_type
| Ecast of expr * coq_type
| Eseqand of expr * expr * coq_type
| Eseqor of expr * expr * coq_type
| Econdition of expr * expr * expr * coq_type
| Esizeof of coq_type * coq_type
| Ealignof of coq_type * coq_type
| Eassign of expr * expr * coq_type
| Eassignop of binary_operation * expr * expr * coq_type * coq_type
| Epostincr of incr_or_decr * expr * coq_type
| Ecomma of expr * expr * coq_type
| Ecall of expr * exprlist * coq_type
| Ebuiltin of external_function * typelist * exprlist * coq_type
| Eloc of block * Int.int * coq_type
| Eparen of expr * coq_type
and exprlist =
| Enil
| Econs of expr * exprlist

val coq_Eindex : expr -> expr -> coq_type -> expr

val coq_Epreincr : incr_or_decr -> expr -> coq_type -> expr

val typeof : expr -> coq_type

type label = ident

type statement =
| Sskip
| Sdo of expr
| Ssequence of statement * statement
| Sifthenelse of expr * statement * statement
| Swhile of expr * statement
| Sdowhile of expr * statement
| Sfor of statement * expr * statement * statement
| Sbreak
| Scontinue
| Sreturn of expr option
| Sswitch of expr * labeled_statements
| Slabel of label * statement
| Sgoto of label
and labeled_statements =
| LSdefault of statement
| LScase of Int.int * statement * labeled_statements

type coq_function = { fn_return : coq_type;
                      fn_params : (ident * coq_type) list;
                      fn_vars : (ident * coq_type) list; fn_body : statement }

val fn_return : coq_function -> coq_type

val fn_params : coq_function -> (ident * coq_type) list

val fn_vars : coq_function -> (ident * coq_type) list

val fn_body : coq_function -> statement

val var_names : (ident * coq_type) list -> ident list

type fundef =
| Internal of coq_function
| External of external_function * typelist * coq_type

val type_of_function : coq_function -> coq_type

val type_of_fundef : fundef -> coq_type

type program = (fundef, coq_type) AST.program

