open AST
open BinInt
open BinNums
open Bool
open Coqlib
open Errors

type signedness =
| Signed
| Unsigned

type intsize =
| I8
| I16
| I32
| IBool

type floatsize =
| F32
| F64

type attr =
  bool
  (* singleton inductive, whose constructor was mk_attr *)

val attr_volatile : attr -> bool

val noattr : attr

type coq_type =
| Tvoid
| Tint of intsize * signedness * attr
| Tfloat of floatsize * attr
| Tpointer of coq_type * attr
| Tarray of coq_type * coq_Z * attr
| Tfunction of typelist * coq_type
| Tstruct of ident * fieldlist * attr
| Tunion of ident * fieldlist * attr
| Tcomp_ptr of ident * attr
and typelist =
| Tnil
| Tcons of coq_type * typelist
and fieldlist =
| Fnil
| Fcons of ident * coq_type * fieldlist

val type_eq : coq_type -> coq_type -> bool

val typelist_eq : typelist -> typelist -> bool

val fieldlist_eq : fieldlist -> fieldlist -> bool

val attr_of_type : coq_type -> attr

val type_int32s : coq_type

val type_bool : coq_type

val typeconv : coq_type -> coq_type

val alignof : coq_type -> coq_Z

val alignof_fields : fieldlist -> coq_Z

val sizeof : coq_type -> coq_Z

val sizeof_struct : fieldlist -> coq_Z -> coq_Z

val sizeof_union : fieldlist -> coq_Z

val field_offset_rec : ident -> fieldlist -> coq_Z -> coq_Z res

val field_offset : ident -> fieldlist -> coq_Z res

type mode =
| By_value of memory_chunk
| By_reference
| By_copy
| By_nothing

val access_mode : coq_type -> mode

val type_is_volatile : coq_type -> bool

val unroll_composite : ident -> coq_type -> coq_type -> coq_type

val type_of_params : (ident * coq_type) list -> typelist

val typ_of_type : coq_type -> typ

val opttyp_of_type : coq_type -> typ option

val typlist_of_typelist : typelist -> typ list

val signature_of_type : typelist -> coq_type -> signature

