open AST
open Floats
open Globalenvs
open Integers
open Values

type eventval =
| EVint of Int.int
| EVfloat of float
| EVptr_global of ident * Int.int

type event =
| Event_syscall of ident * eventval list * eventval
| Event_vload of memory_chunk * ident * Int.int * eventval
| Event_vstore of memory_chunk * ident * Int.int * eventval
| Event_annot of ident * eventval list

type trace = event list

let coq_E0 :  trace =
  []

let block_is_volatile :  ('a1, 'a2) Genv.t -> block -> bool =
  fun ge b  ->
  match Genv.find_var_info ge b with
  | Some gv -> gv.gvar_volatile
  | None -> false

let rec annot_eventvals :     annot_arg list -> eventval list -> eventval list =
  fun targs vargs  ->
  match targs with
  | [] -> vargs
  | a :: targs' ->
    (match a with
     | AA_arg ty ->
       (match vargs with
        | [] -> vargs
        | varg :: vargs' -> varg :: (annot_eventvals targs' vargs'))
     | AA_int n -> (EVint n) :: (annot_eventvals targs' vargs)
     | AA_float n -> (EVfloat n) :: (annot_eventvals targs' vargs))

