open BinInt
open BinNums
open Fappli_IEEE
open Zbool

let join_bits :  coq_Z -> coq_Z -> bool -> coq_Z -> coq_Z -> coq_Z =
  fun mw ew s m e  ->
  Z.add
    (Z.mul (Z.add (if s then Z.pow (Zpos (Coq_xO Coq_xH)) ew else Z0) e)
      (Z.pow (Zpos (Coq_xO Coq_xH)) mw)) m

let split_bits :  coq_Z -> coq_Z -> coq_Z -> (bool * coq_Z) * coq_Z =
  fun mw ew x  ->
  let mm = Z.pow (Zpos (Coq_xO Coq_xH)) mw in
  let em = Z.pow (Zpos (Coq_xO Coq_xH)) ew in
  (((Z.leb (Z.mul mm em) x), (Z.modulo x mm)), (Z.modulo (Z.div x mm) em))

let bits_of_binary_float :  coq_Z -> coq_Z -> binary_float -> coq_Z =
  fun mw ew x ->
  let emax = Z.pow (Zpos (Coq_xO Coq_xH)) (Z.sub ew (Zpos Coq_xH)) in
  let prec = Z.add mw (Zpos Coq_xH) in
  let emin = Z.sub (Z.sub (Zpos (Coq_xI Coq_xH)) emax) prec in
  match x with
  | B754_zero sx -> join_bits mw ew sx Z0 Z0
  | B754_infinity sx ->
    join_bits mw ew sx Z0
      (Z.sub (Z.pow (Zpos (Coq_xO Coq_xH)) ew) (Zpos Coq_xH))
  | B754_nan ->
    join_bits mw ew false
      (Z.sub (Z.pow (Zpos (Coq_xO Coq_xH)) mw) (Zpos Coq_xH))
      (Z.sub (Z.pow (Zpos (Coq_xO Coq_xH)) ew) (Zpos Coq_xH))
  | B754_finite (sx, mx, ex) ->
    if Z.leb (Z.pow (Zpos (Coq_xO Coq_xH)) mw) (Zpos mx)
    then join_bits mw ew sx
           (Z.sub (Zpos mx) (Z.pow (Zpos (Coq_xO Coq_xH)) mw))
           (Z.add (Z.sub ex emin) (Zpos Coq_xH))
    else join_bits mw ew sx (Zpos mx) Z0

let binary_float_of_bits_aux :  coq_Z -> coq_Z -> coq_Z -> full_float =
  fun mw ew x ->
  let emax = Z.pow (Zpos (Coq_xO Coq_xH)) (Z.sub ew (Zpos Coq_xH)) in
  let prec = Z.add mw (Zpos Coq_xH) in
  let emin = Z.sub (Z.sub (Zpos (Coq_xI Coq_xH)) emax) prec in
  (
  let (p, ex) = split_bits mw ew x in
  let (sx, mx) = p in
  if coq_Zeq_bool ex Z0
  then (match mx with
        | Z0 -> F754_zero sx
        | Zpos px -> F754_finite (sx, px, emin)
        | Zneg p0 -> F754_nan)
  else if coq_Zeq_bool ex
            (Z.sub (Z.pow (Zpos (Coq_xO Coq_xH)) ew) (Zpos Coq_xH))
       then if coq_Zeq_bool mx Z0 then F754_infinity sx else F754_nan
       else (match Z.add mx (Z.pow (Zpos (Coq_xO Coq_xH)) mw) with
             | Zpos px ->
               F754_finite (sx, px, (Z.sub (Z.add ex emin) (Zpos Coq_xH)))
             | _ -> F754_nan))

let binary_float_of_bits :  coq_Z -> coq_Z -> coq_Z -> binary_float =
  fun mw ew x  ->
  let emax = Z.pow (Zpos (Coq_xO Coq_xH)) (Z.sub ew (Zpos Coq_xH)) in
  let prec = Z.add mw (Zpos Coq_xH) in
  coq_FF2B prec emax (binary_float_of_bits_aux mw ew x)

type binary32 = binary_float

let b32_of_bits :  coq_Z -> binary32 =
  fun a -> 
  binary_float_of_bits (Zpos (Coq_xI (Coq_xI (Coq_xI (Coq_xO Coq_xH)))))
    (Zpos (Coq_xO (Coq_xO (Coq_xO Coq_xH)))) a

let bits_of_b32 :  binary32 -> coq_Z =
  fun x ->
  bits_of_binary_float (Zpos (Coq_xI (Coq_xI (Coq_xI (Coq_xO Coq_xH)))))
    (Zpos (Coq_xO (Coq_xO (Coq_xO Coq_xH)))) x

type binary64 = binary_float

let b64_opp :  binary_float -> binary_float =
  fun a ->
  coq_Bopp (Zpos (Coq_xI (Coq_xO (Coq_xI (Coq_xO (Coq_xI Coq_xH)))))) (Zpos
    (Coq_xO (Coq_xO (Coq_xO (Coq_xO (Coq_xO (Coq_xO (Coq_xO (Coq_xO (Coq_xO
    (Coq_xO Coq_xH))))))))))) a

let b64_plus :  mode -> binary_float -> binary_float -> binary_float =
  fun a b c ->
  coq_Bplus (Zpos (Coq_xI (Coq_xO (Coq_xI (Coq_xO (Coq_xI Coq_xH)))))) (Zpos
    (Coq_xO (Coq_xO (Coq_xO (Coq_xO (Coq_xO (Coq_xO (Coq_xO (Coq_xO (Coq_xO
    (Coq_xO Coq_xH))))))))))) a b c

let b64_minus :  mode -> binary_float -> binary_float -> binary_float =
  fun a b c ->
  coq_Bminus (Zpos (Coq_xI (Coq_xO (Coq_xI (Coq_xO (Coq_xI Coq_xH)))))) (Zpos
    (Coq_xO (Coq_xO (Coq_xO (Coq_xO (Coq_xO (Coq_xO (Coq_xO (Coq_xO (Coq_xO
    (Coq_xO Coq_xH))))))))))) a b c

let b64_mult :  mode -> binary_float -> binary_float -> binary_float =
  fun a b c ->
  coq_Bmult (Zpos (Coq_xI (Coq_xO (Coq_xI (Coq_xO (Coq_xI Coq_xH)))))) (Zpos
    (Coq_xO (Coq_xO (Coq_xO (Coq_xO (Coq_xO (Coq_xO (Coq_xO (Coq_xO (Coq_xO
    (Coq_xO Coq_xH))))))))))) a b c

let b64_div :  mode -> binary_float -> binary_float -> binary_float =
  fun a b c ->
  coq_Bdiv (Zpos (Coq_xI (Coq_xO (Coq_xI (Coq_xO (Coq_xI Coq_xH)))))) (Zpos
    (Coq_xO (Coq_xO (Coq_xO (Coq_xO (Coq_xO (Coq_xO (Coq_xO (Coq_xO (Coq_xO
    (Coq_xO Coq_xH))))))))))) a b c

let b64_of_bits :  coq_Z -> binary64 =
  fun a ->
  binary_float_of_bits (Zpos (Coq_xO (Coq_xO (Coq_xI (Coq_xO (Coq_xI
    Coq_xH)))))) (Zpos (Coq_xI (Coq_xI (Coq_xO Coq_xH)))) a

let bits_of_b64 :  binary64 -> coq_Z =
  fun x ->
  bits_of_binary_float (Zpos (Coq_xO (Coq_xO (Coq_xI (Coq_xO (Coq_xI
    Coq_xH)))))) (Zpos (Coq_xI (Coq_xI (Coq_xO Coq_xH)))) x

