open BinInt
open BinNums
open BinPos
open Datatypes
open Fappli_IEEE
open Fappli_IEEE_bits
open Fcore_Zaux
open Integers

type float = binary64

module Float = 
 struct 
  (** val zero : float **)
  
  let zero =
    B754_zero false
  
  (** val eq_dec : float -> float -> bool **)
  
  let eq_dec f1 f2 =
    match f1 with
    | B754_zero b ->
      (match f2 with
       | B754_zero b0 ->
         if b then if b0 then true else false else if b0 then false else true
       | _ -> false)
    | B754_infinity b ->
      (match f2 with
       | B754_infinity b0 ->
         if b then if b0 then true else false else if b0 then false else true
       | _ -> false)
    | B754_nan ->
      (match f2 with
       | B754_nan -> true
       | _ -> false)
    | B754_finite (b, m, e) ->
      (match f2 with
       | B754_finite (b0, m0, e1) ->
         if b
         then if b0
              then let s = Pos.eq_dec m m0 in
                   if s then Z.eq_dec e e1 else false
              else false
         else if b0
              then false
              else let s = Pos.eq_dec m m0 in
                   if s then Z.eq_dec e e1 else false
       | _ -> false)
  
  (** val neg : float -> float **)
  
  let neg x =
    b64_opp x
  
  (** val abs : float -> float **)
  
  let abs x = match x with
  | B754_zero b -> B754_zero false
  | B754_infinity b -> B754_infinity false
  | B754_nan -> x
  | B754_finite (b, m, e) -> B754_finite (false, m, e)
  
  (** val binary_normalize64 : coq_Z -> coq_Z -> bool -> float **)
  
  let binary_normalize64 m e s =
    binary_normalize (Zpos (Coq_xI (Coq_xO (Coq_xI (Coq_xO (Coq_xI
      Coq_xH)))))) (Zpos (Coq_xO (Coq_xO (Coq_xO (Coq_xO (Coq_xO (Coq_xO
      (Coq_xO (Coq_xO (Coq_xO (Coq_xO Coq_xH))))))))))) Coq_mode_NE m e s
  
  (** val binary_normalize32 : coq_Z -> coq_Z -> bool -> binary32 **)
  
  let binary_normalize32 m e s =
    binary_normalize (Zpos (Coq_xO (Coq_xO (Coq_xO (Coq_xI Coq_xH))))) (Zpos
      (Coq_xO (Coq_xO (Coq_xO (Coq_xO (Coq_xO (Coq_xO (Coq_xO Coq_xH))))))))
      Coq_mode_NE m e s
  
  (** val floatofbinary32 : binary32 -> float **)
  
  let floatofbinary32 = function
  | B754_finite (s, m, e) -> binary_normalize64 (cond_Zopp s (Zpos m)) e s
  | x -> x
  
  (** val binary32offloat : float -> binary32 **)
  
  let binary32offloat = function
  | B754_finite (s, m, e) -> binary_normalize32 (cond_Zopp s (Zpos m)) e s
  | x -> x
  
  (** val singleoffloat : float -> float **)
  
  let singleoffloat f =
    floatofbinary32 (binary32offloat f)
  
  (** val coq_Zoffloat : float -> coq_Z option **)
  
  let coq_Zoffloat = function
  | B754_zero b -> Some Z0
  | B754_finite (s, m, e0) ->
    (match e0 with
     | Z0 -> Some (cond_Zopp s (Zpos m))
     | Zpos e ->
       Some (Z.mul (cond_Zopp s (Zpos m)) (Z.pow_pos (radix_val radix2) e))
     | Zneg e ->
       Some (cond_Zopp s (Z.div (Zpos m) (Z.pow_pos (radix_val radix2) e))))
  | _ -> None
  
  (** val intoffloat : float -> Int.int option **)
  
  let intoffloat f =
    match coq_Zoffloat f with
    | Some n ->
      if (&&) (Z.leb Int.min_signed n) (Z.leb n Int.max_signed)
      then Some (Int.repr n)
      else None
    | None -> None
  
  (** val intuoffloat : float -> Int.int option **)
  
  let intuoffloat f =
    match coq_Zoffloat f with
    | Some n ->
      if (&&) (Z.leb Z0 n) (Z.leb n Int.max_unsigned)
      then Some (Int.repr n)
      else None
    | None -> None
  
  (** val build_from_parsed_obligation_1 :
      coq_Z -> coq_Z -> positive -> positive -> coq_Z -> positive ->
      binary_float **)
  
  let build_from_parsed_obligation_1  :
      coq_Z -> coq_Z -> positive -> positive -> coq_Z -> positive ->
      binary_float =
    fun prec emax coqzz_base intPart expPart p ->
    failwith "absurd case" (* absurd case *)
  
  (** val build_from_parsed_obligation_2 :
      coq_Z -> coq_Z -> positive -> positive -> coq_Z -> positive -> positive
      -> binary_float **)
  
  let build_from_parsed_obligation_2 :
      coq_Z -> coq_Z -> positive -> positive -> coq_Z -> positive -> positive
      -> binary_float =
    fun prec emax coqzz_base intPart expPart p anonymous ->
    failwith "absurd case" (* absurd case *)
  
  (** val build_from_parsed :
      coq_Z -> coq_Z -> positive -> positive -> coq_Z -> binary_float **)
  
  let build_from_parsed prec emax coqzz_base intPart expPart =
    match expPart with
  | Z0 -> binary_normalize prec emax Coq_mode_NE (Zpos intPart) Z0 false
  | Zpos p ->
    binary_normalize prec emax Coq_mode_NE
      (Z.mul (Zpos intPart) (Z.pow_pos (Zpos coqzz_base) p)) Z0 false
  | Zneg p ->
    let exp = Z.pow_pos (Zpos coqzz_base) p in
    (match exp with
     | Z0 -> build_from_parsed_obligation_1 prec emax coqzz_base intPart expPart p
     | Zpos p0 ->
       coq_FF2B prec emax
         (let (p1, lz) =
            coq_Fdiv_core_binary prec (Zpos intPart) Z0 (Zpos p0) Z0
          in
          let (mz, ez) = p1 in
          (match mz with
           | Zpos mz0 ->
             binary_round_aux prec emax Coq_mode_NE (xorb false false) mz0 ez
               lz
           | _ -> F754_nan))
     | Zneg p0 ->
       build_from_parsed_obligation_2 prec emax coqzz_base intPart expPart p p0)
  
  (** val build_from_parsed64 : positive -> positive -> coq_Z -> float **)
  
  let build_from_parsed64 coqzz_base intPart expPart =
    build_from_parsed (Zpos (Coq_xI (Coq_xO (Coq_xI (Coq_xO (Coq_xI
      Coq_xH)))))) (Zpos (Coq_xO (Coq_xO (Coq_xO (Coq_xO (Coq_xO (Coq_xO
      (Coq_xO (Coq_xO (Coq_xO (Coq_xO Coq_xH))))))))))) coqzz_base intPart expPart
  
  (** val build_from_parsed32 : positive -> positive -> coq_Z -> float **)
  
  let build_from_parsed32 coqzz_base intPart expPart =
    floatofbinary32
      (build_from_parsed (Zpos (Coq_xO (Coq_xO (Coq_xO (Coq_xI Coq_xH)))))
        (Zpos (Coq_xO (Coq_xO (Coq_xO (Coq_xO (Coq_xO (Coq_xO (Coq_xO
        Coq_xH)))))))) coqzz_base intPart expPart)
  
  (** val floatofint : Int.int -> float **)
  
  let floatofint n =
    binary_normalize64 (Int.signed n) Z0 false
  
  (** val floatofintu : Int.int -> float **)
  
  let floatofintu n =
    binary_normalize64 (Int.unsigned n) Z0 false
  
  (** val add : float -> float -> float **)
  
  let add x y =
    b64_plus Coq_mode_NE x y
  
  (** val sub : float -> float -> float **)
  
  let sub x y =
    b64_minus Coq_mode_NE x y
  
  (** val mul : float -> float -> float **)
  
  let mul x y =
    b64_mult Coq_mode_NE x y
  
  (** val div : float -> float -> float **)
  
  let div x y =
    b64_div Coq_mode_NE x y
  
  (** val order_float : float -> float -> Datatypes.comparison option **)
  
  let order_float f1 f2 =
    match f1 with
    | B754_zero b ->
      (match f2 with
       | B754_zero b0 -> Some Eq
       | B754_infinity b0 -> if b0 then Some Gt else Some Lt
       | B754_nan -> None
       | B754_finite (b0, m, e) -> if b0 then Some Gt else Some Lt)
    | B754_infinity b ->
      if b
      then (match f2 with
            | B754_infinity b0 -> if b0 then Some Eq else Some Lt
            | B754_nan -> None
            | _ -> Some Lt)
      else (match f2 with
            | B754_infinity b0 -> if b0 then Some Gt else Some Eq
            | B754_nan -> None
            | _ -> Some Gt)
    | B754_nan -> None
    | B754_finite (s1, m1, e1) ->
      if s1
      then (match f2 with
            | B754_zero b -> Some Lt
            | B754_infinity b -> if b then Some Gt else Some Lt
            | B754_nan -> None
            | B754_finite (s2, m2, e2) ->
              if s1
              then if s2
                   then (match Z.compare e1 e2 with
                         | Eq ->
                           Some (coq_CompOpp (Pos.compare_cont m1 m2 Eq))
                         | Lt -> Some Gt
                         | Gt -> Some Lt)
                   else Some Lt
              else if s2
                   then Some Gt
                   else (match Z.compare e1 e2 with
                         | Eq -> Some (Pos.compare_cont m1 m2 Eq)
                         | x -> Some x))
      else (match f2 with
            | B754_zero b -> Some Gt
            | B754_infinity b -> if b then Some Gt else Some Lt
            | B754_nan -> None
            | B754_finite (s2, m2, e2) ->
              if s1
              then if s2
                   then (match Z.compare e1 e2 with
                         | Eq ->
                           Some (coq_CompOpp (Pos.compare_cont m1 m2 Eq))
                         | Lt -> Some Gt
                         | Gt -> Some Lt)
                   else Some Lt
              else if s2
                   then Some Gt
                   else (match Z.compare e1 e2 with
                         | Eq -> Some (Pos.compare_cont m1 m2 Eq)
                         | x -> Some x))
  
  (** val cmp : comparison -> float -> float -> bool **)
  
  let cmp c f1 f2 =
    match c with
    | Ceq ->
      (match order_float f1 f2 with
       | Some c0 ->
         (match c0 with
          | Eq -> true
          | _ -> false)
       | None -> false)
    | Cne ->
      (match order_float f1 f2 with
       | Some c0 ->
         (match c0 with
          | Eq -> false
          | _ -> true)
       | None -> true)
    | Clt ->
      (match order_float f1 f2 with
       | Some c0 ->
         (match c0 with
          | Lt -> true
          | _ -> false)
       | None -> false)
    | Cle ->
      (match order_float f1 f2 with
       | Some c0 ->
         (match c0 with
          | Gt -> false
          | _ -> true)
       | None -> false)
    | Cgt ->
      (match order_float f1 f2 with
       | Some c0 ->
         (match c0 with
          | Gt -> true
          | _ -> false)
       | None -> false)
    | Cge ->
      (match order_float f1 f2 with
       | Some c0 ->
         (match c0 with
          | Lt -> false
          | _ -> true)
       | None -> false)
  
  (** val bits_of_double : float -> Int64.int **)
  
  let bits_of_double f =
    Int64.repr (bits_of_b64 f)
  
  (** val double_of_bits : Int64.int -> float **)
  
  let double_of_bits b =
    b64_of_bits (Int64.unsigned b)
  
  (** val bits_of_single : float -> Int.int **)
  
  let bits_of_single f =
    Int.repr (bits_of_b32 (binary32offloat f))
  
  (** val single_of_bits : Int.int -> float **)
  
  let single_of_bits b =
    floatofbinary32 (b32_of_bits (Int.unsigned b))
  
  (** val from_words : Int.int -> Int.int -> float **)
  
  let from_words hi lo =
    double_of_bits
      (Int64.coq_or
        (Int64.shl (Int64.repr (Int.unsigned hi))
          (Int64.repr (Zpos (Coq_xO (Coq_xO (Coq_xO (Coq_xO (Coq_xO
            Coq_xH)))))))) (Int64.repr (Int.unsigned lo)))
  
  (** val ox8000_0000 : Int.int **)
  
  let ox8000_0000 =
    Int.repr Int.half_modulus
  
  (** val ox4330_0000 : Int.int **)
  
  let ox4330_0000 =
    Int.repr (Zpos (Coq_xO (Coq_xO (Coq_xO (Coq_xO (Coq_xO (Coq_xO (Coq_xO
      (Coq_xO (Coq_xO (Coq_xO (Coq_xO (Coq_xO (Coq_xO (Coq_xO (Coq_xO (Coq_xO
      (Coq_xO (Coq_xO (Coq_xO (Coq_xO (Coq_xI (Coq_xI (Coq_xO (Coq_xO (Coq_xI
      (Coq_xI (Coq_xO (Coq_xO (Coq_xO (Coq_xO
      Coq_xH)))))))))))))))))))))))))))))))
 end

