open BinNums
open BinPos
open Coqlib
open Datatypes
open Int0
open Machregs
open Ordered
open OrderedType
open Peano
open Registers

module OrderedReg : 
 sig 
  type t = positive
  
  val compare : t -> t -> positive coq_Compare
  
  val eq_dec : t -> t -> bool
 end

module OrderedRegReg : 
 sig 
  type t = positive * positive
  
  val compare : t -> t -> t coq_Compare
  
  val eq_dec : t -> t -> bool
 end

module OrderedMreg : 
 sig 
  type t = IndexedMreg.t
  
  val compare : t -> t -> t coq_Compare
  
  val eq_dec : t -> t -> bool
 end

module OrderedRegMreg : 
 sig 
  type t = positive * IndexedMreg.t
  
  val compare : t -> t -> t coq_Compare
  
  val eq_dec : t -> t -> bool
 end

module SetRegReg : 
 sig 
  module X' : 
   sig 
    type t = positive * positive
    
    val eq_dec : (positive * positive) -> (positive * positive) -> bool
    
    val compare :
      (positive * positive) -> (positive * positive) -> comparison
   end
  
  module MSet : 
   sig 
    module Raw : 
     sig 
      type elt = positive * positive
      
      type tree =
      | Leaf
      | Node of Z_as_Int.t * tree * (positive * positive) * tree
      
      val empty : tree
      
      val is_empty : tree -> bool
      
      val mem : (positive * positive) -> tree -> bool
      
      val min_elt : tree -> elt option
      
      val max_elt : tree -> elt option
      
      val choose : tree -> elt option
      
      val fold : (elt -> 'a1 -> 'a1) -> tree -> 'a1 -> 'a1
      
      val elements_aux :
        (positive * positive) list -> tree -> (positive * positive) list
      
      val elements : tree -> (positive * positive) list
      
      val rev_elements_aux :
        (positive * positive) list -> tree -> (positive * positive) list
      
      val rev_elements : tree -> (positive * positive) list
      
      val cardinal : tree -> nat
      
      val maxdepth : tree -> nat
      
      val mindepth : tree -> nat
      
      val for_all : (elt -> bool) -> tree -> bool
      
      val exists_ : (elt -> bool) -> tree -> bool
      
      type enumeration =
      | End
      | More of elt * tree * enumeration
      
      val cons : tree -> enumeration -> enumeration
      
      val compare_more :
        (positive * positive) -> (enumeration -> comparison) -> enumeration
        -> comparison
      
      val compare_cont :
        tree -> (enumeration -> comparison) -> enumeration -> comparison
      
      val compare_end : enumeration -> comparison
      
      val compare : tree -> tree -> comparison
      
      val equal : tree -> tree -> bool
      
      val subsetl : (tree -> bool) -> (positive * positive) -> tree -> bool
      
      val subsetr : (tree -> bool) -> (positive * positive) -> tree -> bool
      
      val subset : tree -> tree -> bool
      
      type t = tree
      
      val height : t -> Z_as_Int.t
      
      val singleton : (positive * positive) -> tree
      
      val create : t -> (positive * positive) -> t -> tree
      
      val assert_false : t -> (positive * positive) -> t -> tree
      
      val bal : t -> (positive * positive) -> t -> tree
      
      val add : (positive * positive) -> tree -> tree
      
      val join : tree -> elt -> t -> t
      
      val remove_min : tree -> elt -> t -> t * elt
      
      val merge : tree -> tree -> tree
      
      val remove : (positive * positive) -> tree -> tree
      
      val concat : tree -> tree -> tree
      
      type triple = { t_left : t; t_in : bool; t_right : t }
      
      val t_left : triple -> t
      
      val t_in : triple -> bool
      
      val t_right : triple -> t
      
      val split : (positive * positive) -> tree -> triple
      
      val inter : tree -> tree -> tree
      
      val diff : tree -> tree -> tree
      
      val union : tree -> tree -> tree
      
      val filter : (elt -> bool) -> tree -> tree
      
      val partition : (elt -> bool) -> t -> t * t
      
      val ltb_tree : (positive * positive) -> tree -> bool
      
      val gtb_tree : (positive * positive) -> tree -> bool
      
      val isok : tree -> bool
      
      module MX : 
       sig 
        module OrderTac : 
         sig 
          module OTF : 
           sig 
            type t = positive * positive
            
            val compare :
              (positive * positive) -> (positive * positive) -> comparison
            
            val eq_dec :
              (positive * positive) -> (positive * positive) -> bool
           end
          
          module TO : 
           sig 
            type t = positive * positive
            
            val compare :
              (positive * positive) -> (positive * positive) -> comparison
            
            val eq_dec :
              (positive * positive) -> (positive * positive) -> bool
           end
         end
        
        val eq_dec : (positive * positive) -> (positive * positive) -> bool
        
        val lt_dec : (positive * positive) -> (positive * positive) -> bool
        
        val eqb : (positive * positive) -> (positive * positive) -> bool
       end
      
      type coq_R_min_elt =
      | R_min_elt_0 of tree
      | R_min_elt_1 of tree * Z_as_Int.t * tree * (positive * positive)
         * tree
      | R_min_elt_2 of tree * Z_as_Int.t * tree * (positive * positive)
         * tree * Z_as_Int.t * tree * (positive * positive) * tree
         * elt option * coq_R_min_elt
      
      type coq_R_max_elt =
      | R_max_elt_0 of tree
      | R_max_elt_1 of tree * Z_as_Int.t * tree * (positive * positive)
         * tree
      | R_max_elt_2 of tree * Z_as_Int.t * tree * (positive * positive)
         * tree * Z_as_Int.t * tree * (positive * positive) * tree
         * elt option * coq_R_max_elt
      
      module L : 
       sig 
        module MO : 
         sig 
          module OrderTac : 
           sig 
            module OTF : 
             sig 
              type t = positive * positive
              
              val compare :
                (positive * positive) -> (positive * positive) -> comparison
              
              val eq_dec :
                (positive * positive) -> (positive * positive) -> bool
             end
            
            module TO : 
             sig 
              type t = positive * positive
              
              val compare :
                (positive * positive) -> (positive * positive) -> comparison
              
              val eq_dec :
                (positive * positive) -> (positive * positive) -> bool
             end
           end
          
          val eq_dec : (positive * positive) -> (positive * positive) -> bool
          
          val lt_dec : (positive * positive) -> (positive * positive) -> bool
          
          val eqb : (positive * positive) -> (positive * positive) -> bool
         end
       end
      
      val flatten_e : enumeration -> elt list
      
      type coq_R_bal =
      | R_bal_0 of t * (positive * positive) * t
      | R_bal_1 of t * (positive * positive) * t * Z_as_Int.t * tree
         * (positive * positive) * tree
      | R_bal_2 of t * (positive * positive) * t * Z_as_Int.t * tree
         * (positive * positive) * tree
      | R_bal_3 of t * (positive * positive) * t * Z_as_Int.t * tree
         * (positive * positive) * tree * Z_as_Int.t * tree
         * (positive * positive) * tree
      | R_bal_4 of t * (positive * positive) * t
      | R_bal_5 of t * (positive * positive) * t * Z_as_Int.t * tree
         * (positive * positive) * tree
      | R_bal_6 of t * (positive * positive) * t * Z_as_Int.t * tree
         * (positive * positive) * tree
      | R_bal_7 of t * (positive * positive) * t * Z_as_Int.t * tree
         * (positive * positive) * tree * Z_as_Int.t * tree
         * (positive * positive) * tree
      | R_bal_8 of t * (positive * positive) * t
      
      type coq_R_remove_min =
      | R_remove_min_0 of tree * elt * t
      | R_remove_min_1 of tree * elt * t * Z_as_Int.t * tree
         * (positive * positive) * tree * (t * elt) * coq_R_remove_min * 
         t * elt
      
      type coq_R_merge =
      | R_merge_0 of tree * tree
      | R_merge_1 of tree * tree * Z_as_Int.t * tree * (positive * positive)
         * tree
      | R_merge_2 of tree * tree * Z_as_Int.t * tree * (positive * positive)
         * tree * Z_as_Int.t * tree * (positive * positive) * tree * 
         t * elt
      
      type coq_R_concat =
      | R_concat_0 of tree * tree
      | R_concat_1 of tree * tree * Z_as_Int.t * tree * (positive * positive)
         * tree
      | R_concat_2 of tree * tree * Z_as_Int.t * tree * (positive * positive)
         * tree * Z_as_Int.t * tree * (positive * positive) * tree * 
         t * elt
      
      type coq_R_inter =
      | R_inter_0 of tree * tree
      | R_inter_1 of tree * tree * Z_as_Int.t * tree * (positive * positive)
         * tree
      | R_inter_2 of tree * tree * Z_as_Int.t * tree * (positive * positive)
         * tree * Z_as_Int.t * tree * (positive * positive) * tree * 
         t * bool * t * tree * coq_R_inter * tree * coq_R_inter
      | R_inter_3 of tree * tree * Z_as_Int.t * tree * (positive * positive)
         * tree * Z_as_Int.t * tree * (positive * positive) * tree * 
         t * bool * t * tree * coq_R_inter * tree * coq_R_inter
      
      type coq_R_diff =
      | R_diff_0 of tree * tree
      | R_diff_1 of tree * tree * Z_as_Int.t * tree * (positive * positive)
         * tree
      | R_diff_2 of tree * tree * Z_as_Int.t * tree * (positive * positive)
         * tree * Z_as_Int.t * tree * (positive * positive) * tree * 
         t * bool * t * tree * coq_R_diff * tree * coq_R_diff
      | R_diff_3 of tree * tree * Z_as_Int.t * tree * (positive * positive)
         * tree * Z_as_Int.t * tree * (positive * positive) * tree * 
         t * bool * t * tree * coq_R_diff * tree * coq_R_diff
      
      type coq_R_union =
      | R_union_0 of tree * tree
      | R_union_1 of tree * tree * Z_as_Int.t * tree * (positive * positive)
         * tree
      | R_union_2 of tree * tree * Z_as_Int.t * tree * (positive * positive)
         * tree * Z_as_Int.t * tree * (positive * positive) * tree * 
         t * bool * t * tree * coq_R_union * tree * coq_R_union
     end
    
    module E : 
     sig 
      type t = positive * positive
      
      val compare :
        (positive * positive) -> (positive * positive) -> comparison
      
      val eq_dec : (positive * positive) -> (positive * positive) -> bool
     end
    
    type elt = positive * positive
    
    type t_ =
      Raw.t
      (* singleton inductive, whose constructor was Mkt *)
    
    val this : t_ -> Raw.t
    
    type t = t_
    
    val mem : elt -> t -> bool
    
    val add : elt -> t -> t
    
    val remove : elt -> t -> t
    
    val singleton : elt -> t
    
    val union : t -> t -> t
    
    val inter : t -> t -> t
    
    val diff : t -> t -> t
    
    val equal : t -> t -> bool
    
    val subset : t -> t -> bool
    
    val empty : t
    
    val is_empty : t -> bool
    
    val elements : t -> elt list
    
    val choose : t -> elt option
    
    val fold : (elt -> 'a1 -> 'a1) -> t -> 'a1 -> 'a1
    
    val cardinal : t -> nat
    
    val filter : (elt -> bool) -> t -> t
    
    val for_all : (elt -> bool) -> t -> bool
    
    val exists_ : (elt -> bool) -> t -> bool
    
    val partition : (elt -> bool) -> t -> t * t
    
    val eq_dec : t -> t -> bool
    
    val compare : t -> t -> comparison
    
    val min_elt : t -> elt option
    
    val max_elt : t -> elt option
   end
  
  type elt = positive * positive
  
  type t = MSet.t
  
  val empty : t
  
  val is_empty : t -> bool
  
  val mem : elt -> t -> bool
  
  val add : elt -> t -> t
  
  val singleton : elt -> t
  
  val remove : elt -> t -> t
  
  val union : t -> t -> t
  
  val inter : t -> t -> t
  
  val diff : t -> t -> t
  
  val eq_dec : t -> t -> bool
  
  val equal : t -> t -> bool
  
  val subset : t -> t -> bool
  
  val fold : (elt -> 'a1 -> 'a1) -> t -> 'a1 -> 'a1
  
  val for_all : (elt -> bool) -> t -> bool
  
  val exists_ : (elt -> bool) -> t -> bool
  
  val filter : (elt -> bool) -> t -> t
  
  val partition : (elt -> bool) -> t -> t * t
  
  val cardinal : t -> nat
  
  val elements : t -> elt list
  
  val choose : t -> elt option
  
  module MF : 
   sig 
    val eqb : (positive * positive) -> (positive * positive) -> bool
   end
  
  val min_elt : t -> elt option
  
  val max_elt : t -> elt option
  
  val compare : t -> t -> t coq_Compare
  
  module E : 
   sig 
    type t = positive * positive
    
    val compare :
      (positive * positive) -> (positive * positive) -> (positive * positive)
      coq_Compare
    
    val eq_dec : (positive * positive) -> (positive * positive) -> bool
   end
 end

module SetRegMreg : 
 sig 
  module X' : 
   sig 
    type t = positive * IndexedMreg.t
    
    val eq_dec :
      (positive * IndexedMreg.t) -> (positive * IndexedMreg.t) -> bool
    
    val compare :
      (positive * IndexedMreg.t) -> (positive * IndexedMreg.t) -> comparison
   end
  
  module MSet : 
   sig 
    module Raw : 
     sig 
      type elt = positive * IndexedMreg.t
      
      type tree =
      | Leaf
      | Node of Z_as_Int.t * tree * (positive * IndexedMreg.t) * tree
      
      val empty : tree
      
      val is_empty : tree -> bool
      
      val mem : (positive * IndexedMreg.t) -> tree -> bool
      
      val min_elt : tree -> elt option
      
      val max_elt : tree -> elt option
      
      val choose : tree -> elt option
      
      val fold : (elt -> 'a1 -> 'a1) -> tree -> 'a1 -> 'a1
      
      val elements_aux :
        (positive * IndexedMreg.t) list -> tree -> (positive * IndexedMreg.t)
        list
      
      val elements : tree -> (positive * IndexedMreg.t) list
      
      val rev_elements_aux :
        (positive * IndexedMreg.t) list -> tree -> (positive * IndexedMreg.t)
        list
      
      val rev_elements : tree -> (positive * IndexedMreg.t) list
      
      val cardinal : tree -> nat
      
      val maxdepth : tree -> nat
      
      val mindepth : tree -> nat
      
      val for_all : (elt -> bool) -> tree -> bool
      
      val exists_ : (elt -> bool) -> tree -> bool
      
      type enumeration =
      | End
      | More of elt * tree * enumeration
      
      val cons : tree -> enumeration -> enumeration
      
      val compare_more :
        (positive * IndexedMreg.t) -> (enumeration -> comparison) ->
        enumeration -> comparison
      
      val compare_cont :
        tree -> (enumeration -> comparison) -> enumeration -> comparison
      
      val compare_end : enumeration -> comparison
      
      val compare : tree -> tree -> comparison
      
      val equal : tree -> tree -> bool
      
      val subsetl :
        (tree -> bool) -> (positive * IndexedMreg.t) -> tree -> bool
      
      val subsetr :
        (tree -> bool) -> (positive * IndexedMreg.t) -> tree -> bool
      
      val subset : tree -> tree -> bool
      
      type t = tree
      
      val height : t -> Z_as_Int.t
      
      val singleton : (positive * IndexedMreg.t) -> tree
      
      val create : t -> (positive * IndexedMreg.t) -> t -> tree
      
      val assert_false : t -> (positive * IndexedMreg.t) -> t -> tree
      
      val bal : t -> (positive * IndexedMreg.t) -> t -> tree
      
      val add : (positive * IndexedMreg.t) -> tree -> tree
      
      val join : tree -> elt -> t -> t
      
      val remove_min : tree -> elt -> t -> t * elt
      
      val merge : tree -> tree -> tree
      
      val remove : (positive * IndexedMreg.t) -> tree -> tree
      
      val concat : tree -> tree -> tree
      
      type triple = { t_left : t; t_in : bool; t_right : t }
      
      val t_left : triple -> t
      
      val t_in : triple -> bool
      
      val t_right : triple -> t
      
      val split : (positive * IndexedMreg.t) -> tree -> triple
      
      val inter : tree -> tree -> tree
      
      val diff : tree -> tree -> tree
      
      val union : tree -> tree -> tree
      
      val filter : (elt -> bool) -> tree -> tree
      
      val partition : (elt -> bool) -> t -> t * t
      
      val ltb_tree : (positive * IndexedMreg.t) -> tree -> bool
      
      val gtb_tree : (positive * IndexedMreg.t) -> tree -> bool
      
      val isok : tree -> bool
      
      module MX : 
       sig 
        module OrderTac : 
         sig 
          module OTF : 
           sig 
            type t = positive * IndexedMreg.t
            
            val compare :
              (positive * IndexedMreg.t) -> (positive * IndexedMreg.t) ->
              comparison
            
            val eq_dec :
              (positive * IndexedMreg.t) -> (positive * IndexedMreg.t) ->
              bool
           end
          
          module TO : 
           sig 
            type t = positive * IndexedMreg.t
            
            val compare :
              (positive * IndexedMreg.t) -> (positive * IndexedMreg.t) ->
              comparison
            
            val eq_dec :
              (positive * IndexedMreg.t) -> (positive * IndexedMreg.t) ->
              bool
           end
         end
        
        val eq_dec :
          (positive * IndexedMreg.t) -> (positive * IndexedMreg.t) -> bool
        
        val lt_dec :
          (positive * IndexedMreg.t) -> (positive * IndexedMreg.t) -> bool
        
        val eqb :
          (positive * IndexedMreg.t) -> (positive * IndexedMreg.t) -> bool
       end
      
      type coq_R_min_elt =
      | R_min_elt_0 of tree
      | R_min_elt_1 of tree * Z_as_Int.t * tree * (positive * IndexedMreg.t)
         * tree
      | R_min_elt_2 of tree * Z_as_Int.t * tree * (positive * IndexedMreg.t)
         * tree * Z_as_Int.t * tree * (positive * IndexedMreg.t) * tree
         * elt option * coq_R_min_elt
      
      type coq_R_max_elt =
      | R_max_elt_0 of tree
      | R_max_elt_1 of tree * Z_as_Int.t * tree * (positive * IndexedMreg.t)
         * tree
      | R_max_elt_2 of tree * Z_as_Int.t * tree * (positive * IndexedMreg.t)
         * tree * Z_as_Int.t * tree * (positive * IndexedMreg.t) * tree
         * elt option * coq_R_max_elt
      
      module L : 
       sig 
        module MO : 
         sig 
          module OrderTac : 
           sig 
            module OTF : 
             sig 
              type t = positive * IndexedMreg.t
              
              val compare :
                (positive * IndexedMreg.t) -> (positive * IndexedMreg.t) ->
                comparison
              
              val eq_dec :
                (positive * IndexedMreg.t) -> (positive * IndexedMreg.t) ->
                bool
             end
            
            module TO : 
             sig 
              type t = positive * IndexedMreg.t
              
              val compare :
                (positive * IndexedMreg.t) -> (positive * IndexedMreg.t) ->
                comparison
              
              val eq_dec :
                (positive * IndexedMreg.t) -> (positive * IndexedMreg.t) ->
                bool
             end
           end
          
          val eq_dec :
            (positive * IndexedMreg.t) -> (positive * IndexedMreg.t) -> bool
          
          val lt_dec :
            (positive * IndexedMreg.t) -> (positive * IndexedMreg.t) -> bool
          
          val eqb :
            (positive * IndexedMreg.t) -> (positive * IndexedMreg.t) -> bool
         end
       end
      
      val flatten_e : enumeration -> elt list
      
      type coq_R_bal =
      | R_bal_0 of t * (positive * IndexedMreg.t) * t
      | R_bal_1 of t * (positive * IndexedMreg.t) * t * Z_as_Int.t * 
         tree * (positive * IndexedMreg.t) * tree
      | R_bal_2 of t * (positive * IndexedMreg.t) * t * Z_as_Int.t * 
         tree * (positive * IndexedMreg.t) * tree
      | R_bal_3 of t * (positive * IndexedMreg.t) * t * Z_as_Int.t * 
         tree * (positive * IndexedMreg.t) * tree * Z_as_Int.t * tree
         * (positive * IndexedMreg.t) * tree
      | R_bal_4 of t * (positive * IndexedMreg.t) * t
      | R_bal_5 of t * (positive * IndexedMreg.t) * t * Z_as_Int.t * 
         tree * (positive * IndexedMreg.t) * tree
      | R_bal_6 of t * (positive * IndexedMreg.t) * t * Z_as_Int.t * 
         tree * (positive * IndexedMreg.t) * tree
      | R_bal_7 of t * (positive * IndexedMreg.t) * t * Z_as_Int.t * 
         tree * (positive * IndexedMreg.t) * tree * Z_as_Int.t * tree
         * (positive * IndexedMreg.t) * tree
      | R_bal_8 of t * (positive * IndexedMreg.t) * t
      
      type coq_R_remove_min =
      | R_remove_min_0 of tree * elt * t
      | R_remove_min_1 of tree * elt * t * Z_as_Int.t * tree
         * (positive * IndexedMreg.t) * tree * (t * elt) * coq_R_remove_min
         * t * elt
      
      type coq_R_merge =
      | R_merge_0 of tree * tree
      | R_merge_1 of tree * tree * Z_as_Int.t * tree
         * (positive * IndexedMreg.t) * tree
      | R_merge_2 of tree * tree * Z_as_Int.t * tree
         * (positive * IndexedMreg.t) * tree * Z_as_Int.t * tree
         * (positive * IndexedMreg.t) * tree * t * elt
      
      type coq_R_concat =
      | R_concat_0 of tree * tree
      | R_concat_1 of tree * tree * Z_as_Int.t * tree
         * (positive * IndexedMreg.t) * tree
      | R_concat_2 of tree * tree * Z_as_Int.t * tree
         * (positive * IndexedMreg.t) * tree * Z_as_Int.t * tree
         * (positive * IndexedMreg.t) * tree * t * elt
      
      type coq_R_inter =
      | R_inter_0 of tree * tree
      | R_inter_1 of tree * tree * Z_as_Int.t * tree
         * (positive * IndexedMreg.t) * tree
      | R_inter_2 of tree * tree * Z_as_Int.t * tree
         * (positive * IndexedMreg.t) * tree * Z_as_Int.t * tree
         * (positive * IndexedMreg.t) * tree * t * bool * t * tree
         * coq_R_inter * tree * coq_R_inter
      | R_inter_3 of tree * tree * Z_as_Int.t * tree
         * (positive * IndexedMreg.t) * tree * Z_as_Int.t * tree
         * (positive * IndexedMreg.t) * tree * t * bool * t * tree
         * coq_R_inter * tree * coq_R_inter
      
      type coq_R_diff =
      | R_diff_0 of tree * tree
      | R_diff_1 of tree * tree * Z_as_Int.t * tree
         * (positive * IndexedMreg.t) * tree
      | R_diff_2 of tree * tree * Z_as_Int.t * tree
         * (positive * IndexedMreg.t) * tree * Z_as_Int.t * tree
         * (positive * IndexedMreg.t) * tree * t * bool * t * tree
         * coq_R_diff * tree * coq_R_diff
      | R_diff_3 of tree * tree * Z_as_Int.t * tree
         * (positive * IndexedMreg.t) * tree * Z_as_Int.t * tree
         * (positive * IndexedMreg.t) * tree * t * bool * t * tree
         * coq_R_diff * tree * coq_R_diff
      
      type coq_R_union =
      | R_union_0 of tree * tree
      | R_union_1 of tree * tree * Z_as_Int.t * tree
         * (positive * IndexedMreg.t) * tree
      | R_union_2 of tree * tree * Z_as_Int.t * tree
         * (positive * IndexedMreg.t) * tree * Z_as_Int.t * tree
         * (positive * IndexedMreg.t) * tree * t * bool * t * tree
         * coq_R_union * tree * coq_R_union
     end
    
    module E : 
     sig 
      type t = positive * IndexedMreg.t
      
      val compare :
        (positive * IndexedMreg.t) -> (positive * IndexedMreg.t) ->
        comparison
      
      val eq_dec :
        (positive * IndexedMreg.t) -> (positive * IndexedMreg.t) -> bool
     end
    
    type elt = positive * IndexedMreg.t
    
    type t_ =
      Raw.t
      (* singleton inductive, whose constructor was Mkt *)
    
    val this : t_ -> Raw.t
    
    type t = t_
    
    val mem : elt -> t -> bool
    
    val add : elt -> t -> t
    
    val remove : elt -> t -> t
    
    val singleton : elt -> t
    
    val union : t -> t -> t
    
    val inter : t -> t -> t
    
    val diff : t -> t -> t
    
    val equal : t -> t -> bool
    
    val subset : t -> t -> bool
    
    val empty : t
    
    val is_empty : t -> bool
    
    val elements : t -> elt list
    
    val choose : t -> elt option
    
    val fold : (elt -> 'a1 -> 'a1) -> t -> 'a1 -> 'a1
    
    val cardinal : t -> nat
    
    val filter : (elt -> bool) -> t -> t
    
    val for_all : (elt -> bool) -> t -> bool
    
    val exists_ : (elt -> bool) -> t -> bool
    
    val partition : (elt -> bool) -> t -> t * t
    
    val eq_dec : t -> t -> bool
    
    val compare : t -> t -> comparison
    
    val min_elt : t -> elt option
    
    val max_elt : t -> elt option
   end
  
  type elt = positive * IndexedMreg.t
  
  type t = MSet.t
  
  val empty : t
  
  val is_empty : t -> bool
  
  val mem : elt -> t -> bool
  
  val add : elt -> t -> t
  
  val singleton : elt -> t
  
  val remove : elt -> t -> t
  
  val union : t -> t -> t
  
  val inter : t -> t -> t
  
  val diff : t -> t -> t
  
  val eq_dec : t -> t -> bool
  
  val equal : t -> t -> bool
  
  val subset : t -> t -> bool
  
  val fold : (elt -> 'a1 -> 'a1) -> t -> 'a1 -> 'a1
  
  val for_all : (elt -> bool) -> t -> bool
  
  val exists_ : (elt -> bool) -> t -> bool
  
  val filter : (elt -> bool) -> t -> t
  
  val partition : (elt -> bool) -> t -> t * t
  
  val cardinal : t -> nat
  
  val elements : t -> elt list
  
  val choose : t -> elt option
  
  module MF : 
   sig 
    val eqb :
      (positive * IndexedMreg.t) -> (positive * IndexedMreg.t) -> bool
   end
  
  val min_elt : t -> elt option
  
  val max_elt : t -> elt option
  
  val compare : t -> t -> t coq_Compare
  
  module E : 
   sig 
    type t = positive * IndexedMreg.t
    
    val compare :
      (positive * IndexedMreg.t) -> (positive * IndexedMreg.t) ->
      (positive * IndexedMreg.t) coq_Compare
    
    val eq_dec :
      (positive * IndexedMreg.t) -> (positive * IndexedMreg.t) -> bool
   end
 end

type graph = { interf_reg_reg : SetRegReg.t; interf_reg_mreg : SetRegMreg.t;
               pref_reg_reg : SetRegReg.t; pref_reg_mreg : SetRegMreg.t }

val interf_reg_reg : graph -> SetRegReg.t

val interf_reg_mreg : graph -> SetRegMreg.t

val pref_reg_reg : graph -> SetRegReg.t

val pref_reg_mreg : graph -> SetRegMreg.t

val empty_graph : graph

val ordered_pair : reg -> reg -> reg * reg

val add_interf : reg -> reg -> graph -> graph

val add_interf_mreg : reg -> mreg -> graph -> graph

val add_pref : reg -> reg -> graph -> graph

val add_pref_mreg : reg -> mreg -> graph -> graph

val add_intf2 : (reg * reg) -> Regset.t -> Regset.t

val add_intf1 : (reg * mreg) -> Regset.t -> Regset.t

val all_interf_regs : graph -> Regset.t

