open AST
open BinNums
open Datatypes
open Locations
open Maps
open Op

type node = positive

type instruction =
| Lnop of node
| Lop of operation * loc list * loc * node
| Lload of memory_chunk * addressing * loc list * loc * node
| Lstore of memory_chunk * addressing * loc list * loc * node
| Lcall of signature * (loc, ident) sum * loc list * loc * node
| Ltailcall of signature * (loc, ident) sum * loc list
| Lbuiltin of external_function * loc list * loc * node
| Lcond of condition * loc list * node * node
| Ljumptable of loc * node list
| Lreturn of loc option

type code = instruction PTree.t

type coq_function = { fn_sig : signature; fn_params : loc list;
                      fn_stacksize : coq_Z; fn_code : code;
                      fn_entrypoint : node }

let fn_sig :  coq_function -> signature =
  fun x  ->
    x.fn_sig

let fn_params :  coq_function -> loc list =
  fun x  ->
    x.fn_params

let fn_stacksize :  coq_function -> coq_Z =
  fun x  ->
    x.fn_stacksize

let fn_code :  coq_function -> code =
  fun x  ->
    x.fn_code

let fn_entrypoint :  coq_function -> node =
  fun x  ->
    x.fn_entrypoint

type fundef = coq_function AST.fundef

type program = (fundef, unit) AST.program

let successors_instr :  instruction -> node list =
    function
| Lnop s -> s :: []
| Lop (op, args, res, s) -> s :: []
| Lload (chunk, addr, args, dst, s) -> s :: []
| Lstore (chunk, addr, args, src, s) -> s :: []
| Lcall (sig0, ros, args, res, s) -> s :: []
| Lbuiltin (ef, args, res, s) -> s :: []
| Lcond (cond, args, ifso, ifnot) -> ifso :: (ifnot :: [])
| Ljumptable (arg, tbl) -> tbl
| _ -> []

let successors :  coq_function -> node list PTree.t =
  fun f  ->
  PTree.map1 successors_instr f.fn_code

