open AST
open BinNums
open Datatypes
open Locations
open Maps
open Op

type node = positive

type instruction =
| Lnop of node
| Lop of operation * loc list * loc * node
| Lload of memory_chunk * addressing * loc list * loc * node
| Lstore of memory_chunk * addressing * loc list * loc * node
| Lcall of signature * (loc, ident) sum * loc list * loc * node
| Ltailcall of signature * (loc, ident) sum * loc list
| Lbuiltin of external_function * loc list * loc * node
| Lcond of condition * loc list * node * node
| Ljumptable of loc * node list
| Lreturn of loc option

type code = instruction PTree.t

type coq_function = { fn_sig : signature; fn_params : loc list;
                      fn_stacksize : coq_Z; fn_code : code;
                      fn_entrypoint : node }

val fn_sig : coq_function -> signature

val fn_params : coq_function -> loc list

val fn_stacksize : coq_function -> coq_Z

val fn_code : coq_function -> code

val fn_entrypoint : coq_function -> node

type fundef = coq_function AST.fundef

type program = (fundef, unit) AST.program

val successors_instr : instruction -> node list

val successors : coq_function -> node list PTree.t

