open AST
open BinNums
open Datatypes
open Locations
open Op

type label = positive

type instruction =
| Lop of operation * loc list * loc
| Lload of memory_chunk * addressing * loc list * loc
| Lstore of memory_chunk * addressing * loc list * loc
| Lcall of signature * (loc, ident) sum * loc list * loc
| Ltailcall of signature * (loc, ident) sum * loc list
| Lbuiltin of external_function * loc list * loc
| Llabel of label
| Lgoto of label
| Lcond of condition * loc list * label
| Ljumptable of loc * label list
| Lreturn of loc option

type code = instruction list

type coq_function = { fn_sig : signature; fn_params : loc list;
                      fn_stacksize : coq_Z; fn_code : code }

let fn_sig :  coq_function -> signature =
  fun x  ->
    x.fn_sig

let fn_params :  coq_function -> loc list =
  fun x  ->
    x.fn_params

let fn_stacksize :  coq_function -> coq_Z =
  fun x  ->
    x.fn_stacksize

let fn_code :  coq_function -> code =
  fun x  ->
    x.fn_code

type fundef = coq_function AST.fundef

type program = (fundef, unit) AST.program

