open AST
open BinNums
open Bool
open Coqlib
open Datatypes
open Errors
open Int0
open LTL
open LTLin
open Lattice
open Maps
open Op
open Ordered
open OrderedType
open Peano

module DS = 
 struct 
  module L = 
   struct 
    type t = bool
    
    (** val beq : t -> t -> bool **)
    
    let beq =
      eqb
    
    (** val bot : bool **)
    
    let bot =
      false
    
    (** val top : bool **)
    
    let top =
      true
    
    (** val lub : t -> t -> bool **)
    
    let lub x y =
      (||) x y
   end
  
  (** val fixpoint :
      positive list PTree.t -> (positive -> L.t -> L.t) -> (positive * L.t)
      list -> L.t PMap.t option **)
  
  let fixpoint :
      positive list PTree.t -> (positive -> L.t -> L.t) -> (positive * L.t)
      list -> L.t PMap.t option =
    fun x y z ->
    failwith "AXIOM TO BE REALIZED"
 end

let reachable_aux :  LTL.coq_function -> bool PMap.t option =
  fun f  ->
  DS.fixpoint (successors f) (fun pc r -> r) ((f.fn_entrypoint, true) :: [])

let reachable :  LTL.coq_function -> bool PMap.t =
  fun f  ->
  match reachable_aux f with
  | Some rs -> rs
  | None -> PMap.init true

let enumerate_aux :  LTL.coq_function -> bool PMap.t -> node list =
    Linearizeaux.enumerate_aux

module Nodeset = 
 struct 
  module X' = 
   struct 
    type t = positive
    
    (** val eq_dec : positive -> positive -> bool **)
    
    let eq_dec =
      OrderedPositive.eq_dec
    
    (** val compare : positive -> positive -> comparison **)
    
    let compare x y =
      match OrderedPositive.compare x y with
      | LT -> Lt
      | EQ -> Eq
      | GT -> Gt
   end
  
  module MSet = 
   struct 
    module Raw = 
     struct 
      type elt = positive
      
      type tree =
      | Leaf
      | Node of Z_as_Int.t * tree * positive * tree
      
      (** val empty : tree **)
      
      let empty =
        Leaf
      
      (** val is_empty : tree -> bool **)
      
      let is_empty = function
      | Leaf -> true
      | Node (t1, t2, t3, t4) -> false
      
      (** val mem : positive -> tree -> bool **)
      
      let rec mem x = function
      | Leaf -> false
      | Node (t1, l, k, r) ->
        (match OrderedPositive.compare x k with
         | LT -> mem x l
         | EQ -> true
         | GT -> mem x r)
      
      (** val min_elt : tree -> elt option **)
      
      let rec min_elt = function
      | Leaf -> None
      | Node (t1, l, x, r) ->
        (match l with
         | Leaf -> Some x
         | Node (t2, t3, t4, t5) -> min_elt l)
      
      (** val max_elt : tree -> elt option **)
      
      let rec max_elt = function
      | Leaf -> None
      | Node (t1, l, x, r) ->
        (match r with
         | Leaf -> Some x
         | Node (t2, t3, t4, t5) -> max_elt r)
      
      (** val choose : tree -> elt option **)
      
      let choose =
        min_elt
      
      (** val fold : (elt -> 'a1 -> 'a1) -> tree -> 'a1 -> 'a1 **)
      
      let rec fold f t0 coqzz_base =
        match t0 with
        | Leaf -> coqzz_base
        | Node (t1, l, x, r) -> fold f r (f x (fold f l coqzz_base))
      
      (** val elements_aux : positive list -> tree -> positive list **)
      
      let rec elements_aux acc = function
      | Leaf -> acc
      | Node (t0, l, x, r) -> elements_aux (x :: (elements_aux acc r)) l
      
      (** val elements : tree -> positive list **)
      
      let elements =
        elements_aux []
      
      (** val rev_elements_aux : positive list -> tree -> positive list **)
      
      let rec rev_elements_aux acc = function
      | Leaf -> acc
      | Node (t0, l, x, r) ->
        rev_elements_aux (x :: (rev_elements_aux acc l)) r
      
      (** val rev_elements : tree -> positive list **)
      
      let rev_elements =
        rev_elements_aux []
      
      (** val cardinal : tree -> nat **)
      
      let rec cardinal = function
      | Leaf -> O
      | Node (t0, l, t1, r) -> S (plus (cardinal l) (cardinal r))
      
      (** val maxdepth : tree -> nat **)
      
      let rec maxdepth = function
      | Leaf -> O
      | Node (t0, l, t1, r) -> S (max (maxdepth l) (maxdepth r))
      
      (** val mindepth : tree -> nat **)
      
      let rec mindepth = function
      | Leaf -> O
      | Node (t0, l, t1, r) -> S (min (mindepth l) (mindepth r))
      
      (** val for_all : (elt -> bool) -> tree -> bool **)
      
      let rec for_all f = function
      | Leaf -> true
      | Node (t0, l, x, r) ->
        if if f x then for_all f l else false then for_all f r else false
      
      (** val exists_ : (elt -> bool) -> tree -> bool **)
      
      let rec exists_ f = function
      | Leaf -> false
      | Node (t0, l, x, r) ->
        if if f x then true else exists_ f l then true else exists_ f r
      
      type enumeration =
      | End
      | More of elt * tree * enumeration
      
      (** val cons : tree -> enumeration -> enumeration **)
      
      let rec cons s e =
        match s with
        | Leaf -> e
        | Node (t0, l, x, r) -> cons l (More (x, r, e))
      
      (** val compare_more :
          positive -> (enumeration -> comparison) -> enumeration ->
          comparison **)
      
      let compare_more x1 cont = function
      | End -> Gt
      | More (x2, r2, e3) ->
        (match OrderedPositive.compare x1 x2 with
         | LT -> Lt
         | EQ -> cont (cons r2 e3)
         | GT -> Gt)
      
      (** val compare_cont :
          tree -> (enumeration -> comparison) -> enumeration -> comparison **)
      
      let rec compare_cont s1 cont e2 =
        match s1 with
        | Leaf -> cont e2
        | Node (t0, l1, x1, r1) ->
          compare_cont l1 (compare_more x1 (compare_cont r1 cont)) e2
      
      (** val compare_end : enumeration -> comparison **)
      
      let compare_end = function
      | End -> Eq
      | More (e, t0, e0) -> Lt
      
      (** val compare : tree -> tree -> comparison **)
      
      let compare s1 s2 =
        compare_cont s1 compare_end (cons s2 End)
      
      (** val equal : tree -> tree -> bool **)
      
      let equal s1 s2 =
        match compare s1 s2 with
        | Eq -> true
        | _ -> false
      
      (** val subsetl : (tree -> bool) -> positive -> tree -> bool **)
      
      let rec subsetl subset_l1 x1 s2 = match s2 with
      | Leaf -> false
      | Node (t0, l2, x2, r2) ->
        (match OrderedPositive.compare x1 x2 with
         | LT -> subsetl subset_l1 x1 l2
         | EQ -> subset_l1 l2
         | GT -> if mem x1 r2 then subset_l1 s2 else false)
      
      (** val subsetr : (tree -> bool) -> positive -> tree -> bool **)
      
      let rec subsetr subset_r1 x1 s2 = match s2 with
      | Leaf -> false
      | Node (t0, l2, x2, r2) ->
        (match OrderedPositive.compare x1 x2 with
         | LT -> if mem x1 l2 then subset_r1 s2 else false
         | EQ -> subset_r1 r2
         | GT -> subsetr subset_r1 x1 r2)
      
      (** val subset : tree -> tree -> bool **)
      
      let rec subset s1 s2 =
        match s1 with
        | Leaf -> true
        | Node (t0, l1, x1, r1) ->
          (match s2 with
           | Leaf -> false
           | Node (t1, l2, x2, r2) ->
             (match OrderedPositive.compare x1 x2 with
              | LT ->
                if subsetl (subset l1) x1 l2 then subset r1 s2 else false
              | EQ -> if subset l1 l2 then subset r1 r2 else false
              | GT ->
                if subsetr (subset r1) x1 r2 then subset l1 s2 else false))
      
      type t = tree
      
      (** val height : t -> Z_as_Int.t **)
      
      let height = function
      | Leaf -> Z_as_Int._0
      | Node (h, t0, t1, t2) -> h
      
      (** val singleton : positive -> tree **)
      
      let singleton x =
        Node (Z_as_Int._1, Leaf, x, Leaf)
      
      (** val create : t -> positive -> t -> tree **)
      
      let create l x r =
        Node
          ((Z_as_Int.plus (Z_as_Int.max (height l) (height r)) Z_as_Int._1),
          l, x, r)
      
      (** val assert_false : t -> positive -> t -> tree **)
      
      let assert_false =
        create
      
      (** val bal : t -> positive -> t -> tree **)
      
      let bal l x r =
        let hl = height l in
        let hr = height r in
        if Z_as_Int.gt_le_dec hl (Z_as_Int.plus hr Z_as_Int._2)
        then (match l with
              | Leaf -> assert_false l x r
              | Node (t0, ll, lx, lr) ->
                if Z_as_Int.ge_lt_dec (height ll) (height lr)
                then create ll lx (create lr x r)
                else (match lr with
                      | Leaf -> assert_false l x r
                      | Node (t1, lrl, lrx, lrr) ->
                        create (create ll lx lrl) lrx (create lrr x r)))
        else if Z_as_Int.gt_le_dec hr (Z_as_Int.plus hl Z_as_Int._2)
             then (match r with
                   | Leaf -> assert_false l x r
                   | Node (t0, rl, rx, rr) ->
                     if Z_as_Int.ge_lt_dec (height rr) (height rl)
                     then create (create l x rl) rx rr
                     else (match rl with
                           | Leaf -> assert_false l x r
                           | Node (t1, rll, rlx, rlr) ->
                             create (create l x rll) rlx (create rlr rx rr)))
             else create l x r
      
      (** val add : positive -> tree -> tree **)
      
      let rec add x = function
      | Leaf -> Node (Z_as_Int._1, Leaf, x, Leaf)
      | Node (h, l, y, r) ->
        (match OrderedPositive.compare x y with
         | LT -> bal (add x l) y r
         | EQ -> Node (h, l, y, r)
         | GT -> bal l y (add x r))
      
      (** val join : tree -> elt -> t -> t **)
      
      let rec join l = match l with
      | Leaf -> add
      | Node (lh, ll, lx, lr) ->
        (fun x ->
          let rec join_aux r = match r with
          | Leaf -> add x l
          | Node (rh, rl, rx, rr) ->
            if Z_as_Int.gt_le_dec lh (Z_as_Int.plus rh Z_as_Int._2)
            then bal ll lx (join lr x r)
            else if Z_as_Int.gt_le_dec rh (Z_as_Int.plus lh Z_as_Int._2)
                 then bal (join_aux rl) rx rr
                 else create l x r
          in join_aux)
      
      (** val remove_min : tree -> elt -> t -> t * elt **)
      
      let rec remove_min l x r =
        match l with
        | Leaf -> (r, x)
        | Node (lh, ll, lx, lr) ->
          let (l', m) = remove_min ll lx lr in ((bal l' x r), m)
      
      (** val merge : tree -> tree -> tree **)
      
      let merge s1 s2 =
        match s1 with
        | Leaf -> s2
        | Node (t0, t1, t2, t3) ->
          (match s2 with
           | Leaf -> s1
           | Node (t4, l2, x2, r2) ->
             let (s2', m) = remove_min l2 x2 r2 in bal s1 m s2')
      
      (** val remove : positive -> tree -> tree **)
      
      let rec remove x = function
      | Leaf -> Leaf
      | Node (t0, l, y, r) ->
        (match OrderedPositive.compare x y with
         | LT -> bal (remove x l) y r
         | EQ -> merge l r
         | GT -> bal l y (remove x r))
      
      (** val concat : tree -> tree -> tree **)
      
      let concat s1 s2 =
        match s1 with
        | Leaf -> s2
        | Node (t0, t1, t2, t3) ->
          (match s2 with
           | Leaf -> s1
           | Node (t4, l2, x2, r2) ->
             let (s2', m) = remove_min l2 x2 r2 in join s1 m s2')
      
      type triple = { t_left : t; t_in : bool; t_right : t }
      
      (** val t_left : triple -> t **)
      
      let t_left x = x.t_left
      
      (** val t_in : triple -> bool **)
      
      let t_in x = x.t_in
      
      (** val t_right : triple -> t **)
      
      let t_right x = x.t_right
      
      (** val split : positive -> tree -> triple **)
      
      let rec split x = function
      | Leaf -> { t_left = Leaf; t_in = false; t_right = Leaf }
      | Node (t0, l, y, r) ->
        (match OrderedPositive.compare x y with
         | LT ->
           let { t_left = ll; t_in = b; t_right = rl } = split x l in
           { t_left = ll; t_in = b; t_right = (join rl y r) }
         | EQ -> { t_left = l; t_in = true; t_right = r }
         | GT ->
           let { t_left = rl; t_in = b; t_right = rr } = split x r in
           { t_left = (join l y rl); t_in = b; t_right = rr })
      
      (** val inter : tree -> tree -> tree **)
      
      let rec inter s1 s2 =
        match s1 with
        | Leaf -> Leaf
        | Node (t0, l1, x1, r1) ->
          (match s2 with
           | Leaf -> Leaf
           | Node (t1, t2, t3, t4) ->
             let { t_left = l2'; t_in = pres; t_right = r2' } = split x1 s2
             in
             if pres
             then join (inter l1 l2') x1 (inter r1 r2')
             else concat (inter l1 l2') (inter r1 r2'))
      
      (** val diff : tree -> tree -> tree **)
      
      let rec diff s1 s2 =
        match s1 with
        | Leaf -> Leaf
        | Node (t0, l1, x1, r1) ->
          (match s2 with
           | Leaf -> s1
           | Node (t1, t2, t3, t4) ->
             let { t_left = l2'; t_in = pres; t_right = r2' } = split x1 s2
             in
             if pres
             then concat (diff l1 l2') (diff r1 r2')
             else join (diff l1 l2') x1 (diff r1 r2'))
      
      (** val union : tree -> tree -> tree **)
      
      let rec union s1 s2 =
        match s1 with
        | Leaf -> s2
        | Node (t0, l1, x1, r1) ->
          (match s2 with
           | Leaf -> s1
           | Node (t1, t2, t3, t4) ->
             let { t_left = l2'; t_in = x; t_right = r2' } = split x1 s2 in
             join (union l1 l2') x1 (union r1 r2'))
      
      (** val filter : (elt -> bool) -> tree -> tree **)
      
      let rec filter f = function
      | Leaf -> Leaf
      | Node (t0, l, x, r) ->
        let l' = filter f l in
        let r' = filter f r in if f x then join l' x r' else concat l' r'
      
      (** val partition : (elt -> bool) -> t -> t * t **)
      
      let rec partition f = function
      | Leaf -> (Leaf, Leaf)
      | Node (t0, l, x, r) ->
        let (l1, l2) = partition f l in
        let (r1, r2) = partition f r in
        if f x
        then ((join l1 x r1), (concat l2 r2))
        else ((concat l1 r1), (join l2 x r2))
      
      (** val ltb_tree : positive -> tree -> bool **)
      
      let rec ltb_tree x = function
      | Leaf -> true
      | Node (t0, l, y, r) ->
        (match OrderedPositive.compare x y with
         | GT -> (&&) (ltb_tree x l) (ltb_tree x r)
         | _ -> false)
      
      (** val gtb_tree : positive -> tree -> bool **)
      
      let rec gtb_tree x = function
      | Leaf -> true
      | Node (t0, l, y, r) ->
        (match OrderedPositive.compare x y with
         | LT -> (&&) (gtb_tree x l) (gtb_tree x r)
         | _ -> false)
      
      (** val isok : tree -> bool **)
      
      let rec isok = function
      | Leaf -> true
      | Node (t0, l, x, r) ->
        (&&) ((&&) ((&&) (isok l) (isok r)) (ltb_tree x l)) (gtb_tree x r)
      
      module MX = 
       struct 
        module OrderTac = 
         struct 
          module OTF = 
           struct 
            type t = positive
            
            (** val compare : positive -> positive -> comparison **)
            
            let compare x y =
              match OrderedPositive.compare x y with
              | LT -> Lt
              | EQ -> Eq
              | GT -> Gt
            
            (** val eq_dec : positive -> positive -> bool **)
            
            let eq_dec =
              X'.eq_dec
           end
          
          module TO = 
           struct 
            type t = positive
            
            (** val compare : positive -> positive -> comparison **)
            
            let compare x y =
              match OrderedPositive.compare x y with
              | LT -> Lt
              | EQ -> Eq
              | GT -> Gt
            
            (** val eq_dec : positive -> positive -> bool **)
            
            let eq_dec =
              OTF.eq_dec
           end
         end
        
        (** val eq_dec : positive -> positive -> bool **)
        
        let eq_dec =
          X'.eq_dec
        
        (** val lt_dec : positive -> positive -> bool **)
        
        let lt_dec x y =
          let c =
            coq_CompSpec2Type x y
              (match OrderedPositive.compare x y with
               | LT -> Lt
               | EQ -> Eq
               | GT -> Gt)
          in
          (match c with
           | CompLtT -> true
           | _ -> false)
        
        (** val eqb : positive -> positive -> bool **)
        
        let eqb x y =
          if eq_dec x y then true else false
       end
      
      type coq_R_min_elt =
      | R_min_elt_0 of tree
      | R_min_elt_1 of tree * Z_as_Int.t * tree * positive * tree
      | R_min_elt_2 of tree * Z_as_Int.t * tree * positive * tree
         * Z_as_Int.t * tree * positive * tree * elt option * coq_R_min_elt
      
      type coq_R_max_elt =
      | R_max_elt_0 of tree
      | R_max_elt_1 of tree * Z_as_Int.t * tree * positive * tree
      | R_max_elt_2 of tree * Z_as_Int.t * tree * positive * tree
         * Z_as_Int.t * tree * positive * tree * elt option * coq_R_max_elt
      
      module L = 
       struct 
        module MO = 
         struct 
          module OrderTac = 
           struct 
            module OTF = 
             struct 
              type t = positive
              
              (** val compare : positive -> positive -> comparison **)
              
              let compare x y =
                match OrderedPositive.compare x y with
                | LT -> Lt
                | EQ -> Eq
                | GT -> Gt
              
              (** val eq_dec : positive -> positive -> bool **)
              
              let eq_dec =
                X'.eq_dec
             end
            
            module TO = 
             struct 
              type t = positive
              
              (** val compare : positive -> positive -> comparison **)
              
              let compare x y =
                match OrderedPositive.compare x y with
                | LT -> Lt
                | EQ -> Eq
                | GT -> Gt
              
              (** val eq_dec : positive -> positive -> bool **)
              
              let eq_dec =
                OTF.eq_dec
             end
           end
          
          (** val eq_dec : positive -> positive -> bool **)
          
          let eq_dec =
            X'.eq_dec
          
          (** val lt_dec : positive -> positive -> bool **)
          
          let lt_dec x y =
            let c =
              coq_CompSpec2Type x y
                (match OrderedPositive.compare x y with
                 | LT -> Lt
                 | EQ -> Eq
                 | GT -> Gt)
            in
            (match c with
             | CompLtT -> true
             | _ -> false)
          
          (** val eqb : positive -> positive -> bool **)
          
          let eqb x y =
            if eq_dec x y then true else false
         end
       end
      
      (** val flatten_e : enumeration -> elt list **)
      
      let rec flatten_e = function
      | End -> []
      | More (x, t0, r) -> x :: (app (elements t0) (flatten_e r))
      
      type coq_R_bal =
      | R_bal_0 of t * positive * t
      | R_bal_1 of t * positive * t * Z_as_Int.t * tree * positive * tree
      | R_bal_2 of t * positive * t * Z_as_Int.t * tree * positive * tree
      | R_bal_3 of t * positive * t * Z_as_Int.t * tree * positive * 
         tree * Z_as_Int.t * tree * positive * tree
      | R_bal_4 of t * positive * t
      | R_bal_5 of t * positive * t * Z_as_Int.t * tree * positive * tree
      | R_bal_6 of t * positive * t * Z_as_Int.t * tree * positive * tree
      | R_bal_7 of t * positive * t * Z_as_Int.t * tree * positive * 
         tree * Z_as_Int.t * tree * positive * tree
      | R_bal_8 of t * positive * t
      
      type coq_R_remove_min =
      | R_remove_min_0 of tree * elt * t
      | R_remove_min_1 of tree * elt * t * Z_as_Int.t * tree * positive
         * tree * (t * elt) * coq_R_remove_min * t * elt
      
      type coq_R_merge =
      | R_merge_0 of tree * tree
      | R_merge_1 of tree * tree * Z_as_Int.t * tree * positive * tree
      | R_merge_2 of tree * tree * Z_as_Int.t * tree * positive * tree
         * Z_as_Int.t * tree * positive * tree * t * elt
      
      type coq_R_concat =
      | R_concat_0 of tree * tree
      | R_concat_1 of tree * tree * Z_as_Int.t * tree * positive * tree
      | R_concat_2 of tree * tree * Z_as_Int.t * tree * positive * tree
         * Z_as_Int.t * tree * positive * tree * t * elt
      
      type coq_R_inter =
      | R_inter_0 of tree * tree
      | R_inter_1 of tree * tree * Z_as_Int.t * tree * positive * tree
      | R_inter_2 of tree * tree * Z_as_Int.t * tree * positive * tree
         * Z_as_Int.t * tree * positive * tree * t * bool * t * tree
         * coq_R_inter * tree * coq_R_inter
      | R_inter_3 of tree * tree * Z_as_Int.t * tree * positive * tree
         * Z_as_Int.t * tree * positive * tree * t * bool * t * tree
         * coq_R_inter * tree * coq_R_inter
      
      type coq_R_diff =
      | R_diff_0 of tree * tree
      | R_diff_1 of tree * tree * Z_as_Int.t * tree * positive * tree
      | R_diff_2 of tree * tree * Z_as_Int.t * tree * positive * tree
         * Z_as_Int.t * tree * positive * tree * t * bool * t * tree
         * coq_R_diff * tree * coq_R_diff
      | R_diff_3 of tree * tree * Z_as_Int.t * tree * positive * tree
         * Z_as_Int.t * tree * positive * tree * t * bool * t * tree
         * coq_R_diff * tree * coq_R_diff
      
      type coq_R_union =
      | R_union_0 of tree * tree
      | R_union_1 of tree * tree * Z_as_Int.t * tree * positive * tree
      | R_union_2 of tree * tree * Z_as_Int.t * tree * positive * tree
         * Z_as_Int.t * tree * positive * tree * t * bool * t * tree
         * coq_R_union * tree * coq_R_union
     end
    
    module E = 
     struct 
      type t = positive
      
      (** val compare : positive -> positive -> comparison **)
      
      let compare x y =
        match OrderedPositive.compare x y with
        | LT -> Lt
        | EQ -> Eq
        | GT -> Gt
      
      (** val eq_dec : positive -> positive -> bool **)
      
      let eq_dec =
        X'.eq_dec
     end
    
    type elt = positive
    
    type t_ =
      Raw.t
      (* singleton inductive, whose constructor was Mkt *)
    
    (** val this : t_ -> Raw.t **)
    
    let this t0 =
      t0
    
    type t = t_
    
    (** val mem : elt -> t -> bool **)
    
    let mem x s =
      Raw.mem x (this s)
    
    (** val add : elt -> t -> t **)
    
    let add x s =
      Raw.add x (this s)
    
    (** val remove : elt -> t -> t **)
    
    let remove x s =
      Raw.remove x (this s)
    
    (** val singleton : elt -> t **)
    
    let singleton x =
      Raw.singleton x
    
    (** val union : t -> t -> t **)
    
    let union s s' =
      Raw.union (this s) (this s')
    
    (** val inter : t -> t -> t **)
    
    let inter s s' =
      Raw.inter (this s) (this s')
    
    (** val diff : t -> t -> t **)
    
    let diff s s' =
      Raw.diff (this s) (this s')
    
    (** val equal : t -> t -> bool **)
    
    let equal s s' =
      Raw.equal (this s) (this s')
    
    (** val subset : t -> t -> bool **)
    
    let subset s s' =
      Raw.subset (this s) (this s')
    
    (** val empty : t **)
    
    let empty =
      Raw.empty
    
    (** val is_empty : t -> bool **)
    
    let is_empty s =
      Raw.is_empty (this s)
    
    (** val elements : t -> elt list **)
    
    let elements s =
      Raw.elements (this s)
    
    (** val choose : t -> elt option **)
    
    let choose s =
      Raw.choose (this s)
    
    (** val fold : (elt -> 'a1 -> 'a1) -> t -> 'a1 -> 'a1 **)
    
    let fold f s =
      Raw.fold f (this s)
    
    (** val cardinal : t -> nat **)
    
    let cardinal s =
      Raw.cardinal (this s)
    
    (** val filter : (elt -> bool) -> t -> t **)
    
    let filter f s =
      Raw.filter f (this s)
    
    (** val for_all : (elt -> bool) -> t -> bool **)
    
    let for_all f s =
      Raw.for_all f (this s)
    
    (** val exists_ : (elt -> bool) -> t -> bool **)
    
    let exists_ f s =
      Raw.exists_ f (this s)
    
    (** val partition : (elt -> bool) -> t -> t * t **)
    
    let partition f s =
      let p = Raw.partition f (this s) in ((fst p), (snd p))
    
    (** val eq_dec : t -> t -> bool **)
    
    let eq_dec s s' =
      let b = Raw.equal s s' in if b then true else false
    
    (** val compare : t -> t -> comparison **)
    
    let compare s s' =
      Raw.compare (this s) (this s')
    
    (** val min_elt : t -> elt option **)
    
    let min_elt s =
      Raw.min_elt (this s)
    
    (** val max_elt : t -> elt option **)
    
    let max_elt s =
      Raw.max_elt (this s)
   end
  
  type elt = positive
  
  type t = MSet.t
  
  (** val empty : t **)
  
  let empty =
    MSet.empty
  
  (** val is_empty : t -> bool **)
  
  let is_empty =
    MSet.is_empty
  
  (** val mem : elt -> t -> bool **)
  
  let mem =
    MSet.mem
  
  (** val add : elt -> t -> t **)
  
  let add =
    MSet.add
  
  (** val singleton : elt -> t **)
  
  let singleton =
    MSet.singleton
  
  (** val remove : elt -> t -> t **)
  
  let remove =
    MSet.remove
  
  (** val union : t -> t -> t **)
  
  let union =
    MSet.union
  
  (** val inter : t -> t -> t **)
  
  let inter =
    MSet.inter
  
  (** val diff : t -> t -> t **)
  
  let diff =
    MSet.diff
  
  (** val eq_dec : t -> t -> bool **)
  
  let eq_dec =
    MSet.eq_dec
  
  (** val equal : t -> t -> bool **)
  
  let equal =
    MSet.equal
  
  (** val subset : t -> t -> bool **)
  
  let subset =
    MSet.subset
  
  (** val fold : (elt -> 'a1 -> 'a1) -> t -> 'a1 -> 'a1 **)
  
  let fold x x0 x1 =
    MSet.fold x x0 x1
  
  (** val for_all : (elt -> bool) -> t -> bool **)
  
  let for_all =
    MSet.for_all
  
  (** val exists_ : (elt -> bool) -> t -> bool **)
  
  let exists_ =
    MSet.exists_
  
  (** val filter : (elt -> bool) -> t -> t **)
  
  let filter =
    MSet.filter
  
  (** val partition : (elt -> bool) -> t -> t * t **)
  
  let partition =
    MSet.partition
  
  (** val cardinal : t -> nat **)
  
  let cardinal =
    MSet.cardinal
  
  (** val elements : t -> elt list **)
  
  let elements =
    MSet.elements
  
  (** val choose : t -> elt option **)
  
  let choose =
    MSet.choose
  
  module MF = 
   struct 
    (** val eqb : positive -> positive -> bool **)
    
    let eqb x y =
      if MSet.E.eq_dec x y then true else false
   end
  
  (** val min_elt : t -> elt option **)
  
  let min_elt =
    MSet.min_elt
  
  (** val max_elt : t -> elt option **)
  
  let max_elt =
    MSet.max_elt
  
  (** val compare : t -> t -> t coq_Compare **)
  
  let compare s s' =
    let c = coq_CompSpec2Type s s' (MSet.compare s s') in
    (match c with
     | CompEqT -> EQ
     | CompLtT -> LT
     | CompGtT -> GT)
  
  module E = 
   struct 
    type t = positive
    
    (** val compare : positive -> positive -> positive coq_Compare **)
    
    let compare =
      OrderedPositive.compare
    
    (** val eq_dec : positive -> positive -> bool **)
    
    let eq_dec =
      OrderedPositive.eq_dec
   end
 end

let rec nodeset_of_list :  node list -> Nodeset.t -> Nodeset.t res =
  fun l s  ->
  match l with
  | [] -> OK s
  | hd :: tl ->
    if Nodeset.mem hd s
    then Error
           (msg
             ('L'::('i'::('n'::('e'::('a'::('r'::('i'::('z'::('e'::(':'::(' '::('d'::('u'::('p'::('l'::('i'::('c'::('a'::('t'::('e'::('s'::(' '::('i'::('n'::(' '::('e'::('n'::('u'::('m'::('e'::('r'::('a'::('t'::('i'::('o'::('n'::[])))))))))))))))))))))))))))))))))))))
    else nodeset_of_list tl (Nodeset.add hd s)

let check_reachable_aux :     bool PMap.t -> Nodeset.t -> bool -> node -> LTL.instruction -> bool =
  fun reach s ok pc i  ->
  if PMap.get pc reach then (&&) ok (Nodeset.mem pc s) else ok

let check_reachable :     LTL.coq_function -> bool PMap.t -> Nodeset.t -> bool =
  fun f reach s  ->
  PTree.fold (check_reachable_aux reach s) (LTL.fn_code f) true

let enumerate :  LTL.coq_function -> node list res =
  fun f  ->
  let reach = reachable f in
  let enum = enumerate_aux f reach in
  (match nodeset_of_list enum Nodeset.empty with
   | OK x ->
     if check_reachable f reach x
     then OK enum
     else Error
            (msg
              ('L'::('i'::('n'::('e'::('a'::('r'::('i'::('z'::('e'::(':'::(' '::('w'::('r'::('o'::('n'::('g'::(' '::('e'::('n'::('u'::('m'::('e'::('r'::('a'::('t'::('i'::('o'::('n'::[])))))))))))))))))))))))))))))
   | Error msg0 -> Error msg0)

let rec starts_with :  label -> code -> bool =
  fun lbl  ->
    function
| [] -> false
| i :: k' ->
  (match i with
   | Llabel lbl' -> if peq lbl lbl' then true else starts_with lbl k'
   | _ -> false)

let add_branch :  label -> code -> code =
  fun s k  ->
  if starts_with s k then k else (Lgoto s) :: k

let linearize_instr :  LTL.instruction -> code -> code =
  fun b k  ->
  match b with
  | Lnop s -> add_branch s k
  | LTL.Lop (op, args, res0, s) -> (Lop (op, args, res0)) :: (add_branch s k)
  | LTL.Lload (chunk, addr, args, dst, s) ->
    (Lload (chunk, addr, args, dst)) :: (add_branch s k)
  | LTL.Lstore (chunk, addr, args, src, s) ->
    (Lstore (chunk, addr, args, src)) :: (add_branch s k)
  | LTL.Lcall (sig0, ros, args, res0, s) ->
    (Lcall (sig0, ros, args, res0)) :: (add_branch s k)
  | LTL.Ltailcall (sig0, ros, args) -> (Ltailcall (sig0, ros, args)) :: k
  | LTL.Lbuiltin (ef, args, res0, s) ->
    (Lbuiltin (ef, args, res0)) :: (add_branch s k)
  | LTL.Lcond (cond, args, s1, s2) ->
    if starts_with s1 k
    then (Lcond ((negate_condition cond), args, s2)) :: (add_branch s1 k)
    else (Lcond (cond, args, s1)) :: (add_branch s2 k)
  | LTL.Ljumptable (arg, tbl) -> (Ljumptable (arg, tbl)) :: k
  | LTL.Lreturn or0 -> (Lreturn or0) :: k

let rec linearize_body :  LTL.coq_function -> node list -> code =
  fun f  ->
    function
| [] -> []
| pc :: rem ->
  (match PTree.get pc (LTL.fn_code f) with
   | Some b -> (Llabel pc) :: (linearize_instr b (linearize_body f rem))
   | None -> linearize_body f rem)

let transf_function :  LTL.coq_function -> coq_function res =
  fun f  ->
  match enumerate f with
  | OK x ->
    OK { fn_sig = LTL.fn_sig f; fn_params = LTL.fn_params f; fn_stacksize =
      LTL.fn_stacksize f; fn_code =
      (add_branch f.fn_entrypoint (linearize_body f x)) }
  | Error msg0 -> Error msg0

let transf_fundef :  LTL.fundef -> fundef res =
  fun f  ->
  transf_partial_fundef transf_function f

let transf_program :  LTL.program -> program res =
  fun p  ->
  transform_partial_program transf_fundef p

