open AST
open BinInt
open BinNums
open Coqlib
open Machregs

type slot =
| Local of coq_Z * typ
| Incoming of coq_Z * typ
| Outgoing of coq_Z * typ

let slot_type :  slot -> typ =
    function
| Local (ofs, ty) -> ty
| Incoming (ofs, ty) -> ty
| Outgoing (ofs, ty) -> ty

let slot_eq :  slot -> slot -> bool =
  fun p q ->
  let typ_eq = fun t1 t2 ->
    match t1 with
    | Tint ->
      (match t2 with
       | Tint -> true
       | Tfloat -> false)
    | Tfloat ->
      (match t2 with
       | Tint -> false
       | Tfloat -> true)
  in
  match p with
  | Local (x, x0) ->
    (match q with
     | Local (z0, t0) -> if zeq x z0 then typ_eq x0 t0 else false
     | _ -> false)
  | Incoming (x, x0) ->
    (match q with
     | Incoming (z0, t0) -> if zeq x z0 then typ_eq x0 t0 else false
     | _ -> false)
  | Outgoing (x, x0) ->
    (match q with
     | Outgoing (z0, t0) -> if zeq x z0 then typ_eq x0 t0 else false
     | _ -> false)

let typesize :  typ -> coq_Z =
    function
| Tint -> Zpos Coq_xH
| Tfloat -> Zpos (Coq_xO Coq_xH)

type loc =
| R of mreg
| S of slot

module Loc = 
 struct 
  (** val coq_type : loc -> typ **)
  
  let coq_type = function
  | R r -> mreg_type r
  | S s -> slot_type s
  
  (** val eq : loc -> loc -> bool **)
  
  let eq p q =
    match p with
    | R x ->
      (match q with
       | R m0 -> mreg_eq x m0
       | S s -> false)
    | S x ->
      (match q with
       | R m -> false
       | S s0 -> slot_eq x s0)
  
  (** val overlap_aux : typ -> coq_Z -> coq_Z -> bool **)
  
  let overlap_aux t1 d1 d2 =
    if zeq d1 d2
    then true
    else (match t1 with
          | Tint -> false
          | Tfloat -> if zeq (Z.add d1 (Zpos Coq_xH)) d2 then true else false)
  
  (** val overlap : loc -> loc -> bool **)
  
  let overlap l1 l2 =
    match l1 with
    | R m -> false
    | S s ->
      (match s with
       | Local (z, t) -> false
       | Incoming (d1, t1) ->
         (match l2 with
          | R m -> false
          | S s0 ->
            (match s0 with
             | Incoming (d2, t2) ->
               (||) (overlap_aux t1 d1 d2) (overlap_aux t2 d2 d1)
             | _ -> false))
       | Outgoing (d1, t1) ->
         (match l2 with
          | R m -> false
          | S s0 ->
            (match s0 with
             | Outgoing (d2, t2) ->
               (||) (overlap_aux t1 d1 d2) (overlap_aux t2 d2 d1)
             | _ -> false)))
  
  (** val diff_dec : loc -> loc -> bool **)
  
  let diff_dec l1 l2 =
    if eq l1 l2 then false else if overlap l1 l2 then false else true
 end

