open BinNums
open Coqlib
open Datatypes
open EquivDec
open List0

module PTree = 
 struct 
  type elt = positive
  
  (** val elt_eq : positive -> positive -> bool **)
  
  let elt_eq x y =
    peq x y
  
  type 'a tree =
  | Leaf
  | Node of 'a tree * 'a option * 'a tree
  
  type 'a t = 'a tree
  
  (** val empty : 'a1 t **)
  
  let empty =
    Leaf
  
  (** val get : positive -> 'a1 t -> 'a1 option **)
  
  let rec get i = function
  | Leaf -> None
  | Node (l, o, r) ->
    (match i with
     | Coq_xI ii -> get ii r
     | Coq_xO ii -> get ii l
     | Coq_xH -> o)
  
  (** val set : positive -> 'a1 -> 'a1 t -> 'a1 t **)
  
  let rec set i v = function
  | Leaf ->
    (match i with
     | Coq_xI ii -> Node (Leaf, None, (set ii v Leaf))
     | Coq_xO ii -> Node ((set ii v Leaf), None, Leaf)
     | Coq_xH -> Node (Leaf, (Some v), Leaf))
  | Node (l, o, r) ->
    (match i with
     | Coq_xI ii -> Node (l, o, (set ii v r))
     | Coq_xO ii -> Node ((set ii v l), o, r)
     | Coq_xH -> Node (l, (Some v), r))
  
  (** val remove : positive -> 'a1 t -> 'a1 t **)
  
  let rec remove i m =
    match i with
    | Coq_xI ii ->
      (match m with
       | Leaf -> Leaf
       | Node (l, o, r) ->
         (match l with
          | Leaf ->
            (match o with
             | Some a -> Node (l, o, (remove ii r))
             | None ->
               (match remove ii r with
                | Leaf -> Leaf
                | Node (t0, o0, t1) -> Node (Leaf, None, (Node (t0, o0, t1)))))
          | Node (t0, o0, t1) -> Node (l, o, (remove ii r))))
    | Coq_xO ii ->
      (match m with
       | Leaf -> Leaf
       | Node (l, o, r) ->
         (match o with
          | Some a -> Node ((remove ii l), o, r)
          | None ->
            (match r with
             | Leaf ->
               (match remove ii l with
                | Leaf -> Leaf
                | Node (t0, o0, t1) -> Node ((Node (t0, o0, t1)), None, Leaf))
             | Node (t0, o0, t1) -> Node ((remove ii l), o, r))))
    | Coq_xH ->
      (match m with
       | Leaf -> Leaf
       | Node (l, o, r) ->
         (match l with
          | Leaf ->
            (match r with
             | Leaf -> Leaf
             | Node (t0, o0, t1) -> Node (l, None, r))
          | Node (t0, o0, t1) -> Node (l, None, r)))
  
  (** val bempty : 'a1 t -> bool **)
  
  let rec bempty = function
  | Leaf -> true
  | Node (l, o, r) ->
    (match o with
     | Some a -> false
     | None -> (&&) (bempty l) (bempty r))
  
  (** val beq : ('a1 -> 'a1 -> bool) -> 'a1 t -> 'a1 t -> bool **)
  
  let rec beq : ('a1 -> 'a1 -> bool) -> 'a1 t -> 'a1 t -> bool =
    fun beqA m1 m2 ->
    match m1 with
    | Leaf -> bempty m2
    | Node (l1, o1, r1) ->
      (match m2 with
       | Leaf -> bempty m1
       | Node (l2, o2, r2) ->
         (&&)
           ((&&)
             (match o1 with
              | Some y1 ->
                (match o2 with
                 | Some y2 -> beqA y1 y2
                 | None -> false)
              | None ->
                (match o2 with
                 | Some a -> false
                 | None -> true)) (beq beqA l1 l2)) (beq beqA r1 r2))
  
  (** val append : positive -> positive -> positive **)
  
  let rec append i j =
    match i with
    | Coq_xI ii -> Coq_xI (append ii j)
    | Coq_xO ii -> Coq_xO (append ii j)
    | Coq_xH -> j
  
  (** val xmap : (positive -> 'a1 -> 'a2) -> 'a1 t -> positive -> 'a2 t **)
  
  let rec xmap f m i =
    match m with
    | Leaf -> Leaf
    | Node (l, o, r) ->
      Node ((xmap f l (append i (Coq_xO Coq_xH))), (option_map (f i) o),
        (xmap f r (append i (Coq_xI Coq_xH))))
  
  (** val map : (positive -> 'a1 -> 'a2) -> 'a1 t -> 'a2 t **)
  
  let map f m =
    xmap f m Coq_xH
  
  (** val map1 : ('a1 -> 'a2) -> 'a1 t -> 'a2 t **)
  
  let rec map1 f = function
  | Leaf -> Leaf
  | Node (l, o, r) -> Node ((map1 f l), (option_map f o), (map1 f r))
  
  (** val coq_Node' : 'a1 t -> 'a1 option -> 'a1 t -> 'a1 t **)
  
  let coq_Node' l x r =
    match l with
    | Leaf ->
      (match x with
       | Some a -> Node (l, x, r)
       | None ->
         (match r with
          | Leaf -> Leaf
          | Node (t0, o, t1) -> Node (l, x, r)))
    | Node (t0, o, t1) -> Node (l, x, r)
  
  (** val filter1 : ('a1 -> bool) -> 'a1 t -> 'a1 t **)
  
  let rec filter1 pred = function
  | Leaf -> Leaf
  | Node (l, o, r) ->
    let o' =
      match o with
      | Some x -> if pred x then o else None
      | None -> None
    in
    coq_Node' (filter1 pred l) o' (filter1 pred r)
  
  (** val xcombine_l :
      ('a1 option -> 'a2 option -> 'a3 option) -> 'a1 t -> 'a3 t **)
  
  let rec xcombine_l f = function
  | Leaf -> Leaf
  | Node (l, o, r) -> coq_Node' (xcombine_l f l) (f o None) (xcombine_l f r)
  
  (** val xcombine_r :
      ('a1 option -> 'a2 option -> 'a3 option) -> 'a2 t -> 'a3 t **)
  
  let rec xcombine_r f = function
  | Leaf -> Leaf
  | Node (l, o, r) -> coq_Node' (xcombine_r f l) (f None o) (xcombine_r f r)
  
  (** val combine :
      ('a1 option -> 'a2 option -> 'a3 option) -> 'a1 t -> 'a2 t -> 'a3 t **)
  
  let rec combine f m1 m2 =
    match m1 with
    | Leaf -> xcombine_r f m2
    | Node (l1, o1, r1) ->
      (match m2 with
       | Leaf -> xcombine_l f m1
       | Node (l2, o2, r2) ->
         coq_Node' (combine f l1 l2) (f o1 o2) (combine f r1 r2))
  
  (** val xelements : 'a1 t -> positive -> (positive * 'a1) list **)
  
  let rec xelements m i =
    match m with
    | Leaf -> []
    | Node (l, o, r) ->
      (match o with
       | Some x ->
         app (xelements l (append i (Coq_xO Coq_xH))) ((i,
           x) :: (xelements r (append i (Coq_xI Coq_xH))))
       | None ->
         app (xelements l (append i (Coq_xO Coq_xH)))
           (xelements r (append i (Coq_xI Coq_xH))))
  
  (** val elements : 'a1 t -> (positive * 'a1) list **)
  
  let elements m =
    xelements m Coq_xH
  
  (** val xget : positive -> positive -> 'a1 t -> 'a1 option **)
  
  let rec xget i j m =
    match i with
    | Coq_xI ii ->
      (match j with
       | Coq_xI jj -> xget ii jj m
       | Coq_xO p -> None
       | Coq_xH -> get i m)
    | Coq_xO ii ->
      (match j with
       | Coq_xI p -> None
       | Coq_xO jj -> xget ii jj m
       | Coq_xH -> get i m)
    | Coq_xH ->
      (match j with
       | Coq_xH -> get i m
       | _ -> None)
  
  (** val xkeys : 'a1 t -> positive -> positive list **)
  
  let xkeys m i =
    List0.map fst (xelements m i)
  
  (** val xfold :
      ('a2 -> positive -> 'a1 -> 'a2) -> positive -> 'a1 t -> 'a2 -> 'a2 **)
  
  let rec xfold f i m v =
    match m with
    | Leaf -> v
    | Node (l, o, r) ->
      (match o with
       | Some x ->
         let v1 = xfold f (append i (Coq_xO Coq_xH)) l v in
         let v2 = f v1 i x in xfold f (append i (Coq_xI Coq_xH)) r v2
       | None ->
         let v1 = xfold f (append i (Coq_xO Coq_xH)) l v in
         xfold f (append i (Coq_xI Coq_xH)) r v1)
  
  (** val fold : ('a2 -> positive -> 'a1 -> 'a2) -> 'a1 t -> 'a2 -> 'a2 **)
  
  let fold f m v =
    xfold f Coq_xH m v
 end

module PMap = 
 struct 
  type elt = positive
  
  (** val elt_eq : positive -> positive -> bool **)
  
  let elt_eq x y =
    peq x y
  
  type 'a t = 'a * 'a PTree.t
  
  (** val init : 'a1 -> 'a1 * 'a1 PTree.t **)
  
  let init : 'a1 -> 'a1 * 'a1 PTree.t =
    fun x ->
    (x, PTree.empty)
  
  (** val get : positive -> 'a1 t -> 'a1 **)
  
  let get i m =
    match PTree.get i (snd m) with
    | Some x -> x
    | None -> fst m
  
  (** val set : positive -> 'a1 -> 'a1 t -> 'a1 * 'a1 PTree.t **)
  
  let set : positive -> 'a1 -> 'a1 t -> 'a1 * 'a1 PTree.t =
    fun i x m ->
    ((fst m), (PTree.set i x (snd m)))
  
  (** val map : ('a1 -> 'a2) -> 'a1 t -> 'a2 t **)
  
  let map f m =
    ((f (fst m)), (PTree.map1 f (snd m)))
 end

module ZIndexed = 
 struct 
  type t = coq_Z
  
  (** val index : coq_Z -> positive **)
  
  let index = function
  | Z0 -> Coq_xH
  | Zpos p -> Coq_xO p
  | Zneg p -> Coq_xI p
  
  (** val eq : coq_Z -> coq_Z -> bool **)
  
  let eq x y =
    zeq x y
 end

module ZMap = 
 struct 
  type elt = ZIndexed.t
  
  (** val elt_eq : ZIndexed.t -> ZIndexed.t -> bool **)
  
  let elt_eq x y =
    ZIndexed.eq x y
  
  type 'x t = 'x PMap.t
  
  (** val init : 'a1 -> 'a1 * 'a1 PTree.t **)
  
  let init : 'a1 -> 'a1 * 'a1 PTree.t =
    fun x ->
    PMap.init x
  
  (** val get : ZIndexed.t -> 'a1 t -> 'a1 **)
  
  let get i m =
    PMap.get (ZIndexed.index i) m
  
  (** val set : ZIndexed.t -> 'a1 -> 'a1 t -> 'a1 * 'a1 PTree.t **)
  
  let set : ZIndexed.t -> 'a1 -> 'a1 t -> 'a1 * 'a1 PTree.t =
    fun i v m ->
    PMap.set (ZIndexed.index i) v m
  
  (** val map : ('a1 -> 'a2) -> 'a1 t -> 'a2 t **)
  
  let map : ('a1 -> 'a2) -> 'a1 t -> 'a2 t =
    fun f m ->
    PMap.map f m
 end

module PTree_Properties = 
 struct 
  (** val cardinal : 'a1 PTree.t -> nat **)
  
  let cardinal x =
    length (PTree.elements x)
  
  (** val for_all : 'a1 PTree.t -> (PTree.elt -> 'a1 -> bool) -> bool **)
  
  let for_all m f =
    PTree.fold (fun b x a -> (&&) b (f x a)) m true
  
  (** val exists_ : 'a1 PTree.t -> (PTree.elt -> 'a1 -> bool) -> bool **)
  
  let exists_ m f =
    PTree.fold (fun b x a -> (||) b (f x a)) m false
  
  (** val coq_Equal_dec :
      'a1 coq_EqDec -> 'a1 PTree.t -> 'a1 PTree.t -> bool **)
  
  let coq_Equal_dec eqAdec m1 m2 =
    let filtered_var =
      PTree.beq (fun a1 a2 -> proj_sumbool (equiv_dec eqAdec a1 a2)) m1 m2
    in
    if filtered_var then true else false
  
  (** val coq_Equal_EqDec : 'a1 coq_EqDec -> 'a1 PTree.t coq_EqDec **)
  
  let coq_Equal_EqDec eqAdec =
    coq_Equal_dec eqAdec
 end

