open AST
open BinInt
open BinNums
open Coqlib
open Datatypes
open Integers
open Maps
open Memdata
open Memtype
open Values

module Mem : 
 sig 
  type mem' = { mem_contents : memval ZMap.t ZMap.t;
                mem_access : (coq_Z -> perm_kind -> permission option) ZMap.t;
                nextblock : block }
  
  val mem_contents : mem' -> memval ZMap.t ZMap.t
  
  val mem_access : mem' -> (coq_Z -> perm_kind -> permission option) ZMap.t
  
  val nextblock : mem' -> block
  
  type mem = mem'
  
  val perm_order_dec : permission -> permission -> bool
  
  val perm_order'_dec : permission option -> permission -> bool
  
  val perm_dec : mem -> block -> coq_Z -> perm_kind -> permission -> bool
  
  val range_perm_dec :
    mem -> block -> coq_Z -> coq_Z -> perm_kind -> permission -> bool
  
  val valid_access_dec :
    mem -> memory_chunk -> block -> coq_Z -> permission -> bool
  
  val valid_pointer : mem -> block -> coq_Z -> bool
  
  val weak_valid_pointer : mem -> block -> coq_Z -> bool
  
  val empty : mem
  
  val nullptr : block
  
  val alloc : mem -> coq_Z -> coq_Z -> mem' * block
  
  val unchecked_free : mem -> block -> coq_Z -> coq_Z -> mem
  
  val free : mem -> block -> coq_Z -> coq_Z -> mem option
  
  val free_list : mem -> ((block * coq_Z) * coq_Z) list -> mem option
  
  val getN : nat -> coq_Z -> memval ZMap.t -> memval list
  
  val load : memory_chunk -> mem -> block -> coq_Z -> coq_val option
  
  val loadv : memory_chunk -> mem -> coq_val -> coq_val option
  
  val loadbytes : mem -> block -> coq_Z -> coq_Z -> memval list option
  
  val setN : memval list -> coq_Z -> memval ZMap.t -> memval ZMap.t
  
  val store : memory_chunk -> mem -> block -> coq_Z -> coq_val -> mem option
  
  val storev : memory_chunk -> mem -> coq_val -> coq_val -> mem option
  
  val storebytes : mem -> block -> coq_Z -> memval list -> mem option
  
  val drop_perm : mem -> block -> coq_Z -> coq_Z -> permission -> mem option
  
  val valid_access_store :
    mem -> memory_chunk -> block -> coq_Z -> coq_val -> mem
  
  val range_perm_storebytes : mem -> block -> coq_Z -> memval list -> mem
  
  val range_perm_free : mem -> block -> coq_Z -> coq_Z -> mem
  
  val range_perm_drop_2 : mem -> block -> coq_Z -> coq_Z -> permission -> mem
  
  val flat_inj : block -> meminj
 end

