open Datatypes
open List0

type 'reg moves = ('reg * 'reg) list

type 'reg state =
| State of 'reg moves * 'reg moves * 'reg moves

let rec split_move :     ('a1 -> 'a1 -> bool) -> 'a1 moves -> 'a1 -> (('a1 moves * 'a1) * 'a1
    moves) option =
  fun reg_eq m r  ->
  match m with
  | [] -> None
  | p :: tl ->
    let (s, d) = p in
    if reg_eq s r
    then Some (([], d), tl)
    else (match split_move reg_eq tl r with
          | Some p0 ->
            let (p1, after) = p0 in
            let (before, d') = p1 in Some ((((s, d) :: before), d'), after)
          | None -> None)

let rec is_last_source :  ('a1 -> 'a1 -> bool) -> 'a1 -> 'a1 moves -> bool =
  fun reg_eq r  ->
    function
| [] -> false
| p :: tl ->
  let (s, d) = p in
  (match tl with
   | [] -> if reg_eq s r then true else false
   | p0 :: l -> is_last_source reg_eq r tl)

let rec replace_last_source :  'a1 -> 'a1 moves -> 'a1 moves =
  fun r  ->
    function
| [] -> []
| s_d :: tl ->
  let (s, d) = s_d in
  (match tl with
   | [] -> (r, d) :: []
   | p :: l -> s_d :: (replace_last_source r tl))

let final_state :  'a1 state -> bool =
    function
| State (mu, sigma, tau) ->
  (match mu with
   | [] ->
     (match sigma with
      | [] -> true
      | p :: l -> false)
   | p :: l -> false)

let parmove_step :     ('a1 -> 'a1 -> bool) -> ('a1 -> 'a1) -> 'a1 state -> 'a1 state =
  fun reg_eq temp st  ->
    match st with
| State (t, sigma, l) ->
  (match t with
   | [] ->
     (match sigma with
      | [] -> st
      | p :: b ->
        let (s, d) = p in
        (match split_move reg_eq t d with
         | Some p0 ->
           let (p1, t2) = p0 in
           let (t1, r) = p1 in
           State ((app t1 t2), ((d, r) :: ((s, d) :: b)), l)
         | None ->
           (match b with
            | [] -> State (t, [], ((s, d) :: l))
            | p0 :: l0 ->
              if is_last_source reg_eq d b
              then State (t, (replace_last_source (temp d) b), ((s,
                     d) :: ((d, (temp d)) :: l)))
              else State (t, b, ((s, d) :: l)))))
   | p :: tl ->
     let (s, d) = p in
     (match sigma with
      | [] ->
        if reg_eq s d
        then State (tl, [], l)
        else State (tl, ((s, d) :: []), l)
      | p0 :: b ->
        let (s0, d0) = p0 in
        (match split_move reg_eq t d0 with
         | Some p1 ->
           let (p2, t2) = p1 in
           let (t1, r) = p2 in
           State ((app t1 t2), ((d0, r) :: ((s0, d0) :: b)), l)
         | None ->
           (match b with
            | [] -> State (t, [], ((s0, d0) :: l))
            | p1 :: l0 ->
              if is_last_source reg_eq d0 b
              then State (t, (replace_last_source (temp d0) b), ((s0,
                     d0) :: ((d0, (temp d0)) :: l)))
              else State (t, b, ((s0, d0) :: l))))))

let rec parmove_aux :     ('a1 -> 'a1 -> bool) -> ('a1 -> 'a1) -> 'a1 state -> 'a1 moves =
  fun reg_eq temp x  ->
  if final_state x
  then let (State (mu, sigma, tau)) = x in tau
  else parmove_aux reg_eq temp (parmove_step reg_eq temp x)

let parmove :     ('a1 -> 'a1 -> bool) -> ('a1 -> 'a1) -> 'a1 moves -> 'a1 moves =
  fun reg_eq temp mu  ->
  rev (parmove_aux reg_eq temp (State (mu, [], [])))

let parmove2 :     ('a1 -> 'a1 -> bool) -> ('a1 -> 'a1) -> 'a1 list -> 'a1 list -> 'a1 moves =
  fun reg_eq temp sl dl  ->
  parmove reg_eq temp (combine sl dl)

