open BinNums
open BinPos
open Coqlib
open Datatypes
open Iteration
open Maps

type node = positive

type graph = node list PTree.t

module Sort = 
 struct 
  (** val merge : positive list -> positive list -> positive list **)
  
  let rec merge l1 l2 =
    let rec merge_aux l3 =
      match l1 with
      | [] -> l3
      | a1 :: l1' ->
        (match l3 with
         | [] -> l1
         | a2 :: l2' ->
           if plt a2 a1 then a2 :: (merge_aux l2') else a1 :: (merge l1' l3))
    in merge_aux l2
  
  (** val merge_list_to_stack :
      positive list option list -> positive list -> positive list option list **)
  
  let rec merge_list_to_stack stack l =
    match stack with
    | [] -> (Some l) :: []
    | y :: stack' ->
      (match y with
       | Some l' -> None :: (merge_list_to_stack stack' (merge l' l))
       | None -> (Some l) :: stack')
  
  (** val merge_stack : positive list option list -> positive list **)
  
  let rec merge_stack = function
  | [] -> []
  | y :: stack' ->
    (match y with
     | Some l -> merge l (merge_stack stack')
     | None -> merge_stack stack')
  
  (** val iter_merge :
      positive list option list -> positive list -> positive list **)
  
  let rec iter_merge stack = function
  | [] -> merge_stack stack
  | a :: l' -> iter_merge (merge_list_to_stack stack (a :: [])) l'
  
  (** val sort : positive list -> positive list **)
  
  let sort : positive list -> positive list = fun x ->
    iter_merge [] x
   
  (** val flatten_stack : positive list option list -> positive list **)
  
  let rec flatten_stack : positive list option list -> positive list = function
  | [] -> []
  | o :: stack' ->
    (match o with
     | Some l -> app l (flatten_stack stack')
     | None -> flatten_stack stack')
 end

type state = { gr : graph; wrk : (node * node list) list;
               map : positive PTree.t; next : positive }

let gr :  state -> graph =
  fun x  ->
    x.gr

let wrk :  state -> (node * node list) list =
  fun x  ->
    x.wrk

let map :  state -> positive PTree.t =
  fun x  ->
    x.map

let next :  state -> positive =
  fun x  ->
    x.next

let init_state :  graph -> node -> state =
  fun g root  ->
  match PTree.get root g with
  | Some succs ->
    { gr = (PTree.remove root g); wrk = ((root, (Sort.sort succs)) :: []);
      map = PTree.empty; next = Coq_xH }
  | None -> { gr = g; wrk = []; map = PTree.empty; next = Coq_xH }

let transition :  state -> (positive PTree.t, state) sum =
  fun s  ->
  match s.wrk with
  | [] -> Coq_inl s.map
  | p :: l ->
    let (x, l0) = p in
    (match l0 with
     | [] ->
       Coq_inr { gr = s.gr; wrk = l; map = (PTree.set x s.next s.map); next =
         (Pos.succ s.next) }
     | y :: succs_x ->
       (match PTree.get y s.gr with
        | Some succs_y ->
          Coq_inr { gr = (PTree.remove y s.gr); wrk = ((y,
            (Sort.sort succs_y)) :: ((x, succs_x) :: l)); map = s.map; next =
            s.next }
        | None ->
          Coq_inr { gr = s.gr; wrk = ((x, succs_x) :: l); map = s.map; next =
            s.next }))

let postorder :  graph -> node -> positive PTree.t =
  fun g root  ->
  WfIter.iterate transition (init_state g root)

