open AST
open BinNums
open Datatypes
open Maps
open Op
open Registers

type node = positive

type instruction =
| Inop of node
| Iop of operation * reg list * reg * node
| Iload of memory_chunk * addressing * reg list * reg * node
| Istore of memory_chunk * addressing * reg list * reg * node
| Icall of signature * (reg, ident) sum * reg list * reg * node
| Itailcall of signature * (reg, ident) sum * reg list
| Ibuiltin of external_function * reg list * reg * node
| Icond of condition * reg list * node * node
| Ijumptable of reg * node list
| Ireturn of reg option

type code = instruction PTree.t

type coq_function = { fn_sig : signature; fn_params : reg list;
                      fn_stacksize : coq_Z; fn_code : code;
                      fn_entrypoint : node }

let fn_sig :  coq_function -> signature =
  fun x  ->
    x.fn_sig

let fn_params :  coq_function -> reg list =
  fun x  ->
    x.fn_params

let fn_stacksize :  coq_function -> coq_Z =
  fun x  ->
    x.fn_stacksize

let fn_code :  coq_function -> code =
  fun x  ->
    x.fn_code

let fn_entrypoint :  coq_function -> node =
  fun x  ->
    x.fn_entrypoint

type fundef = coq_function AST.fundef

type program = (fundef, unit) AST.program

let successors_instr :  instruction -> node list =
    function
| Inop s -> s :: []
| Iop (op, args, res, s) -> s :: []
| Iload (chunk, addr, args, dst, s) -> s :: []
| Istore (chunk, addr, args, src, s) -> s :: []
| Icall (sig0, ros, args, res, s) -> s :: []
| Ibuiltin (ef, args, res, s) -> s :: []
| Icond (cond, args, ifso, ifnot) -> ifso :: (ifnot :: [])
| Ijumptable (arg, tbl) -> tbl
| _ -> []

let successors :  coq_function -> node list PTree.t =
  fun f  ->
  PTree.map1 successors_instr f.fn_code

let transf_function :     (node -> instruction -> instruction) -> coq_function -> coq_function =
  fun transf f  ->
  { fn_sig = f.fn_sig; fn_params = f.fn_params; fn_stacksize =
    f.fn_stacksize; fn_code = (PTree.map transf f.fn_code); fn_entrypoint =
    f.fn_entrypoint }

