open AST
open BinNums
open Datatypes
open Maps
open Op
open Registers

type node = positive

type instruction =
| Inop of node
| Iop of operation * reg list * reg * node
| Iload of memory_chunk * addressing * reg list * reg * node
| Istore of memory_chunk * addressing * reg list * reg * node
| Icall of signature * (reg, ident) sum * reg list * reg * node
| Itailcall of signature * (reg, ident) sum * reg list
| Ibuiltin of external_function * reg list * reg * node
| Icond of condition * reg list * node * node
| Ijumptable of reg * node list
| Ireturn of reg option

type code = instruction PTree.t

type coq_function = { fn_sig : signature; fn_params : reg list;
                      fn_stacksize : coq_Z; fn_code : code;
                      fn_entrypoint : node }

val fn_sig : coq_function -> signature

val fn_params : coq_function -> reg list

val fn_stacksize : coq_function -> coq_Z

val fn_code : coq_function -> code

val fn_entrypoint : coq_function -> node

type fundef = coq_function AST.fundef

type program = (fundef, unit) AST.program

val successors_instr : instruction -> node list

val successors : coq_function -> node list PTree.t

val transf_function :
  (node -> instruction -> instruction) -> coq_function -> coq_function

