open AST
open BinNums
open CminorSel
open Datatypes
open Floats
open Integers
open Op

let symbol_is_external :  ident -> bool =
  fun id -> Configuration.system =
    "macosx" && C2C.atom_is_extern id

let addrsymbol :  ident -> Int.int -> expr =
  fun id ofs  ->
  if symbol_is_external id
  then if Int.eq ofs Int.zero
       then Eop ((Oindirectsymbol id), Enil)
       else Eop ((Olea (Aindexed ofs)), (Econs ((Eop ((Oindirectsymbol id),
              Enil)), Enil)))
  else Eop ((Olea (Aglobal (id, ofs))), Enil)

let addrstack :  Int.int -> expr =
  fun ofs  ->
  Eop ((Olea (Ainstack ofs)), Enil)

let notint :  expr -> expr =
  fun e  ->
  Eop ((Oxorimm Int.mone), (Econs (e, Enil)))

let offset_addressing :  addressing -> Int.int -> addressing =
  fun a ofs  ->
  match a with
  | Aindexed n -> Aindexed (Int.add n ofs)
  | Aindexed2 n -> Aindexed2 (Int.add n ofs)
  | Ascaled (sc, n) -> Ascaled (sc, (Int.add n ofs))
  | Aindexed2scaled (sc, n) -> Aindexed2scaled (sc, (Int.add n ofs))
  | Aglobal (id, n) -> Aglobal (id, (Int.add n ofs))
  | Abased (id, n) -> Abased (id, (Int.add n ofs))
  | Abasedscaled (sc, id, n) -> Abasedscaled (sc, id, (Int.add n ofs))
  | Ainstack n -> Ainstack (Int.add n ofs)

type addimm_cases =
| Coq_addimm_case1 of Int.int
| Coq_addimm_case2 of addressing * exprlist
| Coq_addimm_default of expr

let addimm_match :  expr -> addimm_cases =
    function
| Eop (o, args) ->
  (match o with
   | Ointconst m ->
     (match args with
      | Enil -> Coq_addimm_case1 m
      | Econs (e0, e1) ->
        Coq_addimm_default (Eop ((Ointconst m), (Econs (e0, e1)))))
   | Olea addr -> Coq_addimm_case2 (addr, args)
   | x -> Coq_addimm_default (Eop (x, args)))
| x -> Coq_addimm_default x

let addimm :  Int.int -> expr -> expr =
  fun n e  ->
  if Int.eq n Int.zero
  then e
  else (match addimm_match e with
        | Coq_addimm_case1 m -> Eop ((Ointconst (Int.add n m)), Enil)
        | Coq_addimm_case2 (addr, args) ->
          Eop ((Olea (offset_addressing addr n)), args)
        | Coq_addimm_default e0 ->
          Eop ((Olea (Aindexed n)), (Econs (e0, Enil))))

type add_cases =
| Coq_add_case1 of Int.int * expr
| Coq_add_case2 of expr * Int.int
| Coq_add_case3 of Int.int * expr * Int.int * expr
| Coq_add_case4 of Int.int * expr * Int.int * Int.int * expr
| Coq_add_case5 of Int.int * Int.int * expr * Int.int * expr
| Coq_add_case6 of Int.int * expr * ident * Int.int
| Coq_add_case7 of ident * Int.int * Int.int * expr
| Coq_add_case8 of Int.int * Int.int * expr * ident * Int.int
| Coq_add_case9 of ident * Int.int * Int.int * Int.int * expr
| Coq_add_case10 of Int.int * Int.int * expr * expr
| Coq_add_case11 of expr * Int.int * Int.int * expr
| Coq_add_case12 of Int.int * expr * expr
| Coq_add_case13 of expr * Int.int * expr
| Coq_add_default of expr * expr

let add_match :  expr -> expr -> add_cases =
  fun e1 e2  ->
  match e1 with
  | Eop (o, e) ->
    (match o with
     | Ointconst n1 ->
       (match e with
        | Enil -> Coq_add_case1 (n1, e2)
        | Econs (e0, e3) ->
          let e4 = Eop ((Ointconst n1), (Econs (e0, e3))) in
          (match e2 with
           | Eop (o0, e5) ->
             (match o0 with
              | Ointconst n2 ->
                (match e5 with
                 | Enil -> Coq_add_case2 (e4, n2)
                 | Econs (e6, e7) ->
                   Coq_add_default (e4, (Eop ((Ointconst n2), (Econs (e6,
                     e7))))))
              | Olea a ->
                (match a with
                 | Aindexed n ->
                   (match e5 with
                    | Enil ->
                      Coq_add_default (e4, (Eop ((Olea (Aindexed n)), Enil)))
                    | Econs (t2, e6) ->
                      (match e6 with
                       | Enil -> Coq_add_case13 (e4, n, t2)
                       | Econs (e7, e8) ->
                         Coq_add_default (e4, (Eop ((Olea (Aindexed n)),
                           (Econs (t2, (Econs (e7, e8)))))))))
                 | Ascaled (sc, n) ->
                   (match e5 with
                    | Enil ->
                      Coq_add_default (e4, (Eop ((Olea (Ascaled (sc, n))),
                        Enil)))
                    | Econs (t2, e6) ->
                      (match e6 with
                       | Enil -> Coq_add_case11 (e4, sc, n, t2)
                       | Econs (e7, e8) ->
                         Coq_add_default (e4, (Eop ((Olea (Ascaled (sc, n))),
                           (Econs (t2, (Econs (e7, e8)))))))))
                 | x -> Coq_add_default (e4, (Eop ((Olea x), e5))))
              | x -> Coq_add_default (e4, (Eop (x, e5))))
           | x -> Coq_add_default (e4, x)))
     | Olea a ->
       (match a with
        | Aindexed n ->
          (match e with
           | Enil ->
             let e3 = Eop ((Olea (Aindexed n)), Enil) in
             (match e2 with
              | Eop (o0, e0) ->
                (match o0 with
                 | Ointconst n2 ->
                   (match e0 with
                    | Enil -> Coq_add_case2 (e3, n2)
                    | Econs (e4, e5) ->
                      Coq_add_default (e3, (Eop ((Ointconst n2), (Econs (e4,
                        e5))))))
                 | Olea a0 ->
                   (match a0 with
                    | Aindexed n0 ->
                      (match e0 with
                       | Enil ->
                         Coq_add_default (e3, (Eop ((Olea (Aindexed n0)),
                           Enil)))
                       | Econs (t2, e4) ->
                         (match e4 with
                          | Enil -> Coq_add_case13 (e3, n0, t2)
                          | Econs (e5, e6) ->
                            Coq_add_default (e3, (Eop ((Olea (Aindexed n0)),
                              (Econs (t2, (Econs (e5, e6)))))))))
                    | Ascaled (sc, n0) ->
                      (match e0 with
                       | Enil ->
                         Coq_add_default (e3, (Eop ((Olea (Ascaled (sc,
                           n0))), Enil)))
                       | Econs (t2, e4) ->
                         (match e4 with
                          | Enil -> Coq_add_case11 (e3, sc, n0, t2)
                          | Econs (e5, e6) ->
                            Coq_add_default (e3, (Eop ((Olea (Ascaled (sc,
                              n0))), (Econs (t2, (Econs (e5, e6)))))))))
                    | x -> Coq_add_default (e3, (Eop ((Olea x), e0))))
                 | x -> Coq_add_default (e3, (Eop (x, e0))))
              | x -> Coq_add_default (e3, x))
           | Econs (t1, e0) ->
             (match e0 with
              | Enil ->
                (match e2 with
                 | Eop (o0, e3) ->
                   (match o0 with
                    | Ointconst n2 ->
                      (match e3 with
                       | Enil ->
                         Coq_add_case2 ((Eop ((Olea (Aindexed n)), (Econs
                           (t1, Enil)))), n2)
                       | Econs (e4, e5) ->
                         Coq_add_case12 (n, t1, (Eop ((Ointconst n2), (Econs
                           (e4, e5))))))
                    | Olea a0 ->
                      (match a0 with
                       | Aindexed n0 ->
                         (match e3 with
                          | Enil ->
                            Coq_add_case12 (n, t1, (Eop ((Olea (Aindexed
                              n0)), Enil)))
                          | Econs (t2, e4) ->
                            (match e4 with
                             | Enil -> Coq_add_case3 (n, t1, n0, t2)
                             | Econs (e5, e6) ->
                               Coq_add_case12 (n, t1, (Eop ((Olea (Aindexed
                                 n0)), (Econs (t2, (Econs (e5, e6)))))))))
                       | Ascaled (sc, n0) ->
                         (match e3 with
                          | Enil ->
                            Coq_add_case12 (n, t1, (Eop ((Olea (Ascaled (sc,
                              n0))), Enil)))
                          | Econs (t2, e4) ->
                            (match e4 with
                             | Enil -> Coq_add_case4 (n, t1, sc, n0, t2)
                             | Econs (e5, e6) ->
                               Coq_add_case12 (n, t1, (Eop ((Olea (Ascaled
                                 (sc, n0))), (Econs (t2, (Econs (e5,
                                 e6)))))))))
                       | Aglobal (id, ofs) ->
                         (match e3 with
                          | Enil -> Coq_add_case6 (n, t1, id, ofs)
                          | Econs (e4, e5) ->
                            Coq_add_case12 (n, t1, (Eop ((Olea (Aglobal (id,
                              ofs))), (Econs (e4, e5))))))
                       | x -> Coq_add_case12 (n, t1, (Eop ((Olea x), e3))))
                    | x -> Coq_add_case12 (n, t1, (Eop (x, e3))))
                 | x -> Coq_add_case12 (n, t1, x))
              | Econs (e3, e4) ->
                let e5 = Eop ((Olea (Aindexed n)), (Econs (t1, (Econs (e3,
                  e4)))))
                in
                (match e2 with
                 | Eop (o0, e6) ->
                   (match o0 with
                    | Ointconst n2 ->
                      (match e6 with
                       | Enil -> Coq_add_case2 (e5, n2)
                       | Econs (e7, e8) ->
                         Coq_add_default (e5, (Eop ((Ointconst n2), (Econs
                           (e7, e8))))))
                    | Olea a0 ->
                      (match a0 with
                       | Aindexed n0 ->
                         (match e6 with
                          | Enil ->
                            Coq_add_default (e5, (Eop ((Olea (Aindexed n0)),
                              Enil)))
                          | Econs (t2, e7) ->
                            (match e7 with
                             | Enil -> Coq_add_case13 (e5, n0, t2)
                             | Econs (e8, e9) ->
                               Coq_add_default (e5, (Eop ((Olea (Aindexed
                                 n0)), (Econs (t2, (Econs (e8, e9)))))))))
                       | Ascaled (sc, n0) ->
                         (match e6 with
                          | Enil ->
                            Coq_add_default (e5, (Eop ((Olea (Ascaled (sc,
                              n0))), Enil)))
                          | Econs (t2, e7) ->
                            (match e7 with
                             | Enil -> Coq_add_case11 (e5, sc, n0, t2)
                             | Econs (e8, e9) ->
                               Coq_add_default (e5, (Eop ((Olea (Ascaled (sc,
                                 n0))), (Econs (t2, (Econs (e8, e9)))))))))
                       | x -> Coq_add_default (e5, (Eop ((Olea x), e6))))
                    | x -> Coq_add_default (e5, (Eop (x, e6))))
                 | x -> Coq_add_default (e5, x))))
        | Ascaled (sc, n) ->
          (match e with
           | Enil ->
             let e3 = Eop ((Olea (Ascaled (sc, n))), Enil) in
             (match e2 with
              | Eop (o0, e0) ->
                (match o0 with
                 | Ointconst n2 ->
                   (match e0 with
                    | Enil -> Coq_add_case2 (e3, n2)
                    | Econs (e4, e5) ->
                      Coq_add_default (e3, (Eop ((Ointconst n2), (Econs (e4,
                        e5))))))
                 | Olea a0 ->
                   (match a0 with
                    | Aindexed n0 ->
                      (match e0 with
                       | Enil ->
                         Coq_add_default (e3, (Eop ((Olea (Aindexed n0)),
                           Enil)))
                       | Econs (t2, e4) ->
                         (match e4 with
                          | Enil -> Coq_add_case13 (e3, n0, t2)
                          | Econs (e5, e6) ->
                            Coq_add_default (e3, (Eop ((Olea (Aindexed n0)),
                              (Econs (t2, (Econs (e5, e6)))))))))
                    | Ascaled (sc0, n0) ->
                      (match e0 with
                       | Enil ->
                         Coq_add_default (e3, (Eop ((Olea (Ascaled (sc0,
                           n0))), Enil)))
                       | Econs (t2, e4) ->
                         (match e4 with
                          | Enil -> Coq_add_case11 (e3, sc0, n0, t2)
                          | Econs (e5, e6) ->
                            Coq_add_default (e3, (Eop ((Olea (Ascaled (sc0,
                              n0))), (Econs (t2, (Econs (e5, e6)))))))))
                    | x -> Coq_add_default (e3, (Eop ((Olea x), e0))))
                 | x -> Coq_add_default (e3, (Eop (x, e0))))
              | x -> Coq_add_default (e3, x))
           | Econs (t1, e0) ->
             (match e0 with
              | Enil ->
                (match e2 with
                 | Eop (o0, e3) ->
                   (match o0 with
                    | Ointconst n2 ->
                      (match e3 with
                       | Enil ->
                         Coq_add_case2 ((Eop ((Olea (Ascaled (sc, n))),
                           (Econs (t1, Enil)))), n2)
                       | Econs (e4, e5) ->
                         Coq_add_case10 (sc, n, t1, (Eop ((Ointconst n2),
                           (Econs (e4, e5))))))
                    | Olea a0 ->
                      (match a0 with
                       | Aindexed n0 ->
                         (match e3 with
                          | Enil ->
                            Coq_add_case10 (sc, n, t1, (Eop ((Olea (Aindexed
                              n0)), Enil)))
                          | Econs (t2, e4) ->
                            (match e4 with
                             | Enil -> Coq_add_case5 (sc, n, t1, n0, t2)
                             | Econs (e5, e6) ->
                               Coq_add_case10 (sc, n, t1, (Eop ((Olea
                                 (Aindexed n0)), (Econs (t2, (Econs (e5,
                                 e6)))))))))
                       | Aglobal (id, ofs) ->
                         (match e3 with
                          | Enil -> Coq_add_case8 (sc, n, t1, id, ofs)
                          | Econs (e4, e5) ->
                            Coq_add_case10 (sc, n, t1, (Eop ((Olea (Aglobal
                              (id, ofs))), (Econs (e4, e5))))))
                       | x ->
                         Coq_add_case10 (sc, n, t1, (Eop ((Olea x), e3))))
                    | x -> Coq_add_case10 (sc, n, t1, (Eop (x, e3))))
                 | x -> Coq_add_case10 (sc, n, t1, x))
              | Econs (e3, e4) ->
                let e5 = Eop ((Olea (Ascaled (sc, n))), (Econs (t1, (Econs
                  (e3, e4)))))
                in
                (match e2 with
                 | Eop (o0, e6) ->
                   (match o0 with
                    | Ointconst n2 ->
                      (match e6 with
                       | Enil -> Coq_add_case2 (e5, n2)
                       | Econs (e7, e8) ->
                         Coq_add_default (e5, (Eop ((Ointconst n2), (Econs
                           (e7, e8))))))
                    | Olea a0 ->
                      (match a0 with
                       | Aindexed n0 ->
                         (match e6 with
                          | Enil ->
                            Coq_add_default (e5, (Eop ((Olea (Aindexed n0)),
                              Enil)))
                          | Econs (t2, e7) ->
                            (match e7 with
                             | Enil -> Coq_add_case13 (e5, n0, t2)
                             | Econs (e8, e9) ->
                               Coq_add_default (e5, (Eop ((Olea (Aindexed
                                 n0)), (Econs (t2, (Econs (e8, e9)))))))))
                       | Ascaled (sc0, n0) ->
                         (match e6 with
                          | Enil ->
                            Coq_add_default (e5, (Eop ((Olea (Ascaled (sc0,
                              n0))), Enil)))
                          | Econs (t2, e7) ->
                            (match e7 with
                             | Enil -> Coq_add_case11 (e5, sc0, n0, t2)
                             | Econs (e8, e9) ->
                               Coq_add_default (e5, (Eop ((Olea (Ascaled
                                 (sc0, n0))), (Econs (t2, (Econs (e8,
                                 e9)))))))))
                       | x -> Coq_add_default (e5, (Eop ((Olea x), e6))))
                    | x -> Coq_add_default (e5, (Eop (x, e6))))
                 | x -> Coq_add_default (e5, x))))
        | Aglobal (id, ofs) ->
          (match e with
           | Enil ->
             let e3 = Eop ((Olea (Aglobal (id, ofs))), Enil) in
             (match e2 with
              | Eop (o0, e0) ->
                (match o0 with
                 | Ointconst n2 ->
                   (match e0 with
                    | Enil -> Coq_add_case2 (e3, n2)
                    | Econs (e4, e5) ->
                      Coq_add_default (e3, (Eop ((Ointconst n2), (Econs (e4,
                        e5))))))
                 | Olea a0 ->
                   (match a0 with
                    | Aindexed n ->
                      (match e0 with
                       | Enil ->
                         Coq_add_default (e3, (Eop ((Olea (Aindexed n)),
                           Enil)))
                       | Econs (t2, e4) ->
                         (match e4 with
                          | Enil -> Coq_add_case7 (id, ofs, n, t2)
                          | Econs (e5, e6) ->
                            Coq_add_default (e3, (Eop ((Olea (Aindexed n)),
                              (Econs (t2, (Econs (e5, e6)))))))))
                    | Ascaled (sc, n) ->
                      (match e0 with
                       | Enil ->
                         Coq_add_default (e3, (Eop ((Olea (Ascaled (sc, n))),
                           Enil)))
                       | Econs (t2, e4) ->
                         (match e4 with
                          | Enil -> Coq_add_case9 (id, ofs, sc, n, t2)
                          | Econs (e5, e6) ->
                            Coq_add_default (e3, (Eop ((Olea (Ascaled (sc,
                              n))), (Econs (t2, (Econs (e5, e6)))))))))
                    | x -> Coq_add_default (e3, (Eop ((Olea x), e0))))
                 | x -> Coq_add_default (e3, (Eop (x, e0))))
              | x -> Coq_add_default (e3, x))
           | Econs (e0, e3) ->
             let e4 = Eop ((Olea (Aglobal (id, ofs))), (Econs (e0, e3))) in
             (match e2 with
              | Eop (o0, e5) ->
                (match o0 with
                 | Ointconst n2 ->
                   (match e5 with
                    | Enil -> Coq_add_case2 (e4, n2)
                    | Econs (e6, e7) ->
                      Coq_add_default (e4, (Eop ((Ointconst n2), (Econs (e6,
                        e7))))))
                 | Olea a0 ->
                   (match a0 with
                    | Aindexed n ->
                      (match e5 with
                       | Enil ->
                         Coq_add_default (e4, (Eop ((Olea (Aindexed n)),
                           Enil)))
                       | Econs (t2, e6) ->
                         (match e6 with
                          | Enil -> Coq_add_case13 (e4, n, t2)
                          | Econs (e7, e8) ->
                            Coq_add_default (e4, (Eop ((Olea (Aindexed n)),
                              (Econs (t2, (Econs (e7, e8)))))))))
                    | Ascaled (sc, n) ->
                      (match e5 with
                       | Enil ->
                         Coq_add_default (e4, (Eop ((Olea (Ascaled (sc, n))),
                           Enil)))
                       | Econs (t2, e6) ->
                         (match e6 with
                          | Enil -> Coq_add_case11 (e4, sc, n, t2)
                          | Econs (e7, e8) ->
                            Coq_add_default (e4, (Eop ((Olea (Ascaled (sc,
                              n))), (Econs (t2, (Econs (e7, e8)))))))))
                    | x -> Coq_add_default (e4, (Eop ((Olea x), e5))))
                 | x -> Coq_add_default (e4, (Eop (x, e5))))
              | x -> Coq_add_default (e4, x)))
        | x ->
          let e3 = Eop ((Olea x), e) in
          (match e2 with
           | Eop (o0, e0) ->
             (match o0 with
              | Ointconst n2 ->
                (match e0 with
                 | Enil -> Coq_add_case2 (e3, n2)
                 | Econs (e4, e5) ->
                   Coq_add_default (e3, (Eop ((Ointconst n2), (Econs (e4,
                     e5))))))
              | Olea a0 ->
                (match a0 with
                 | Aindexed n ->
                   (match e0 with
                    | Enil ->
                      Coq_add_default (e3, (Eop ((Olea (Aindexed n)), Enil)))
                    | Econs (t2, e4) ->
                      (match e4 with
                       | Enil -> Coq_add_case13 (e3, n, t2)
                       | Econs (e5, e6) ->
                         Coq_add_default (e3, (Eop ((Olea (Aindexed n)),
                           (Econs (t2, (Econs (e5, e6)))))))))
                 | Ascaled (sc, n) ->
                   (match e0 with
                    | Enil ->
                      Coq_add_default (e3, (Eop ((Olea (Ascaled (sc, n))),
                        Enil)))
                    | Econs (t2, e4) ->
                      (match e4 with
                       | Enil -> Coq_add_case11 (e3, sc, n, t2)
                       | Econs (e5, e6) ->
                         Coq_add_default (e3, (Eop ((Olea (Ascaled (sc, n))),
                           (Econs (t2, (Econs (e5, e6)))))))))
                 | x0 -> Coq_add_default (e3, (Eop ((Olea x0), e0))))
              | x0 -> Coq_add_default (e3, (Eop (x0, e0))))
           | x0 -> Coq_add_default (e3, x0)))
     | x ->
       let e3 = Eop (x, e) in
       (match e2 with
        | Eop (o0, e0) ->
          (match o0 with
           | Ointconst n2 ->
             (match e0 with
              | Enil -> Coq_add_case2 (e3, n2)
              | Econs (e4, e5) ->
                Coq_add_default (e3, (Eop ((Ointconst n2), (Econs (e4,
                  e5))))))
           | Olea a ->
             (match a with
              | Aindexed n ->
                (match e0 with
                 | Enil ->
                   Coq_add_default (e3, (Eop ((Olea (Aindexed n)), Enil)))
                 | Econs (t2, e4) ->
                   (match e4 with
                    | Enil -> Coq_add_case13 (e3, n, t2)
                    | Econs (e5, e6) ->
                      Coq_add_default (e3, (Eop ((Olea (Aindexed n)), (Econs
                        (t2, (Econs (e5, e6)))))))))
              | Ascaled (sc, n) ->
                (match e0 with
                 | Enil ->
                   Coq_add_default (e3, (Eop ((Olea (Ascaled (sc, n))),
                     Enil)))
                 | Econs (t2, e4) ->
                   (match e4 with
                    | Enil -> Coq_add_case11 (e3, sc, n, t2)
                    | Econs (e5, e6) ->
                      Coq_add_default (e3, (Eop ((Olea (Ascaled (sc, n))),
                        (Econs (t2, (Econs (e5, e6)))))))))
              | x0 -> Coq_add_default (e3, (Eop ((Olea x0), e0))))
           | x0 -> Coq_add_default (e3, (Eop (x0, e0))))
        | x0 -> Coq_add_default (e3, x0)))
  | x ->
    (match e2 with
     | Eop (o, e) ->
       (match o with
        | Ointconst n2 ->
          (match e with
           | Enil -> Coq_add_case2 (x, n2)
           | Econs (e0, e3) ->
             Coq_add_default (x, (Eop ((Ointconst n2), (Econs (e0, e3))))))
        | Olea a ->
          (match a with
           | Aindexed n ->
             (match e with
              | Enil ->
                Coq_add_default (x, (Eop ((Olea (Aindexed n)), Enil)))
              | Econs (t2, e0) ->
                (match e0 with
                 | Enil -> Coq_add_case13 (x, n, t2)
                 | Econs (e3, e4) ->
                   Coq_add_default (x, (Eop ((Olea (Aindexed n)), (Econs (t2,
                     (Econs (e3, e4)))))))))
           | Ascaled (sc, n) ->
             (match e with
              | Enil ->
                Coq_add_default (x, (Eop ((Olea (Ascaled (sc, n))), Enil)))
              | Econs (t2, e0) ->
                (match e0 with
                 | Enil -> Coq_add_case11 (x, sc, n, t2)
                 | Econs (e3, e4) ->
                   Coq_add_default (x, (Eop ((Olea (Ascaled (sc, n))), (Econs
                     (t2, (Econs (e3, e4)))))))))
           | x0 -> Coq_add_default (x, (Eop ((Olea x0), e))))
        | x0 -> Coq_add_default (x, (Eop (x0, e))))
     | x0 -> Coq_add_default (x, x0))

let add :  expr -> expr -> expr =
  fun e1 e2  ->
  match add_match e1 e2 with
  | Coq_add_case1 (n1, t2) -> addimm n1 t2
  | Coq_add_case2 (t1, n2) -> addimm n2 t1
  | Coq_add_case3 (n1, t1, n2, t2) ->
    Eop ((Olea (Aindexed2 (Int.add n1 n2))), (Econs (t1, (Econs (t2,
      Enil)))))
  | Coq_add_case4 (n1, t1, sc, n2, t2) ->
    Eop ((Olea (Aindexed2scaled (sc, (Int.add n1 n2)))), (Econs (t1, (Econs
      (t2, Enil)))))
  | Coq_add_case5 (sc, n1, t1, n2, t2) ->
    Eop ((Olea (Aindexed2scaled (sc, (Int.add n1 n2)))), (Econs (t2, (Econs
      (t1, Enil)))))
  | Coq_add_case6 (n1, t1, id, ofs) ->
    Eop ((Olea (Abased (id, (Int.add ofs n1)))), (Econs (t1, Enil)))
  | Coq_add_case7 (id, ofs, n2, t2) ->
    Eop ((Olea (Abased (id, (Int.add ofs n2)))), (Econs (t2, Enil)))
  | Coq_add_case8 (sc, n1, t1, id, ofs) ->
    Eop ((Olea (Abasedscaled (sc, id, (Int.add ofs n1)))), (Econs (t1,
      Enil)))
  | Coq_add_case9 (id, ofs, sc, n2, t2) ->
    Eop ((Olea (Abasedscaled (sc, id, (Int.add ofs n2)))), (Econs (t2,
      Enil)))
  | Coq_add_case10 (sc, n, t1, t2) ->
    Eop ((Olea (Aindexed2scaled (sc, n))), (Econs (t2, (Econs (t1, Enil)))))
  | Coq_add_case11 (t1, sc, n, t2) ->
    Eop ((Olea (Aindexed2scaled (sc, n))), (Econs (t1, (Econs (t2, Enil)))))
  | Coq_add_case12 (n, t1, t2) ->
    Eop ((Olea (Aindexed2 n)), (Econs (t1, (Econs (t2, Enil)))))
  | Coq_add_case13 (t1, n, t2) ->
    Eop ((Olea (Aindexed2 n)), (Econs (t1, (Econs (t2, Enil)))))
  | Coq_add_default (e3, e4) ->
    Eop ((Olea (Aindexed2 Int.zero)), (Econs (e3, (Econs (e4, Enil)))))

type sub_cases =
| Coq_sub_case1 of expr * Int.int
| Coq_sub_case2 of Int.int * expr * Int.int * expr
| Coq_sub_case3 of Int.int * expr * expr
| Coq_sub_case4 of expr * Int.int * expr
| Coq_sub_default of expr * expr

let sub_match :  expr -> expr -> sub_cases =
  fun e1 e2  ->
  match e1 with
  | Eop (o, e) ->
    (match o with
     | Olea a ->
       (match a with
        | Aindexed n1 ->
          (match e with
           | Enil ->
             let e3 = Eop ((Olea (Aindexed n1)), Enil) in
             (match e2 with
              | Eop (o0, e0) ->
                (match o0 with
                 | Ointconst n2 ->
                   (match e0 with
                    | Enil -> Coq_sub_case1 (e3, n2)
                    | Econs (e4, e5) ->
                      Coq_sub_default (e3, (Eop ((Ointconst n2), (Econs (e4,
                        e5))))))
                 | Olea a0 ->
                   (match a0 with
                    | Aindexed n2 ->
                      (match e0 with
                       | Enil ->
                         Coq_sub_default (e3, (Eop ((Olea (Aindexed n2)),
                           Enil)))
                       | Econs (t2, e4) ->
                         (match e4 with
                          | Enil -> Coq_sub_case4 (e3, n2, t2)
                          | Econs (e5, e6) ->
                            Coq_sub_default (e3, (Eop ((Olea (Aindexed n2)),
                              (Econs (t2, (Econs (e5, e6)))))))))
                    | x -> Coq_sub_default (e3, (Eop ((Olea x), e0))))
                 | x -> Coq_sub_default (e3, (Eop (x, e0))))
              | x -> Coq_sub_default (e3, x))
           | Econs (t1, e0) ->
             (match e0 with
              | Enil ->
                (match e2 with
                 | Eop (o0, e3) ->
                   (match o0 with
                    | Ointconst n2 ->
                      (match e3 with
                       | Enil ->
                         Coq_sub_case1 ((Eop ((Olea (Aindexed n1)), (Econs
                           (t1, Enil)))), n2)
                       | Econs (e4, e5) ->
                         Coq_sub_case3 (n1, t1, (Eop ((Ointconst n2), (Econs
                           (e4, e5))))))
                    | Olea a0 ->
                      (match a0 with
                       | Aindexed n2 ->
                         (match e3 with
                          | Enil ->
                            Coq_sub_case3 (n1, t1, (Eop ((Olea (Aindexed
                              n2)), Enil)))
                          | Econs (t2, e4) ->
                            (match e4 with
                             | Enil -> Coq_sub_case2 (n1, t1, n2, t2)
                             | Econs (e5, e6) ->
                               Coq_sub_case3 (n1, t1, (Eop ((Olea (Aindexed
                                 n2)), (Econs (t2, (Econs (e5, e6)))))))))
                       | x -> Coq_sub_case3 (n1, t1, (Eop ((Olea x), e3))))
                    | x -> Coq_sub_case3 (n1, t1, (Eop (x, e3))))
                 | x -> Coq_sub_case3 (n1, t1, x))
              | Econs (e3, e4) ->
                let e5 = Eop ((Olea (Aindexed n1)), (Econs (t1, (Econs (e3,
                  e4)))))
                in
                (match e2 with
                 | Eop (o0, e6) ->
                   (match o0 with
                    | Ointconst n2 ->
                      (match e6 with
                       | Enil -> Coq_sub_case1 (e5, n2)
                       | Econs (e7, e8) ->
                         Coq_sub_default (e5, (Eop ((Ointconst n2), (Econs
                           (e7, e8))))))
                    | Olea a0 ->
                      (match a0 with
                       | Aindexed n2 ->
                         (match e6 with
                          | Enil ->
                            Coq_sub_default (e5, (Eop ((Olea (Aindexed n2)),
                              Enil)))
                          | Econs (t2, e7) ->
                            (match e7 with
                             | Enil -> Coq_sub_case4 (e5, n2, t2)
                             | Econs (e8, e9) ->
                               Coq_sub_default (e5, (Eop ((Olea (Aindexed
                                 n2)), (Econs (t2, (Econs (e8, e9)))))))))
                       | x -> Coq_sub_default (e5, (Eop ((Olea x), e6))))
                    | x -> Coq_sub_default (e5, (Eop (x, e6))))
                 | x -> Coq_sub_default (e5, x))))
        | x ->
          let e3 = Eop ((Olea x), e) in
          (match e2 with
           | Eop (o0, e0) ->
             (match o0 with
              | Ointconst n2 ->
                (match e0 with
                 | Enil -> Coq_sub_case1 (e3, n2)
                 | Econs (e4, e5) ->
                   Coq_sub_default (e3, (Eop ((Ointconst n2), (Econs (e4,
                     e5))))))
              | Olea a0 ->
                (match a0 with
                 | Aindexed n2 ->
                   (match e0 with
                    | Enil ->
                      Coq_sub_default (e3, (Eop ((Olea (Aindexed n2)),
                        Enil)))
                    | Econs (t2, e4) ->
                      (match e4 with
                       | Enil -> Coq_sub_case4 (e3, n2, t2)
                       | Econs (e5, e6) ->
                         Coq_sub_default (e3, (Eop ((Olea (Aindexed n2)),
                           (Econs (t2, (Econs (e5, e6)))))))))
                 | x0 -> Coq_sub_default (e3, (Eop ((Olea x0), e0))))
              | x0 -> Coq_sub_default (e3, (Eop (x0, e0))))
           | x0 -> Coq_sub_default (e3, x0)))
     | x ->
       let e3 = Eop (x, e) in
       (match e2 with
        | Eop (o0, e0) ->
          (match o0 with
           | Ointconst n2 ->
             (match e0 with
              | Enil -> Coq_sub_case1 (e3, n2)
              | Econs (e4, e5) ->
                Coq_sub_default (e3, (Eop ((Ointconst n2), (Econs (e4,
                  e5))))))
           | Olea a ->
             (match a with
              | Aindexed n2 ->
                (match e0 with
                 | Enil ->
                   Coq_sub_default (e3, (Eop ((Olea (Aindexed n2)), Enil)))
                 | Econs (t2, e4) ->
                   (match e4 with
                    | Enil -> Coq_sub_case4 (e3, n2, t2)
                    | Econs (e5, e6) ->
                      Coq_sub_default (e3, (Eop ((Olea (Aindexed n2)), (Econs
                        (t2, (Econs (e5, e6)))))))))
              | x0 -> Coq_sub_default (e3, (Eop ((Olea x0), e0))))
           | x0 -> Coq_sub_default (e3, (Eop (x0, e0))))
        | x0 -> Coq_sub_default (e3, x0)))
  | x ->
    (match e2 with
     | Eop (o, e) ->
       (match o with
        | Ointconst n2 ->
          (match e with
           | Enil -> Coq_sub_case1 (x, n2)
           | Econs (e0, e3) ->
             Coq_sub_default (x, (Eop ((Ointconst n2), (Econs (e0, e3))))))
        | Olea a ->
          (match a with
           | Aindexed n2 ->
             (match e with
              | Enil ->
                Coq_sub_default (x, (Eop ((Olea (Aindexed n2)), Enil)))
              | Econs (t2, e0) ->
                (match e0 with
                 | Enil -> Coq_sub_case4 (x, n2, t2)
                 | Econs (e3, e4) ->
                   Coq_sub_default (x, (Eop ((Olea (Aindexed n2)), (Econs
                     (t2, (Econs (e3, e4)))))))))
           | x0 -> Coq_sub_default (x, (Eop ((Olea x0), e))))
        | x0 -> Coq_sub_default (x, (Eop (x0, e))))
     | x0 -> Coq_sub_default (x, x0))

let sub :  expr -> expr -> expr =
  fun e1 e2  ->
  match sub_match e1 e2 with
  | Coq_sub_case1 (t1, n2) -> addimm (Int.neg n2) t1
  | Coq_sub_case2 (n1, t1, n2, t2) ->
    addimm (Int.sub n1 n2) (Eop (Osub, (Econs (t1, (Econs (t2, Enil))))))
  | Coq_sub_case3 (n1, t1, t2) ->
    addimm n1 (Eop (Osub, (Econs (t1, (Econs (t2, Enil))))))
  | Coq_sub_case4 (t1, n2, t2) ->
    addimm (Int.neg n2) (Eop (Osub, (Econs (t1, (Econs (t2, Enil))))))
  | Coq_sub_default (e3, e4) -> Eop (Osub, (Econs (e3, (Econs (e4, Enil)))))

let negint :  expr -> expr =
  fun e  ->
  Eop (Oneg, (Econs (e, Enil)))

let shift_is_scale :  Int.int -> bool =
  fun n  ->
  (||)
    ((||) (Int.eq n (Int.repr (Zpos Coq_xH)))
      (Int.eq n (Int.repr (Zpos (Coq_xO Coq_xH)))))
    (Int.eq n (Int.repr (Zpos (Coq_xI Coq_xH))))

type shlimm_cases =
| Coq_shlimm_case1 of Int.int
| Coq_shlimm_case2 of Int.int * expr
| Coq_shlimm_case3 of Int.int * expr
| Coq_shlimm_default of expr

let shlimm_match :  expr -> shlimm_cases =
    function
| Eop (o, e) ->
  (match o with
   | Ointconst n1 ->
     (match e with
      | Enil -> Coq_shlimm_case1 n1
      | Econs (e0, e2) ->
        Coq_shlimm_default (Eop ((Ointconst n1), (Econs (e0, e2)))))
   | Oshlimm n1 ->
     (match e with
      | Enil -> Coq_shlimm_default (Eop ((Oshlimm n1), Enil))
      | Econs (t1, e0) ->
        (match e0 with
         | Enil -> Coq_shlimm_case2 (n1, t1)
         | Econs (e2, e3) ->
           Coq_shlimm_default (Eop ((Oshlimm n1), (Econs (t1, (Econs (e2,
             e3))))))))
   | Olea a ->
     (match a with
      | Aindexed n1 ->
        (match e with
         | Enil -> Coq_shlimm_default (Eop ((Olea (Aindexed n1)), Enil))
         | Econs (t1, e0) ->
           (match e0 with
            | Enil -> Coq_shlimm_case3 (n1, t1)
            | Econs (e2, e3) ->
              Coq_shlimm_default (Eop ((Olea (Aindexed n1)), (Econs (t1,
                (Econs (e2, e3))))))))
      | x -> Coq_shlimm_default (Eop ((Olea x), e)))
   | x -> Coq_shlimm_default (Eop (x, e)))
| x -> Coq_shlimm_default x

let shlimm :  expr -> Int.int -> expr =
  fun e1 n  ->
  if Int.eq n Int.zero
  then e1
  else (match shlimm_match e1 with
        | Coq_shlimm_case1 n1 -> Eop ((Ointconst (Int.shl n1 n)), Enil)
        | Coq_shlimm_case2 (n1, t1) ->
          if Int.ltu (Int.add n n1) Int.iwordsize
          then Eop ((Oshlimm (Int.add n n1)), (Econs (t1, Enil)))
          else Eop ((Oshlimm n), (Econs (e1, Enil)))
        | Coq_shlimm_case3 (n1, t1) ->
          if shift_is_scale n
          then Eop ((Olea (Ascaled ((Int.shl Int.one n), (Int.shl n1 n)))),
                 (Econs (t1, Enil)))
          else Eop ((Oshlimm n), (Econs (e1, Enil)))
        | Coq_shlimm_default e2 ->
          if shift_is_scale n
          then Eop ((Olea (Ascaled ((Int.shl Int.one n), Int.zero))), (Econs
                 (e2, Enil)))
          else Eop ((Oshlimm n), (Econs (e2, Enil))))

type shruimm_cases =
| Coq_shruimm_case1 of Int.int
| Coq_shruimm_case2 of Int.int * expr
| Coq_shruimm_default of expr

let shruimm_match :  expr -> shruimm_cases =
    function
| Eop (o, e) ->
  (match o with
   | Ointconst n1 ->
     (match e with
      | Enil -> Coq_shruimm_case1 n1
      | Econs (e0, e2) ->
        Coq_shruimm_default (Eop ((Ointconst n1), (Econs (e0, e2)))))
   | Oshruimm n1 ->
     (match e with
      | Enil -> Coq_shruimm_default (Eop ((Oshruimm n1), Enil))
      | Econs (t1, e0) ->
        (match e0 with
         | Enil -> Coq_shruimm_case2 (n1, t1)
         | Econs (e2, e3) ->
           Coq_shruimm_default (Eop ((Oshruimm n1), (Econs (t1, (Econs (e2,
             e3))))))))
   | x -> Coq_shruimm_default (Eop (x, e)))
| x -> Coq_shruimm_default x

let shruimm :  expr -> Int.int -> expr =
  fun e1 n  ->
  if Int.eq n Int.zero
  then e1
  else (match shruimm_match e1 with
        | Coq_shruimm_case1 n1 -> Eop ((Ointconst (Int.shru n1 n)), Enil)
        | Coq_shruimm_case2 (n1, t1) ->
          if Int.ltu (Int.add n n1) Int.iwordsize
          then Eop ((Oshruimm (Int.add n n1)), (Econs (t1, Enil)))
          else Eop ((Oshruimm n), (Econs (e1, Enil)))
        | Coq_shruimm_default e2 -> Eop ((Oshruimm n), (Econs (e2, Enil))))

type shrimm_cases =
| Coq_shrimm_case1 of Int.int
| Coq_shrimm_case2 of Int.int * expr
| Coq_shrimm_default of expr

let shrimm_match :  expr -> shrimm_cases =
    function
| Eop (o, e) ->
  (match o with
   | Ointconst n1 ->
     (match e with
      | Enil -> Coq_shrimm_case1 n1
      | Econs (e0, e2) ->
        Coq_shrimm_default (Eop ((Ointconst n1), (Econs (e0, e2)))))
   | Oshrimm n1 ->
     (match e with
      | Enil -> Coq_shrimm_default (Eop ((Oshrimm n1), Enil))
      | Econs (t1, e0) ->
        (match e0 with
         | Enil -> Coq_shrimm_case2 (n1, t1)
         | Econs (e2, e3) ->
           Coq_shrimm_default (Eop ((Oshrimm n1), (Econs (t1, (Econs (e2,
             e3))))))))
   | x -> Coq_shrimm_default (Eop (x, e)))
| x -> Coq_shrimm_default x

let shrimm :  expr -> Int.int -> expr =
  fun e1 n  ->
  if Int.eq n Int.zero
  then e1
  else (match shrimm_match e1 with
        | Coq_shrimm_case1 n1 -> Eop ((Ointconst (Int.shr n1 n)), Enil)
        | Coq_shrimm_case2 (n1, t1) ->
          if Int.ltu (Int.add n n1) Int.iwordsize
          then Eop ((Oshrimm (Int.add n n1)), (Econs (t1, Enil)))
          else Eop ((Oshrimm n), (Econs (e1, Enil)))
        | Coq_shrimm_default e2 -> Eop ((Oshrimm n), (Econs (e2, Enil))))

let mulimm_base :  Int.int -> expr -> expr =
  fun n1 e2  ->
  match Int.one_bits n1 with
  | [] -> Eop ((Omulimm n1), (Econs (e2, Enil)))
  | i :: l ->
    (match l with
     | [] -> shlimm e2 i
     | j :: l0 ->
       (match l0 with
        | [] ->
          Elet (e2, (add (shlimm (Eletvar O) i) (shlimm (Eletvar O) j)))
        | i0 :: l1 -> Eop ((Omulimm n1), (Econs (e2, Enil)))))

type mulimm_cases =
| Coq_mulimm_case1 of Int.int
| Coq_mulimm_case2 of Int.int * expr
| Coq_mulimm_default of expr

let mulimm_match :  expr -> mulimm_cases =
    function
| Eop (o, e) ->
  (match o with
   | Ointconst n2 ->
     (match e with
      | Enil -> Coq_mulimm_case1 n2
      | Econs (e0, e1) ->
        Coq_mulimm_default (Eop ((Ointconst n2), (Econs (e0, e1)))))
   | Olea a ->
     (match a with
      | Aindexed n2 ->
        (match e with
         | Enil -> Coq_mulimm_default (Eop ((Olea (Aindexed n2)), Enil))
         | Econs (t2, e0) ->
           (match e0 with
            | Enil -> Coq_mulimm_case2 (n2, t2)
            | Econs (e1, e3) ->
              Coq_mulimm_default (Eop ((Olea (Aindexed n2)), (Econs (t2,
                (Econs (e1, e3))))))))
      | x -> Coq_mulimm_default (Eop ((Olea x), e)))
   | x -> Coq_mulimm_default (Eop (x, e)))
| x -> Coq_mulimm_default x

let mulimm :  Int.int -> expr -> expr =
  fun n1 e2  ->
  if Int.eq n1 Int.zero
  then Eop ((Ointconst Int.zero), Enil)
  else if Int.eq n1 Int.one
       then e2
       else (match mulimm_match e2 with
             | Coq_mulimm_case1 n2 -> Eop ((Ointconst (Int.mul n1 n2)), Enil)
             | Coq_mulimm_case2 (n2, t2) ->
               addimm (Int.mul n1 n2) (mulimm_base n1 t2)
             | Coq_mulimm_default e3 -> mulimm_base n1 e3)

type mul_cases =
| Coq_mul_case1 of Int.int * expr
| Coq_mul_case2 of expr * Int.int
| Coq_mul_default of expr * expr

let mul_match :  expr -> expr -> mul_cases =
  fun e1 e2  ->
  match e1 with
  | Eop (o, e) ->
    (match o with
     | Ointconst n1 ->
       (match e with
        | Enil -> Coq_mul_case1 (n1, e2)
        | Econs (e0, e3) ->
          let e4 = Eop ((Ointconst n1), (Econs (e0, e3))) in
          (match e2 with
           | Eop (o0, e5) ->
             (match o0 with
              | Ointconst n2 ->
                (match e5 with
                 | Enil -> Coq_mul_case2 (e4, n2)
                 | Econs (e6, e7) ->
                   Coq_mul_default (e4, (Eop ((Ointconst n2), (Econs (e6,
                     e7))))))
              | x -> Coq_mul_default (e4, (Eop (x, e5))))
           | x -> Coq_mul_default (e4, x)))
     | x ->
       let e3 = Eop (x, e) in
       (match e2 with
        | Eop (o0, e0) ->
          (match o0 with
           | Ointconst n2 ->
             (match e0 with
              | Enil -> Coq_mul_case2 (e3, n2)
              | Econs (e4, e5) ->
                Coq_mul_default (e3, (Eop ((Ointconst n2), (Econs (e4,
                  e5))))))
           | x0 -> Coq_mul_default (e3, (Eop (x0, e0))))
        | x0 -> Coq_mul_default (e3, x0)))
  | x ->
    (match e2 with
     | Eop (o, e) ->
       (match o with
        | Ointconst n2 ->
          (match e with
           | Enil -> Coq_mul_case2 (x, n2)
           | Econs (e0, e3) ->
             Coq_mul_default (x, (Eop ((Ointconst n2), (Econs (e0, e3))))))
        | x0 -> Coq_mul_default (x, (Eop (x0, e))))
     | x0 -> Coq_mul_default (x, x0))

let mul :  expr -> expr -> expr =
  fun e1 e2  ->
  match mul_match e1 e2 with
  | Coq_mul_case1 (n1, t2) -> mulimm n1 t2
  | Coq_mul_case2 (t1, n2) -> mulimm n2 t1
  | Coq_mul_default (e3, e4) -> Eop (Omul, (Econs (e3, (Econs (e4, Enil)))))

type andimm_cases =
| Coq_andimm_case1 of Int.int
| Coq_andimm_case2 of Int.int * expr
| Coq_andimm_case3 of expr
| Coq_andimm_case4 of expr
| Coq_andimm_default of expr

let andimm_match :  expr -> andimm_cases =
    function
| Eop (o, e) ->
  (match o with
   | Ointconst n2 ->
     (match e with
      | Enil -> Coq_andimm_case1 n2
      | Econs (e0, e1) ->
        Coq_andimm_default (Eop ((Ointconst n2), (Econs (e0, e1)))))
   | Ocast8unsigned ->
     (match e with
      | Enil -> Coq_andimm_default (Eop (Ocast8unsigned, Enil))
      | Econs (t2, e0) ->
        (match e0 with
         | Enil -> Coq_andimm_case3 t2
         | Econs (e1, e3) ->
           Coq_andimm_default (Eop (Ocast8unsigned, (Econs (t2, (Econs (e1,
             e3))))))))
   | Ocast16unsigned ->
     (match e with
      | Enil -> Coq_andimm_default (Eop (Ocast16unsigned, Enil))
      | Econs (t2, e0) ->
        (match e0 with
         | Enil -> Coq_andimm_case4 t2
         | Econs (e1, e3) ->
           Coq_andimm_default (Eop (Ocast16unsigned, (Econs (t2, (Econs (e1,
             e3))))))))
   | Oandimm n2 ->
     (match e with
      | Enil -> Coq_andimm_default (Eop ((Oandimm n2), Enil))
      | Econs (t2, e0) ->
        (match e0 with
         | Enil -> Coq_andimm_case2 (n2, t2)
         | Econs (e1, e3) ->
           Coq_andimm_default (Eop ((Oandimm n2), (Econs (t2, (Econs (e1,
             e3))))))))
   | x -> Coq_andimm_default (Eop (x, e)))
| x -> Coq_andimm_default x

let andimm :  Int.int -> expr -> expr =
  fun n1 e2  ->
  if Int.eq n1 Int.zero
  then Eop ((Ointconst Int.zero), Enil)
  else if Int.eq n1 Int.mone
       then e2
       else (match andimm_match e2 with
             | Coq_andimm_case1 n2 ->
               Eop ((Ointconst (Int.coq_and n1 n2)), Enil)
             | Coq_andimm_case2 (n2, t2) ->
               Eop ((Oandimm (Int.coq_and n1 n2)), (Econs (t2, Enil)))
             | Coq_andimm_case3 t2 ->
               Eop ((Oandimm
                 (Int.coq_and n1
                   (Int.repr (Zpos (Coq_xI (Coq_xI (Coq_xI (Coq_xI (Coq_xI
                     (Coq_xI (Coq_xI Coq_xH))))))))))), (Econs (t2, Enil)))
             | Coq_andimm_case4 t2 ->
               Eop ((Oandimm
                 (Int.coq_and n1
                   (Int.repr (Zpos (Coq_xI (Coq_xI (Coq_xI (Coq_xI (Coq_xI
                     (Coq_xI (Coq_xI (Coq_xI (Coq_xI (Coq_xI (Coq_xI (Coq_xI
                     (Coq_xI (Coq_xI (Coq_xI Coq_xH))))))))))))))))))),
                 (Econs (t2, Enil)))
             | Coq_andimm_default e3 ->
               Eop ((Oandimm n1), (Econs (e3, Enil))))

type and_cases =
| Coq_and_case1 of Int.int * expr
| Coq_and_case2 of expr * Int.int
| Coq_and_default of expr * expr

let and_match :  expr -> expr -> and_cases =
  fun e1 e2  ->
  match e1 with
  | Eop (o, e) ->
    (match o with
     | Ointconst n1 ->
       (match e with
        | Enil -> Coq_and_case1 (n1, e2)
        | Econs (e0, e3) ->
          let e4 = Eop ((Ointconst n1), (Econs (e0, e3))) in
          (match e2 with
           | Eop (o0, e5) ->
             (match o0 with
              | Ointconst n2 ->
                (match e5 with
                 | Enil -> Coq_and_case2 (e4, n2)
                 | Econs (e6, e7) ->
                   Coq_and_default (e4, (Eop ((Ointconst n2), (Econs (e6,
                     e7))))))
              | x -> Coq_and_default (e4, (Eop (x, e5))))
           | x -> Coq_and_default (e4, x)))
     | x ->
       let e3 = Eop (x, e) in
       (match e2 with
        | Eop (o0, e0) ->
          (match o0 with
           | Ointconst n2 ->
             (match e0 with
              | Enil -> Coq_and_case2 (e3, n2)
              | Econs (e4, e5) ->
                Coq_and_default (e3, (Eop ((Ointconst n2), (Econs (e4,
                  e5))))))
           | x0 -> Coq_and_default (e3, (Eop (x0, e0))))
        | x0 -> Coq_and_default (e3, x0)))
  | x ->
    (match e2 with
     | Eop (o, e) ->
       (match o with
        | Ointconst n2 ->
          (match e with
           | Enil -> Coq_and_case2 (x, n2)
           | Econs (e0, e3) ->
             Coq_and_default (x, (Eop ((Ointconst n2), (Econs (e0, e3))))))
        | x0 -> Coq_and_default (x, (Eop (x0, e))))
     | x0 -> Coq_and_default (x, x0))

let coq_and :  expr -> expr -> expr =
  fun e1 e2  ->
  match and_match e1 e2 with
  | Coq_and_case1 (n1, t2) -> andimm n1 t2
  | Coq_and_case2 (t1, n2) -> andimm n2 t1
  | Coq_and_default (e3, e4) -> Eop (Oand, (Econs (e3, (Econs (e4, Enil)))))

type orimm_cases =
| Coq_orimm_case1 of Int.int
| Coq_orimm_case2 of Int.int * expr
| Coq_orimm_default of expr

let orimm_match :  expr -> orimm_cases =
    function
| Eop (o, e) ->
  (match o with
   | Ointconst n2 ->
     (match e with
      | Enil -> Coq_orimm_case1 n2
      | Econs (e0, e1) ->
        Coq_orimm_default (Eop ((Ointconst n2), (Econs (e0, e1)))))
   | Oorimm n2 ->
     (match e with
      | Enil -> Coq_orimm_default (Eop ((Oorimm n2), Enil))
      | Econs (t2, e0) ->
        (match e0 with
         | Enil -> Coq_orimm_case2 (n2, t2)
         | Econs (e1, e3) ->
           Coq_orimm_default (Eop ((Oorimm n2), (Econs (t2, (Econs (e1,
             e3))))))))
   | x -> Coq_orimm_default (Eop (x, e)))
| x -> Coq_orimm_default x

let orimm :  Int.int -> expr -> expr =
  fun n1 e2  ->
  if Int.eq n1 Int.zero
  then e2
  else if Int.eq n1 Int.mone
       then Eop ((Ointconst Int.mone), Enil)
       else (match orimm_match e2 with
             | Coq_orimm_case1 n2 ->
               Eop ((Ointconst (Int.coq_or n1 n2)), Enil)
             | Coq_orimm_case2 (n2, t2) ->
               Eop ((Oorimm (Int.coq_or n1 n2)), (Econs (t2, Enil)))
             | Coq_orimm_default e3 -> Eop ((Oorimm n1), (Econs (e3, Enil))))

let same_expr_pure :  expr -> expr -> bool =
  fun e1 e2  ->
  match e1 with
  | Evar v1 ->
    (match e2 with
     | Evar v2 -> if ident_eq v1 v2 then true else false
     | _ -> false)
  | _ -> false

type or_cases =
| Coq_or_case1 of Int.int * expr
| Coq_or_case2 of expr * Int.int
| Coq_or_case3 of Int.int * expr * Int.int * expr
| Coq_or_case4 of Int.int * expr * Int.int * expr
| Coq_or_default of expr * expr

let or_match :  expr -> expr -> or_cases =
  fun e1 e2  ->
  match e1 with
  | Eop (o, e) ->
    (match o with
     | Ointconst n1 ->
       (match e with
        | Enil -> Coq_or_case1 (n1, e2)
        | Econs (e0, e3) ->
          let e4 = Eop ((Ointconst n1), (Econs (e0, e3))) in
          (match e2 with
           | Eop (o0, e5) ->
             (match o0 with
              | Ointconst n2 ->
                (match e5 with
                 | Enil -> Coq_or_case2 (e4, n2)
                 | Econs (e6, e7) ->
                   Coq_or_default (e4, (Eop ((Ointconst n2), (Econs (e6,
                     e7))))))
              | x -> Coq_or_default (e4, (Eop (x, e5))))
           | x -> Coq_or_default (e4, x)))
     | Oshlimm n1 ->
       (match e with
        | Enil ->
          let e3 = Eop ((Oshlimm n1), Enil) in
          (match e2 with
           | Eop (o0, e0) ->
             (match o0 with
              | Ointconst n2 ->
                (match e0 with
                 | Enil -> Coq_or_case2 (e3, n2)
                 | Econs (e4, e5) ->
                   Coq_or_default (e3, (Eop ((Ointconst n2), (Econs (e4,
                     e5))))))
              | x -> Coq_or_default (e3, (Eop (x, e0))))
           | x -> Coq_or_default (e3, x))
        | Econs (t1, e0) ->
          (match e0 with
           | Enil ->
             let e3 = Eop ((Oshlimm n1), (Econs (t1, Enil))) in
             (match e2 with
              | Eop (o0, e4) ->
                (match o0 with
                 | Ointconst n2 ->
                   (match e4 with
                    | Enil -> Coq_or_case2 (e3, n2)
                    | Econs (e5, e6) ->
                      Coq_or_default (e3, (Eop ((Ointconst n2), (Econs (e5,
                        e6))))))
                 | Oshruimm n2 ->
                   (match e4 with
                    | Enil ->
                      Coq_or_default (e3, (Eop ((Oshruimm n2), Enil)))
                    | Econs (t2, e5) ->
                      (match e5 with
                       | Enil -> Coq_or_case3 (n1, t1, n2, t2)
                       | Econs (e6, e7) ->
                         Coq_or_default (e3, (Eop ((Oshruimm n2), (Econs (t2,
                           (Econs (e6, e7)))))))))
                 | x -> Coq_or_default (e3, (Eop (x, e4))))
              | x -> Coq_or_default (e3, x))
           | Econs (e3, e4) ->
             let e5 = Eop ((Oshlimm n1), (Econs (t1, (Econs (e3, e4))))) in
             (match e2 with
              | Eop (o0, e6) ->
                (match o0 with
                 | Ointconst n2 ->
                   (match e6 with
                    | Enil -> Coq_or_case2 (e5, n2)
                    | Econs (e7, e8) ->
                      Coq_or_default (e5, (Eop ((Ointconst n2), (Econs (e7,
                        e8))))))
                 | x -> Coq_or_default (e5, (Eop (x, e6))))
              | x -> Coq_or_default (e5, x))))
     | Oshruimm n2 ->
       (match e with
        | Enil ->
          let e3 = Eop ((Oshruimm n2), Enil) in
          (match e2 with
           | Eop (o0, e0) ->
             (match o0 with
              | Ointconst n3 ->
                (match e0 with
                 | Enil -> Coq_or_case2 (e3, n3)
                 | Econs (e4, e5) ->
                   Coq_or_default (e3, (Eop ((Ointconst n3), (Econs (e4,
                     e5))))))
              | x -> Coq_or_default (e3, (Eop (x, e0))))
           | x -> Coq_or_default (e3, x))
        | Econs (t2, e0) ->
          (match e0 with
           | Enil ->
             let e3 = Eop ((Oshruimm n2), (Econs (t2, Enil))) in
             (match e2 with
              | Eop (o0, e4) ->
                (match o0 with
                 | Ointconst n3 ->
                   (match e4 with
                    | Enil -> Coq_or_case2 (e3, n3)
                    | Econs (e5, e6) ->
                      Coq_or_default (e3, (Eop ((Ointconst n3), (Econs (e5,
                        e6))))))
                 | Oshlimm n1 ->
                   (match e4 with
                    | Enil -> Coq_or_default (e3, (Eop ((Oshlimm n1), Enil)))
                    | Econs (t1, e5) ->
                      (match e5 with
                       | Enil -> Coq_or_case4 (n2, t2, n1, t1)
                       | Econs (e6, e7) ->
                         Coq_or_default (e3, (Eop ((Oshlimm n1), (Econs (t1,
                           (Econs (e6, e7)))))))))
                 | x -> Coq_or_default (e3, (Eop (x, e4))))
              | x -> Coq_or_default (e3, x))
           | Econs (e3, e4) ->
             let e5 = Eop ((Oshruimm n2), (Econs (t2, (Econs (e3, e4))))) in
             (match e2 with
              | Eop (o0, e6) ->
                (match o0 with
                 | Ointconst n3 ->
                   (match e6 with
                    | Enil -> Coq_or_case2 (e5, n3)
                    | Econs (e7, e8) ->
                      Coq_or_default (e5, (Eop ((Ointconst n3), (Econs (e7,
                        e8))))))
                 | x -> Coq_or_default (e5, (Eop (x, e6))))
              | x -> Coq_or_default (e5, x))))
     | x ->
       let e3 = Eop (x, e) in
       (match e2 with
        | Eop (o0, e0) ->
          (match o0 with
           | Ointconst n2 ->
             (match e0 with
              | Enil -> Coq_or_case2 (e3, n2)
              | Econs (e4, e5) ->
                Coq_or_default (e3, (Eop ((Ointconst n2), (Econs (e4, e5))))))
           | x0 -> Coq_or_default (e3, (Eop (x0, e0))))
        | x0 -> Coq_or_default (e3, x0)))
  | x ->
    (match e2 with
     | Eop (o, e) ->
       (match o with
        | Ointconst n2 ->
          (match e with
           | Enil -> Coq_or_case2 (x, n2)
           | Econs (e0, e3) ->
             Coq_or_default (x, (Eop ((Ointconst n2), (Econs (e0, e3))))))
        | x0 -> Coq_or_default (x, (Eop (x0, e))))
     | x0 -> Coq_or_default (x, x0))

let coq_or :  expr -> expr -> expr =
  fun e1 e2  ->
  match or_match e1 e2 with
  | Coq_or_case1 (n1, t2) -> orimm n1 t2
  | Coq_or_case2 (t1, n2) -> orimm n2 t1
  | Coq_or_case3 (n1, t1, n2, t2) ->
    if (&&) (Int.eq (Int.add n1 n2) Int.iwordsize) (same_expr_pure t1 t2)
    then Eop ((Ororimm n2), (Econs (t1, Enil)))
    else Eop (Oor, (Econs (e1, (Econs (e2, Enil)))))
  | Coq_or_case4 (n2, t2, n1, t1) ->
    if (&&) (Int.eq (Int.add n1 n2) Int.iwordsize) (same_expr_pure t1 t2)
    then Eop ((Ororimm n2), (Econs (t1, Enil)))
    else Eop (Oor, (Econs (e1, (Econs (e2, Enil)))))
  | Coq_or_default (e3, e4) -> Eop (Oor, (Econs (e3, (Econs (e4, Enil)))))

type xorimm_cases =
| Coq_xorimm_case1 of Int.int
| Coq_xorimm_case2 of Int.int * expr
| Coq_xorimm_default of expr

let xorimm_match :  expr -> xorimm_cases =
    function
| Eop (o, e) ->
  (match o with
   | Ointconst n2 ->
     (match e with
      | Enil -> Coq_xorimm_case1 n2
      | Econs (e0, e1) ->
        Coq_xorimm_default (Eop ((Ointconst n2), (Econs (e0, e1)))))
   | Oxorimm n2 ->
     (match e with
      | Enil -> Coq_xorimm_default (Eop ((Oxorimm n2), Enil))
      | Econs (t2, e0) ->
        (match e0 with
         | Enil -> Coq_xorimm_case2 (n2, t2)
         | Econs (e1, e3) ->
           Coq_xorimm_default (Eop ((Oxorimm n2), (Econs (t2, (Econs (e1,
             e3))))))))
   | x -> Coq_xorimm_default (Eop (x, e)))
| x -> Coq_xorimm_default x

let xorimm :  Int.int -> expr -> expr =
  fun n1 e2  ->
  if Int.eq n1 Int.zero
  then e2
  else (match xorimm_match e2 with
        | Coq_xorimm_case1 n2 -> Eop ((Ointconst (Int.xor n1 n2)), Enil)
        | Coq_xorimm_case2 (n2, t2) ->
          Eop ((Oxorimm (Int.xor n1 n2)), (Econs (t2, Enil)))
        | Coq_xorimm_default e3 -> Eop ((Oxorimm n1), (Econs (e3, Enil))))

type xor_cases =
| Coq_xor_case1 of Int.int * expr
| Coq_xor_case2 of expr * Int.int
| Coq_xor_default of expr * expr

let xor_match :  expr -> expr -> xor_cases =
  fun e1 e2  ->
  match e1 with
  | Eop (o, e) ->
    (match o with
     | Ointconst n1 ->
       (match e with
        | Enil -> Coq_xor_case1 (n1, e2)
        | Econs (e0, e3) ->
          let e4 = Eop ((Ointconst n1), (Econs (e0, e3))) in
          (match e2 with
           | Eop (o0, e5) ->
             (match o0 with
              | Ointconst n2 ->
                (match e5 with
                 | Enil -> Coq_xor_case2 (e4, n2)
                 | Econs (e6, e7) ->
                   Coq_xor_default (e4, (Eop ((Ointconst n2), (Econs (e6,
                     e7))))))
              | x -> Coq_xor_default (e4, (Eop (x, e5))))
           | x -> Coq_xor_default (e4, x)))
     | x ->
       let e3 = Eop (x, e) in
       (match e2 with
        | Eop (o0, e0) ->
          (match o0 with
           | Ointconst n2 ->
             (match e0 with
              | Enil -> Coq_xor_case2 (e3, n2)
              | Econs (e4, e5) ->
                Coq_xor_default (e3, (Eop ((Ointconst n2), (Econs (e4,
                  e5))))))
           | x0 -> Coq_xor_default (e3, (Eop (x0, e0))))
        | x0 -> Coq_xor_default (e3, x0)))
  | x ->
    (match e2 with
     | Eop (o, e) ->
       (match o with
        | Ointconst n2 ->
          (match e with
           | Enil -> Coq_xor_case2 (x, n2)
           | Econs (e0, e3) ->
             Coq_xor_default (x, (Eop ((Ointconst n2), (Econs (e0, e3))))))
        | x0 -> Coq_xor_default (x, (Eop (x0, e))))
     | x0 -> Coq_xor_default (x, x0))

let xor :  expr -> expr -> expr =
  fun e1 e2  ->
  match xor_match e1 e2 with
  | Coq_xor_case1 (n1, t2) -> xorimm n1 t2
  | Coq_xor_case2 (t1, n2) -> xorimm n2 t1
  | Coq_xor_default (e3, e4) -> Eop (Oxor, (Econs (e3, (Econs (e4, Enil)))))

let divu :  expr -> expr -> expr =
  fun e1 e2  ->
  Eop (Odivu, (Econs (e1, (Econs (e2, Enil)))))

let modu :  expr -> expr -> expr =
  fun e1 e2  ->
  Eop (Omodu, (Econs (e1, (Econs (e2, Enil)))))

let divs :  expr -> expr -> expr =
  fun e1 e2  ->
  Eop (Odiv, (Econs (e1, (Econs (e2, Enil)))))

let mods :  expr -> expr -> expr =
  fun e1 e2  ->
  Eop (Omod, (Econs (e1, (Econs (e2, Enil)))))

type shl_cases =
| Coq_shl_case1 of Int.int
| Coq_shl_default of expr

let shl_match :  expr -> shl_cases =
    function
| Eop (o, e) ->
  (match o with
   | Ointconst n2 ->
     (match e with
      | Enil -> Coq_shl_case1 n2
      | Econs (e0, e1) ->
        Coq_shl_default (Eop ((Ointconst n2), (Econs (e0, e1)))))
   | x -> Coq_shl_default (Eop (x, e)))
| x -> Coq_shl_default x

let shl :  expr -> expr -> expr =
  fun e1 e2  ->
  match shl_match e2 with
  | Coq_shl_case1 n2 -> shlimm e1 n2
  | Coq_shl_default e3 -> Eop (Oshl, (Econs (e1, (Econs (e3, Enil)))))

type shr_cases =
| Coq_shr_case1 of Int.int
| Coq_shr_default of expr

let shr_match :  expr -> shr_cases =
    function
| Eop (o, e) ->
  (match o with
   | Ointconst n2 ->
     (match e with
      | Enil -> Coq_shr_case1 n2
      | Econs (e0, e1) ->
        Coq_shr_default (Eop ((Ointconst n2), (Econs (e0, e1)))))
   | x -> Coq_shr_default (Eop (x, e)))
| x -> Coq_shr_default x

let shr :  expr -> expr -> expr =
  fun e1 e2  ->
  match shr_match e2 with
  | Coq_shr_case1 n2 -> shrimm e1 n2
  | Coq_shr_default e3 -> Eop (Oshr, (Econs (e1, (Econs (e3, Enil)))))

type shru_cases =
| Coq_shru_case1 of Int.int
| Coq_shru_default of expr

let shru_match :  expr -> shru_cases =
    function
| Eop (o, e) ->
  (match o with
   | Ointconst n2 ->
     (match e with
      | Enil -> Coq_shru_case1 n2
      | Econs (e0, e1) ->
        Coq_shru_default (Eop ((Ointconst n2), (Econs (e0, e1)))))
   | x -> Coq_shru_default (Eop (x, e)))
| x -> Coq_shru_default x

let shru :  expr -> expr -> expr =
  fun e1 e2  ->
  match shru_match e2 with
  | Coq_shru_case1 n2 -> shruimm e1 n2
  | Coq_shru_default e3 -> Eop (Oshru, (Econs (e1, (Econs (e3, Enil)))))

let negf :  expr -> expr =
  fun e  ->
  Eop (Onegf, (Econs (e, Enil)))

let absf :  expr -> expr =
  fun e  ->
  Eop (Oabsf, (Econs (e, Enil)))

let addf :  expr -> expr -> expr =
  fun e1 e2  ->
  Eop (Oaddf, (Econs (e1, (Econs (e2, Enil)))))

let subf :  expr -> expr -> expr =
  fun e1 e2  ->
  Eop (Osubf, (Econs (e1, (Econs (e2, Enil)))))

let mulf :  expr -> expr -> expr =
  fun e1 e2  ->
  Eop (Omulf, (Econs (e1, (Econs (e2, Enil)))))

let divf :  expr -> expr -> expr =
  fun e1 e2  ->
  Eop (Odivf, (Econs (e1, (Econs (e2, Enil)))))

type compimm_cases =
| Coq_compimm_case1 of comparison * Int.int
| Coq_compimm_case2 of condition * exprlist
| Coq_compimm_case3 of condition * exprlist
| Coq_compimm_case4 of Int.int * expr
| Coq_compimm_case5 of Int.int * expr
| Coq_compimm_default of comparison * expr

let compimm_match :  comparison -> expr -> compimm_cases =
  fun c e1  ->
  match c with
  | Ceq ->
    let c0 = Ceq in
    (match e1 with
     | Eop (o, el) ->
       (match o with
        | Ointconst n1 ->
          (match el with
           | Enil -> Coq_compimm_case1 (c0, n1)
           | Econs (e, e0) ->
             Coq_compimm_default (c0, (Eop ((Ointconst n1), (Econs (e,
               e0))))))
        | Oandimm n1 ->
          (match el with
           | Enil -> Coq_compimm_default (c0, (Eop ((Oandimm n1), Enil)))
           | Econs (t1, e) ->
             (match e with
              | Enil -> Coq_compimm_case4 (n1, t1)
              | Econs (e0, e2) ->
                Coq_compimm_default (c0, (Eop ((Oandimm n1), (Econs (t1,
                  (Econs (e0, e2)))))))))
        | Ocmp c1 -> Coq_compimm_case2 (c1, el)
        | x -> Coq_compimm_default (c0, (Eop (x, el))))
     | x -> Coq_compimm_default (c0, x))
  | Cne ->
    let c0 = Cne in
    (match e1 with
     | Eop (o, el) ->
       (match o with
        | Ointconst n1 ->
          (match el with
           | Enil -> Coq_compimm_case1 (c0, n1)
           | Econs (e, e0) ->
             Coq_compimm_default (c0, (Eop ((Ointconst n1), (Econs (e,
               e0))))))
        | Oandimm n1 ->
          (match el with
           | Enil -> Coq_compimm_default (c0, (Eop ((Oandimm n1), Enil)))
           | Econs (t1, e) ->
             (match e with
              | Enil -> Coq_compimm_case5 (n1, t1)
              | Econs (e0, e2) ->
                Coq_compimm_default (c0, (Eop ((Oandimm n1), (Econs (t1,
                  (Econs (e0, e2)))))))))
        | Ocmp c1 -> Coq_compimm_case3 (c1, el)
        | x -> Coq_compimm_default (c0, (Eop (x, el))))
     | x -> Coq_compimm_default (c0, x))
  | x ->
    (match e1 with
     | Eop (o, e) ->
       (match o with
        | Ointconst n1 ->
          (match e with
           | Enil -> Coq_compimm_case1 (x, n1)
           | Econs (e0, e2) ->
             Coq_compimm_default (x, (Eop ((Ointconst n1), (Econs (e0,
               e2))))))
        | x0 -> Coq_compimm_default (x, (Eop (x0, e))))
     | x0 -> Coq_compimm_default (x, x0))

let compimm :     (comparison -> Int.int -> condition) -> (comparison -> Int.int -> Int.int
    -> bool) -> comparison -> expr -> Int.int -> expr =
  fun coqzz_default sem c e1 n2  ->
  match compimm_match c e1 with
  | Coq_compimm_case1 (c0, n1) ->
    Eop ((Ointconst (if sem c0 n1 n2 then Int.one else Int.zero)), Enil)
  | Coq_compimm_case2 (c0, el) ->
    if Int.eq_dec n2 Int.zero
    then Eop ((Ocmp (negate_condition c0)), el)
    else if Int.eq_dec n2 Int.one
         then Eop ((Ocmp c0), el)
         else Eop ((Ointconst Int.zero), Enil)
  | Coq_compimm_case3 (c0, el) ->
    if Int.eq_dec n2 Int.zero
    then Eop ((Ocmp c0), el)
    else if Int.eq_dec n2 Int.one
         then Eop ((Ocmp (negate_condition c0)), el)
         else Eop ((Ointconst Int.one), Enil)
  | Coq_compimm_case4 (n1, t1) ->
    if Int.eq_dec n2 Int.zero
    then Eop ((Ocmp (Cmaskzero n1)), (Econs (t1, Enil)))
    else Eop ((Ocmp (coqzz_default c n2)), (Econs (e1, Enil)))
  | Coq_compimm_case5 (n1, t1) ->
    if Int.eq_dec n2 Int.zero
    then Eop ((Ocmp (Cmasknotzero n1)), (Econs (t1, Enil)))
    else Eop ((Ocmp (coqzz_default c n2)), (Econs (e1, Enil)))
  | Coq_compimm_default (c0, e2) ->
    Eop ((Ocmp (coqzz_default c0 n2)), (Econs (e2, Enil)))

type comp_cases =
| Coq_comp_case1 of Int.int * expr
| Coq_comp_case2 of expr * Int.int
| Coq_comp_default of expr * expr

let comp_match :  expr -> expr -> comp_cases =
  fun e1 e2  ->
  match e1 with
  | Eop (o, e) ->
    (match o with
     | Ointconst n1 ->
       (match e with
        | Enil -> Coq_comp_case1 (n1, e2)
        | Econs (e0, e3) ->
          let e4 = Eop ((Ointconst n1), (Econs (e0, e3))) in
          (match e2 with
           | Eop (o0, e5) ->
             (match o0 with
              | Ointconst n2 ->
                (match e5 with
                 | Enil -> Coq_comp_case2 (e4, n2)
                 | Econs (e6, e7) ->
                   Coq_comp_default (e4, (Eop ((Ointconst n2), (Econs (e6,
                     e7))))))
              | x -> Coq_comp_default (e4, (Eop (x, e5))))
           | x -> Coq_comp_default (e4, x)))
     | x ->
       let e3 = Eop (x, e) in
       (match e2 with
        | Eop (o0, e0) ->
          (match o0 with
           | Ointconst n2 ->
             (match e0 with
              | Enil -> Coq_comp_case2 (e3, n2)
              | Econs (e4, e5) ->
                Coq_comp_default (e3, (Eop ((Ointconst n2), (Econs (e4,
                  e5))))))
           | x0 -> Coq_comp_default (e3, (Eop (x0, e0))))
        | x0 -> Coq_comp_default (e3, x0)))
  | x ->
    (match e2 with
     | Eop (o, e) ->
       (match o with
        | Ointconst n2 ->
          (match e with
           | Enil -> Coq_comp_case2 (x, n2)
           | Econs (e0, e3) ->
             Coq_comp_default (x, (Eop ((Ointconst n2), (Econs (e0, e3))))))
        | x0 -> Coq_comp_default (x, (Eop (x0, e))))
     | x0 -> Coq_comp_default (x, x0))

let comp :  comparison -> expr -> expr -> expr =
  fun c e1 e2  ->
  match comp_match e1 e2 with
  | Coq_comp_case1 (n1, t2) ->
    compimm (fun x x0 -> Ccompimm (x, x0)) Int.cmp (swap_comparison c) t2 n1
  | Coq_comp_case2 (t1, n2) ->
    compimm (fun x x0 -> Ccompimm (x, x0)) Int.cmp c t1 n2
  | Coq_comp_default (e3, e4) ->
    Eop ((Ocmp (Ccomp c)), (Econs (e3, (Econs (e4, Enil)))))

type compu_cases =
| Coq_compu_case1 of Int.int * expr
| Coq_compu_case2 of expr * Int.int
| Coq_compu_default of expr * expr

let compu_match :  expr -> expr -> compu_cases =
  fun e1 e2  ->
  match e1 with
  | Eop (o, e) ->
    (match o with
     | Ointconst n1 ->
       (match e with
        | Enil -> Coq_compu_case1 (n1, e2)
        | Econs (e0, e3) ->
          let e4 = Eop ((Ointconst n1), (Econs (e0, e3))) in
          (match e2 with
           | Eop (o0, e5) ->
             (match o0 with
              | Ointconst n2 ->
                (match e5 with
                 | Enil -> Coq_compu_case2 (e4, n2)
                 | Econs (e6, e7) ->
                   Coq_compu_default (e4, (Eop ((Ointconst n2), (Econs (e6,
                     e7))))))
              | x -> Coq_compu_default (e4, (Eop (x, e5))))
           | x -> Coq_compu_default (e4, x)))
     | x ->
       let e3 = Eop (x, e) in
       (match e2 with
        | Eop (o0, e0) ->
          (match o0 with
           | Ointconst n2 ->
             (match e0 with
              | Enil -> Coq_compu_case2 (e3, n2)
              | Econs (e4, e5) ->
                Coq_compu_default (e3, (Eop ((Ointconst n2), (Econs (e4,
                  e5))))))
           | x0 -> Coq_compu_default (e3, (Eop (x0, e0))))
        | x0 -> Coq_compu_default (e3, x0)))
  | x ->
    (match e2 with
     | Eop (o, e) ->
       (match o with
        | Ointconst n2 ->
          (match e with
           | Enil -> Coq_compu_case2 (x, n2)
           | Econs (e0, e3) ->
             Coq_compu_default (x, (Eop ((Ointconst n2), (Econs (e0, e3))))))
        | x0 -> Coq_compu_default (x, (Eop (x0, e))))
     | x0 -> Coq_compu_default (x, x0))

let compu :  comparison -> expr -> expr -> expr =
  fun c e1 e2  ->
  match compu_match e1 e2 with
  | Coq_compu_case1 (n1, t2) ->
    compimm (fun x x0 -> Ccompuimm (x, x0)) Int.cmpu (swap_comparison c) t2
      n1
  | Coq_compu_case2 (t1, n2) ->
    compimm (fun x x0 -> Ccompuimm (x, x0)) Int.cmpu c t1 n2
  | Coq_compu_default (e3, e4) ->
    Eop ((Ocmp (Ccompu c)), (Econs (e3, (Econs (e4, Enil)))))

let compf :  comparison -> expr -> expr -> expr =
  fun c e1 e2  ->
  Eop ((Ocmp (Ccompf c)), (Econs (e1, (Econs (e2, Enil)))))

type cast8unsigned_cases =
| Coq_cast8unsigned_case1 of Int.int * expr
| Coq_cast8unsigned_default of expr

let cast8unsigned_match :  expr -> cast8unsigned_cases =
    function
| Eop (o, e0) ->
  (match o with
   | Oandimm n ->
     (match e0 with
      | Enil -> Coq_cast8unsigned_default (Eop ((Oandimm n), Enil))
      | Econs (t, e1) ->
        (match e1 with
         | Enil -> Coq_cast8unsigned_case1 (n, t)
         | Econs (e2, e3) ->
           Coq_cast8unsigned_default (Eop ((Oandimm n), (Econs (t, (Econs
             (e2, e3))))))))
   | x -> Coq_cast8unsigned_default (Eop (x, e0)))
| x -> Coq_cast8unsigned_default x

let cast8unsigned :  expr -> expr =
  fun e  ->
  match cast8unsigned_match e with
  | Coq_cast8unsigned_case1 (n, t) ->
    Eop ((Oandimm
      (Int.coq_and
        (Int.repr (Zpos (Coq_xI (Coq_xI (Coq_xI (Coq_xI (Coq_xI (Coq_xI
          (Coq_xI Coq_xH))))))))) n)), (Econs (t, Enil)))
  | Coq_cast8unsigned_default e0 -> Eop (Ocast8unsigned, (Econs (e0, Enil)))

let cast8signed :  expr -> expr =
  fun e  ->
  Eop (Ocast8signed, (Econs (e, Enil)))

type cast16unsigned_cases =
| Coq_cast16unsigned_case1 of Int.int * expr
| Coq_cast16unsigned_default of expr

let cast16unsigned_match :  expr -> cast16unsigned_cases =
    function
| Eop (o, e0) ->
  (match o with
   | Oandimm n ->
     (match e0 with
      | Enil -> Coq_cast16unsigned_default (Eop ((Oandimm n), Enil))
      | Econs (t, e1) ->
        (match e1 with
         | Enil -> Coq_cast16unsigned_case1 (n, t)
         | Econs (e2, e3) ->
           Coq_cast16unsigned_default (Eop ((Oandimm n), (Econs (t, (Econs
             (e2, e3))))))))
   | x -> Coq_cast16unsigned_default (Eop (x, e0)))
| x -> Coq_cast16unsigned_default x

let cast16unsigned :  expr -> expr =
  fun e  ->
  match cast16unsigned_match e with
  | Coq_cast16unsigned_case1 (n, t) ->
    Eop ((Oandimm
      (Int.coq_and
        (Int.repr (Zpos (Coq_xI (Coq_xI (Coq_xI (Coq_xI (Coq_xI (Coq_xI
          (Coq_xI (Coq_xI (Coq_xI (Coq_xI (Coq_xI (Coq_xI (Coq_xI (Coq_xI
          (Coq_xI Coq_xH))))))))))))))))) n)), (Econs (t, Enil)))
  | Coq_cast16unsigned_default e0 ->
    Eop (Ocast16unsigned, (Econs (e0, Enil)))

let cast16signed :  expr -> expr =
  fun e  ->
  Eop (Ocast16signed, (Econs (e, Enil)))

let singleoffloat :  expr -> expr =
  fun e  ->
  Eop (Osingleoffloat, (Econs (e, Enil)))

let intoffloat :  expr -> expr =
  fun e  ->
  Eop (Ointoffloat, (Econs (e, Enil)))

let floatofint :  expr -> expr =
  fun e  ->
  Eop (Ofloatofint, (Econs (e, Enil)))

let intuoffloat :  expr -> expr =
  fun e  ->
  Elet (e, (Elet ((Eop ((Ofloatconst (Float.floatofintu Float.ox8000_0000)),
    Enil)), (Econdition ((Ccompf Clt), (Econs ((Eletvar (S O)), (Econs
    ((Eletvar O), Enil)))), (intoffloat (Eletvar (S O))),
    (addimm Float.ox8000_0000
      (intoffloat (subf (Eletvar (S O)) (Eletvar O)))))))))

let floatofintu :  expr -> expr =
  fun e  ->
  let f = Eop ((Ofloatconst (Float.floatofintu Float.ox8000_0000)), Enil) in
  Elet (e, (Econdition ((Ccompuimm (Clt, Float.ox8000_0000)), (Econs
  ((Eletvar O), Enil)), (floatofint (Eletvar O)),
  (addf (floatofint (addimm (Int.neg Float.ox8000_0000) (Eletvar O))) f))))

type addressing_cases =
| Coq_addressing_case1 of addressing * exprlist
| Coq_addressing_default of expr

let addressing_match :  expr -> addressing_cases =
    function
| Eop (o, args) ->
  (match o with
   | Olea addr -> Coq_addressing_case1 (addr, args)
   | x -> Coq_addressing_default (Eop (x, args)))
| x -> Coq_addressing_default x

let addressing :  memory_chunk -> expr -> addressing * exprlist =
  fun chunk e  ->
  match addressing_match e with
  | Coq_addressing_case1 (addr, args) -> (addr, args)
  | Coq_addressing_default e0 -> ((Aindexed Int.zero), (Econs (e0, Enil)))

