open AST
open BinNums
open Clight
open Cop
open Coqlib
open Ctypes
open Datatypes
open Errors
open Int0
open List0
open Ordered
open OrderedType
open Peano

module VSet = 
 struct 
  module X' = 
   struct 
    type t = positive
    
    (** val eq_dec : positive -> positive -> bool **)
    
    let eq_dec x y =
      OrderedPositive.eq_dec x y
    
    (** val compare : positive -> positive -> comparison **)
    
    let compare : positive -> positive -> comparison = fun x y ->
      match OrderedPositive.compare x y with
      | LT -> Lt
      | EQ -> Eq
      | GT -> Gt
   end
  
  module MSet = 
   struct 
    module Raw = 
     struct 
      type elt = positive
      
      type tree =
      | Leaf
      | Node of Z_as_Int.t * tree * positive * tree
      
      (** val empty : tree **)
      
      let empty =
        Leaf
      
      (** val is_empty : tree -> bool **)
      
      let is_empty = function
      | Leaf -> true
      | Node (t1, t2, t3, t4) -> false
      
      (** val mem : positive -> tree -> bool **)
      
      let rec mem x = function
      | Leaf -> false
      | Node (t1, l, k, r) ->
        (match OrderedPositive.compare x k with
         | LT -> mem x l
         | EQ -> true
         | GT -> mem x r)
      
      (** val min_elt : tree -> elt option **)
      
      let rec min_elt = function
      | Leaf -> None
      | Node (t1, l, x, r) ->
        (match l with
         | Leaf -> Some x
         | Node (t2, t3, t4, t5) -> min_elt l)
      
      (** val max_elt : tree -> elt option **)
      
      let rec max_elt = function
      | Leaf -> None
      | Node (t1, l, x, r) ->
        (match r with
         | Leaf -> Some x
         | Node (t2, t3, t4, t5) -> max_elt r)
      
      (** val choose : tree -> elt option **)
      
      let choose x =
        min_elt x
      
      (** val fold : (elt -> 'a1 -> 'a1) -> tree -> 'a1 -> 'a1 **)
      
      let rec fold f t0 coqzz_base =
        match t0 with
        | Leaf -> coqzz_base
        | Node (t1, l, x, r) -> fold f r (f x (fold f l coqzz_base))
      
      (** val elements_aux : positive list -> tree -> positive list **)
      
      let rec elements_aux acc = function
      | Leaf -> acc
      | Node (t0, l, x, r) -> elements_aux (x :: (elements_aux acc r)) l
      
      (** val elements : tree -> positive list **)
      
      let elements x =
        elements_aux [] x
      
      (** val rev_elements_aux : positive list -> tree -> positive list **)
      
      let rec rev_elements_aux acc = function
      | Leaf -> acc
      | Node (t0, l, x, r) ->
        rev_elements_aux (x :: (rev_elements_aux acc l)) r
      
      (** val rev_elements : tree -> positive list **)
      
      let rev_elements x =
        rev_elements_aux [] x
      
      (** val cardinal : tree -> nat **)
      
      let rec cardinal = function
      | Leaf -> O
      | Node (t0, l, t1, r) -> S (plus (cardinal l) (cardinal r))
      
      (** val maxdepth : tree -> nat **)
      
      let rec maxdepth = function
      | Leaf -> O
      | Node (t0, l, t1, r) -> S (max (maxdepth l) (maxdepth r))
      
      (** val mindepth : tree -> nat **)
      
      let rec mindepth = function
      | Leaf -> O
      | Node (t0, l, t1, r) -> S (min (mindepth l) (mindepth r))
      
      (** val for_all : (elt -> bool) -> tree -> bool **)
      
      let rec for_all f = function
      | Leaf -> true
      | Node (t0, l, x, r) ->
        if if f x then for_all f l else false then for_all f r else false
      
      (** val exists_ : (elt -> bool) -> tree -> bool **)
      
      let rec exists_ f = function
      | Leaf -> false
      | Node (t0, l, x, r) ->
        if if f x then true else exists_ f l then true else exists_ f r
      
      type enumeration =
      | End
      | More of elt * tree * enumeration
      
      (** val cons : tree -> enumeration -> enumeration **)
      
      let rec cons s e =
        match s with
        | Leaf -> e
        | Node (t0, l, x, r) -> cons l (More (x, r, e))
      
      (** val compare_more :
          positive -> (enumeration -> comparison) -> enumeration ->
          comparison **)
      
      let compare_more x1 cont = function
      | End -> Gt
      | More (x2, r2, e3) ->
        (match OrderedPositive.compare x1 x2 with
         | LT -> Lt
         | EQ -> cont (cons r2 e3)
         | GT -> Gt)
      
      (** val compare_cont :
          tree -> (enumeration -> comparison) -> enumeration -> comparison **)
      
      let rec compare_cont s1 cont e2 =
        match s1 with
        | Leaf -> cont e2
        | Node (t0, l1, x1, r1) ->
          compare_cont l1 (compare_more x1 (compare_cont r1 cont)) e2
      
      (** val compare_end : enumeration -> comparison **)
      
      let compare_end = function
      | End -> Eq
      | More (e, t0, e0) -> Lt
      
      (** val compare : tree -> tree -> comparison **)
      
      let compare s1 s2 =
        compare_cont s1 compare_end (cons s2 End)
      
      (** val equal : tree -> tree -> bool **)
      
      let equal s1 s2 =
        match compare s1 s2 with
        | Eq -> true
        | _ -> false
      
      (** val subsetl : (tree -> bool) -> positive -> tree -> bool **)
      
      let rec subsetl subset_l1 x1 s2 = match s2 with
      | Leaf -> false
      | Node (t0, l2, x2, r2) ->
        (match OrderedPositive.compare x1 x2 with
         | LT -> subsetl subset_l1 x1 l2
         | EQ -> subset_l1 l2
         | GT -> if mem x1 r2 then subset_l1 s2 else false)
      
      (** val subsetr : (tree -> bool) -> positive -> tree -> bool **)
      
      let rec subsetr subset_r1 x1 s2 = match s2 with
      | Leaf -> false
      | Node (t0, l2, x2, r2) ->
        (match OrderedPositive.compare x1 x2 with
         | LT -> if mem x1 l2 then subset_r1 s2 else false
         | EQ -> subset_r1 r2
         | GT -> subsetr subset_r1 x1 r2)
      
      (** val subset : tree -> tree -> bool **)
      
      let rec subset s1 s2 =
        match s1 with
        | Leaf -> true
        | Node (t0, l1, x1, r1) ->
          (match s2 with
           | Leaf -> false
           | Node (t1, l2, x2, r2) ->
             (match OrderedPositive.compare x1 x2 with
              | LT ->
                if subsetl (subset l1) x1 l2 then subset r1 s2 else false
              | EQ -> if subset l1 l2 then subset r1 r2 else false
              | GT ->
                if subsetr (subset r1) x1 r2 then subset l1 s2 else false))
      
      type t = tree
      
      (** val height : t -> Z_as_Int.t **)
      
      let height = function
      | Leaf -> Z_as_Int._0
      | Node (h, t0, t1, t2) -> h
      
      (** val singleton : positive -> tree **)
      
      let singleton x =
        Node (Z_as_Int._1, Leaf, x, Leaf)
      
      (** val create : t -> positive -> t -> tree **)
      
      let create l x r =
        Node
          ((Z_as_Int.plus (Z_as_Int.max (height l) (height r)) Z_as_Int._1),
          l, x, r)
      
      (** val assert_false : t -> positive -> t -> tree **)
      
      let assert_false : t -> positive -> t -> tree =
	fun x y z ->
        failwith "assert false in trees"
      
      (** val bal : t -> positive -> t -> tree **)
      
      let bal l x r =
        let hl = height l in
        let hr = height r in
        if Z_as_Int.gt_le_dec hl (Z_as_Int.plus hr Z_as_Int._2)
        then (match l with
              | Leaf -> assert_false l x r
              | Node (t0, ll, lx, lr) ->
                if Z_as_Int.ge_lt_dec (height ll) (height lr)
                then create ll lx (create lr x r)
                else (match lr with
                      | Leaf -> assert_false l x r
                      | Node (t1, lrl, lrx, lrr) ->
                        create (create ll lx lrl) lrx (create lrr x r)))
        else if Z_as_Int.gt_le_dec hr (Z_as_Int.plus hl Z_as_Int._2)
             then (match r with
                   | Leaf -> assert_false l x r
                   | Node (t0, rl, rx, rr) ->
                     if Z_as_Int.ge_lt_dec (height rr) (height rl)
                     then create (create l x rl) rx rr
                     else (match rl with
                           | Leaf -> assert_false l x r
                           | Node (t1, rll, rlx, rlr) ->
                             create (create l x rll) rlx (create rlr rx rr)))
             else create l x r
      
      (** val add : positive -> tree -> tree **)
      
      let rec add x = function
      | Leaf -> Node (Z_as_Int._1, Leaf, x, Leaf)
      | Node (h, l, y, r) ->
        (match OrderedPositive.compare x y with
         | LT -> bal (add x l) y r
         | EQ -> Node (h, l, y, r)
         | GT -> bal l y (add x r))
      
      (** val join : tree -> elt -> t -> t **)
      
      let rec join l x r = match l with
      | Leaf -> add x r
      | Node (lh, ll, lx, lr) ->
          let rec join_aux r = match r with
          | Leaf -> add x l
          | Node (rh, rl, rx, rr) ->
            if Z_as_Int.gt_le_dec lh (Z_as_Int.plus rh Z_as_Int._2)
            then bal ll lx (join lr x r)
            else if Z_as_Int.gt_le_dec rh (Z_as_Int.plus lh Z_as_Int._2)
                 then bal (join_aux rl) rx rr
                 else create l x r
          in join_aux r
      
      (** val remove_min : tree -> elt -> t -> t * elt **)
      
      let rec remove_min l x r =
        match l with
        | Leaf -> (r, x)
        | Node (lh, ll, lx, lr) ->
          let (l', m) = remove_min ll lx lr in ((bal l' x r), m)
      
      (** val merge : tree -> tree -> tree **)
      
      let merge s1 s2 =
        match s1 with
        | Leaf -> s2
        | Node (t0, t1, t2, t3) ->
          (match s2 with
           | Leaf -> s1
           | Node (t4, l2, x2, r2) ->
             let (s2', m) = remove_min l2 x2 r2 in bal s1 m s2')
      
      (** val remove : positive -> tree -> tree **)
      
      let rec remove x = function
      | Leaf -> Leaf
      | Node (t0, l, y, r) ->
        (match OrderedPositive.compare x y with
         | LT -> bal (remove x l) y r
         | EQ -> merge l r
         | GT -> bal l y (remove x r))
      
      (** val concat : tree -> tree -> tree **)
      
      let concat s1 s2 =
        match s1 with
        | Leaf -> s2
        | Node (t0, t1, t2, t3) ->
          (match s2 with
           | Leaf -> s1
           | Node (t4, l2, x2, r2) ->
             let (s2', m) = remove_min l2 x2 r2 in join s1 m s2')
      
      type triple = { t_left : t; t_in : bool; t_right : t }
      
      (** val t_left : triple -> t **)
      
      let t_left x = x.t_left
      
      (** val t_in : triple -> bool **)
      
      let t_in x = x.t_in
      
      (** val t_right : triple -> t **)
      
      let t_right x = x.t_right
      
      (** val split : positive -> tree -> triple **)
      
      let rec split x = function
      | Leaf -> { t_left = Leaf; t_in = false; t_right = Leaf }
      | Node (t0, l, y, r) ->
        (match OrderedPositive.compare x y with
         | LT ->
           let { t_left = ll; t_in = b; t_right = rl } = split x l in
           { t_left = ll; t_in = b; t_right = (join rl y r) }
         | EQ -> { t_left = l; t_in = true; t_right = r }
         | GT ->
           let { t_left = rl; t_in = b; t_right = rr } = split x r in
           { t_left = (join l y rl); t_in = b; t_right = rr })
      
      (** val inter : tree -> tree -> tree **)
      
      let rec inter s1 s2 =
        match s1 with
        | Leaf -> Leaf
        | Node (t0, l1, x1, r1) ->
          (match s2 with
           | Leaf -> Leaf
           | Node (t1, t2, t3, t4) ->
             let { t_left = l2'; t_in = pres; t_right = r2' } = split x1 s2
             in
             if pres
             then join (inter l1 l2') x1 (inter r1 r2')
             else concat (inter l1 l2') (inter r1 r2'))
      
      (** val diff : tree -> tree -> tree **)
      
      let rec diff s1 s2 =
        match s1 with
        | Leaf -> Leaf
        | Node (t0, l1, x1, r1) ->
          (match s2 with
           | Leaf -> s1
           | Node (t1, t2, t3, t4) ->
             let { t_left = l2'; t_in = pres; t_right = r2' } = split x1 s2
             in
             if pres
             then concat (diff l1 l2') (diff r1 r2')
             else join (diff l1 l2') x1 (diff r1 r2'))
      
      (** val union : tree -> tree -> tree **)
      
      let rec union s1 s2 =
        match s1 with
        | Leaf -> s2
        | Node (t0, l1, x1, r1) ->
          (match s2 with
           | Leaf -> s1
           | Node (t1, t2, t3, t4) ->
             let { t_left = l2'; t_in = x; t_right = r2' } = split x1 s2 in
             join (union l1 l2') x1 (union r1 r2'))
      
      (** val filter : (elt -> bool) -> tree -> tree **)
      
      let rec filter f = function
      | Leaf -> Leaf
      | Node (t0, l, x, r) ->
        let l' = filter f l in
        let r' = filter f r in if f x then join l' x r' else concat l' r'
      
      (** val partition : (elt -> bool) -> t -> t * t **)
      
      let rec partition f = function
      | Leaf -> (Leaf, Leaf)
      | Node (t0, l, x, r) ->
        let (l1, l2) = partition f l in
        let (r1, r2) = partition f r in
        if f x
        then ((join l1 x r1), (concat l2 r2))
        else ((concat l1 r1), (join l2 x r2))
      
      (** val ltb_tree : positive -> tree -> bool **)
      
      let rec ltb_tree x = function
      | Leaf -> true
      | Node (t0, l, y, r) ->
        (match OrderedPositive.compare x y with
         | GT -> (&&) (ltb_tree x l) (ltb_tree x r)
         | _ -> false)
      
      (** val gtb_tree : positive -> tree -> bool **)
      
      let rec gtb_tree x = function
      | Leaf -> true
      | Node (t0, l, y, r) ->
        (match OrderedPositive.compare x y with
         | LT -> (&&) (gtb_tree x l) (gtb_tree x r)
         | _ -> false)
      
      (** val isok : tree -> bool **)
      
      let rec isok = function
      | Leaf -> true
      | Node (t0, l, x, r) ->
        (&&) ((&&) ((&&) (isok l) (isok r)) (ltb_tree x l)) (gtb_tree x r)
      
      module MX = 
       struct 
        module OrderTac = 
         struct 
          module OTF = 
           struct 
            type t = positive
            
            (** val compare : positive -> positive -> comparison **)
            
            let compare : positive -> positive -> comparison = fun x y ->
              match OrderedPositive.compare x y with
              | LT -> Lt
              | EQ -> Eq
              | GT -> Gt
            
            (** val eq_dec : positive -> positive -> bool **)
            
            let eq_dec x y =
              X'.eq_dec x y
           end
          
          module TO = 
           struct 
            type t = positive
            
            (** val compare : positive -> positive -> comparison **)
            
            let compare : positive -> positive -> comparison = fun x y ->
              match OrderedPositive.compare x y with
              | LT -> Lt
              | EQ -> Eq
              | GT -> Gt
            
            (** val eq_dec : positive -> positive -> bool **)
            
            let eq_dec x y =
              OTF.eq_dec x y
           end
         end
        
        (** val eq_dec : positive -> positive -> bool **)
        
        let eq_dec x y =
          X'.eq_dec x y
        
        (** val lt_dec : positive -> positive -> bool **)
        
        let lt_dec x y =
          let c =
            coq_CompSpec2Type x y
              (match OrderedPositive.compare x y with
               | LT -> Lt
               | EQ -> Eq
               | GT -> Gt)
          in
          (match c with
           | CompLtT -> true
           | _ -> false)
        
        (** val eqb : positive -> positive -> bool **)
        
        let eqb x y =
          if eq_dec x y then true else false
       end
      
      type coq_R_min_elt =
      | R_min_elt_0 of tree
      | R_min_elt_1 of tree * Z_as_Int.t * tree * positive * tree
      | R_min_elt_2 of tree * Z_as_Int.t * tree * positive * tree
         * Z_as_Int.t * tree * positive * tree * elt option * coq_R_min_elt
      
      type coq_R_max_elt =
      | R_max_elt_0 of tree
      | R_max_elt_1 of tree * Z_as_Int.t * tree * positive * tree
      | R_max_elt_2 of tree * Z_as_Int.t * tree * positive * tree
         * Z_as_Int.t * tree * positive * tree * elt option * coq_R_max_elt
      
      module L = 
       struct 
        module MO = 
         struct 
          module OrderTac = 
           struct 
            module OTF = 
             struct 
              type t = positive
              
              (** val compare : positive -> positive -> comparison **)
              
              let compare x y =
                match OrderedPositive.compare x y with
                | LT -> Lt
                | EQ -> Eq
                | GT -> Gt
              
              (** val eq_dec : positive -> positive -> bool **)
              
              let eq_dec x y =
                X'.eq_dec x y
             end
            
            module TO = 
             struct 
              type t = positive
              
              (** val compare : positive -> positive -> comparison **)
              
              let compare : positive -> positive -> comparison = fun x y ->
                match OrderedPositive.compare x y with
                | LT -> Lt
                | EQ -> Eq
                | GT -> Gt
              
              (** val eq_dec : positive -> positive -> bool **)
              
              let eq_dec x y =
                OTF.eq_dec x y
             end
           end
          
          (** val eq_dec : positive -> positive -> bool **)
          
          let eq_dec x y =
            X'.eq_dec x y
          
          (** val lt_dec : positive -> positive -> bool **)
          
          let lt_dec x y =
            let c =
              coq_CompSpec2Type x y
                (match OrderedPositive.compare x y with
                 | LT -> Lt
                 | EQ -> Eq
                 | GT -> Gt)
            in
            (match c with
             | CompLtT -> true
             | _ -> false)
          
          (** val eqb : positive -> positive -> bool **)
          
          let eqb x y =
            if eq_dec x y then true else false
         end
       end
      
      (** val flatten_e : enumeration -> elt list **)
      
      let rec flatten_e = function
      | End -> []
      | More (x, t0, r) -> x :: (app (elements t0) (flatten_e r))
      
      type coq_R_bal =
      | R_bal_0 of t * positive * t
      | R_bal_1 of t * positive * t * Z_as_Int.t * tree * positive * tree
      | R_bal_2 of t * positive * t * Z_as_Int.t * tree * positive * tree
      | R_bal_3 of t * positive * t * Z_as_Int.t * tree * positive * 
         tree * Z_as_Int.t * tree * positive * tree
      | R_bal_4 of t * positive * t
      | R_bal_5 of t * positive * t * Z_as_Int.t * tree * positive * tree
      | R_bal_6 of t * positive * t * Z_as_Int.t * tree * positive * tree
      | R_bal_7 of t * positive * t * Z_as_Int.t * tree * positive * 
         tree * Z_as_Int.t * tree * positive * tree
      | R_bal_8 of t * positive * t
      
      type coq_R_remove_min =
      | R_remove_min_0 of tree * elt * t
      | R_remove_min_1 of tree * elt * t * Z_as_Int.t * tree * positive
         * tree * (t * elt) * coq_R_remove_min * t * elt
      
      type coq_R_merge =
      | R_merge_0 of tree * tree
      | R_merge_1 of tree * tree * Z_as_Int.t * tree * positive * tree
      | R_merge_2 of tree * tree * Z_as_Int.t * tree * positive * tree
         * Z_as_Int.t * tree * positive * tree * t * elt
      
      type coq_R_concat =
      | R_concat_0 of tree * tree
      | R_concat_1 of tree * tree * Z_as_Int.t * tree * positive * tree
      | R_concat_2 of tree * tree * Z_as_Int.t * tree * positive * tree
         * Z_as_Int.t * tree * positive * tree * t * elt
      
      type coq_R_inter =
      | R_inter_0 of tree * tree
      | R_inter_1 of tree * tree * Z_as_Int.t * tree * positive * tree
      | R_inter_2 of tree * tree * Z_as_Int.t * tree * positive * tree
         * Z_as_Int.t * tree * positive * tree * t * bool * t * tree
         * coq_R_inter * tree * coq_R_inter
      | R_inter_3 of tree * tree * Z_as_Int.t * tree * positive * tree
         * Z_as_Int.t * tree * positive * tree * t * bool * t * tree
         * coq_R_inter * tree * coq_R_inter
      
      type coq_R_diff =
      | R_diff_0 of tree * tree
      | R_diff_1 of tree * tree * Z_as_Int.t * tree * positive * tree
      | R_diff_2 of tree * tree * Z_as_Int.t * tree * positive * tree
         * Z_as_Int.t * tree * positive * tree * t * bool * t * tree
         * coq_R_diff * tree * coq_R_diff
      | R_diff_3 of tree * tree * Z_as_Int.t * tree * positive * tree
         * Z_as_Int.t * tree * positive * tree * t * bool * t * tree
         * coq_R_diff * tree * coq_R_diff
      
      type coq_R_union =
      | R_union_0 of tree * tree
      | R_union_1 of tree * tree * Z_as_Int.t * tree * positive * tree
      | R_union_2 of tree * tree * Z_as_Int.t * tree * positive * tree
         * Z_as_Int.t * tree * positive * tree * t * bool * t * tree
         * coq_R_union * tree * coq_R_union
     end
    
    module E = 
     struct 
      type t = positive
      
      (** val compare : positive -> positive -> comparison **)
      
      let compare : positive -> positive -> comparison = fun x y ->
        match OrderedPositive.compare x y with
        | LT -> Lt
        | EQ -> Eq
        | GT -> Gt
      
      (** val eq_dec : positive -> positive -> bool **)
      
      let eq_dec x y =
        X'.eq_dec x y
     end
    
    type elt = positive
    
    type t_ =
      Raw.t
      (* singleton inductive, whose constructor was Mkt *)
    
    (** val this : t_ -> Raw.t **)
    
    let this : t_ -> Raw.t = fun t0 ->
      t0
    
    type t = t_
    
    (** val mem : elt -> t -> bool **)
    
    let mem x s =
      Raw.mem x (this s)
    
    (** val add : elt -> t -> t **)
    
    let add x s =
      Raw.add x (this s)
    
    (** val remove : elt -> t -> t **)
    
    let remove x s =
      Raw.remove x (this s)
    
    (** val singleton : elt -> t **)
    
    let singleton x =
      Raw.singleton x
    
    (** val union : t -> t -> t **)
    
    let union s s' =
      Raw.union (this s) (this s')
    
    (** val inter : t -> t -> t **)
    
    let inter s s' =
      Raw.inter (this s) (this s')
    
    (** val diff : t -> t -> t **)
    
    let diff s s' =
      Raw.diff (this s) (this s')
    
    (** val equal : t -> t -> bool **)
    
    let equal s s' =
      Raw.equal (this s) (this s')
    
    (** val subset : t -> t -> bool **)
    
    let subset s s' =
      Raw.subset (this s) (this s')
    
    (** val empty : t **)
    
    let empty =
      Raw.empty
    
    (** val is_empty : t -> bool **)
    
    let is_empty s =
      Raw.is_empty (this s)
    
    (** val elements : t -> elt list **)
    
    let elements s =
      Raw.elements (this s)
    
    (** val choose : t -> elt option **)
    
    let choose s =
      Raw.choose (this s)
    
    (** val fold : (elt -> 'a1 -> 'a1) -> t -> 'a1 -> 'a1 **)
    
    let fold f s x =
      Raw.fold f (this s) x
    
    (** val cardinal : t -> nat **)
    
    let cardinal s =
      Raw.cardinal (this s)
    
    (** val filter : (elt -> bool) -> t -> t **)
    
    let filter f s =
      Raw.filter f (this s)
    
    (** val for_all : (elt -> bool) -> t -> bool **)
    
    let for_all f s =
      Raw.for_all f (this s)
    
    (** val exists_ : (elt -> bool) -> t -> bool **)
    
    let exists_ f s =
      Raw.exists_ f (this s)
    
    (** val partition : (elt -> bool) -> t -> t * t **)
    
    let partition f s =
      let p = Raw.partition f (this s) in ((fst p), (snd p))
    
    (** val eq_dec : t -> t -> bool **)
    
    let eq_dec s s' =
      let b = Raw.equal s s' in if b then true else false
    
    (** val compare : t -> t -> comparison **)
    
    let compare s s' =
      Raw.compare (this s) (this s')
    
    (** val min_elt : t -> elt option **)
    
    let min_elt s =
      Raw.min_elt (this s)
    
    (** val max_elt : t -> elt option **)
    
    let max_elt s =
      Raw.max_elt (this s)
   end
  
  type elt = positive
  
  type t = MSet.t
  
  (** val empty : t **)
  
  let empty =
    MSet.empty
  
  (** val is_empty : t -> bool **)
  
  let is_empty : t -> bool = fun x ->
    MSet.is_empty x
  
  (** val mem : elt -> t -> bool **)
  
  let mem : elt -> t -> bool = fun x y -> 
    MSet.mem x y
  
  (** val add : elt -> t -> t **)
  
  let add : elt -> t -> t = fun x y ->
    MSet.add x y
  
  (** val singleton : elt -> t **)
  
  let singleton : elt -> t = fun x ->
    MSet.singleton x
  
  (** val remove : elt -> t -> t **)
  
  let remove : elt -> t -> t = fun x y ->
    MSet.remove x y
  
  (** val union : t -> t -> t **)
  
  let union : t -> t -> t = fun x y ->
    MSet.union x y
  
  (** val inter : t -> t -> t **)
  
  let inter : t -> t -> t = fun x y ->
    MSet.inter x y
  
  (** val diff : t -> t -> t **)
  
  let diff : t -> t -> t = fun x y ->
    MSet.diff x y
  
  (** val eq_dec : t -> t -> bool **)
  
  let eq_dec x y =
    MSet.eq_dec x y
  
  (** val equal : t -> t -> bool **)
  
  let equal x y =
    MSet.equal x y
  
  (** val subset : t -> t -> bool **)
  
  let subset x y =
    MSet.subset x y
  
  (** val fold : (elt -> 'a1 -> 'a1) -> t -> 'a1 -> 'a1 **)
  
  let fold x x0 x1 =
    MSet.fold x x0 x1
  
  (** val for_all : (elt -> bool) -> t -> bool **)
  
  let for_all x y =
    MSet.for_all x y
  
  (** val exists_ : (elt -> bool) -> t -> bool **)
  
  let exists_ x y =
    MSet.exists_ x y
  
  (** val filter : (elt -> bool) -> t -> t **)
  
  let filter x y =
    MSet.filter x y
  
  (** val partition : (elt -> bool) -> t -> t * t **)
  
  let partition x y =
    MSet.partition x y
  
  (** val cardinal : t -> nat **)
  
  let cardinal x =
    MSet.cardinal x
  
  (** val elements : t -> elt list **)
  
  let elements x =
    MSet.elements x
  
  (** val choose : t -> elt option **)
  
  let choose x =
    MSet.choose x
  
  module MF = 
   struct 
    (** val eqb : positive -> positive -> bool **)
    
    let eqb x y =
      if MSet.E.eq_dec x y then true else false
   end
  
  (** val min_elt : t -> elt option **)
  
  let min_elt x =
    MSet.min_elt x
  
  (** val max_elt : t -> elt option **)
  
  let max_elt x =
    MSet.max_elt x
  
  (** val compare : t -> t -> t coq_Compare **)
  
  let compare : t -> t -> t coq_Compare = fun s s' ->
    let c = coq_CompSpec2Type s s' (MSet.compare s s') in
    (match c with
     | CompEqT -> EQ
     | CompLtT -> LT
     | CompGtT -> GT)
  
  module E = 
   struct 
    type t = positive
    
    (** val compare : positive -> positive -> positive coq_Compare **)
    
    let compare : positive -> positive -> positive coq_Compare = fun x y ->
      OrderedPositive.compare x y
    
    (** val eq_dec : positive -> positive -> bool **)
    
    let eq_dec x y =
      OrderedPositive.eq_dec x y
   end
 end

type compilenv = VSet.t

let is_liftable_var :  compilenv -> expr -> ident option =
  fun cenv  ->
    function
| Evar (id, ty) -> if VSet.mem id cenv then Some id else None
| _ -> None

let make_cast :  expr -> coq_type -> expr =
  fun a tto  ->
  match classify_cast (typeof a) tto with
  | Coq_cast_case_neutral -> a
  | Coq_cast_case_i2i (sz2, si2) ->
    (match sz2 with
     | I32 -> a
     | _ -> Ecast (a, tto))
  | Coq_cast_case_f2f sz2 ->
    (match sz2 with
     | F32 -> Ecast (a, tto)
     | F64 -> a)
  | Coq_cast_case_struct (id1, fld1, id2, fld2) -> a
  | Coq_cast_case_union (id1, fld1, id2, fld2) -> a
  | Coq_cast_case_void -> a
  | _ -> Ecast (a, tto)

let rec simpl_expr :  compilenv -> expr -> expr =
  fun cenv a  ->
    match a with
| Evar (id, ty) ->
  if VSet.mem id cenv then Etempvar (id, ty) else Evar (id, ty)
| Etempvar (id, ty) -> Etempvar (id, ty)
| Ederef (a1, ty) -> Ederef ((simpl_expr cenv a1), ty)
| Eaddrof (a1, ty) -> Eaddrof ((simpl_expr cenv a1), ty)
| Eunop (op, a1, ty) -> Eunop (op, (simpl_expr cenv a1), ty)
| Ebinop (op, a1, a2, ty) ->
  Ebinop (op, (simpl_expr cenv a1), (simpl_expr cenv a2), ty)
| Ecast (a1, ty) -> Ecast ((simpl_expr cenv a1), ty)
| Efield (a1, fld, ty) -> Efield ((simpl_expr cenv a1), fld, ty)
| _ -> a

let simpl_exprlist :  compilenv -> expr list -> expr list =
  fun cenv al  ->
  map (simpl_expr cenv) al

let check_temp :  compilenv -> ident -> unit res =
  fun cenv id  ->
  if VSet.mem id cenv
  then Error ((MSG
         ('b'::('a'::('d'::(' '::('t'::('e'::('m'::('p'::('o'::('r'::('a'::('r'::('y'::(' '::[]))))))))))))))) :: ((CTX
         id) :: []))
  else OK ()

let check_opttemp :  compilenv -> ident option -> unit res =
  fun cenv  ->
    function
| Some id -> check_temp cenv id
| None -> OK ()

let rec simpl_stmt :  compilenv -> statement -> statement res =
  fun cenv  ->
    function
| Sassign (a1, a2) ->
  (match is_liftable_var cenv a1 with
   | Some id -> OK (Sset (id, (make_cast (simpl_expr cenv a2) (typeof a1))))
   | None -> OK (Sassign ((simpl_expr cenv a1), (simpl_expr cenv a2))))
| Sset (id, a) ->
  (match check_temp cenv id with
   | OK x -> OK (Sset (id, (simpl_expr cenv a)))
   | Error msg -> Error msg)
| Scall (optid, a, al) ->
  (match check_opttemp cenv optid with
   | OK x ->
     OK (Scall (optid, (simpl_expr cenv a), (simpl_exprlist cenv al)))
   | Error msg -> Error msg)
| Sbuiltin (optid, ef, tyargs, al) ->
  (match check_opttemp cenv optid with
   | OK x -> OK (Sbuiltin (optid, ef, tyargs, (simpl_exprlist cenv al)))
   | Error msg -> Error msg)
| Ssequence (s1, s2) ->
  (match simpl_stmt cenv s1 with
   | OK x ->
     (match simpl_stmt cenv s2 with
      | OK x0 -> OK (Ssequence (x, x0))
      | Error msg -> Error msg)
   | Error msg -> Error msg)
| Sifthenelse (a, s1, s2) ->
  (match simpl_stmt cenv s1 with
   | OK x ->
     (match simpl_stmt cenv s2 with
      | OK x0 -> OK (Sifthenelse ((simpl_expr cenv a), x, x0))
      | Error msg -> Error msg)
   | Error msg -> Error msg)
| Sloop (s1, s2) ->
  (match simpl_stmt cenv s1 with
   | OK x ->
     (match simpl_stmt cenv s2 with
      | OK x0 -> OK (Sloop (x, x0))
      | Error msg -> Error msg)
   | Error msg -> Error msg)
| Sreturn opta -> OK (Sreturn (option_map (simpl_expr cenv) opta))
| Sswitch (a, ls) ->
  (match simpl_lblstmt cenv ls with
   | OK x -> OK (Sswitch ((simpl_expr cenv a), x))
   | Error msg -> Error msg)
| Slabel (lbl, s0) ->
  (match simpl_stmt cenv s0 with
   | OK x -> OK (Slabel (lbl, x))
   | Error msg -> Error msg)
| x -> OK x

and simpl_lblstmt :     compilenv -> labeled_statements -> labeled_statements res =
  fun cenv  ->
    function
| LSdefault s ->
  (match simpl_stmt cenv s with
   | OK x -> OK (LSdefault x)
   | Error msg -> Error msg)
| LScase (n, s, ls1) ->
  (match simpl_stmt cenv s with
   | OK x ->
     (match simpl_lblstmt cenv ls1 with
      | OK x0 -> OK (LScase (n, x, x0))
      | Error msg -> Error msg)
   | Error msg -> Error msg)

let rec store_params :     compilenv -> (ident * coq_type) list -> statement -> statement =
  fun cenv coqzz_params s  ->
  match coqzz_params with
  | [] -> s
  | p :: coqzz_params' ->
    let (id, ty) = p in
    if VSet.mem id cenv
    then store_params cenv coqzz_params' s
    else Ssequence ((Sassign ((Evar (id, ty)), (Etempvar (id, ty)))),
           (store_params cenv coqzz_params' s))

let rec addr_taken_expr :  expr -> VSet.t =
    function
| Ederef (a1, ty) -> addr_taken_expr a1
| Eaddrof (a1, ty) ->
  (match a1 with
   | Evar (id, ty1) -> VSet.singleton id
   | _ -> addr_taken_expr a1)
| Eunop (op, a1, ty) -> addr_taken_expr a1
| Ebinop (op, a1, a2, ty) ->
  VSet.union (addr_taken_expr a1) (addr_taken_expr a2)
| Ecast (a1, ty) -> addr_taken_expr a1
| Efield (a1, fld, ty) -> addr_taken_expr a1
| _ -> VSet.empty

let rec addr_taken_exprlist :  expr list -> VSet.t =
    function
| [] -> VSet.empty
| a :: l' -> VSet.union (addr_taken_expr a) (addr_taken_exprlist l')

let rec addr_taken_stmt :  statement -> VSet.t =
    function
| Sassign (a, b) -> VSet.union (addr_taken_expr a) (addr_taken_expr b)
| Sset (id, a) -> addr_taken_expr a
| Scall (optid, a, bl) ->
  VSet.union (addr_taken_expr a) (addr_taken_exprlist bl)
| Sbuiltin (optid, ef, tyargs, bl) -> addr_taken_exprlist bl
| Ssequence (s1, s2) -> VSet.union (addr_taken_stmt s1) (addr_taken_stmt s2)
| Sifthenelse (a, s1, s2) ->
  VSet.union (addr_taken_expr a)
    (VSet.union (addr_taken_stmt s1) (addr_taken_stmt s2))
| Sloop (s1, s2) -> VSet.union (addr_taken_stmt s1) (addr_taken_stmt s2)
| Sreturn o ->
  (match o with
   | Some a -> addr_taken_expr a
   | None -> VSet.empty)
| Sswitch (a, ls) -> VSet.union (addr_taken_expr a) (addr_taken_lblstmt ls)
| Slabel (lbl, s0) -> addr_taken_stmt s0
| _ -> VSet.empty

and addr_taken_lblstmt :  labeled_statements -> VSet.t =
    function
| LSdefault s -> addr_taken_stmt s
| LScase (n, s, ls') ->
  VSet.union (addr_taken_stmt s) (addr_taken_lblstmt ls')

let add_local_variable :     VSet.t -> (ident * coq_type) -> compilenv -> compilenv =
  fun atk id_ty cenv  ->
  let (id, ty) = id_ty in
  (match access_mode ty with
   | By_value chunk -> if VSet.mem id atk then cenv else VSet.add id cenv
   | _ -> cenv)

let cenv_for :  coq_function -> compilenv =
  fun f  ->
  let atk = addr_taken_stmt f.fn_body in
  fold_right (add_local_variable atk) VSet.empty (app f.fn_params f.fn_vars)

let remove_lifted :     compilenv -> (ident * coq_type) list -> (VSet.elt * coq_type) list =
  fun cenv vars  ->
  filter (fun id_ty -> negb (VSet.mem (fst id_ty) cenv)) vars

let add_lifted :     compilenv -> (ident * coq_type) list -> (ident * coq_type) list ->
    (VSet.elt * coq_type) list =
  fun cenv vars1 vars2  ->
  app (filter (fun id_ty -> VSet.mem (fst id_ty) cenv) vars1) vars2

let transf_function :  coq_function -> coq_function res =
  fun f  ->
  let cenv = cenv_for f in
  (match assertion
           (proj_sumbool
             (list_disjoint_dec ident_eq (var_names f.fn_params)
               (var_names f.fn_temps))) with
   | OK x ->
     (match simpl_stmt cenv f.fn_body with
      | OK x0 ->
        OK { fn_return = f.fn_return; fn_params = f.fn_params; fn_vars =
          (remove_lifted cenv (app f.fn_params f.fn_vars)); fn_temps =
          (add_lifted cenv f.fn_vars f.fn_temps); fn_body =
          (store_params cenv f.fn_params x0) }
      | Error msg -> Error msg)
   | Error msg -> Error msg)

let transf_fundef :  fundef -> fundef res =
    function
| Internal f ->
  (match transf_function f with
   | OK x -> OK (Internal x)
   | Error msg -> Error msg)
| External (ef, targs, tres) -> OK (External (ef, targs, tres))

let transf_program :  program -> program res =
  fun p  ->
  transform_partial_program transf_fundef p

