open BinInt
open BinNums
open Bounds
open Coqlib

let fe_ofs_arg :  coq_Z =
  Z0

type frame_env = { fe_size : coq_Z; fe_ofs_link : coq_Z;
                   fe_ofs_retaddr : coq_Z; fe_ofs_int_local : coq_Z;
                   fe_ofs_int_callee_save : coq_Z;
                   fe_num_int_callee_save : coq_Z;
                   fe_ofs_float_local : coq_Z;
                   fe_ofs_float_callee_save : coq_Z;
                   fe_num_float_callee_save : coq_Z; fe_stack_data : 
                   coq_Z }

let fe_size :  frame_env -> coq_Z =
  fun x  ->
    x.fe_size

let fe_ofs_link :  frame_env -> coq_Z =
  fun x  ->
    x.fe_ofs_link

let fe_ofs_retaddr :  frame_env -> coq_Z =
  fun x  ->
    x.fe_ofs_retaddr

let fe_ofs_int_local :  frame_env -> coq_Z =
  fun x  ->
    x.fe_ofs_int_local

let fe_ofs_int_callee_save :  frame_env -> coq_Z =
  fun x  ->
    x.fe_ofs_int_callee_save

let fe_num_int_callee_save :  frame_env -> coq_Z =
  fun x  ->
    x.fe_num_int_callee_save

let fe_ofs_float_local :  frame_env -> coq_Z =
  fun x  ->
    x.fe_ofs_float_local

let fe_ofs_float_callee_save :  frame_env -> coq_Z =
  fun x  ->
    x.fe_ofs_float_callee_save

let fe_num_float_callee_save :  frame_env -> coq_Z =
  fun x  ->
    x.fe_num_float_callee_save

let fe_stack_data :  frame_env -> coq_Z =
  fun x  ->
    x.fe_stack_data

let make_env :  bounds -> frame_env =
  fun b  ->
  let olink = Z.mul (Zpos (Coq_xO (Coq_xO Coq_xH))) b.bound_outgoing in
  let oil = Z.add olink (Zpos (Coq_xO (Coq_xO Coq_xH))) in
  let oics =
    Z.add oil (Z.mul (Zpos (Coq_xO (Coq_xO Coq_xH))) b.bound_int_local)
  in
  let oendi =
    Z.add oics
      (Z.mul (Zpos (Coq_xO (Coq_xO Coq_xH))) b.bound_int_callee_save)
  in
  let ofl = align oendi (Zpos (Coq_xO (Coq_xO (Coq_xO Coq_xH)))) in
  let ofcs =
    Z.add ofl
      (Z.mul (Zpos (Coq_xO (Coq_xO (Coq_xO Coq_xH)))) b.bound_float_local)
  in
  let ostkdata =
    Z.add ofcs
      (Z.mul (Zpos (Coq_xO (Coq_xO (Coq_xO Coq_xH))))
        b.bound_float_callee_save)
  in
  let oretaddr =
    align (Z.add ostkdata b.bound_stack_data) (Zpos (Coq_xO (Coq_xO Coq_xH)))
  in
  let sz = Z.add oretaddr (Zpos (Coq_xO (Coq_xO Coq_xH))) in
  { fe_size = sz; fe_ofs_link = olink; fe_ofs_retaddr = oretaddr;
  fe_ofs_int_local = oil; fe_ofs_int_callee_save = oics;
  fe_num_int_callee_save = b.bound_int_callee_save; fe_ofs_float_local = ofl;
  fe_ofs_float_callee_save = ofcs; fe_num_float_callee_save =
  b.bound_float_callee_save; fe_stack_data = ostkdata }

