open BinInt
open BinNums
open Coqlib
open Datatypes
open EqNat
open Integers
open Maps

module IntIndexed : 
 sig 
  type t = Int.int
  
  val index : Int.int -> positive
  
  val eq : Int.int -> Int.int -> bool
 end

module IntMap : 
 sig 
  type elt = IntIndexed.t
  
  val elt_eq : IntIndexed.t -> IntIndexed.t -> bool
  
  type 'x t = 'x PMap.t
  
  val init : 'a1 -> 'a1 * 'a1 PTree.t
  
  val get : IntIndexed.t -> 'a1 t -> 'a1
  
  val set : IntIndexed.t -> 'a1 -> 'a1 t -> 'a1 * 'a1 PTree.t
  
  val map : ('a1 -> 'a2) -> 'a1 t -> 'a2 t
 end

type table = (Int.int * nat) list

type comptree =
| CTaction of nat
| CTifeq of Int.int * nat * comptree
| CTiflt of Int.int * comptree * comptree
| CTjumptable of Int.int * Int.int * nat list * comptree

val split_lt : Int.int -> table -> table * table

val split_eq : Int.int -> table -> nat option * table

val split_between :
  nat -> Int.int -> Int.int -> table -> nat IntMap.t * table

val refine_low_bound : coq_Z -> coq_Z -> coq_Z

val refine_high_bound : coq_Z -> coq_Z -> coq_Z

val validate_jumptable : nat IntMap.t -> nat list -> Int.int -> bool

val validate : nat -> table -> comptree -> coq_Z -> coq_Z -> bool

val validate_switch : nat -> table -> comptree -> bool

