open AST
open BinNums
open Conventions
open Coqlib
open Datatypes
open Maps
open Op
open RTL
open Registers

let rec is_return :  nat -> coq_function -> node -> reg -> bool =
  fun n f pc rret  ->
  match n with
  | O -> false
  | S n' ->
    (match PTree.get pc f.fn_code with
     | Some i ->
       (match i with
        | Inop s -> is_return n' f s rret
        | Iop (op, args, dst, s) ->
          (match is_move_operation op args with
           | Some src ->
             if Reg.eq rret src then is_return n' f s dst else false
           | None -> false)
        | Ireturn o ->
          (match o with
           | Some r -> proj_sumbool (Reg.eq r rret)
           | None -> true)
        | _ -> false)
     | None -> false)

let niter :  nat =
  S (S (S (S (S O))))

let transf_instr :  coq_function -> node -> instruction -> instruction =
  fun f pc instr  ->
    match instr with
| Icall (sig0, ros, args, res, s) ->
  if (&&) ((&&) (is_return niter f s res) (tailcall_is_possible sig0))
       (proj_sumbool (opt_typ_eq sig0.sig_res f.fn_sig.sig_res))
  then Itailcall (sig0, ros, args)
  else instr
| _ -> instr

let transf_function :  coq_function -> coq_function =
  fun f  ->
  if zeq f.fn_stacksize Z0 then transf_function (transf_instr f) f else f

let transf_fundef :  fundef -> fundef =
  fun fd  ->
  transf_fundef transf_function fd

let transf_program :  program -> program =
  fun p  ->
  transform_program transf_fundef p

