open AST
open BinInt
open BinNums
open Coqlib
open Datatypes
open Floats
open Integers

type block = coq_Z

val eq_block : coq_Z -> coq_Z -> bool

type coq_val =
| Vundef
| Vint of Int.int
| Vfloat of float
| Vptr of block * Int.int

val coq_Vzero : coq_val

val coq_Vtrue : coq_val

val coq_Vfalse : coq_val

module Val : 
 sig 
  val neg : coq_val -> coq_val
  
  val negf : coq_val -> coq_val
  
  val absf : coq_val -> coq_val
  
  val maketotal : coq_val option -> coq_val
  
  val intoffloat : coq_val -> coq_val option
  
  val intuoffloat : coq_val -> coq_val option
  
  val floatofint : coq_val -> coq_val option
  
  val floatofintu : coq_val -> coq_val option
  
  val floatofwords : coq_val -> coq_val -> coq_val
  
  val negint : coq_val -> coq_val
  
  val notint : coq_val -> coq_val
  
  val of_bool : bool -> coq_val
  
  val boolval : coq_val -> coq_val
  
  val notbool : coq_val -> coq_val
  
  val zero_ext : coq_Z -> coq_val -> coq_val
  
  val sign_ext : coq_Z -> coq_val -> coq_val
  
  val singleoffloat : coq_val -> coq_val
  
  val add : coq_val -> coq_val -> coq_val
  
  val sub : coq_val -> coq_val -> coq_val
  
  val mul : coq_val -> coq_val -> coq_val
  
  val divs : coq_val -> coq_val -> coq_val option
  
  val mods : coq_val -> coq_val -> coq_val option
  
  val divu : coq_val -> coq_val -> coq_val option
  
  val modu : coq_val -> coq_val -> coq_val option
  
  val add_carry : coq_val -> coq_val -> coq_val -> coq_val
  
  val coq_and : coq_val -> coq_val -> coq_val
  
  val coq_or : coq_val -> coq_val -> coq_val
  
  val xor : coq_val -> coq_val -> coq_val
  
  val shl : coq_val -> coq_val -> coq_val
  
  val shr : coq_val -> coq_val -> coq_val
  
  val shr_carry : coq_val -> coq_val -> coq_val
  
  val shrx : coq_val -> coq_val -> coq_val option
  
  val shru : coq_val -> coq_val -> coq_val
  
  val rolm : coq_val -> Int.int -> Int.int -> coq_val
  
  val ror : coq_val -> coq_val -> coq_val
  
  val addf : coq_val -> coq_val -> coq_val
  
  val subf : coq_val -> coq_val -> coq_val
  
  val mulf : coq_val -> coq_val -> coq_val
  
  val divf : coq_val -> coq_val -> coq_val
  
  val cmp_bool : comparison -> coq_val -> coq_val -> bool option
  
  val cmp_different_blocks : comparison -> bool option
  
  val cmpu_bool :
    (block -> coq_Z -> bool) -> comparison -> coq_val -> coq_val -> bool
    option
  
  val cmpf_bool : comparison -> coq_val -> coq_val -> bool option
  
  val of_optbool : bool option -> coq_val
  
  val cmp : comparison -> coq_val -> coq_val -> coq_val
  
  val cmpu :
    (block -> coq_Z -> bool) -> comparison -> coq_val -> coq_val -> coq_val
  
  val cmpf : comparison -> coq_val -> coq_val -> coq_val
  
  val load_result : memory_chunk -> coq_val -> coq_val
 end

type meminj = block -> (block * coq_Z) option

