#!/usr/bin/python
import re, sys, os, os.path, shutil

src_path = "../extraction"
dst_path = "./converted"

val_comment_begin = re.compile('^\(\*\* val (.+) :(.*)$')
val_comment_end = re.compile('^(.*) \*\*\)$')
let_begin = re.compile('^(let(?: rec)?|and) ([^ ]+) (.*)=(.*)$')
ml_filetype = re.compile('^.*\.ml$')
mli_filetype = re.compile('^.*\.mli$')
single_identifier = re.compile('^ +([_a-zA-Z0-9]+) *$')
reserved_word = re.compile('\\b(const|base|params|default)\\b')

# let Coq_existT (x, r)
deconstructor_let = re.compile('let +([^ ]+) *\((.*)\) =')
inner_parenthesis = re.compile('\([^()]*\)')

def unreserve(line):
  return reserved_word.sub('coqzz_\\1', line)

def compute_arity(ml_type):
  while True:
    ml_type2 = inner_parenthesis.sub('', ml_type)
    if ml_type == ml_type2: break
    ml_type = ml_type2
  return ml_type.count('->')

def gobble_type(infile, last):
  acc = ''
  while True:
    match = val_comment_end.match(last)
    if match:
      acc += match.group(1)
      break
    acc += last
    last = infile.readline()
    if last == '': break
  return acc

def dummy_args(arity):
  s = ''
  for i in range(arity):
    s += 'x' + str(i) + ' '
  return s

def process_file(infile, outfile):
  eta_expansed = False
  while True:
    line = infile.readline()
    if line == '': break
    match = val_comment_begin.match(line)
    if match:
      eta_expansed = False
      val_id = match.group(1)
      val_type = gobble_type(infile, match.group(2))
      while True:
        line = infile.readline()
        if line <> "\n": break
      match = let_begin.match(line)
      assert match
      decl_prefix = match.group(1)
      val_id2 = match.group(2)
      assert val_id == val_id2
      args = match.group(3)
      tail = match.group(4)
      print >>outfile, decl_prefix,
      print >>outfile, unreserve(val_id), ":", unreserve(val_type), "="
      if args <> '':
        print >>outfile, "  fun", unreserve(args), "->"
        arity = 0
      else:
        arity = compute_arity(val_type)
      if tail <> '':
        print >>outfile, "  ", unreserve(tail)
    else:
      match = single_identifier.match(line)
      if line == '\n':
        print >>outfile
      else:
        if match and arity > 0:
          assert not eta_expansed
          eta_expansed = True
          args = dummy_args(arity)
          print >>outfile, "fun", args, "->", match.group(1), args
        else:
          line = line.replace('assert false', 'failwith "absurd case"')
          line = deconstructor_let.sub('let (\\1 (\\2)) =', line)
          print >>outfile, unreserve(line),
      arity = 0

try:
  os.mkdir(dst_path)
except OSError:
  pass

for filename in os.listdir(src_path):
  print filename
  if ml_filetype.match(filename):
    with open(os.path.join(src_path, filename), 'r') as infile:
      with open(os.path.join(dst_path, filename), 'w') as outfile:
        process_file(infile, outfile)
  elif mli_filetype.match(filename):
    shutil.copy(os.path.join(src_path, filename), os.path.join(dst_path, filename))
